# cython: language_level=3
from libc.stdint cimport int64_t, uint64_t

cdef extern from "include_glew.h":
    ctypedef struct _cl_context:
        pass
    ctypedef struct _cl_event:
        pass
    ctypedef struct __GLsync:
        pass

    ctypedef unsigned short wchar_t
    ctypedef int ptrdiff_t

    ctypedef unsigned int GLenum
    ctypedef unsigned int GLbitfield
    ctypedef unsigned int GLuint
    ctypedef int GLint
    ctypedef int GLsizei
    ctypedef char GLchar
    ctypedef unsigned char GLboolean
    ctypedef signed char GLbyte
    ctypedef short GLshort
    ctypedef unsigned char GLubyte
    ctypedef unsigned short GLushort
    ctypedef unsigned long GLulong
    ctypedef float GLfloat
    ctypedef float GLclampf
    ctypedef double GLdouble
    ctypedef double GLclampd
    ctypedef int GLfixed
    ctypedef int GLclampx
    ctypedef void GLvoid

    ctypedef int64_t GLint64EXT
    ctypedef uint64_t GLuint64EXT
    ctypedef GLint64EXT GLint64
    ctypedef GLuint64EXT GLuint64
    ctypedef __GLsync *GLsync
    ctypedef char GLcharARB
    ctypedef ptrdiff_t GLintptr
    ctypedef ptrdiff_t GLsizeiptr
    ctypedef _cl_context *cl_context
    ctypedef _cl_event *cl_event
    ctypedef unsigned int GLhandleARB
    ctypedef ptrdiff_t GLintptrARB
    ctypedef ptrdiff_t GLsizeiptrARB
    ctypedef void* GLeglClientBufferEXT
    ctypedef unsigned short GLhalf
    ctypedef GLintptr GLvdpauSurfaceNV
    ctypedef long GLVULKANPROCNV

    ctypedef void *GLeglImageOES  # GL_EXT_EGL_image_storage

    ctypedef void (__stdcall *GLDEBUGPROCAMD)(GLuint id, GLenum category, GLenum severity, GLsizei length, GLchar *message, GLvoid *userParam)
    ctypedef void (__stdcall *GLDEBUGPROCARB)(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, GLchar *message, GLvoid *userParam)

    ctypedef void (__stdcall *GLDEBUGPROC)(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, const GLchar* message, GLvoid* userParam)
    ctypedef void (__stdcall *GLLOGPROCREGAL)(GLenum stream, GLsizei length, const GLchar *message, GLvoid *context)


    GLenum glewInit()
    GLboolean glewIsSupported(char *name)
    GLboolean glewIsExtensionSupported(char *name)
    GLboolean glewGetExtension(char* name)
    GLubyte *glewGetErrorString(GLenum error)
    GLubyte *glewGetString(GLenum name)


    enum:
        GL_VARIANT_DATATYPE_EXT
        GL_UNSIGNED_SHORT_4_4_4_4_REV
        GL_4PASS_2_EXT
        GL_RGBA16F
        GL_HISTOGRAM_LUMINANCE_SIZE
        GL_VERTEX_PROGRAM_TWO_SIDE_ARB
        GLEW_NV_vertex_array_range2
        GL_MODELVIEW21_ARB
        GL_CONTEXT_PROFILE_MASK
        GL_CURRENT_RASTER_DISTANCE
        GLEW_NV_generate_mipmap_sRGB
        GLEW_ARB_conditional_render_inverted
        GL_HISTOGRAM_EXT
        GL_FOG_END
        GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX
        GL_MATRIX11_ARB
        GL_STATIC_ATI
        GL_CONVOLUTION_FORMAT_EXT
        GL_DRAW_INDIRECT_ADDRESS_NV
        GL_SIGNED_LUMINANCE_NV
        GLEW_ARB_texture_compression
        GL_GEOMETRY_SHADER_EXT
        GL_VERTEX_ATTRIB_ARRAY_ENABLED
        GL_PACK_ALIGNMENT
        GL_VIEWPORT_BOUNDS_RANGE
        GL_EXT_multi_draw_arrays
        GL_SYNC_FLAGS_APPLE
        GLEW_NV_vertex_array_range
        GL_UNPACK_COMPRESSED_BLOCK_HEIGHT
        GL_ONE_MINUS_SRC_ALPHA
        GL_DEBUG_SOURCE_API_ARB
        GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB
        GL_IMAGE_BUFFER_EXT
        GLEW_ARB_texture_rgb10_a2ui
        GL_SGIX_packed_6bytes
        GL_RGBA_SIGNED_COMPONENTS_EXT
        GL_MAX_VERTEX_VARYING_COMPONENTS_ARB
        GL_SGI_complex_type
        GL_MAX_SPARSE_TEXTURE_SIZE_ARB
        GL_NV_float_buffer
        GL_SHADER_STORAGE_BUFFER_BINDING
        GL_LOGIC_OP_MODE
        GL_COPY_PIXEL_TOKEN
        GL_RGBA32UI
        GL_DRAW_BUFFER0_ARB
        GL_3DC_X_AMD
        GL_COMPRESSED_SRGB_EXT
        GL_FLOAT_MAT3x4_NV
        GLEW_NV_transform_feedback
        GLEW_ARB_ES3_1_compatibility
        GL_YCBCR_422_APPLE
        GL_GREEN_BIAS
        GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_FAST_SIZE_EXT
        GL_READ_FRAMEBUFFER_BINDING_NV
        GL_MAP1_GRID_SEGMENTS
        GL_NUM_FRAGMENT_REGISTERS_ATI
        GL_BLEND_EQUATION_EXT
        GL_MAP1_TANGENT_EXT
        GLEW_APPLE_texture_range
        GL_MAX_IMAGE_UNITS
        GL_AMD_compressed_ATC_texture
        GL_MAX_PROJECTION_STACK_DEPTH
        GL_VIEW_CLASS_S3TC_DXT1_RGBA
        GL_DRAW_BUFFER4_ATI
        GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB
        GL_UNSIGNALED
        GL_VIEWPORT_POSITION_W_SCALE_X_COEFF_NV
        GL_TESS_GEN_SPACING
        GL_TEXTURE_ALPHA_TYPE
        GL_COMPRESSED_RGBA_ASTC_4x4_KHR
        GL_SOURCE0_ALPHA_ARB
        GL_INTERNALFORMAT_ALPHA_SIZE
        GL_TEXTURE_CUBE_MAP_ARRAY_EXT
        GL_TRANSFORM_FEEDBACK_BUFFER_START_NV
        GL_ALL_COMPLETED_NV
        GL_TEXTURE_COMPRESSION_HINT_ARB
        GLEW_INTEL_fragment_shader_ordering
        GL_SAMPLES_3DFX
        GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI
        GL_SGIX_bali_g_instruments
        GL_SHADER_COMPILER
        GL_MAP1_VERTEX_ATTRIB5_4_NV
        GL_TEXTURE_BORDER_VALUES_NV
        GL_CLIP_DISTANCE5
        GL_MAP2_INDEX_NURBS_SGIX
        GL_ARB_spirv_extensions
        GL_IMAGE_1D_ARRAY
        GL_READ_WRITE_ARB
        GL_PIXEL_MIN_FILTER_EXT
        GLEW_EXT_framebuffer_blit
        GL_TIME_ELAPSED
        GLEW_ARB_derivative_control
        GL_DRAW_BUFFER1_ATI
        GL_MAX_VIEWPORT_DIMS
        GL_INT_SAMPLER_2D_ARRAY_EXT
        GL_KHR_texture_compression_astc_ldr
        GL_MAP1_INDEX_NURBS_SGIX
        GL_CLOSE_PATH_NV
        GL_TEXTURE_CUBE_MAP_ARRAY
        GL_POINT_SPRITE_CULL_CLIP_ATIX
        GL_ARB_seamless_cube_map
        GL_RGBA8_OES
        GL_GEOMETRY_OUTPUT_TYPE
        GLEW_ARB_robustness_application_isolation
        GL_TRANSFORM_FEEDBACK_BUFFER_NV
        GL_EXT_texture_compression_astc_decode_mode
        GL_FLOAT_VEC3
        GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV
        GL_WINDOW_RECTANGLE_EXT
        GL_SGI_complex
        GL_MODELVIEW
        GL_SWIZZLE_STRQ_DQ_ATI
        GL_APPLE_sync
        GL_TEXTURE
        GL_MATRIX2_NV
        GL_MAX_VERTEX_UNIFORM_COMPONENTS
        GL_OUTPUT_VERTEX_EXT
        GL_ARB_shader_image_size
        GL_LUMINANCE_ALPHA_FLOAT32_ATI
        GL_SCREEN_KHR
        GLEW_KHR_texture_compression_astc_hdr
        GL_RGB8I_EXT
        GL_INCR
        GL_INTENSITY8_SNORM
        GL_ONE_MINUS_DST_ALPHA
        GL_EXT_texture_env_dot3
        GL_VIRTUAL_PAGE_SIZE_Z_AMD
        GL_SCISSOR_TEST
        GL_COORD_REPLACE_NV
        GL_NUM_INSTRUCTIONS_TOTAL_ATI
        GL_RGB10_A2_EXT
        GL_ARB_texture_rectangle
        GLEW_EXT_texture_compression_bptc
        GL_MATRIX12_ARB
        GL_MAX_FRAMEBUFFER_SAMPLES
        GL_PROGRAM_INSTRUCTIONS_ARB
        GL_UNSIGNED_IDENTITY_NV
        GL_VBO_FREE_MEMORY_ATI
        GL_BUFFER_SIZE_ARB
        GL_RELATIVE_ROUNDED_RECT4_NV
        GL_ACTIVE_VARYINGS_NV
        GL_DEPTH_SAMPLES_NV
        GLEW_ARB_polygon_offset_clamp
        GL_SGIX_texture_add_env
        GL_TEXTURE_GEN_T
        GL_VERTEX_SUBROUTINE
        GL_SAMPLER_2D_SHADOW
        GL_NOR
        GL_QUERY_BY_REGION_NO_WAIT_NV
        GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI
        GL_COLOR_ATTACHMENT10
        GL_TEXTURE_BINDING_RENDERBUFFER_NV
        GL_HALF_APPLE
        GL_CONSERVATIVE_RASTER_DILATE_NV
        GLEW_EXT_packed_pixels
        GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI
        GL_ARB_copy_image
        GL_POST_CONVOLUTION_ALPHA_SCALE
        GL_VERTEX_ARRAY_RANGE_VALID_NV
        GL_SGIX_texture_mipmap_anisotropic
        GL_VIDEO_COLOR_CONVERSION_MIN_NV
        GL_CLAMP_READ_COLOR_ARB
        GL_ALPHA12
        GL_RED_EXT
        GL_4PASS_0_EXT
        GL_COMPRESSED_RGBA_BPTC_UNORM_EXT
        GL_EVAL_VERTEX_ATTRIB2_NV
        GL_HISTOGRAM_FORMAT_EXT
        GL_COMBINER_BIAS_NV
        GL_BLEND_SRC_RGB_EXT
        GL_TEXTURE_COMPARE_FUNC_ARB
        GL_CLIPPING_INPUT_PRIMITIVES_ARB
        GL_INDEX_ARRAY_LIST_STRIDE_IBM
        GL_TRANSPOSE_AFFINE_2D_NV
        GL_INVALID_FRAMEBUFFER_OPERATION
        GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE
        GL_SAMPLE_LOCATION_PIXEL_GRID_HEIGHT_NV
        GL_CONSTANT_ALPHA
        GL_TEXTURE_RED_TYPE_ARB
        GL_MODULATE_SIGNED_ADD_ATI
        GL_UNDEFINED_VERTEX
        GL_MODELVIEW9_ARB
        GL_COMPRESSED_RGBA_S3TC_DXT3_NV
        GL_OBJECT_ACTIVE_ATTRIBUTES_ARB
        GL_MAP1_VERTEX_ATTRIB2_4_NV
        GL_R5_G6_B5_ICC_SGIX
        GLEW_EXT_point_parameters
        GLEW_ARB_point_parameters
        GL_VERTEX_ATTRIB_ARRAY15_NV
        GL_SPIR_V_BINARY_ARB
        GL_VERTEX_ARRAY_BINDING_APPLE
        GLEW_ARB_sync
        GL_ACCUM_ALPHA_BITS
        GLEW_EXT_window_rectangles
        GL_RGB32I_EXT
        GL_NUM_VIRTUAL_PAGE_SIZES_EXT
        GL_TRANSFORM_FEEDBACK_BUFFER_START
        GL_ALPHA_MAX_CLAMP_INGR
        GL_TANGENT_ARRAY_EXT
        GL_DRAW_BUFFER10_EXT
        GLEW_QCOM_framebuffer_foveated
        GL_BUFFER_DATA_SIZE
        GL_BUFFER_MAP_POINTER
        GL_MAX
        GL_INTERNALFORMAT_SUPPORTED
        GL_VERTEX_ATTRIB_MAP2_ORDER_APPLE
        GLEW_ARB_uniform_buffer_object
        GL_INT64_NV
        GL_SPIR_V_BINARY
        GL_VARIABLE_B_NV
        GL_NUM_COMPRESSED_TEXTURE_FORMATS
        GL_ASYNC_MARKER_SGIX
        GL_LUMINANCE_SIGNED_SGIX
        GL_COLOR_ATTACHMENT12_EXT
        GL_COMPARE_REF_TO_TEXTURE
        GLEW_NV_primitive_restart
        GL_FLOAT_RGBA_MODE_NV
        GL_TEXTURE4
        GL_GEOMETRY_TEXTURE
        GLEW_NV_texture_compression_latc
        GL_QUERY_RESULT_AVAILABLE_ANGLE
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR
        GL_EXT_provoking_vertex
        GL_OUTPUT_TEXTURE_COORD8_EXT
        GLEW_KHR_parallel_shader_compile
        GL_FLOAT16_MAT4x2_AMD
        GL_FIXED_ONLY_ARB
        GL_TEXTURE_COORD_ARRAY_STRIDE
        GL_MODELVIEW22_ARB
        GL_ARB_get_texture_sub_image
        GL_UNPACK_CLIENT_STORAGE_APPLE
        GL_TEXTURE_MAX_CLAMP_S_SGIX
        GL_SAMPLE_COVERAGE_VALUE
        GL_TRANSFORM_FEEDBACK_BUFFER_STRIDE
        GL_MAX_FRAGMENT_UNIFORM_VECTORS
        GL_HISTOGRAM_WIDTH
        GL_UNSIGNED_INT_5_9_9_9_REV_EXT
        GL_ARB_robustness_application_isolation
        GL_EXT_texture_buffer_object
        GLEW_SGIX_depth_texture
        GL_GPU_MEMORY_INFO_EVICTION_COUNT_NVX
        GL_NV_shadow_samplers_array
        GL_NV_explicit_multisample
        GL_DATA_BUFFER_AMD
        GL_ELEMENT_ARRAY_BUFFER_ARB
        GL_VARIABLE_C_NV
        GLEW_EXT_blend_logic_op
        GL_OUTPUT_TEXTURE_COORD3_EXT
        GL_FRAGMENT_SHADER_ATI
        GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY
        GLEW_SGIX_fragments_instrument
        GL_SPOT_EXPONENT
        GL_MAX_GEOMETRY_OUTPUT_VERTICES
        GL_EXT_texture_sRGB_R8
        GL_R16_SNORM
        GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER
        GL_NV_vertex_program3
        GL_COMPRESSED_RED_RGTC1_EXT
        GL_OP_RECIP_SQRT_EXT
        GL_COMPRESSED_RG11_EAC
        GL_PERFQUERY_COUNTER_DESC_LENGTH_MAX_INTEL
        GL_SRC0_ALPHA
        GLEW_ARB_indirect_parameters
        GL_SGIX_resample
        GL_R16F
        GL_MAP2_TEXTURE_COORD_4
        GL_AMD_multi_draw_indirect
        GL_FRAMEBUFFER_FETCH_NONCOHERENT_QCOM
        GL_DEPTH_BOUNDS_EXT
        GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS
        GL_TEXTURE21
        GLEW_ARB_conservative_depth
        GL_LUMINANCE8_ALPHA8_SNORM
        GL_PHASE_SGIX
        GL_CURRENT_QUERY_ARB
        GL_OP_NEGATE_EXT
        GL_PATH_STENCIL_DEPTH_OFFSET_FACTOR_NV
        GL_BINNING_CONTROL_HINT_QCOM
        GL_IMAGE_BINDING_FORMAT
        GL_POLYGON_OFFSET_CLAMP
        GLEW_ARB_copy_image
        GL_ALPHA_FLOAT32_APPLE
        GL_MAX_RED_SGIS
        GL_FOG_INDEX
        GL_MULTICAST_GPUS_NV
        GL_PERSPECTIVE_CORRECTION_HINT
        GL_VERTEX_ARRAY_BUFFER_BINDING
        GL_EXT_EGL_image_array
        GL_SGI_color_matrix
        GL_LEFT
        GL_HP_texture_lighting
        GL_FRAGMENT_LIGHT_SPACE_SGIX
        GL_FLOAT_MAT2x4
        GLEW_EXT_blend_func_extended
        GLEW_ARB_texture_env_crossbar
        GLEW_AMD_shader_atomic_counter_ops
        GL_SPOT_CUTOFF
        GL_PATH_INITIAL_DASH_CAP_NV
        GL_COLOR_MATRIX_HINT
        GL_KHR_robustness
        GL_OP_MOV_EXT
        GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB
        GL_LINES_ADJACENCY
        GL_TEXTURE_BINDING_EXTERNAL_OES
        GL_LIGHT7
        GL_PER_GPU_STORAGE_NV
        GL_EXT_debug_label
        GL_IMAGE_CUBE_EXT
        GL_SIGNED_HILO_NV
        GL_R1UI_C4F_N3F_V3F_SUN
        GL_MAX_COMPUTE_VARIABLE_GROUP_INVOCATIONS_ARB
        GL_EVAL_BIT
        GL_VERSION_1_2_1
        GL_OFFSET_TEXTURE_MATRIX_NV
        GLEW_NV_compute_program5
        GLEW_NV_texgen_emboss
        GLEW_NV_texture_rectangle_compressed
        GL_GEQUAL
        GL_PATH_DASH_OFFSET_RESET_NV
        GL_CONTINUOUS_AMD
        GL_REG_4_ATI
        GL_DEBUG_REGAL
        GL_ELEMENT_ARRAY_APPLE
        GLEW_SGIX_flush_raster
        GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV
        GL_DRAW_BUFFER1_EXT
        GLEW_NV_gpu_program4
        GL_DEBUG_TYPE_MARKER
        GL_TEXTURE_COMPARE_FUNC_EXT
        GL_UNPACK_ALIGNMENT
        GL_PROGRAM_LENGTH_NV
        GLEW_NV_draw_texture
        GLEW_EXT_instanced_arrays
        GLEW_EXT_vertex_array_setXXX
        GLEW_EXT_memory_object_win32
        GL_PIXEL_MAP_I_TO_I_SIZE
        GL_PACK_LSB_FIRST
        GL_TEXTURE_BINDING_3D
        GL_INT_VEC4
        GL_GREMEDY_frame_terminator
        GL_POINT_DISTANCE_ATTENUATION
        GL_TEXTURE_HEIGHT
        GLEW_SGIX_framezoom
        GL_COUNT_DOWN_NV
        GL_MAX_CLIP_DISTANCES_EXT
        GL_EXT_stencil_wrap
        GL_TEXTURE_WIDTH
        GL_ARB_texture_storage_multisample
        GLEW_ARB_stencil_texturing
        GLEW_ARM_shader_framebuffer_fetch_depth_stencil
        GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT
        GL_MAGNITUDE_SCALE_NV
        GL_NUM_VIRTUAL_PAGE_SIZES_ARB
        GLEW_NV_float_buffer
        GLEW_EXT_cmyka
        GL_TEXTURE1_ARB
        GL_SAMPLE_MASK_EXT
        GL_SRC2_ALPHA
        GL_LAYER_PROVOKING_VERTEX_EXT
        GL_FRAGMENT_INTERPOLATION_OFFSET_BITS
        GL_DOUBLE_MAT4
        GL_SURFACE_STATE_NV
        GL_CURRENT_RASTER_NORMAL_EXT
        GL_EXT_texture_compression_astc_decode_mode_rgb9e5
        GLEW_EXT_pixel_transform
        GL_NV_viewport_array2
        GL_CUBE_MAP_XN_SGIX
        GL_PERFQUERY_GLOBAL_CONTEXT_INTEL
        GL_SGIX_datapipe
        GL_BOOL_VEC2
        GL_IGLOO_SWAPTMESH_SGIX
        GL_PATH_ERROR_POSITION_NV
        GLEW_ARB_texture_barrier
        GL_NEAREST_MIPMAP_NEAREST
        GL_FRAMEBUFFER_DEFAULT_SAMPLES
        GL_INT_VEC3
        GL_SPARE0_PLUS_SECONDARY_COLOR_NV
        GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_ANGLE
        GLEW_APPLE_rgb_422
        GLEW_NV_vertex_program1_1
        GL_SRC_ALPHA
        GL_BACK_PRIMARY_COLOR_NV
        GL_SPARSE_STORAGE_BIT_ARB
        GL_EXT_geometry_shader4
        GL_VERTEX_ARRAY_POINTER_EXT
        GLEW_SGI_color_matrix
        GL_DOUBLE_MAT2
        GLEW_NV_instanced_arrays
        GL_OP_EXP_BASE_2_EXT
        GL_TRACK_MATRIX_TRANSFORM_NV
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT
        GL_NV_light_max_exponent
        GLEW_ARB_get_texture_sub_image
        GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB
        GL_VIEWPORT_SWIZZLE_NEGATIVE_W_NV
        GL_MAX_ASYNC_READ_PIXELS_SGIX
        GL_EVAL_VERTEX_ATTRIB4_NV
        GL_MAP2_VERTEX_ATTRIB4_4_NV
        GL_SAMPLER_EXTERNAL_2D_Y2Y_EXT
        GL_COMPRESSED_RGB
        GL_RED_SNORM
        GL_REGAL_log
        GL_UNSIGNED_INT_SAMPLER_CUBE
        GLEW_VERSION_3_1
        GL_INCLUSIVE_EXT
        GL_TEXTURE_FILTER_CONTROL_EXT
        GL_NV_packed_depth_stencil
        GL_RGBA_ICC_SGIX
        GL_MAX_CULL_DISTANCES_EXT
        GL_WEIGHTED_AVERAGE_EXT
        GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT
        GL_VERSION_1_1
        GL_TEXTURE_BINDING_2D_ARRAY_EXT
        GL_MAP1_TEXTURE_COORD_2_NURBS_SGIX
        GL_AMD_seamless_cubemap_per_texture
        GL_MAP_FLUSH_EXPLICIT_BIT
        GL_MULTISAMPLE_BUFFER_BIT5_QCOM
        GL_SAMPLER_CUBE_MAP_ARRAY_EXT
        GL_READ_PIXELS_FORMAT
        GL_LAYOUT_LINEAR_INTEL
        GL_TEXTURE28_ARB
        GLEW_ATIX_texture_env_route
        GL_PROGRAM_PIPELINE
        GL_TEXTURE_RED_SIZE
        GL_RGBA32F
        GL_TIMEOUT_IGNORED
        GL_TEXCOORD1_BIT_PGI
        GL_POINT_SPRITE_R_MODE_NV
        GL_MAX_FOG_LAYERS_POINTS_SGIX
        GL_NV_copy_depth_to_color
        GL_VERTEX_BINDING_STRIDE
        GL_GLYPH_HEIGHT_BIT_NV
        GL_TRIANGLES
        GL_MAX_RATIONAL_EVAL_ORDER_NV
        GL_1PASS_SGIS
        GL_DRAW_BUFFER6_ARB
        GL_ACTIVE_UNIFORMS
        GL_BUMP_ROT_MATRIX_ATI
        GLEW_EXT_sparse_texture
        GL_MODELVIEW2_ARB
        GL_TEXTURE_DEPTH_QCOM
        GL_INTERLACE_SGIX
        GL_RGBA_EXTENDED_RANGE_SGIX
        GL_COLOR_MATRIX
        GL_EXT_shader_framebuffer_fetch
        GL_HP_image_transform
        GL_INTENSITY8I_EXT
        GL_FRONT_FACE
        GL_MIN_MAP_BUFFER_ALIGNMENT
        GL_SHADER_OBJECT_ARB
        GL_HANDLE_TYPE_OPAQUE_WIN32_KMT_EXT
        GL_INTERLACE_READ_INGR
        GL_UNIFORM_BLOCK_NAME_LENGTH
        GL_SGIX_icc_texture
        GL_MAX_CONVOLUTION_WIDTH
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR
        GL_MAX_COMPUTE_WORK_GROUP_COUNT
        GL_MULTISAMPLE_BUFFER_BIT1_QCOM
        GL_ELEMENT_ARRAY_ATI
        GL_IMAGE_TEXEL_SIZE
        GL_COLOR_ATTACHMENT3_EXT
        GL_VIRTUAL_PAGE_SIZE_Z_EXT
        GL_TEXTURE_3D
        GL_APPLE_specular_vector
        GL_UNSIGNED_NORMALIZED_EXT
        GLEW_APPLE_vertex_array_object
        GL_CURRENT_PALETTE_MATRIX_ARB
        GL_MAX_WINDOW_RECTANGLES_EXT
        GL_SLIM8U_SGIX
        GL_TEXTURE28
        GL_PN_TRIANGLES_POINT_MODE_ATI
        GL_PIXEL_MAP_G_TO_G_SIZE
        GL_FOG_COORDINATE_ARRAY_STRIDE_EXT
        GL_ANGLE_framebuffer_blit
        GL_INT_SAMPLER_3D
        GLEW_S3_s3tc
        GL_RGB16UI
        GL_RENDERBUFFER_GREEN_SIZE
        GL_INT_IMAGE_1D_EXT
        GL_OUTPUT_TEXTURE_COORD11_EXT
        GL_STENCIL_OP_VALUE_AMD
        GL_READ_FRAMEBUFFER_BINDING_ANGLE
        GLEW_SGIX_pixel_texture_bits
        GL_COORD_REPLACE
        GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB
        GL_SEPARATE_SPECULAR_COLOR
        GL_MATRIX6_NV
        GL_STORAGE_CLIENT_APPLE
        GL_ALPHA_ICC_SGIX
        GL_FLOAT16_MAT3x2_AMD
        GL_CLAMP_TO_EDGE_EXT
        GL_PIXEL_PACK_BUFFER_BINDING_ARB
        GL_CLIP_DISTANCE3_APPLE
        GL_COMPRESSED_RGBA_ASTC_5x5_KHR
        GL_COMPATIBLE_SUBROUTINES
        GL_LINE_WIDTH_RANGE
        GL_QUERY_RESULT_NO_WAIT_AMD
        GL_VERTEX_ID_NV
        GLEW_ANGLE_texture_compression_dxt3
        GL_WIDE_LINE_HINT_PGI
        GL_MAX_VERTEX_ATTRIB_RELATIVE_OFFSET
        GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS
        GL_MATRIX0_NV
        GL_INCR_WRAP_EXT
        GL_NV_shader_buffer_load
        GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV
        GLEW_SGIX_texture_phase
        GL_UNSIGNED_INT8_VEC4_NV
        GL_INDEX_BITS
        GLEW_SGIX_texture_lod_bias
        GL_UNSIGNED_INT_5_9_9_9_REV_APPLE
        GL_NV_fragment_program2
        GL_SGIX_subsample
        GL_MAX_FRAMEBUFFER_LAYERS
        GL_4PASS_3_SGIS
        GL_FUNC_ADD_EXT
        GL_EXT_clip_cull_distance
        GL_PIXEL_TRANSFORM_2D_MATRIX_EXT
        GL_R32I
        GL_NV_parameter_buffer_object2
        GL_SGIS_multitexture
        GL_DISTANCE_ATTENUATION_EXT
        GL_PARTIAL_SUCCESS_NV
        GL_IMAGE_3D
        GLEW_SGIX_fog_texture
        GLEW_NV_texture_compression_vtc
        GL_MAT_COLOR_INDEXES_BIT_PGI
        GL_OBJECT_BUFFER_SIZE_ATI
        GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS
        GL_ROUNDED_RECT2_NV
        GL_SAMPLER_CUBE_SHADOW
        GL_EXT_vertex_array_setXXX
        GL_SPARSE_BUFFER_PAGE_SIZE_ARB
        GL_PROXY_TEXTURE_COLOR_TABLE_SGI
        GL_SHADER_STORAGE_BUFFER_SIZE
        GLEW_ARB_fragment_shader_interlock
        GL_2PASS_1_EXT
        GL_CULL_FACE_MODE
        GLEW_ARB_texture_query_lod
        GL_MAP1_INDEX
        GL_MAX_WIDTH
        GL_NV_point_sprite
        GL_SGIX_fog_factor_to_alpha
        GL_FOG_TYPE_SGIX
        GL_PERFQUERY_FLUSH_INTEL
        GL_DOUBLE_MAT2x4
        GLEW_AMD_gpu_shader_int16
        GL_MAP_INVALIDATE_BUFFER_BIT_EXT
        GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT
        GL_TIMEOUT_EXPIRED_APPLE
        GL_MAX_CONVOLUTION_WIDTH_EXT
        GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB
        GLEW_NV_half_float
        GL_ALLOW_DRAW_WIN_HINT_PGI
        GL_POLYGON_OFFSET_CLAMP_EXT
        GL_CON_4_ATI
        GL_MALI_SHADER_BINARY_ARM
        GL_DRAW_BUFFER13
        GL_SAMPLE_COVERAGE_INVERT_ARB
        GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT
        GL_LUMINANCE_SNORM
        GL_STENCIL_INDEX4_EXT
        GL_UNPACK_ROW_LENGTH
        GL_MAX_FRAGMENT_LIGHTS_EXT
        GL_TEXTURE_1D_STACK_BINDING_MESAX
        GL_OBJECT_TYPE_APPLE
        GL_TEXTURE12
        GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV
        GL_DRAW_PIXELS_APPLE
        GL_PIXEL_MAP_I_TO_R
        GLEW_AMD_name_gen_delete
        GL_GUILTY_CONTEXT_RESET_ARB
        GL_SLUMINANCE8_ALPHA8_NV
        GLEW_NV_shadow_samplers_array
        GL_COLOR_ATTACHMENT5
        GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT
        GL_RGBA16_EXT
        GL_ALL_STATIC_DATA_IBM
        GLEW_SGIS_texture_filter4
        GL_NORMAL_ARRAY_STRIDE
        GL_QUAD_STRIP
        GL_EXT_misc_attribute
        GL_ELEMENT_ARRAY_TYPE_APPLE
        GL_SIGNED_ALPHA8_NV
        GL_STENCIL_BUFFER_BIT0_QCOM
        GL_PRIMITIVE_RESTART_NV
        GL_ARB_tessellation_shader
        GL_RG16_SNORM_EXT
        GL_MAT_EMISSION_BIT_PGI
        GL_ARB_shader_bit_encoding
        GL_VERTEX_SUBROUTINE_UNIFORM
        GL_DONT_CARE
        GL_AMD_performance_monitor
        GL_SRC0_RGB
        GL_PIXEL_BUFFER_BARRIER_BIT_EXT
        GL_ARB_draw_elements_base_vertex
        GL_SAMPLER_2D_ARRAY_NV
        GL_FONT_GLYPHS_AVAILABLE_NV
        GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV
        GL_OUTPUT_TEXTURE_COORD29_EXT
        GL_MAX_MODELVIEW_STACK_DEPTH
        GL_ATTRIB_ARRAY_SIZE_NV
        GL_RGBA4_EXT
        GL_INT_IMAGE_2D_EXT
        GL_CON_6_ATI
        GL_TRANSPOSE_AFFINE_3D_NV
        GL_UNSIGNED_INT_VEC2_EXT
        GL_CUBIC_EXT
        GL_SLUMINANCE_NV
        GL_NV_pixel_data_range
        GL_MAJOR_VERSION
        GLEW_EXT_multiview_draw_buffers
        GL_QUERY_RESULT_NO_WAIT
        GL_POLYGON_STIPPLE_BIT
        GL_LIGHTEN_NV
        GL_V2F
        GL_PROGRAM_BINDING_ARB
        GLEW_EXT_cull_vertex
        GL_TEXCOORD4_BIT_PGI
        GLEW_SGIX_fragment_lighting_space
        GL_COLOR_INDEX8_EXT
        GL_GEOMETRY_DEFORMATION_BIT_SGIX
        GLEW_ARB_gl_spirv
        GL_BLEND_EQUATION_RGB_EXT
        GL_STACK_UNDERFLOW
        GL_OP_DOT3_EXT
        GL_DRAW_BUFFER12
        GL_RGBA16F_ARB
        GL_DRAW_BUFFER11_NV
        GL_TEXTURE_TARGET_QCOM
        GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI
        GL_ARB_derivative_control
        GL_SPECULAR
        GL_SPARE1_NV
        GL_COMPRESSED_RGBA_ARB
        GLEW_NV_register_combiners2
        GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_ARB
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV
        GLEW_SGI_complex_type
        GL_POLYGON_OFFSET_POINT_NV
        GL_DIFFERENCE_NV
        GL_PIXEL_MAP_G_TO_G
        GL_COLOR_TABLE_SCALE_SGI
        GL_LUMINANCE8
        GL_SYNC_FENCE_APPLE
        GL_VERTEX_ARRAY_BINDING
        GL_COMPRESSED_RGB_S3TC_DXT1_ANGLE
        GL_EFFECTIVE_RASTER_SAMPLES_EXT
        GL_INT_IMAGE_2D_RECT
        GL_TEXTURE_RANGE_POINTER_APPLE
        GL_UNIFORM_IS_ROW_MAJOR
        GL_EXT_polygon_offset_clamp
        GL_LIGHTEN_KHR
        GLEW_VERSION_1_2_1
        GL_OUTPUT_TEXTURE_COORD20_EXT
        GL_RGB5_EXT
        GL_OR
        GL_NV_fragment_shader_interlock
        GLEW_VERSION_4_2
        GL_IMAGE_CLASS_1_X_32
        GL_VIEWPORT_SWIZZLE_POSITIVE_X_NV
        GL_NEGATIVE_Y_EXT
        GL_MAX_PROGRAM_ATTRIBS_ARB
        GL_SMOOTH_LINE_WIDTH_RANGE
        GL_FONT_MAX_ADVANCE_HEIGHT_BIT_NV
        GLEW_NV_video_capture
        GLEW_ATI_vertex_streams
        GL_ARB_ES2_compatibility
        GLEW_EXT_EGL_image_array
        GL_VERSION_4_2
        GL_LINEAR_MIPMAP_NEAREST
        GL_MODELVIEW_PROJECTION_NV
        GL_COMBINER_CD_OUTPUT_NV
        GL_AUX2
        GL_RGB16F_EXT
        GL_LAYERED_SGIX
        GLEW_IBM_vertex_array_lists
        GL_POLYGON_TOKEN
        GL_READ_PIXELS
        GLEW_ERROR_NO_GL_VERSION
        GL_CURRENT_MATRIX_STACK_DEPTH_ARB
        GL_COVERAGE_MODULATION_TABLE_SIZE_NV
        GL_QUERY_COUNTER_BITS_ARB
        GL_QUERY_BUFFER_AMD
        GL_UNSIGNED_INT16_VEC2_NV
        GL_NV_fragment_program4
        GL_FRAGMENT_NORMAL_EXT
        GL_RESAMPLE_ZERO_FILL_OML
        GLEW_ATI_element_array
        GLEW_ARB_matrix_palette
        GL_EXT_stencil_clear_tag
        GL_CURRENT_MATRIX_ARB
        GL_BUFFER_ACCESS_ARB
        GL_CURRENT_RASTER_SECONDARY_COLOR
        GL_TEXTURE24
        GLEW_ARB_seamless_cubemap_per_texture
        GL_APPLE_texture_max_level
        GL_FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR
        GLEW_ARB_cull_distance
        GL_SHADER_INCLUDE_ARB
        GL_FLOAT_R_NV
        GL_CLIP_DISTANCE2_EXT
        GL_DEPTH_CLEAR_VALUE
        GL_AMD_texture_gather_bias_lod
        GL_POST_TEXTURE_FILTER_BIAS_SGIX
        GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT
        GL_REPLACEMENT_CODE_SUN
        GL_DEBUG_CATEGORY_PERFORMANCE_AMD
        GL_COLOR_ARRAY_COUNT_EXT
        GL_TEXTURE_COVERAGE_SAMPLES_NV
        GLEW_3DFX_texture_compression_FXT1
        GLEW_NV_shader_thread_shuffle
        GL_TEXTURE_MAX_CLAMP_T_SGIX
        GL_RG16I
        GL_MAX_TEXTURE_IMAGE_UNITS
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR
        GL_VERSION_4_3
        GL_DRAW_BUFFER1_NV
        GL_NUM_SPARSE_LEVELS_ARB
        GL_TEXTURE_COORD_ARRAY
        GL_INT_SAMPLER_BUFFER_EXT
        GL_ADD_SIGNED
        GL_HSL_LUMINOSITY_KHR
        GL_MAX_LAYERS
        GL_VIDEO_CAPTURE_FRAME_WIDTH_NV
        GLEW_EXT_debug_marker
        GL_SYNC_X11_FENCE_EXT
        GL_COLOR_BUFFER_BIT1_QCOM
        GLEW_ARB_texture_rectangle
        GL_VIEW_CLASS_16_BITS
        GL_TEXTURE9
        GL_TEXTURE8
        GL_FRAGMENTS_INSTRUMENT_MAX_SGIX
        GL_IBM_rasterpos_clip
        GL_ARB_invalidate_subdata
        GL_FETCH_PER_SAMPLE_ARM
        GL_NV_multisample_filter_hint
        GL_COMPARE_REF_TO_TEXTURE_EXT
        GL_STORAGE_PRIVATE_APPLE
        GL_NV_depth_buffer_float
        GL_ACCUM_CLEAR_VALUE
        GL_SUBPIXEL_PRECISION_BIAS_X_BITS_NV
        GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS
        GL_VERTEX_ATTRIB_ARRAY2_NV
        GL_SGIX_decimation
        GL_SGIX_fragment_lighting_space
        GL_COMPRESSED_RED_GREEN_RGTC2_EXT
        GL_CONDITION_SATISFIED_APPLE
        GL_EVAL_2D_NV
        GL_DELETE_STATUS
        GL_SHADER_SOURCE_LENGTH
        GLEW_NV_draw_instanced
        GL_APPLE_color_buffer_packed_float
        GLEW_EXT_packed_depth_stencil
        GLEW_ARB_fragment_program
        GL_DEPTH_RANGE
        GL_OBJECT_SHADER_SOURCE_LENGTH_ARB
        GL_COMPRESSED_RED
        GLEW_SGI_fft
        GL_COMPRESSED_LUMINANCE_ALPHA_3DC_ATI
        GLEW_EXT_read_format_bgra
        GL_COLOR_INDEX12_EXT
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL
        GL_W_EXT
        GL_MAX_CLIENT_ATTRIB_STACK_DEPTH
        GL_COMBINER3_NV
        GL_CLIP_DISTANCE1_APPLE
        GL_ARB_depth_texture
        GL_INVERT_RGB_NV
        GL_NUM_SPIR_V_EXTENSIONS
        GL_LUMINANCE4_ALPHA4_EXT
        GLEW_SUN_slice_accum
        GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV
        GL_TEXTURE_SAMPLES
        GL_COLOR_INDEXES
        GL_DYNAMIC_ATI
        GL_EXT_texture_object
        GL_TEXTURE14
        GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB
        GL_UNSIGNED_INT_10F_11F_11F_REV_EXT
        GL_REPLACE_VALUE_AMD
        GL_SAMPLER_EXTERNAL_OES
        GL_LINE_WIDTH_GRANULARITY
        GLEW_SGIX_slim
        GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV
        GL_DRAW_BUFFER0
        GLEW_EXT_x11_sync_object
        GL_FIXED_ONLY
        GL_MAX_DUAL_SOURCE_DRAW_BUFFERS_EXT
        GLEW_SGIX_bali_r_instruments
        GL_AUX0
        GL_HINT_BIT
        GL_TEXTURE_COMPARE_FUNC
        GLEW_QCOM_driver_control
        GL_ONE_MINUS_SRC1_ALPHA_EXT
        GL_EXT_multi_draw_indirect
        GL_TEXTURE29
        GL_ARB_point_parameters
        GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB
        GL_SET_AMD
        GL_LUMINANCE_ALPHA32F_ARB
        GLEW_EXT_texture_compression_astc_decode_mode_rgb9e5
        GL_MINMAX_FORMAT_EXT
        GL_COMPLEX_UNSIGNED_INT_SGI
        GL_COLOR_TABLE_BLUE_SIZE_SGI
        GL_PROJECTION
        GL_TEXTURE26_ARB
        GLEW_EXT_scene_marker
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR
        GL_FLOAT
        GL_STREAM_READ
        GL_INDEX_ARRAY_TYPE
        GL_OPERAND1_RGB_EXT
        GL_EXT_histogram
        GLEW_3DFX_multisample
        GL_AMD_shader_atomic_counter_ops
        GL_COMPRESSED_RGBA_S3TC_DXT1_NV
        GL_SOURCE0_ALPHA
        GL_TEXTURE_BUFFER_ARB
        GL_IMAGE_BINDING_NAME_EXT
        GL_SRGB_ALPHA
        GL_LUMINANCE4_EXT
        GL_UNPACK_SKIP_IMAGES_EXT
        GLEW_ARB_texture_buffer_object_rgb32
        GL_ARB_map_buffer_range
        GLEW_ARB_texture_storage
        GLEW_GREMEDY_string_marker
        GL_INTENSITY16_EXTENDED_RANGE_SGIX
        GL_COLOR_ARRAY_STRIDE
        GL_TEXTURE_GREEN_TYPE
        GL_COLOR_ATTACHMENT1_EXT
        GL_FONT_HAS_KERNING_BIT_NV
        GL_TEXTURE_SWIZZLE_A_EXT
        GL_POST_CONVOLUTION_GREEN_BIAS
        GLEW_ARB_vertex_program
        GL_OUTPUT_POINT_SIZE_ATIX
        GL_MAX_GEOMETRY_SHADER_INVOCATIONS
        GL_APPLE_flush_buffer_range
        GLEW_EXT_texture_integer
        GL_EXT_shader_implicit_conversions
        GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS
        GL_BUFFER_MAP_OFFSET
        GL_MAX_VARYING_FLOATS
        GL_DRAW_BUFFER15
        GL_EXT_multiview_draw_buffers
        GL_RGB8UI
        GL_DRAW_BUFFER5_ARB
        GLEW_VERSION_3_2
        GL_PIXEL_COUNT_NV
        GL_UNSIGNED_INT64_AMD
        GL_STATIC_COPY
        GL_SOURCE0_RGB_ARB
        GL_YCBCR_MESA
        GL_TEXTURE_RECTANGLE_EXT
        GL_VIBRANCE_BIAS_NV
        GL_UNIFORM_ARRAY_STRIDE
        GLEW_ANGLE_instanced_arrays
        GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY
        GL_COLOR_MATERIAL_FACE
        GLEW_NV_copy_buffer
        GL_MIN_BLUE_SGIS
        GLEW_NV_gpu_program_fp64
        GL_ATI_shader_texture_lod
        GL_IBM_static_data
        GLEW_ANGLE_framebuffer_blit
        GL_COLOR_TABLE_SGI
        GL_TEXTURE_FLOAT_COMPONENTS_NV
        GL_STEREO
        GL_CUBE_MAP_ZN_SGIX
        GL_VERTEX_STREAM2_ATI
        GL_TEXTURE_BLUE_TYPE
        GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI
        GL_MAX_GEOMETRY_INPUT_COMPONENTS_EXT
        GL_ANGLE_texture_compression_dxt5
        GL_RGBA16F_EXT
        GL_PROGRAM_OBJECT_EXT
        GL_ANGLE_translated_shader_source
        GLEW_NV_bgr
        GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS
        GL_COLOR_ATTACHMENT2
        GL_UNSIGNED_BYTE_2_3_3_REV
        GL_TEXTURE_3D_BINDING_EXT
        GL_MAX_GEOMETRY_INPUT_COMPONENTS
        GL_SAMPLER_2D_ARB
        GL_MAP1_VERTEX_ATTRIB12_4_NV
        GL_MAX_RASTER_SAMPLES_EXT
        GL_CODE_REGAL
        GL_TRANSFORM_FEEDBACK_BUFFER_EXT
        GL_ARB_separate_shader_objects
        GL_NV_texture_env_combine4
        GL_MAX_TEXTURE_COORDS_ARB
        GL_COMPRESSED_SRGB_ALPHA_PVRTC_2BPPV1_EXT
        GL_NV_fence
        GL_RASTER_FIXED_SAMPLE_LOCATIONS_EXT
        GL_PIXEL_MAP_I_TO_B
        GL_TEXTURE2_SGIS
        GL_VERTEX_ARRAY_COUNT_EXT
        GL_NV_shader_atomic_float
        GL_RED_SCALE
        GLEW_EXT_histogram
        GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING
        GL_SIGNED_RGBA_NV
        GL_OFFSET_TEXTURE_2D_SCALE_NV
        GL_FRAMEBUFFER_DEFAULT_FIXED_SAMPLE_LOCATIONS
        GL_CONSERVATIVE_RASTER_MODE_POST_SNAP_NV
        GL_RENDERBUFFER_BLUE_SIZE_EXT
        GLEW_EXT_geometry_shader4
        GL_VERTEX_ARRAY_LENGTH_NV
        GLEW_GREMEDY_frame_terminator
        GL_TEXTURE_COORD_ARRAY_POINTER_EXT
        GL_DRAW_BUFFER2_NV
        GL_DRAW_BUFFER15_NV
        GL_FRAGMENT_TEXTURE
        GL_VERTEX_TEXTURE
        GL_QUERY_RESULT_AVAILABLE_NV
        GL_SAMPLE_COVERAGE_ARB
        GL_DOUBLE_MAT4_EXT
        GL_ASYNC_READ_PIXELS_SGIX
        GL_EDGE_FLAG_ARRAY_LIST_IBM
        GL_SAMPLER_1D_ARRAY
        GL_RGBA16_SNORM
        GL_SECONDARY_COLOR_ARRAY_LENGTH_NV
        GL_TIME_ELAPSED_EXT
        GL_PLUS_DARKER_NV
        GL_DEPTH24_STENCIL8
        GL_OP_MULTIPLY_MATRIX_EXT
        GL_ARB_gpu_shader5
        GL_TRANSFORM_FEEDBACK_BARRIER_BIT
        GL_POINT_BIT
        GL_COLOR_TABLE_LUMINANCE_SIZE_SGI
        GLEW_ATIX_point_sprites
        GL_DOUBLE
        GL_VIEW_CLASS_128_BITS
        GL_SGIX_fog_patchy
        GLEW_NV_shader_thread_group
        GL_LAYOUT_DEFAULT_INTEL
        GL_NV_vertex_program2_option
        GL_VERTEX_STREAM7_ATI
        GL_VIEW_CLASS_96_BITS
        GL_ARB_texture_query_levels
        GL_CONFORMANT_NV
        GL_EXT_texture
        GLEW_NV_vertex_program2
        GL_PIXEL_MAP_R_TO_R
        GL_DEPTH_BUFFER_BIT3_QCOM
        GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT
        GL_SYNC_CL_EVENT_ARB
        GL_TEXTURE_GREEN_SIZE_EXT
        GL_MAX_ELEMENTS_VERTICES_EXT
        GL_OPERAND1_RGB
        GL_R
        GL_CONVOLUTION_HINT_SGIX
        GL_PIXEL_MAG_FILTER_EXT
        GL_NUM_SHADING_LANGUAGE_VERSIONS
        GL_SGIX_impact_pixel_texture
        GL_FRAGMENT_PROGRAM_ARB
        GL_SMOOTH_QUADRATIC_CURVE_TO_NV
        GL_FLOAT16_VEC4_NV
        GL_RELATIVE_LARGE_CCW_ARC_TO_NV
        GL_BUFFER_OBJECT_APPLE
        GL_RG_INTEGER
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT
        GL_BITMAP
        GL_RGB32F_ARB
        GL_MESA_ycbcr_texture
        GL_NORMAL_MAP_NV
        GLEW_ARB_robust_buffer_access_behavior
        GL_QUERY_WAIT_NV
        GL_VERTEX_ATTRIB_RELATIVE_OFFSET
        GL_FENCE_CONDITION_NV
        GL_SAMPLES_ARB
        GLEW_NV_vertex_program4
        GL_PATH_STROKE_MASK_NV
        GL_MAP1_VERTEX_ATTRIB13_4_NV
        GL_GLYPH_VERTICAL_BEARING_ADVANCE_BIT_NV
        GL_T2F_V3F
        GL_POLYGON_SMOOTH_HINT
        GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB
        GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE
        GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_EXT
        GL_LUMINANCE_FLOAT16_ATI
        GL_EXT_texture_cube_map_array
        GLEW_EXT_blend_equation_separate
        GL_GREEN_BIT_ATI
        GL_NV_fragment_program
        GL_INTENSITY16F_ARB
        GL_SELECTION_BUFFER_POINTER
        GLEW_ARB_imaging
        GL_FUNC_REVERSE_SUBTRACT
        GL_MAX_SAMPLES_EXT
        GL_VERSION_1_2
        GL_RED_NV
        GL_UNSIGNED_SHORT_8_8_REV_APPLE
        GLEW_ARB_internalformat_query2
        GL_COMPUTE_SHADER_INVOCATIONS_ARB
        GL_SOURCE0_RGB_EXT
        GL_STEREO_PROJECTION_ORTHO_NV
        GL_CONVOLUTION_BORDER_MODE_EXT
        GL_APPLE_texture_format_BGRA8888
        GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB
        GL_COLOR_SUM
        GL_DEBUG_SOURCE_SHADER_COMPILER_ARB
        GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI
        GLEW_VERSION_4_6
        GL_OPERAND0_ALPHA_ARB
        GL_CLIP_PLANE5
        GLEW_ARB_texture_buffer_range
        GL_SOURCE1_RGB_ARB
        GL_INGR_color_clamp
        GL_DYNAMIC_DRAW_ARB
        GL_BUFFER_GPU_ADDRESS_NV
        GLEW_EXT_color_subtable
        GL_MALI_PROGRAM_BINARY_ARM
        GL_TEXTURE31
        GL_ARB_texture_compression_bptc
        GL_NV_depth_clamp
        GL_DEVICE_NODE_MASK_EXT
        GL_NURBS_KNOT_COUNT_SGIX
        GL_EYE_DISTANCE_TO_POINT_SGIS
        GLEW_EXT_shader_group_vote
        GL_VIDEO_BUFFER_NV
        GLEW_NV_conditional_render
        GL_AMD_interleaved_elements
        GL_HSL_HUE_NV
        GL_PROGRAM_ALU_INSTRUCTIONS_ARB
        GL_ACTIVE_SUBROUTINE_MAX_LENGTH
        GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY
        GL_RENDERBUFFER_ALPHA_SIZE
        GL_ALWAYS_SOFT_HINT_PGI
        GL_BLUE_SCALE
        GL_FLOAT_MAT3
        GL_MATRIX8_ARB
        GL_MAP_ATTRIB_V_ORDER_NV
        GLEW_SGIX_color_matrix_accuracy
        GL_PROXY_TEXTURE_CUBE_MAP_ARRAY_ARB
        GL_GPU_OPTIMIZED_QCOM
        GLEW_EXT_geometry_shader
        GL_MAP2_VERTEX_ATTRIB14_4_NV
        GL_TESS_CONTROL_SUBROUTINE
        GL_PATH_CLIENT_LENGTH_NV
        GL_MAX_PROGRAM_OUTPUT_VERTICES_NV
        GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV
        GL_POINT_SIZE_MAX
        GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV
        GL_COLOR_ARRAY_BUFFER_BINDING
        GLEW_ARB_shader_image_size
        GLEW_SGIX_occlusion_instrument
        GL_TEXTURE27
        GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_TEST
        GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET
        GL_UNSIGNED_BYTE_3_3_2_EXT
        GL_TRIANGLES_ADJACENCY
        GL_DRAW_BUFFER7_ARB
        GL_CURRENT_VERTEX_WEIGHT_EXT
        GL_CLIP_DISTANCE5_EXT
        GLEW_NV_texture_shader2
        GL_TRANSPOSE_TEXTURE_MATRIX
        GL_TESS_EVALUATION_PROGRAM_NV
        GL_FOG_HINT
        GL_ARB_texture_rg
        GL_PACK_RESAMPLE_SGIX
        GL_OFFSET_HILO_TEXTURE_2D_NV
        GL_EXT_conservative_depth
        GL_SAMPLE_ALPHA_TO_ONE_EXT
        GL_LUMINANCE_ALPHA32F_EXT
        GLEW_ARB_shader_image_load_store
        GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT
        GL_2PASS_0_EXT
        GL_VIEWPORT_SUBPIXEL_BITS
        GL_OUTPUT_TEXTURE_COORD0_EXT
        GL_MAX_VERTEX_IMAGE_UNIFORMS
        GL_COLOR_ATTACHMENT4_EXT
        GL_QUADRATIC_ATTENUATION
        GL_PACK_SKIP_PIXELS
        GLEW_ARB_buffer_storage
        GLEW_AMD_query_buffer_object
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_CONTROL_SHADER
        GL_MAX_GEOMETRY_PROGRAM_INVOCATIONS_NV
        GL_BUFFER_FLUSHING_UNMAP_APPLE
        GL_SRC_OVER_NV
        GL_STRICT_DEPTHFUNC_HINT_PGI
        GL_POST_CONVOLUTION_RED_BIAS
        GL_DEBUG_CATEGORY_OTHER_AMD
        GL_PIXEL_TRANSFORM_2D_EXT
        GLEW_ARB_compute_shader
        GL_KHR_blend_equation_advanced_coherent
        GL_NV_texture_shader2
        GL_ALPHA_BIAS
        GL_TEXTURE_ALPHA_SIZE
        GL_TESS_CONTROL_OUTPUT_VERTICES
        GLEW_EXT_texture_sRGB_R8
        GLEW_NV_texture_array
        GL_HILO16_NV
        GL_MATRIX16_ARB
        GL_DRAW_BUFFER1_ARB
        GL_PREVIOUS_EXT
        GL_SAMPLE_BUFFERS_EXT
        GL_UNSIGNED_INT_SAMPLER_BUFFER
        GL_MAX_BLUE_SGIS
        GL_ANGLE_framebuffer_multisample
        GL_FOG_OFFSET_SGIX
        GL_R1UI_V3F_SUN
        GLEW_ARB_shading_language_include
        GL_COMPLEX_FLOAT_SGI
        GL_3D_COLOR_TEXTURE
        GL_CLAMP_TO_BORDER_SGIS
        GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB
        GL_LUMINANCE16_ALPHA16_SNORM
        GL_PACK_RESAMPLE_OML
        GL_OFFSET_TEXTURE_SCALE_NV
        GL_IMAGE_BIT
        GL_DEBUG_OUTPUT
        GLEW_ARB_robustness
        GL_SGIX_bali_r_instruments
        GL_IMAGE_2D_RECT
        GL_OUTPUT_COLOR0_EXT
        GL_REGAL_enable
        GL_DECR_WRAP
        GL_RESET_NOTIFICATION_STRATEGY
        GL_COEFF
        GL_ADD_ATI
        GL_LOG_STATUS_REGAL
        GL_VIDEO_CAPTURE_FRAME_HEIGHT_NV
        GL_UNIFORM_BUFFER_BINDING_EXT
        GL_MULTISAMPLE_LINE_WIDTH_RANGE_ARB
        GL_SUNX_constant_data
        GL_POINT_SPRITE_NV
        GL_RG_SNORM
        GL_SIGNED_ALPHA_NV
        GL_TEXTURE_GEN_MODE
        GL_DITHER
        GL_ARB_framebuffer_no_attachments
        GL_COLOR_BUFFER_BIT6_QCOM
        GL_DSDT8_NV
        GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR
        GL_SAMPLER_CUBE_SHADOW_EXT
        GL_TESSELLATION_MODE_AMD
        GLEW_SGIX_dvc
        GL_TESS_EVALUATION_SHADER_BIT
        GL_VERTEX_WEIGHT_ARRAY_EXT
        GL_TEXTURE_SWIZZLE_R_EXT
        GL_COMPRESSED_RGBA_S3TC_DXT3_EXT
        GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_NV
        GLEW_KHR_texture_compression_astc_sliced_3d
        GL_NVX_conditional_render
        GL_PROVOKING_VERTEX_EXT
        GL_INVALID_OPERATION
        GL_BGR_INTEGER
        GLEW_AMD_sparse_texture
        GL_SGIX_bali_timer_instruments
        GL_RGB32I
        GL_INTENSITY_EXT
        GL_READ_FRAMEBUFFER_BINDING_APPLE
        GL_SIGNED_HILO8_NV
        GL_ATI_texture_float
        GL_OUTPUT_TEXTURE_COORD16_EXT
        GLEW_SGIS_texture4D
        GL_MODELVIEW19_ARB
        GL_TEXTURE_VIEW_MIN_LEVEL
        GLEW_EXT_memory_object_fd
        GL_CONVOLUTION_FORMAT
        GLEW_EXT_geometry_point_size
        GL_422_REV_AVERAGE_EXT
        GL_PACK_SKIP_IMAGES_EXT
        GL_TEXTURE_TILING_EXT
        GL_TEXTURE_RECTANGLE_NV
        GL_VERSION_3_0
        GL_CONVOLUTION_HEIGHT_EXT
        GL_MATRIX3_NV
        GL_EXT_blend_minmax
        GL_DEPTH32F_STENCIL8
        GL_LUMINANCE_ALPHA32UI_EXT
        GL_R16_EXT
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT
        GL_INTERNALFORMAT_STENCIL_SIZE
        GLEW_ARB_framebuffer_object
        GL_COLOR_ATTACHMENT8_NV
        GLEW_ARB_pixel_buffer_object
        GL_CMYKA_EXT
        GL_ONE_MINUS_SRC_COLOR
        GL_AMD_vertex_shader_tessellator
        GL_PROGRAM_PIPELINE_OBJECT_EXT
        GL_PROXY_TEXTURE_RECTANGLE
        GL_BALI_NUM_PRIMS_CLIP_RESULT_INSTRUMENT
        GL_LOCAL_CONSTANT_DATATYPE_EXT
        GL_SRGB_WRITE
        GL_TESS_GEN_MODE
        GL_TEXTURE_MAX_LEVEL_SGIS
        GL_MIXED_STENCIL_SAMPLES_SUPPORTED_NV
        GL_TRANSFORM_HINT_APPLE
        GLEW_ARB_texture_gather
        GL_3DVISION_STEREO_NV
        GL_IMAGE_CLASS_4_X_8
        GL_COLOR_MATERIAL_PARAMETER
        GL_GLYPH_HORIZONTAL_BEARING_X_BIT_NV
        GL_PROGRAM_FORMAT_ARB
        GL_CONIC_CURVE_TO_NV
        GL_TRANSFORM_FEEDBACK_RECORD_NV
        GL_SUN_convolution_border_modes
        GL_SHADER_BINARY_FORMAT_SPIR_V_ARB
        GL_UNSIGNED_INT_10_10_10_2_EXT
        GL_RASTERIZER_DISCARD
        GL_COLOR_ATTACHMENT9_EXT
        GL_RENDERBUFFER_BINDING_EXT
        GLEW_ARB_gpu_shader_fp64
        GLEW_SGIX_pixel_tiles
        GL_COMPRESSED_RGBA_ASTC_8x5_KHR
        GL_INT64_VEC4_ARB
        GLEW_SGIX_mpeg2
        GL_TEXTURE_1D_STACK_MESAX
        GL_CONVOLUTION_BORDER_COLOR
        GL_DYNAMIC_STORAGE_BIT_EXT
        GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE
        GL_NV_blend_equation_advanced
        GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
        GL_PIXEL_MAP_S_TO_S
        GL_RENDERBUFFER_COLOR_SAMPLES_NV
        GL_MAX_VERTEX_ATTRIB_STRIDE
        GL_MAX_VARYING_COMPONENTS_EXT
        GL_TEXTURE23
        GL_TEXTURE_STORAGE_SPARSE_BIT_AMD
        GLEW_ARB_gpu_shader5
        GL_CLIP_PLANE0
        GL_COLOR_ATTACHMENT13_NV
        GL_INVERSE_TRANSPOSE_NV
        GL_ALPHA_BITS
        GL_LUMINANCE16F_EXT
        GL_MODULATE_ADD_ATIX
        GL_RGB16_EXT
        GL_RGBA2_EXT
        GL_UNSIGNED_SHORT_4_4_4_4
        GL_ROUNDED_RECT4_NV
        GL_DRAW_BUFFER12_EXT
        GL_REG_5_ATI
        GL_HI_BIAS_NV
        GLEW_NV_vertex_program3
        GL_PACK_CMYK_HINT_EXT
        GL_LUMINANCE12_ALPHA12
        GL_TRANSFORM_FEEDBACK_BUFFER
        GL_AMD_conservative_depth
        GL_TEXTURE_POINT_SPRITE_ATIX
        GL_TRIANGLES_ADJACENCY_ARB
        GL_ACCUM_ADJACENT_PAIRS_NV
        GL_UNSIGNED_SHORT_5_5_5_1
        GL_AMD_blend_minmax_factor
        GLEW_NV_transform_feedback2
        GL_PERFORMANCE_MONITOR_AMD
        GLEW_ARB_multi_bind
        GL_OUTPUT_TEXTURE_COORD19_EXT
        GL_MAX_COLOR_ATTACHMENTS
        GL_OBJECT_LINEAR
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR
        GL_NV_shader_atomic_counters
        GL_SAMPLER_2D_RECT
        GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS
        GL_DRAW_BUFFER15_ATI
        GL_REG_1_ATI
        GL_ARB_draw_indirect
        GL_DRAW_BUFFER7_ATI
        GL_FOG_BIT
        GLEW_MESA_shader_integer_functions
        GL_MAX_VERTEX_UNIFORM_BLOCKS
        GL_PRIMITIVES_GENERATED
        GL_UNSIGNED_INT16_NV
        GL_COMPRESSED_TEXTURE_FORMATS
        GLEW_EXT_shader_framebuffer_fetch
        GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_WRITE
        GL_HANDLE_TYPE_D3D12_FENCE_EXT
        GL_IMAGE_2D_RECT_EXT
        GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT
        GL_HISTOGRAM_BLUE_SIZE_EXT
        GL_SIGNED_LUMINANCE8_NV
        GL_MAX_SHADER_COMPILER_THREADS_KHR
        GL_R16
        GL_FOG_COORD_ARRAY_TYPE
        GL_R5_G6_B5_A8_ICC_SGIX
        GL_COLOR_TABLE_BIAS
        GL_PROGRAMMABLE_SAMPLE_LOCATION_ARB
        GL_OVR_multiview
        GL_TEXTURE3_ARB
        GL_VIEWPORT_SWIZZLE_NEGATIVE_Y_NV
        GL_VERTEX_ATTRIB_ARRAY_NORMALIZED
        GL_UNPACK_IMAGE_HEIGHT
        GL_IMAGE_1D_EXT
        GL_TEXTURE_MATERIAL_FACE_EXT
        GL_BUFFER
        GL_PIXEL_PACK_BUFFER_BINDING
        GL_LUMINANCE_ALPHA_EXTENDED_RANGE_SGIX
        GL_VERTEX_DATA_HINT_PGI
        GL_POLYGON_OFFSET_LINE_NV
        GL_VERTEX_STATE_PROGRAM_NV
        GL_DOUBLE_MAT3x4_EXT
        GLEW_EXT_texture_compression_latc
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR
        GLEW_NV_fragment_coverage_to_color
        GL_SYNC_CONDITION
        GL_MAX_SHADER_COMBINED_LOCAL_STORAGE_SIZE_EXT
        GL_INT_IMAGE_CUBE_EXT
        GL_SEPARATE_ATTRIBS
        GL_NUM_SAMPLE_COUNTS
        GL_FOG_COORD_ARRAY_BUFFER_BINDING
        GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY
        GLEW_EXT_texture_format_BGRA8888
        GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX
        GL_EXT_texture_format_BGRA8888
        GL_LUMINANCE16_ALPHA16_EXT
        GL_POINT_NV
        GL_NV_fbo_color_attachments
        GLEW_NV_bindless_multi_draw_indirect
        GL_SUBPIXEL_BITS
        GLEW_EXT_shader_non_constant_global_initializers
        GL_ARB_occlusion_query2
        GL_UNSIGNED_SHORT
        GL_TEXTURE_SHADOW
        GL_CULL_FACE
        GL_VIEW_CLASS_8_BITS
        GL_DEPENDENT_RGB_TEXTURE_3D_NV
        GL_INGR_interlace_read
        GLEW_IBM_multimode_draw_arrays
        GL_ELEMENT_ARRAY_BUFFER
        GL_ONE_MINUS_DST_COLOR
        GL_EXT_shader_texture_lod
        GLEW_ARB_shader_draw_parameters
        GL_CURRENT_BINORMAL_EXT
        GL_NV_command_list
        GL_MAX_SPARSE_3D_TEXTURE_SIZE_ARB
        GL_NV_vertex_program4
        GLEW_EXT_texture_object
        GL_TIMESTAMP
        GLEW_NV_vdpau_interop
        GL_MODELVIEW23_ARB
        GL_CONVOLUTION_FILTER_BIAS_EXT
        GL_VERTEX_ATTRIB_ARRAY5_NV
        GL_ARB_shading_language_packing
        GL_FRAGMENT_SHADER_DISCARDS_SAMPLES_EXT
        GL_MIN_PROGRAM_TEXEL_OFFSET_NV
        GL_RGBA16I
        GL_TEXTURE18_ARB
        GLEW_WIN_swap_hint
        GL_MAP1_COLOR_4_NURBS_SGIX
        GL_HP_occlusion_test
        GL_PERFQUERY_COUNTER_DURATION_NORM_INTEL
        GL_MODELVIEW_STACK_DEPTH
        GL_COLOR_TABLE_WIDTH_EXT
        GL_ATIX_vertex_shader_output_point_size
        GL_STEREO_PROJECTION_PERSPECTIVE_NV
        GL_ARB_gpu_shader_int64
        GL_MIRROR_CLAMP_TO_EDGE_ATI
        GL_SRGB8_ALPHA8
        GL_DEDICATED_MEMORY_OBJECT_EXT
        GL_COLOR_SUM_CLAMP_NV
        GLEW_MESA_pack_invert
        GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI
        GL_BUFFER_USAGE_ARB
        GL_ARB_program_interface_query
        GL_INTENSITY32I_EXT
        GL_TEXTURE_SWIZZLE_R
        GL_RGB10
        GL_ARB_texture_border_clamp
        GL_FRAGMENT_COVERAGE_TO_COLOR_NV
        GL_ARB_texture_non_power_of_two
        GL_NV_parameter_buffer_object
        GL_VERSION_4_1
        GL_EXT_clear_texture
        GL_ALPHA_REF_COMMAND_NV
        GL_HSL_LUMINOSITY_NV
        GL_ALL_SHADER_BITS
        GLEW_ARB_shader_viewport_layer_array
        GL_ARB_fragment_coord_conventions
        GL_VERTEX_ATTRIB_ARRAY_STRIDE
        GL_SGIX_scalebias_hint
        GL_SAMPLES_SGIS
        GL_ACTIVE_VARIABLES
        GL_DEPTH_BUFFER_FLOAT_MODE_NV
        GL_HISTOGRAM
        GL_N3F_V3F
        GL_TEXTURE_REDUCTION_MODE_EXT
        GL_STEREO_CONVERGENCE_NV
        GL_VERTEX_ARRAY_TYPE_EXT
        GL_SGIS_detail_texture
        GL_FORMAT_SUBSAMPLE_244_244_OML
        GL_UNSIGNED_NORMALIZED
        GL_CURRENT_VERTEX_ATTRIB
        GL_DOUBLE_MAT2x3
        GL_MAP1_COLOR_4
        GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE
        GL_SAMPLE_BUFFERS_ARB
        GL_SAMPLE_MASK_VALUE_NV
        GL_SOURCE3_RGB_NV
        GL_STENCIL_INDEX8
        GL_TEXTURE_CUBE_MAP_POSITIVE_Z
        GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB
        GL_CLAMP_TO_EDGE
        GL_QUERY_RESULT_ARB
        GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB
        GL_INDEX_LOGIC_OP
        GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB
        GL_FRAMEBUFFER_BARRIER_BIT_EXT
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT
        GL_TEXTURE_MAX_LOD
        GL_HSL_COLOR_NV
        GL_TEXTURE_2D_STACK_BINDING_MESAX
        GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_EXT
        GL_TRANSFORM_FEEDBACK_OVERFLOW_ARB
        GL_ATI_map_object_buffer
        GL_RGB10_A2
        GL_TEXTURE6_ARB
        GL_PIXEL_MAP_B_TO_B_SIZE
        GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB
        GL_SHADER_CONSISTENT_NV
        GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV
        GL_SRGB
        GL_VIDEO_CAPTURE_FIELD_LOWER_HEIGHT_NV
        GL_IMAGE_CLASS_1_X_8
        GL_SGIX_cylinder_texgen
        GL_MULTIVIEW_EXT
        GL_VIEWPORT_INDEX_PROVOKING_VERTEX
        GL_MULTISAMPLE_BUFFER_BIT3_QCOM
        GL_ARB_compressed_texture_pixel_storage
        GL_SAMPLE_MASK_INVERT_SGIS
        GLEW_ARB_clear_texture
        GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_EVALUATION_SHADER
        GL_UTF8_NV
        GL_SLIM12S_SGIX
        GL_TEXTURE_INDEX_SIZE_EXT
        GL_TRANSFORM_FEEDBACK_VARYING
        GL_SCALAR_EXT
        GL_FRAMEBUFFER_COMPLETE
        GL_COMPUTE_WORK_GROUP_SIZE
        GL_UNSIGNED_INT64_VEC3_ARB
        GL_FLOAT_VEC4
        GL_CLIENT_MAPPED_BUFFER_BARRIER_BIT_EXT
        GL_DECAL
        GL_REFERENCED_BY_GEOMETRY_SHADER
        GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE
        GL_MATRIX14_ARB
        GL_ARC_TO_NV
        GL_TRANSFORM_FEEDBACK_BARRIER_BIT_EXT
        GL_MAT_SHININESS_BIT_PGI
        GLEW_ARB_seamless_cube_map
        GLEW_INTEL_texture_scissor
        GL_LUMINANCE_ALPHA_FLOAT16_ATI
        GL_LAYOUT_GENERAL_EXT
        GLEW_SGIX_async
        GL_SAMPLER_2D_MULTISAMPLE_ARRAY
        GL_TEXTURE_BINDING_BUFFER_ARB
        GL_EXT_cmyka
        GL_MAP2_VERTEX_ATTRIB6_4_NV
        GL_CLIP_NEAR_HINT_PGI
        GL_VARIABLE_E_NV
        GL_ONE_EXT
        GLEW_ARB_clear_buffer_object
        GL_OUTPUT_TEXTURE_COORD7_EXT
        GL_ELEMENT_ADDRESS_COMMAND_NV
        GL_PROXY_TEXTURE_CUBE_MAP
        GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_NV
        GL_MAX_TEXTURE_LOD_BIAS_EXT
        GL_ALPHA4_EXT
        GL_LUMINANCE4_ALPHA4
        GL_NATIVE_GRAPHICS_HANDLE_PGI
        GL_DST_OVER_NV
        GLEW_EXT_vertex_array_bgra
        GL_BLEND_DST_RGB
        GL_RGB16UI_EXT
        GL_TEXTURE_IMAGE_TYPE
        GL_EXT_blend_equation_separate
        GL_MAX_VERTEX_STREAMS
        GLEW_ARB_shadow_ambient
        GL_DOUBLE_MAT3x2_EXT
        GL_AMD_framebuffer_sample_positions
        GL_PROGRAM_TARGET_NV
        GLEW_EXT_texture_sRGB_decode
        GLEW_QCOM_alpha_test
        GL_COMBINE_ARB
        GL_BOOL_VEC4_ARB
        GL_DEPTH_BUFFER_BIT6_QCOM
        GL_TEXTURE_BINDING_2D_ARRAY
        GL_PATH_COVER_DEPTH_FUNC_NV
        GL_NV_texture_expand_normal
        GL_CURRENT_FOG_COORDINATE
        GL_FONT_UNDERLINE_THICKNESS_BIT_NV
        GL_COLOR_ATTACHMENT3
        GL_EXT_shadow_samplers
        GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH
        GL_NV_texture_barrier
        GL_DYNAMIC_COPY_ARB
        GL_SGIX_vertex_array_object
        GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI
        GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV
        GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_APPLE
        GL_UNSIGNED_INT_IMAGE_2D_ARRAY_EXT
        GL_PATH_STROKE_BOUND_NV
        GL_PARALLEL_ARRAYS_INTEL
        GL_LUMINANCE_ALPHA_INTEGER_EXT
        GL_VERTEX_ATTRIB_ARRAY1_NV
        GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB
        GL_IBM_cull_vertex
        GL_RENDERBUFFER_STENCIL_SIZE
        GL_EDGE_FLAG_ARRAY_LENGTH_NV
        GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV
        GLEW_AMD_shader_stencil_export
        GL_TEXTURE_VIEW_MIN_LAYER_EXT
        GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_EXT
        GL_RENDER_MODE
        GL_MAX_MIPMAP_ANISOTROPY_SGIX
        GL_SUN_triangle_list
        GL_ALPHA16_SNORM
        GL_INVALID_INDEX
        GL_RGB_INTEGER_EXT
        GL_ATIX_texture_env_route
        GL_STEREO_SEPARATION_NV
        GL_MAX_SERVER_WAIT_TIMEOUT_APPLE
        GL_PACK_SUBSAMPLE_RATE_SGIX
        GL_TEXTURE24_ARB
        GL_DEBUG_SEVERITY_LOW
        GL_POINT_SIZE_MIN_ARB
        GL_MAP2_COLOR_4
        GL_EVAL_VERTEX_ATTRIB10_NV
        GL_EXT_shader_image_load_formatted
        GL_INT64_VEC2_ARB
        GL_NEGATIVE_ONE_TO_ONE
        GL_GENERATE_MIPMAP_HINT
        GL_EDGE_FLAG_ARRAY_BUFFER_BINDING
        GL_VERTEX_PROGRAM_TWO_SIDE
        GL_PIXEL_UNPACK_BUFFER_NV
        GL_VERTICAL_LINE_TO_NV
        GL_UNIFORM_BLOCK_REFERENCED_BY_COMPUTE_SHADER
        GL_MAP2_VERTEX_ATTRIB1_4_NV
        GL_COLOR_ARRAY_ADDRESS_NV
        GL_FRAMEBUFFER_ATTACHMENT_ANGLE
        GL_MODELVIEW10_ARB
        GL_IMAGE_COMPATIBILITY_CLASS
        GL_GLOBAL_ALPHA_FACTOR_SUN
        GL_EYE_LINEAR
        GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV
        GL_TEXTURE_2D_STACK_MESAX
        GL_CLIP_DEPTH_MODE
        GL_REPLACE
        GL_EXT_sRGB
        GL_DEPTH_TEXTURE_MODE_ARB
        GL_ALIASED_LINE_WIDTH_RANGE
        GLEW_SGIX_igloo_interface
        GL_VERTEX_ATTRIB_ARRAY_INTEGER
        GL_SECONDARY_COLOR_ARRAY_SIZE
        GL_GUILTY_CONTEXT_RESET
        GL_ALPHA_TEST_REF
        GL_PROGRAM_OBJECT_ARB
        GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM
        GL_TEXCOORD2_BIT_PGI
        GLEW_AMD_shader_ballot
        GLEW_ARB_texture_env_add
        GL_IMAGE_BINDING_ACCESS_EXT
        GL_DRAW_BUFFER5_ATI
        GL_ALPHA4
        GLEW_OES_byte_coordinates
        GL_MIN_FRAGMENT_INTERPOLATION_OFFSET
        GL_VERTEX_BINDING_OFFSET
        GL_DARKEN_NV
        GL_DEPTH_COMPONENT32F
        GL_LUMINANCE32I_EXT
        GL_UNSIGNED_INT_S8_S8_8_8_NV
        GL_MATRIX3_ARB
        GL_PIXEL_MAP_A_TO_A_SIZE
        GL_EXT_float_blend
        GL_SIGNED_RGB_NV
        GL_DEPTH_COMPONENT32
        GL_MAP2_TEXTURE_COORD_3_NURBS_SGIX
        GL_ERROR_REGAL
        GLEW_AMD_vertex_shader_tessellator
        GL_MAX_GEOMETRY_ATOMIC_COUNTERS
        GLEW_AMD_pinned_memory
        GL_MINUS_NV
        GL_QCOM_perfmon_global_mode
        GL_FRONT
        GL_SAMPLE_MASK_VALUE
        GL_RASTER_POSITION_UNCLIPPED_IBM
        GL_ARB_internalformat_query2
        GL_VERTEX_WEIGHTING_EXT
        GLEW_NV_shader_atomic_counters
        GL_SAMPLER_1D_SHADOW_ARB
        GL_RGB16_SNORM
        GL_ARB_parallel_shader_compile
        GL_TEXTURE_FETCH_BARRIER_BIT
        GL_COMPLETION_STATUS_ARB
        GLEW_WIN_specular_fog
        GLEW_EXT_422_pixels
        GL_NV_path_rendering
        GL_ALPHA_MAX_SGIX
        GL_RENDERBUFFER
        GL_RGBA_INTEGER_MODE_EXT
        GL_TESS_CONTROL_PROGRAM_PARAMETER_BUFFER_NV
        GL_SWIZZLE_STRQ_ATI
        GL_TRIANGLES_ADJACENCY_EXT
        GL_TEXTURE_RECTANGLE
        GL_MAX_DRAW_BUFFERS_NV
        GL_SRC_ATOP_NV
        GL_AMD_shader_ballot
        GL_INT_SAMPLER_2D_EXT
        GL_TEXTURE_BLUE_TYPE_ARB
        GL_VERTEX_SHADER_LOCALS_EXT
        GL_EXT_sRGB_write_control
        GL_KTX_Z_REGION
        GL_KTX_STENCIL_REGION
        GL_R1UI_C4UB_V3F_SUN
        GLEW_EXT_stencil_wrap
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT
        GL_BLUE_MIN_CLAMP_INGR
        GL_QUERY_OBJECT_AMD
        GL_TESS_EVALUATION_SHADER_INVOCATIONS_ARB
        GL_TEXTURE_SWIZZLE_RGBA
        GL_BUMP_ENVMAP_ATI
        GL_VERTEX_ATTRIB_MAP1_SIZE_APPLE
        GL_COLOR_ATTACHMENT6
        GL_LOCATION
        GL_Z6Y10Z6CB10Z6Y10Z6CR10_422_NV
        GL_NV_read_depth_stencil
        GL_NUM_WINDOW_RECTANGLES_EXT
        GL_MAP_UNSYNCHRONIZED_BIT_EXT
        GL_VERSION_1_5
        GL_DOUBLE_MAT3x2
        GLEW_ARM_rgba8
        GL_PROGRAM_BINARY_FORMATS
        GL_EXT_coordinate_frame
        GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2
        GL_EXT_texture_rectangle
        GL_WRITE_ONLY_ARB
        GL_MATRIX19_ARB
        GL_MAX_VERTEX_VARYING_COMPONENTS_EXT
        GL_DEBUG_LOGGED_MESSAGES_ARB
        GL_R16UI
        GL_ACTIVE_ATTRIBUTES
        GLEW_APPLE_vertex_array_range
        GL_LGPU_SEPARATE_STORAGE_BIT_NVX
        GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS
        GL_MAX_ARRAY_TEXTURE_LAYERS
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_COMPUTE_SHADER
        GL_DRAW_BUFFER14_ARB
        GL_IGLOO_LMC_COLOR_SGIX
        GL_RGBA8I
        GL_BUFFER_SIZE
        GL_VERTEX_ARRAY_LIST_STRIDE_IBM
        GL_DOUBLE_MAT3_EXT
        GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV
        GL_ARB_robust_buffer_access_behavior
        GL_COMPRESSED_SRGB_PVRTC_4BPPV1_EXT
        GL_ATI_element_array
        GL_TEXTURE_3D_EXT
        GL_COLOR_ATTACHMENT13_EXT
        GL_FONT_TARGET_UNAVAILABLE_NV
        GL_EXT_sparse_texture
        GL_OP_MADD_EXT
        GLEW_ATI_text_fragment_shader
        GL_UNSIGNED_INT_24_8_NV
        GL_MAX_LIST_NESTING
        GL_OUTPUT_TEXTURE_COORD27_EXT
        GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV
        GL_TERMINATE_SEQUENCE_COMMAND_NV
        GL_LIGHT_MODEL_COLOR_CONTROL
        GL_DT_BIAS_NV
        GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB
        GLEW_NV_viewport_array
        GL_MAP1_TEXTURE_COORD_2
        GL_COLOR
        GL_ARRAY_BUFFER_BINDING_ARB
        GL_SOURCE1_ALPHA_ARB
        GL_SAMPLE_MASK_NV
        GL_DEPTH_COMPONENT32F_NV
        GL_APPLE_copy_texture_levels
        GL_BGR_NV
        GL_PIXEL_PACK_BUFFER_EXT
        GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT
        GL_TEXTURE_MIN_LOD
        GL_COMPUTE_SHADER
        GL_DRAW_FRAMEBUFFER
        GL_REFLECTION_MAP_ARB
        GLEW_ARB_transform_feedback_instanced
        GLEW_NV_packed_depth_stencil
        GL_BUFFER_MAP_POINTER_ARB
        GL_FOG_SPECULAR_TEXTURE_WIN
        GL_PIXEL_UNPACK_BUFFER_BINDING
        GL_MAX_COMBINED_UNIFORM_BLOCKS
        GL_LUMINANCE_ALPHA8I_EXT
        GL_ELEMENT_ARRAY_LENGTH_NV
        GL_TEXTURE_ASTC_DECODE_PRECISION_EXT
        GL_SAMPLE_MASK_SGIS
        GL_COMPRESSED_ALPHA_ARB
        GL_TEXTURE_BINDING_CUBE_MAP_ARB
        GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV
        GL_EXT_texture_filter_anisotropic
        GL_XOR_NV
        GLEW_EXT_pixel_transform_color_table
        GLEW_EXT_semaphore_fd
        GL_SGIX_nonlinear_lighting_pervertex
        GL_IMAGE_BINDING_ACCESS
        GL_R16F_EXT
        GL_WRAP_BORDER
        GL_SHORT
        GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT
        GL_PATCH_VERTICES
        GL_TEXTURE_APPLICATION_MODE_EXT
        GL_INT_IMAGE_2D_MULTISAMPLE_EXT
        GL_DRAW_INDIRECT_BUFFER_BINDING
        GL_LINE
        GL_MAP2_TEXTURE_COORD_1
        GL_VERSION_3_1
        GL_ARRAY_BUFFER_BINDING
        GL_SGIX_ycrcba
        GL_RG16_EXT
        GL_SPHERE_MAP
        GL_SOURCE3_ALPHA_NV
        GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT
        GL_EXT_convolution
        GL_PASS_THROUGH_NV
        GL_INTERLEAVED_ATTRIBS_EXT
        GL_RGB16_SIGNED_SGIX
        GLEW_NV_multisample_coverage
        GL_ALPHA
        GL_SLUMINANCE8_EXT
        GL_OBJECT_DISTANCE_TO_LINE_SGIS
        GL_INTEL_fragment_shader_ordering
        GL_STENCIL_BACK_REF
        GL_ATI_texture_compression_3dc
        GLEW_EXT_clip_cull_distance
        GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT
        GL_RGB_FLOAT32_APPLE
        GLEW_SGIX_calligraphic_fragment
        GL_TEXTURE_NUM_LEVELS_QCOM
        GL_SUBTRACT_ARB
        GL_VIDEO_COLOR_CONVERSION_MAX_NV
        GL_POLYGON_OFFSET_BIAS_EXT
        GL_MAP1_TEXTURE_COORD_3
        GL_ELEMENT_ARRAY_BUFFER_BINDING
        GL_INDEX_ARRAY_STRIDE_EXT
        GL_TEXTURE_MAX_LEVEL
        GLEW_APPLE_copy_texture_levels
        GL_VERTEX_ARRAY_TYPE
        GL_SYNC_FLUSH_COMMANDS_BIT_APPLE
        GL_COLOR_TABLE_WIDTH_SGI
        GLEW_ARB_color_buffer_float
        GL_LIST_INDEX
        GLEW_EXT_texture_type_2_10_10_10_REV
        GL_INT_SAMPLER_3D_EXT
        GL_TEXTURE17
        GL_MAX_SPOT_EXPONENT_NV
        GL_STENCIL_BACK_VALUE_MASK
        GL_SUN_read_video_pixels
        GL_COPY_READ_BUFFER_BINDING
        GL_COMPRESSED_SRGB8_ETC2
        GL_COLOR_ATTACHMENT5_NV
        GL_TRANSFORM_FEEDBACK_BINDING_NV
        GL_REFERENCED_BY_COMPUTE_SHADER
        GL_R1UI_C3F_V3F_SUN
        GL_KHR_robust_buffer_access_behavior
        GL_ALPHA_FLOAT32_ATI
        GL_ALIASED_POINT_SIZE_RANGE
        GLEW_APPLE_float_pixels
        GL_EXT_multiple_textures
        GLEW_NV_texture_shader3
        GL_MAP1_VERTEX_ATTRIB9_4_NV
        GL_HANDLE_TYPE_OPAQUE_WIN32_EXT
        GL_NUM_ACTIVE_VARIABLES
        GL_AMD_debug_output
        GL_MULTISAMPLE_BUFFER_BIT4_QCOM
        GL_PRIMITIVE_RESTART_INDEX_NV
        GL_PRIMARY_COLOR
        GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER
        GL_VERTICES_SUBMITTED_ARB
        GL_MAX_TEXTURE_UNITS
        GL_ARB_sparse_buffer
        GL_COLOR_ARRAY_LIST_IBM
        GL_INT_SAMPLER_1D_ARRAY
        GL_SIGNED_INTENSITY_NV
        GLEW_ARB_draw_indirect
        GL_UNSIGNED_INT8_VEC2_NV
        GLEW_NV_uniform_buffer_unified_memory
        GLEW_EXT_multi_draw_arrays
        GL_DRAW_BUFFER2
        GL_MAX_TEXTURE_COORD_SETS_SGIS
        GLEW_ARB_texture_cube_map
        GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV
        GLEW_ARB_half_float_pixel
        GLEW_EXT_shared_texture_palette
        GL_MAX_SAMPLE_MASK_WORDS_NV
        GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT
        GLEW_ERROR_NO_GLX_DISPLAY
        GL_DEPTH_SCALE
        GL_DOT_PRODUCT_TEXTURE_3D_NV
        GL_COMPRESSED_RED_RGTC1
        GL_EXT_texture_compression_rgtc
        GL_RG32F
        GL_MAX_DRAW_BUFFERS_EXT
        GL_IBM_texture_mirrored_repeat
        GL_ALPHA_SIGNED_SGIX
        GL_EXT_shader_image_load_store
        GL_ARB_clip_control
        GL_FONT_HEIGHT_BIT_NV
        GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER
        GL_FOVEATION_ENABLE_BIT_QCOM
        GL_EXT_vertex_array
        GL_BUFFER_VARIABLE
        GL_LAST_VERTEX_CONVENTION
        GL_TEXTURE25
        GL_NV_vertex_program1_1
        GL_MAX_COMPUTE_UNIFORM_BLOCKS
        GL_IMAGE_CLASS_1_X_16
        GL_NV_present_video
        GL_COMPRESSED_SIGNED_RG11_EAC
        GLEW_SGIX_pixel_texture
        GL_TRANSFORM_FEEDBACK_VARYINGS_EXT
        GL_TEXTURE31_ARB
        GL_LOAD
        GL_NAME_LENGTH
        GL_FRAGMENT_MATERIAL_EXT
        GL_PACK_IMAGE_HEIGHT_EXT
        GL_STEREO_PROJECTION_NV
        GL_INDEX_ARRAY_LENGTH_NV
        GL_TEXTURE_UNSIGNED_REMAP_MODE_NV
        GL_INCR_WRAP
        GL_TESS_GEN_VERTEX_ORDER
        GL_UNSIGNED_INT8_NV
        GL_CLAMP_VERTEX_COLOR_ARB
        GL_CONSTANT_ALPHA_EXT
        GL_HALF_FLOAT_ARB
        GL_COLOR_RENDERABLE
        GL_PRESERVE_ATI
        GL_LESS
        GL_GREEN_NV
        GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT
        GL_DRAW_BUFFER11_ARB
        GL_UNSIGNED_INT_VEC4_EXT
        GL_COLOR_SUM_EXT
        GL_MAX_SERVER_WAIT_TIMEOUT
        GL_PROXY_COLOR_TABLE_SGI
        GL_VERTEX_STREAM6_ATI
        GL_TRANSFORM_FEEDBACK_PAUSED
        GL_COLOR_ATTACHMENT11
        GL_INT_IMAGE_2D_RECT_EXT
        GL_OP_CLAMP_EXT
        GL_NV_gpu_program5_mem_extended
        GL_NV_bgr
        GL_CLIP_PLANE1
        GL_ATOMIC_COUNTER_BUFFER_START
        GL_ARB_gl_spirv
        GL_MAX_ASYNC_DRAW_PIXELS_SGIX
        GL_SIGNED_NEGATE_NV
        GL_MATRIX_INDEX_ARRAY_STRIDE_ARB
        GL_RGBA8_EXT
        GLEW_ARB_sample_shading
        GL_LIGHT_MODEL_LOCAL_VIEWER
        GL_OBJECT_PLANE
        GL_FAILURE_NV
        GL_CONVOLUTION_FILTER_SCALE
        GL_MAX_TESS_GEN_LEVEL
        GLEW_ARB_map_buffer_range
        GL_ALPHA16_ICC_SGIX
        GL_NV_register_combiners2
        GL_ARB_shader_texture_image_samples
        GL_POINT
        GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION
        GL_MIRROR_CLAMP_TO_EDGE_EXT
        GL_DECR_WRAP_EXT
        GL_PATH_DASH_CAPS_NV
        GL_LAYOUT_SHADER_READ_ONLY_EXT
        GL_MAX_VARYING_COMPONENTS
        GL_FULL_SUPPORT
        GL_FRAMEBUFFER_SAMPLE_LOCATION_PIXEL_GRID_ARB
        GLEW_SGIX_nonlinear_lighting_pervertex
        GL_DSDT_MAG_VIB_NV
        GL_TEXTURE_BINDING_1D_ARRAY
        GL_LAYOUT_COLOR_ATTACHMENT_EXT
        GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB
        GL_COLOR_TABLE_BLUE_SIZE_EXT
        GL_NORMAL_ARRAY
        GL_UNSIGNED_INT_SAMPLER_2D_EXT
        GL_IMAGE_CUBE_MAP_ARRAY_EXT
        GL_PRIMITIVE_ID_NV
        GL_C4UB_V2F
        GL_FLOAT_RGB32_NV
        GL_OP_POWER_EXT
        GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS_EXT
        GL_SHADER_TYPE
        GL_SHADOW_AMBIENT_SGIX
        GL_CLAMP_TO_BORDER_ARB
        GL_FRAGMENT_LIGHT0_EXT
        GL_INTERLACE_READ_OML
        GL_LIST_BASE
        GL_MAX_VERTEX_UNIFORM_VECTORS
        GL_SIGNED_RGB8_NV
        GL_COLOR_INDEX2_EXT
        GLEW_NV_shadow_samplers_cube
        GL_STENCIL_REF
        GL_SAMPLER_CUBE_MAP_ARRAY
        GLEW_AMD_stencil_operation_extended
        GL_SOURCE1_ALPHA
        GL_SAMPLER_1D_ARRAY_SHADOW
        GL_LINE_STRIP_ADJACENCY_ARB
        GL_PARAMETER_BUFFER_BINDING_ARB
        GL_UNDEFINED_VERTEX_EXT
        GLEW_ARB_texture_float
        GL_EGL_GENERATE_RESET_ON_VIDEO_MEMORY_PURGE_NV
        GL_PLUS_CLAMPED_ALPHA_NV
        GL_SGIX_cube_map
        GL_FOG_COORDINATE_ARRAY_EXT
        GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS_ARB
        GL_VIEW_CLASS_S3TC_DXT1_RGB
        GL_OUTPUT_TEXTURE_COORD15_EXT
        GL_EVAL_VERTEX_ATTRIB11_NV
        GL_EXT_texture_rg
        GL_DEPTH_CLAMP_FAR_AMD
        GL_UPPER_LEFT
        GL_REDUCE
        GL_CCW
        GL_FIELDS_NV
        GL_EXT_bindable_uniform
        GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV
        GL_R1UI_T2F_N3F_V3F_SUN
        GL_OML_resample
        GL_FRAMEBUFFER_EXT
        GL_ACTIVE_PROGRAM_EXT
        GL_ACCUM_BLUE_BITS
        GL_EDGEFLAG_BIT_PGI
        GL_PIXEL_MAP_A_TO_A
        GL_COMBINE
        GL_MAX_UNIFORM_BLOCK_SIZE
        GL_ARB_texture_compression
        GL_ARB_vertex_array_bgra
        GLEW_ARB_enhanced_layouts
        GL_RG8I
        GL_MAX_SAMPLES_NV
        GL_ARB_vertex_attrib_64bit
        GLEW_SGIX_spotlight_cutoff
        GL_EXT_fog_coord
        GL_C4F_N3F_V3F
        GL_UNSIGNED_INT_IMAGE_CUBE_EXT
        GL_EXT_texture_sRGB_decode
        GL_EDGE_FLAG_ARRAY_COUNT_EXT
        GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS
        GL_TEXTURE_POINT_MODE_ATIX
        GL_EMBOSS_MAP_NV
        GL_COLOR_ATTACHMENT8_EXT
        GL_RELATIVE_SMALL_CCW_ARC_TO_NV
        GL_COMBINER_CD_DOT_PRODUCT_NV
        GL_MAX_TEXTURE_MAX_ANISOTROPY
        GL_VERTEX_ID_SWIZZLE_AMD
        GL_R8
        GL_COLOR_BUFFER_BIT5_QCOM
        GL_EXT_gpu_shader4
        GL_TEXTURE_BLUE_SIZE_EXT
        GL_INT_IMAGE_1D
        GL_TEXTURE_BINDING_1D
        GL_UNSIGNED_INT64_VEC4_ARB
        GL_EXT_timer_query
        GL_ARB_compute_shader
        GL_MAP2_COLOR_4_NURBS_SGIX
        GL_UNIFORM_BLOCK_INDEX
        GL_DEBUG_SEVERITY_MEDIUM_ARB
        GL_TIMESTAMP_ANGLE
        GL_Z4Y12Z4CB12Z4CR12_444_NV
        GL_MAX_CULL_DISTANCES
        GL_COLOR3_BIT_PGI
        GL_MAX_FRAGMENT_INPUT_COMPONENTS
        GL_MAX_ACTIVE_LIGHTS_EXT
        GL_UNSIGNED_INT_VEC4
        GL_IMAGE_2D_ARRAY_EXT
        GL_3D
        GL_EXTENSIONS
        GL_FONT_UNINTELLIGIBLE_NV
        GL_MAX_RENDERBUFFER_SIZE_EXT
        GLEW_REGAL_enable
        GL_EMISSION
        GL_ZERO_EXT
        GL_APPLE_vertex_program_evaluators
        GL_TEXTURE30_ARB
        GL_ELEMENT_ARRAY_BARRIER_BIT_EXT
        GL_IGLOO_COLORNORMAL_SGIX
        GL_FOG_COORD
        GL_INT
        GL_FEEDBACK_BUFFER_POINTER
        GL_ANGLE_depth_texture
        GL_POST_COLOR_MATRIX_RED_BIAS_SGI
        GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH
        GL_ANGLE_pack_reverse_row_order
        GL_MAX_COMPUTE_VARIABLE_GROUP_SIZE_ARB
        GL_MINUS_CLAMPED_NV
        GL_PER_STAGE_CONSTANTS_NV
        GL_COMPRESSED_RGBA_ASTC_10x6_KHR
        GL_TEXTURE_LOD_BIAS
        GL_VERTEX_PROGRAM_POINT_SIZE
        GL_COLOR_ARRAY_SIZE_EXT
        GL_TEXTURE_WRAP_R_EXT
        GL_DOT3_RGBA_EXT
        GLEW_SUN_triangle_list
        GL_SAMPLER_2D_SHADOW_EXT
        GLEW_VERSION_2_0
        GL_TEXTURE15_ARB
        GL_4_BYTES
        GL_FLOAT_32_UNSIGNED_INT_24_8_REV
        GL_MODELVIEW6_ARB
        GL_ATI_texture_mirror_once
        GL_AUX1
        GL_EXT_copy_image
        GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY
        GLEW_EXT_polygon_offset_clamp
        GL_SMOOTH_CUBIC_CURVE_TO_NV
        GL_MAX_COMPUTE_ATOMIC_COUNTERS
        GLEW_EXT_float_blend
        GL_HSL_HUE_KHR
        GL_DRAW_BUFFER4_EXT
        GL_SGIX_fog_scale
        GL_LINEARLIGHT_NV
        GL_SECONDARY_COLOR_ARRAY_POINTER
        GL_TEXTURE_MAG_FILTER
        GLEW_NV_read_depth
        GL_ATTRIB_STACK_DEPTH
        GL_CURRENT_OCCLUSION_QUERY_ID_NV
        GL_ARB_depth_clamp
        GL_SRGB_EXT
        GL_TEXTURE_IMAGE_FORMAT
        GL_EXT_geometry_shader
        GL_STRICT_SCISSOR_HINT_PGI
        GL_TEXTURE_2D_ARRAY_EXT
        GL_LUMINANCE_ALPHA32I_EXT
        GL_VIEW_CLASS_RGTC2_RG
        GL_PROJECTION_STACK_DEPTH
        GL_FLOAT_MAT4
        GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT
        GL_CONVOLUTION_BORDER_MODE
        GLEW_EXT_shader_pixel_local_storage2
        GLEW_EXT_sparse_texture2
        GLEW_SGIX_mpeg1
        GL_EXT_polygon_offset
        GL_TEXTURE_COORD_ARRAY_TYPE_EXT
        GLEW_NV_framebuffer_multisample
        GL_VERSION_1_4
        GL_HARDMIX_NV
        GL_UNSIGNED_INT64_VEC2_NV
        GL_FRAMEBUFFER_RENDERABLE_LAYERED
        GLEW_INTEL_parallel_arrays
        GL_OPERAND2_RGB_ARB
        GL_MODELVIEW28_ARB
        GL_NEGATIVE_X_EXT
        GL_GLOBAL_ALPHA_SUN
        GL_BOOL_VEC3_ARB
        GL_GEOMETRY_SHADER_BIT_EXT
        GL_COMBINER_COMPONENT_USAGE_NV
        GL_ATTRIBUTE_ADDRESS_COMMAND_NV
        GL_FOG_COORD_ARRAY_ADDRESS_NV
        GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE
        GL_PROGRAM_ERROR_POSITION_ARB
        GL_MAX_COMBINED_CLIP_AND_CULL_DISTANCES
        GL_NV_internalformat_sample_query
        GL_RECLAIM_MEMORY_HINT_PGI
        GL_UNIFORM_BLOCK_DATA_SIZE
        GL_EXT_memory_object_win32
        GL_PATH_STENCIL_DEPTH_OFFSET_UNITS_NV
        GL_422_REV_EXT
        GL_OUTPUT_TEXTURE_COORD13_EXT
        GL_RG16F_EXT
        GLEW_ARB_vertex_shader
        GL_TABLE_TOO_LARGE
        GLEW_APPLE_fence
        GLEW_AMD_multi_draw_indirect
        GL_LUMINANCE_ALPHA
        GL_IMAGE_BUFFER
        GL_UNPACK_SUBSAMPLE_RATE_SGIX
        GL_SHADOW_ATTENUATION_EXT
        GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV
        GL_FRAMEBUFFER_DEFAULT_LAYERS
        GL_ARB_shader_clock
        GL_TEXTURE_BASE_LEVEL_SGIS
        GL_AFFINE_2D_NV
        GL_OPERAND2_RGB
        GL_4PASS_1_SGIS
        GL_HANDLE_TYPE_D3D12_TILEPOOL_EXT
        GL_STATIC_READ_ARB
        GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY
        GLEW_ARB_texture_rg
        GL_POLYGON_OFFSET_COMMAND_NV
        GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV
        GL_TEXTURE_STENCIL_SIZE_EXT
        GL_Y_EXT
        GL_EDGE_FLAG_ARRAY_POINTER
        GL_SGI_color_table
        GL_EXT_clip_volume_hint
        GL_CONSTANT_ARB
        GLEW_VERSION_1_2
        GL_COMPRESSED_SLUMINANCE
        GLEW_NV_occlusion_query
        GL_COMBINE_RGB_EXT
        GL_DYNAMIC_READ
        GL_RGB_FLOAT16_ATI
        GL_LINE_LOOP
        GL_MULTICAST_PROGRAMMABLE_SAMPLE_LOCATION_NV
        GL_TEXTURE_GEN_Q
        GL_POST_COLOR_MATRIX_GREEN_SCALE
        GL_EXT_draw_buffers
        GL_COLOR_SUM_ARB
        GL_SOURCE1_ALPHA_EXT
        GL_MAX_LGPU_GPUS_NVX
        GL_EYE_POINT_SGIS
        GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT
        GL_TIMEOUT_IGNORED_APPLE
        GL_QUERY
        GL_GEOMETRY_PROGRAM_NV
        GLEW_EXT_texture_cube_map_array
        GL_MAX_SAMPLES
        GL_MAX_TEXTURE_COORDS
        GL_ATI_pn_triangles
        GL_LUMINANCE16_ALPHA16_EXTENDED_RANGE_SGIX
        GLEW_NV_blend_square
        GL_ELEMENT_ARRAY_POINTER_APPLE
        GL_TEXTURE_GEN_S
        GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM
        GL_IMAGE_2D_ARRAY
        GLEW_SGIS_line_texgen
        GL_VERTEX_ARRAY_EXT
        GL_ACTIVE_UNIFORM_MAX_LENGTH
        GL_CURRENT_VERTEX_ATTRIB_ARB
        GL_UNPACK_SKIP_IMAGES_NV
        GL_MAX_VERTEX_STREAMS_ATI
        GLEW_SGIX_texture_range
        GLEW_MESA_ycbcr_texture
        GL_LIGHT_MODEL_COLOR_CONTROL_EXT
        GL_DOT_PRODUCT_TEXTURE_1D_NV
        GL_DOUBLE_MAT4x2
        GL_MAP_UNSYNCHRONIZED_BIT
        GL_MAX_PROGRAM_PARAMETERS_ARB
        GL_PROGRAM
        GL_INT_IMAGE_2D_ARRAY
        GL_POINT_SIZE
        GL_ARB_transform_feedback_instanced
        GL_NV_texture_border_clamp
        GL_INTEL_conservative_rasterization
        GLEW_EXT_texture_edge_clamp
        GL_INNOCENT_CONTEXT_RESET
        GL_OFFSET
        GL_DRAW_BUFFER_EXT
        GL_ALPHA_TEST_FUNC
        GL_TEXTURE_LEQUAL_R_SGIX
        GL_OVERLAY_NV
        GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV
        GL_LINE_TOKEN
        GL_R32F_EXT
        GL_NV_shader_atomic_fp16_vector
        GL_MAX_ELEMENTS_INDICES_EXT
        GL_IMAGE_BINDING_LEVEL_EXT
        GL_DOUBLE_MAT3
        GL_UNPACK_SKIP_PIXELS
        GL_FOVEATION_SCALED_BIN_METHOD_BIT_QCOM
        GL_CLIP_DISTANCE0_APPLE
        GL_TEXTURE_COORD_ARRAY_POINTER
        GL_STENCIL_BACK_PASS_DEPTH_PASS
        GL_GEOMETRY_INPUT_TYPE
        GL_MODELVIEW12_ARB
        GL_CURRENT_BIT
        GL_OBJECT_TYPE
        GL_MATRIX10_ARB
        GL_COMPLEX_BYTE_SGI
        GL_MINMAX_SINK_EXT
        GL_PATH_FILL_COVER_MODE_NV
        GL_VENDOR
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB
        GL_SYNC_FLAGS
        GLEW_EXT_texture_env_add
        GLEW_ARB_half_float_vertex
        GL_RESAMPLE_DECIMATE_SGIX
        GLEW_ARB_bindless_texture
        GL_SOFTLIGHT_KHR
        GL_SGIX_instrument_error
        GL_UNPACK_RESAMPLE_OML
        GLEW_KHR_texture_compression_astc_ldr
        GL_TEXTURE25_ARB
        GLEW_ARB_vertex_attrib_binding
        GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB
        GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_NV
        GL_FOG_COORDINATE_ARRAY_POINTER_EXT
        GL_ELEMENT_ARRAY_BARRIER_BIT
        GL_MULTISAMPLE_BIT_3DFX
        GL_HISTOGRAM_WIDTH_EXT
        GL_MISSING_REGAL
        GL_MAX_MULTIVIEW_BUFFERS_EXT
        GL_ARB_explicit_uniform_location
        GLEW_SGIX_blend_cadd
        GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT
        GL_DRAW_BUFFER7_EXT
        GL_FOG_COORDINATE_EXT
        GL_IMAGE_1D
        GLEW_NV_texture_npot_2D_mipmap
        GL_COMPRESSED_SRGB_S3TC_DXT1_EXT
        GL_PGI_misc_hints
        GL_LUMINANCE32F_EXT
        GL_OML_subsample
        GL_PATH_DASH_ARRAY_COUNT_NV
        GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT
        GL_GLYPH_HORIZONTAL_BEARING_ADVANCE_BIT_NV
        GLEW_NV_shader_noperspective_interpolation
        GL_COLOR_MATRIX_STACK_DEPTH
        GL_TEXTURE_SHADER_NV
        GL_MAX_NAME_STACK_DEPTH
        GL_SGIX_igloo_interface
        GL_FLOAT_RGB16_NV
        GL_INT_SAMPLER_2D_RECT_EXT
        GL_EXT_shadow_funcs
        GL_INTERNALFORMAT_PREFERRED
        GL_TEXTURE_USAGE_ANGLE
        GLEW_EXT_draw_range_elements
        GL_DOMAIN
        GL_FOG_DISTANCE_MODE_NV
        GL_QUERY_ALL_EVENT_BITS_AMD
        GL_BINORMAL_ARRAY_POINTER_EXT
        GL_RGBA4_S3TC
        GL_NO_ERROR
        GL_VERTEX_ATTRIB_BINDING
        GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB
        GLEW_AMD_conservative_depth
        GL_EDGE_FLAG
        GLEW_NV_geometry_shader_passthrough
        GL_LUMINANCE_ALPHA16F_EXT
        GL_RELEASED_APPLE
        GL_HISTOGRAM_RED_SIZE_EXT
        GL_LOSE_CONTEXT_ON_RESET
        GL_ALPHA16_EXT
        GL_NV_pack_subimage
        GL_MAX_VERTEX_SHADER_LOCALS_EXT
        GL_NUM_PROGRAM_BINARY_FORMATS
        GL_RGB_SIGNED_SGIX
        GLEW_ARB_sampler_objects
        GL_INTENSITY8UI_EXT
        GL_NV_draw_texture
        GL_INTENSITY_FLOAT32_ATI
        GL_MAP1_GRID_DOMAIN
        GL_BGRA8_EXT
        GL_EXT_vertex_shader
        GL_INDEX_BIT_PGI
        GL_OPERAND2_ALPHA
        GL_EXT_shader_group_vote
        GL_ARB_polygon_offset_clamp
        GL_MAX_SUBROUTINES
        GL_NV_vdpau_interop
        GL_4PASS_1_EXT
        GL_MAP1_TEXTURE_COORD_3_NURBS_SGIX
        GL_MAX_VARYING_VECTORS
        GL_DEPTH_COMPONENT
        GL_TEXTURE_COMPONENTS
        GL_MODELVIEW_MATRIX
        GL_LUMINANCE_ALPHA8UI_EXT
        GL_SRC1_ALPHA_EXT
        GLEW_EXT_transform_feedback
        GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING
        GL_TRANSPOSE_PROGRAM_MATRIX_EXT
        GL_VIEWPORT_POSITION_W_SCALE_NV
        GL_SUBPIXEL_PRECISION_BIAS_Y_BITS_NV
        GL_EXT_gpu_program_parameters
        GL_NUM_COMPATIBLE_SUBROUTINES
        GLEW_SGIX_fog_factor_to_alpha
        GL_UNSIGNED_INT_VEC3
        GL_MIN_ALPHA_SGIS
        GLEW_ARM_shader_framebuffer_fetch
        GL_INTENSITY8
        GL_MAX_COMPUTE_SHARED_MEMORY_SIZE
        GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB
        GL_LUMINANCE16_EXT
        GL_SAMPLE_ALPHA_TO_MASK_SGIS
        GL_EXT_direct_state_access
        GLEW_ANGLE_texture_usage
        GL_VERTEX_ARRAY_POINTER
        GL_LINEAR_ATTENUATION
        GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB
        GL_COMPLEX_UNSIGNED_BYTE_SGI
        GLEW_SGIX_vertex_array_object
        GL_SGIX_dvc
        GL_VIEW_CLASS_48_BITS
        GL_SHADER_IMAGE_STORE
        GL_MAX_COMPUTE_WORK_GROUP_SIZE
        GL_MAP_INVALIDATE_RANGE_BIT_EXT
        GL_VERTEX_ARRAY_SIZE_EXT
        GL_VERSION_4_6
        GL_COLOR_ATTACHMENT4_NV
        GL_EXTERNAL_VIRTUAL_MEMORY_BUFFER_AMD
        GL_TEXTURE_COMPRESSED
        GL_SQUARE_NV
        GL_DRAW_BUFFER0_ATI
        GL_SHADING_LANGUAGE_VERSION
        GL_NV_explicit_attrib_location
        GL_AND_INVERTED
        GL_FIXED
        GL_COLOR_TABLE_INTENSITY_SIZE_EXT
        GL_RGB8UI_EXT
        GL_DEPTH_STENCIL
        GL_OP_MIN_EXT
        GL_TEXTURE5_ARB
        GL_ARB_explicit_attrib_location
        GL_EXT_draw_buffers_indexed
        GL_EXT_abgr
        GL_INDEX_ARRAY_LIST_IBM
        GL_ARB_robustness
        GL_NV_vertex_program2
        GL_AMD_vertex_shader_layer
        GL_EXT_compressed_ETC1_RGB8_sub_texture
        GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV
        GL_RELATIVE_MOVE_TO_NV
        GLEW_EXT_stencil_two_side
        GLEW_ANGLE_framebuffer_multisample
        GL_SGIX_depth_pass_instrument
        GL_MATRIX5_ARB
        GL_GLYPH_VERTICAL_BEARING_X_BIT_NV
        GL_ALPHA16UI_EXT
        GL_FOG_START
        GL_TRIANGLE_STRIP_ADJACENCY_ARB
        GL_NO_RESET_NOTIFICATION_ARB
        GL_VERTEX_PROGRAM_NV
        GL_INTENSITY16UI_EXT
        GL_COLOR_TABLE_GREEN_SIZE_EXT
        GL_ATOMIC_COUNTER_BUFFER
        GL_CLAMP_TO_EDGE_SGIS
        GL_3DFX_texture_compression_FXT1
        GL_NV_texture_compression_vtc
        GL_SAMPLE_LOCATION_PIXEL_GRID_WIDTH_ARB
        GL_COMPRESSED_SLUMINANCE_ALPHA
        GL_STATE_RESTORE
        GL_COLOR_ATTACHMENT9
        GL_OBJECT_BUFFER_USAGE_ATI
        GL_MATRIX_INDEX_ARRAY_SIZE_ARB
        GL_EXT_index_material
        GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS
        GL_DRAW_INDIRECT_BUFFER
        GL_NORMAL_ARRAY_COUNT_EXT
        GL_IGLOO_FULLSCREEN_SGIX
        GL_MAX_COMBINED_IMAGE_UNIFORMS
        GL_EXT_draw_buffers2
        GL_COVERAGE_MODULATION_NV
        GL_RESET_NOTIFICATION_STRATEGY_ARB
        GL_GEOMETRY_OUTPUT_TYPE_EXT
        GLEW_NV_pack_subimage
        GL_ARB_texture_multisample
        GL_AMD_depth_clamp_separate
        GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB
        GL_POST_COLOR_MATRIX_GREEN_BIAS
        GL_OP_MUL_EXT
        GL_DRAW_BUFFER15_ARB
        GL_OP_SET_GE_EXT
        GL_FLOAT_MAT3x2
        GL_PACK_ROW_LENGTH_NV
        GL_LUMINANCE_ALPHA_ICC_SGIX
        GLEW_ARB_texture_swizzle
        GL_SGIX_texture_phase
        GL_RENDERBUFFER_STENCIL_SIZE_EXT
        GLEW_EXT_texture_lod_bias
        GLEW_ARB_shader_bit_encoding
        GL_MAP_READ_BIT
        GL_SAMPLE_ALPHA_TO_COVERAGE_ARB
        GLEW_ATI_fragment_shader
        GL_MAX_SHADER_BUFFER_ADDRESS_NV
        GLEW_SGIX_resample
        GL_EXT_texture_view
        GL_ANGLE_program_binary
        GL_SAMPLER_2D_ARRAY_SHADOW
        GL_SGIS_line_texgen
        GL_BLEND_EQUATION_ALPHA
        GL_NV_vertex_attrib_integer_64bit
        GL_UNSIGNED_INT_IMAGE_1D_ARRAY_EXT
        GL_FRAGMENT_COLOR_EXT
        GL_COMPRESSED_SRGB_ALPHA_EXT
        GL_FRAMEBUFFER_UNDEFINED
        GL_OUTPUT_TEXTURE_COORD5_EXT
        GL_DEBUG_SOURCE_APPLICATION
        GL_SRC_COLOR
        GL_HIGH_FLOAT
        GLEW_NV_viewport_swizzle
        GL_S3_s3tc
        GL_BUMP_TARGET_ATI
        GLEW_NV_conservative_raster_dilate
        GL_SGIX_mpeg1
        GL_FLOAT16_VEC2_NV
        GL_VERTEX_ATTRIB_ARRAY11_NV
        GL_TEXTURE_OUTPUT_ALPHA_ATIX
        GL_SAMPLE_LOCATION_NV
        GL_INDEX_ARRAY
        GL_ALPHA_INTEGER_EXT
        GLEW_ARB_compatibility
        GL_SMOOTH
        GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS
        GL_GREEN_BITS
        GL_STENCIL_INDEX
        GL_PROGRAM_OUTPUT
        GL_NEAREST
        GL_POST_TEXTURE_FILTER_SCALE_SGIX
        GL_QUERY_COUNTER_BITS
        GL_SAMPLE_LOCATION_PIXEL_GRID_HEIGHT_ARB
        GLEW_KHR_robust_buffer_access_behavior
        GL_DOT3_RGBA
        GL_MAX_TRANSFORM_FEEDBACK_BUFFERS
        GLEW_APPLE_pixel_buffer
        GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS
        GLEW_ARB_pipeline_statistics_query
        GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV
        GLEW_ARB_explicit_attrib_location
        GL_ARB_conditional_render_inverted
        GLEW_VERSION_3_3
        GL_DEPENDENT_HILO_TEXTURE_2D_NV
        GL_MAX_VERTEX_ATTRIBS
        GL_TESS_EVALUATION_SHADER
        GL_MAX_PROGRAM_RESULT_COMPONENTS_NV
        GL_PATH_STROKE_BOUNDING_BOX_NV
        GL_FRAGMENTS_INSTRUMENT_SGIX
        GL_ARB_shader_stencil_export
        GL_COLOR_ATTACHMENT0_NV
        GL_SECONDARY_COLOR_ARRAY_TYPE
        GL_PIXEL_TRANSFORM_SGI
        GLEW_EXT_light_texture
        GL_PERCENTAGE_AMD
        GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS
        GL_PN_TRIANGLES_ATI
        GL_UNSIGNED_INT_5_9_9_9_REV
        GL_LO_SCALE_NV
        GLEW_OML_interlace
        GL_UNSIGNED_INT_8_8_8_8
        GL_TEXTURE_1D_ARRAY
        GL_PROXY_TEXTURE_CUBE_MAP_EXT
        GL_MAX_COMBINED_CLIP_AND_CULL_DISTANCES_EXT
        GL_FLOAT_MAT2x4_NV
        GL_COMBINER5_NV
        GL_HISTOGRAM_RED_SIZE
        GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT
        GL_TEXTURE14_ARB
        GL_LUMINANCE16_ALPHA16
        GL_CLIP_DISTANCE7_APPLE
        GL_TEXTURE_VIEW_NUM_LAYERS
        GL_AND_REVERSE
        GL_APPLE_row_bytes
        GLEW_ARB_texture_border_clamp
        GL_QUERY_COUNTER_BITS_ANGLE
        GL_MAP_COHERENT_BIT_EXT
        GL_DUP_LAST_CUBIC_CURVE_TO_NV
        GL_LINEARBURN_NV
        GL_KTX_buffer_region
        GLEW_ARB_texture_non_power_of_two
        GL_TEXTURE_CUBE_MAP_SEAMLESS
        GL_TEXTURE_INTENSITY_TYPE
        GLEW_ATI_shader_texture_lod
        GLEW_NV_image_formats
        GL_ISOLINES
        GL_MATRIX22_ARB
        GL_TEXTURE_GREEN_TYPE_ARB
        GL_TEXTURE_COORD_ARRAY_LENGTH_NV
        GL_DEPTH_COMPONENTS
        GL_CLIP_DISTANCE3
        GL_STENCIL_BACK_PASS_DEPTH_FAIL
        GL_MAX_DEBUG_LOGGED_MESSAGES
        GL_VIDEO_CAPTURE_TO_422_SUPPORTED_NV
        GL_GEOMETRY_OUTPUT_TYPE_ARB
        GLEW_ANGLE_texture_compression_dxt1
        GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS
        GL_DEPTH32F_STENCIL8_NV
        GL_NV_gpu_program5
        GL_DRAW_BUFFER2_ARB
        GL_NOP_COMMAND_NV
        GL_UNSIGNED_INT8_VEC3_NV
        GL_NORMALIZE
        GL_SPARE0_NV
        GL_POST_COLOR_MATRIX_COLOR_TABLE
        GL_DEPTH_BUFFER_BIT1_QCOM
        GL_4PASS_3_EXT
        GL_MAX_FRAGMENT_SHADER_STORAGE_BLOCKS
        GL_EXT_framebuffer_sRGB
        GLEW_ARB_point_sprite
        GL_COLOR_TABLE_FORMAT_SGI
        GL_SCALED_RESOLVE_FASTEST_EXT
        GL_SWIZZLE_STQ_ATI
        GL_CONSERVATIVE_RASTER_MODE_PRE_SNAP_TRIANGLES_NV
        GL_MIRRORED_REPEAT_IBM
        GL_CAVEAT_SUPPORT
        GL_TEXTURE_COMPRESSED_BLOCK_HEIGHT
        GL_DRAW_BUFFER9_EXT
        GL_ALPHA_FLOAT16_APPLE
        GLEW_ARB_occlusion_query2
        GL_MIRROR_CLAMP_TO_EDGE
        GL_MAX_GEOMETRY_UNIFORM_COMPONENTS
        GL_TEXTURE_SRGB_DECODE_EXT
        GL_ARB_texture_swizzle
        GL_CURRENT_QUERY_ANGLE
        GL_EVAL_VERTEX_ATTRIB12_NV
        GL_ARB_vertex_attrib_binding
        GL_MATRIX13_ARB
        GL_VERTEX_ARRAY_SIZE
        GL_LUMINANCE_ALPHA_FLOAT32_APPLE
        GL_MAP2_VERTEX_ATTRIB2_4_NV
        GL_RENDERBUFFER_DEPTH_SIZE
        GL_TEXTURE_CUBE_MAP_ARRAY_OES
        GL_FOG_BLEND_ALPHA_SGIX
        GL_COMPRESSED_SIGNED_R11_EAC
        GL_IMAGE_2D_MULTISAMPLE_ARRAY
        GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB
        GL_PATH_FILL_MASK_NV
        GL_NV_packed_float_linear
        GLEW_APPLE_element_array
        GL_EXT_texture_array
        GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN
        GL_WEIGHT_SUM_UNITY_ARB
        GL_FACTOR_MIN_AMD
        GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV
        GL_BUFFER_MAP_LENGTH
        GL_INTENSITY
        GL_MAX_SPARSE_TEXTURE_SIZE_AMD
        GL_COMPRESSED_RGBA_S3TC_DXT5_EXT
        GL_TEXTURE_LUMINANCE_SIZE
        GL_STRICT_LIGHTING_HINT_PGI
        GL_UNPACK_CONSTANT_DATA_SUNX
        GL_CONVOLUTION_WIDTH_EXT
        GL_MATRIX4_ARB
        GLEW_AMD_compressed_ATC_texture
        GL_REPLACE_EXT
        GL_INTENSITY8_EXT
        GL_ALPHA32F_ARB
        GL_CONSTANT
        GLEW_EXT_packed_float
        GL_OPTIMAL_TILING_EXT
        GLEW_QCOM_binning_control
        GL_EXT_semaphore
        GL_NV_shader_thread_shuffle
        GL_CURRENT_RASTER_POSITION_VALID
        GL_ALPHA_TEST_REF_QCOM
        GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT
        GL_PRIMARY_COLOR_NV
        GL_TEXTURE10
        GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB
        GLEW_SGIX_texture_scale_bias
        GL_TEXTURE_MAX_CLAMP_R_SGIX
        GL_MODELVIEW0_ARB
        GL_POLYGON_SMOOTH
        GL_FOG_COLOR
        GL_STENCIL_EXT
        GL_INT_VEC3_ARB
        GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB
        GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV
        GL_DRAW_BUFFER8_ATI
        GLEW_APPLE_vertex_program_evaluators
        GL_RGBA16I_EXT
        GL_MAX_COMBINED_ATOMIC_COUNTERS
        GLEW_SGIX_vertex_preclip_hint
        GL_COLOR_TABLE_FORMAT_EXT
        GLEW_ARB_transform_feedback2
        GLEW_EXT_direct_state_access
        GL_MAX_DRAW_BUFFERS
        GL_POST_CONVOLUTION_BLUE_BIAS
        GL_LOG_HTTP_REGAL
        GL_SGIX_reference_plane
        GL_DEPTH_STENCIL_TO_RGBA_NV
        GL_RGB_FLOAT16_APPLE
        GL_WAIT_FAILED
        GL_INT_SAMPLER_1D_ARRAY_EXT
        GL_APPLE_clip_distance
        GL_CONVOLUTION_1D_EXT
        GL_LOSE_CONTEXT_ON_RESET_ARB
        GLEW_NV_depth_range_unclamped
        GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE
        GL_CURRENT_COLOR
        GL_AMD_vertex_shader_viewport_index
        GL_TEXTURE_DEPTH
        GL_FRAMEBUFFER_BINDING
        GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL
        GL_EXT_texture_shared_exponent
        GL_ARB_multi_draw_indirect
        GL_INDEX_ARRAY_EXT
        GL_INT_SAMPLER_2D_ARRAY
        GL_AMD_shader_stencil_value_export
        GL_ARB_texture_buffer_object_rgb32
        GL_COMPRESSED_RGB_S3TC_DXT1_EXT
        GL_LOG_WARNING_REGAL
        GL_TEXTURE_BIT
        GL_CLIENT_PIXEL_STORE_BIT
        GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV
        GL_SRC1_COLOR
        GL_CLAMP_VERTEX_COLOR
        GL_SGIX_image_compression
        GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY
        GL_SGIX_fragment_specular_lighting
        GL_RASTERIZER_DISCARD_NV
        GL_ARB_texture_storage
        GL_COMPRESSED_LUMINANCE_ALPHA_ARB
        GL_CLIP_DISTANCE0
        GL_TRANSLATE_3D_NV
        GL_RGBA32I
        GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT
        GL_FIELD_LOWER_NV
        GL_DEBUG_CATEGORY_SHADER_COMPILER_AMD
        GL_INTERNALFORMAT_BLUE_SIZE
        GL_HISTOGRAM_FORMAT
        GL_MAX_PROGRAM_GENERIC_RESULTS_NV
        GL_VERTEX_SHADER
        GL_SECONDARY_COLOR_NV
        GL_ARB_sync
        GL_MAX_LIGHTS
        GL_MAX_FRAGMENT_INTERPOLATION_OFFSET
        GL_MAX_SPARSE_TEXTURE_SIZE_EXT
        GL_FENCE_APPLE
        GL_EXT_shader_non_constant_global_initializers
        GL_SOURCE2_RGB_ARB
        GL_RG8
        GL_TEXTURE_CUBE_MAP_NEGATIVE_X
        GL_LIGHT_ENV_MODE_EXT
        GL_RGB16F_ARB
        GL_COLOR_ATTACHMENT14_EXT
        GL_TRACK_MATRIX_NV
        GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_ARB
        GL_ARB_cl_event
        GL_LINE_WIDTH
        GL_OP_CROSS_PRODUCT_EXT
        GLEW_AMD_framebuffer_sample_positions
        GL_RASTER_SAMPLES_EXT
        GL_SELECTED_TEXTURE_SGIS
        GL_ALPHA8
        GL_MAX_COLOR_ATTACHMENTS_NV
        GL_APPLE_ycbcr_422
        GLEW_NV_fragment_program4
        GL_BLEND_PREMULTIPLIED_SRC_NV
        GL_TEXTURE_COMPARE_OPERATOR_SGIX
        GL_TEXTURE_UPDATE_BARRIER_BIT_EXT
        GL_COMPUTE_SHADER_INVOCATIONS
        GL_ARB_map_buffer_alignment
        GL_TEXTURE_SPARSE_EXT
        GLEW_NV_shader_atomic_fp16_vector
        GL_STENCIL_TEST_TWO_SIDE_EXT
        GL_PATH_GEN_COEFF_NV
        GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT
        GL_ALPHA_FLOAT16_ATI
        GL_TEXTURE8_ARB
        GL_QCOM_writeonly_rendering
        GL_TEXTURE_ENV_COLOR
        GL_MAP_FLUSH_EXPLICIT_BIT_EXT
        GL_CONTEXT_FLAG_NO_ERROR_BIT_KHR
        GL_TEXTURE17_ARB
        GL_XOR
        GL_RGB5
        GL_ONE_MINUS_CONSTANT_COLOR_EXT
        GL_INT_SAMPLER_CUBE
        GL_WRITEONLY_RENDERING_QCOM
        GL_TEXTURE_2D_BINDING_EXT
        GL_UNSIGNED_INT_IMAGE_2D_RECT_EXT
        GL_MAX_TEXTURE_STACK_DEPTH
        GL_EXT_texture_edge_clamp
        GL_FRAGMENT_SUBROUTINE_UNIFORM
        GL_IMAGE_2D_MULTISAMPLE_EXT
        GL_FLOAT_RGBA_NV
        GL_READ_PIXEL_DATA_RANGE_NV
        GL_UNSIGNED_INT_24_8_EXT
        GL_MAP2_VERTEX_ATTRIB12_4_NV
        GL_ACCUM_RED_BITS
        GL_RG8UI
        GL_STEREO_CUTOFF_NV
        GL_PIXEL_UNPACK_BUFFER_BINDING_ARB
        GL_UNIFORM_SGIX
        GL_MAP2_VERTEX_ATTRIB8_4_NV
        GL_EXT_win32_keyed_mutex
        GLEW_SGIX_texture_coordinate_clamp
        GL_SUPERSAMPLE_SCALE_Y_NV
        GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT
        GL_MAX_TEXTURE_BUFFER_SIZE_ARB
        GL_SGIS_shared_multisample
        GL_UNSIGNED_INT_10F_11F_11F_REV_APPLE
        GL_OBJECT_INFO_LOG_LENGTH_ARB
        GL_HISTOGRAM_SINK
        GL_DRAW_BUFFER15_EXT
        GL_NV_conservative_raster_dilate
        GLEW_INTEL_map_texture
        GL_DEBUG_SEVERITY_MEDIUM
        GL_CND0_ATI
        GL_POINT_SPRITE
        GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB
        GL_DIFFERENCE_KHR
        GLEW_NV_texture_barrier
        GL_TEXTURE_CUBE_MAP_ARB
        GL_READ_BUFFER
        GLEW_ARB_sparse_texture
        GL_SGIX_blend_alpha_minmax
        GL_ARB_shading_language_100
        GL_SGI_fft
        GL_LUMINANCE8_EXT
        GL_INTENSITY_EXTENDED_RANGE_SGIX
        GL_POST_CONVOLUTION_GREEN_SCALE_EXT
        GLEW_APPLE_texture_packed_float
        GL_FRAGMENT_PROGRAM_INTERPOLATION_OFFSET_BITS_NV
        GL_OP_FRAC_EXT
        GL_EVAL_VERTEX_ATTRIB8_NV
        GL_TEXTURE_CUBE_MAP_EXT
        GL_FLOAT16_MAT2x3_AMD
        GL_VERTEX_ARRAY_ADDRESS_NV
        GL_MAP1_TEXTURE_COORD_1_NURBS_SGIX
        GL_MAX_SAMPLES_APPLE
        GL_ALREADY_SIGNALED
        GL_MAX_PROGRAM_PATCH_ATTRIBS_NV
        GL_COMPRESSED_RGBA_ASTC_6x6_KHR
        GLEW_SGIX_blend_alpha_minmax
        GL_POINT_FADE_THRESHOLD_SIZE
        GL_GPU_MEMORY_INFO_TOTAL_AVAILABLE_MEMORY_NVX
        GL_PIXEL_CUBIC_WEIGHT_EXT
        GLEW_SGIX_async_pixel
        GL_MIN_INTENSITY_SGIS
        GL_MATRIX27_ARB
        GL_DRAW_ARRAYS_INSTANCED_COMMAND_NV
        GL_FLOAT16_MAT2_AMD
        GL_COMPRESSED_RGBA_S3TC_DXT1_ANGLE
        GL_DSDT8_MAG8_NV
        GL_GREEN_INTEGER
        GL_LINE_QUALITY_HINT_SGIX
        GL_CIRCULAR_CW_ARC_TO_NV
        GL_FRAMEBUFFER_INCOMPLETE_INSUFFICIENT_SHADER_COMBINED_LOCAL_STORAGE_EXT
        GL_EXT_post_depth_coverage
        GL_CONTEXT_FLAGS
        GL_UNSIGNED_INT_SAMPLER_BUFFER_AMD
        GLEW_EXT_draw_buffers2
        GL_EXT_cull_vertex
        GL_LAYOUT_TRANSFER_DST_EXT
        GL_MULTIPLY_NV
        GL_PROXY_TEXTURE_2D_STACK_MESAX
        GL_STENCIL_SAMPLES_NV
        GLEW_SGIX_polynomial_ffd
        GL_ARB_bindless_texture
        GL_PATCH_DEFAULT_INNER_LEVEL
        GL_TEXTURE_LUMINANCE_TYPE_ARB
        GLEW_AMD_depth_clamp_separate
        GL_SGIX_async_pixel
        GL_INVALID_ENUM
        GLEW_NV_platform_binary
        GL_PIXEL_MAP_I_TO_I
        GL_PIXEL_BUFFER_BARRIER_BIT
        GL_FRAGMENT_DEPTH_EXT
        GL_RG_EXT
        GL_BLEND_DST_RGB_EXT
        GL_INT_IMAGE_CUBE
        GL_TRANSLATE_2D_NV
        GL_CLIPPING_OUTPUT_PRIMITIVES
        GL_LUMINANCE8_ALPHA8
        GL_MAP2_VERTEX_ATTRIB7_4_NV
        GL_RGB16_SNORM_EXT
        GL_TESS_CONTROL_TEXTURE
        GL_TRANSPOSE_PROJECTION_MATRIX
        GL_INT_IMAGE_2D
        GL_SECONDARY_COLOR_ARRAY_LIST_IBM
        GL_VIRTUAL_PAGE_SIZE_X_ARB
        GL_TEXTURE_ENV
        GL_MITER_TRUNCATE_NV
        GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT
        GL_HSL_SATURATION_KHR
        GL_UNDEFINED_APPLE
        GL_UNSIGNED_INT_IMAGE_2D_RECT
        GL_COLOR_ATTACHMENT9_NV
        GL_ANGLE_texture_compression_dxt3
        GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT
        GLEW_VERSION_4_5
        GL_NV_shadow_samplers_cube
        GL_MATRIX_INDEX_ARRAY_TYPE_ARB
        GLEW_NV_explicit_multisample
        GLEW_ATIX_vertex_shader_output_point_size
        GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT
        GL_COLOR_BUFFER_BIT2_QCOM
        GL_TESS_EVALUATION_PROGRAM_PARAMETER_BUFFER_NV
        GL_TEXTURE_BINDING_RECTANGLE_ARB
        GL_NORMAL_MAP
        GL_EXT_semaphore_win32
        GL_PATH_STENCIL_REF_NV
        GL_DS_BIAS_NV
        GLEW_ARB_multitexture
        GL_EYE_DISTANCE_TO_LINE_SGIS
        GL_MAX_RENDERBUFFER_SIZE
        GL_AMD_gpu_shader_int16
        GL_TEXTURE_2D_MULTISAMPLE
        GLEW_VERSION_1_3
        GL_MAX_VIEWPORTS_NV
        GL_DST_ATOP_NV
        GL_DRAW_BUFFER14_NV
        GL_CURRENT_PROGRAM
        GL_MAP1_NORMAL
        GL_CLIP_DISTANCE1
        GL_DISPATCH_INDIRECT_BUFFER
        GLEW_NV_fragment_program2
        GL_FONT_X_MIN_BOUNDS_BIT_NV
        GL_INTERLEAVED_ATTRIBS_NV
        GL_SAMPLES
        GL_LUMINANCE_INTEGER_EXT
        GL_DRAW_BUFFER5
        GL_RELATIVE_LARGE_CW_ARC_TO_NV
        GL_DEBUG_CATEGORY_UNDEFINED_BEHAVIOR_AMD
        GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT
        GL_TEXTURE20_ARB
        GL_MAX_DEBUG_MESSAGE_LENGTH_AMD
        GL_POST_COLOR_MATRIX_RED_SCALE
        GL_VERTEX_PRECLIP_HINT_SGIX
        GL_INVALID_VALUE
        GL_MAP2_GRID_SEGMENTS
        GL_SLUMINANCE8_ALPHA8
        GL_VERTEX_ATTRIB_ARRAY0_NV
        GL_ARB_draw_buffers
        GLEW_ANGLE_program_binary
        GL_V3F
        GL_DEPTH_PASS_INSTRUMENT_SGIX
        GL_ARB_base_instance
        GL_EXT_framebuffer_blit
        GL_TEXTURE_LOD_BIAS_EXT
        GL_RELATIVE_HORIZONTAL_LINE_TO_NV
        GL_NV_draw_instanced
        GL_INVALID_FRAMEBUFFER_OPERATION_EXT
        GL_PARAMETER_BUFFER_BINDING
        GL_MATRIX7_NV
        GLEW_EXT_index_array_formats
        GL_DOUBLE_MAT2x4_EXT
        GL_EXT_external_buffer
        GL_POINTS
        GL_INT_IMAGE_3D_EXT
        GL_VERTEX_PROGRAM_BINDING_NV
        GL_MAX_RECTANGLE_TEXTURE_SIZE
        GL_LIST_MODE
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS
        GL_OBJECT_POINT_SGIS
        GLEW_EXT_polygon_offset
        GL_DOUBLE_VEC3
        GL_ARM_shader_framebuffer_fetch
        GLEW_VERSION_1_1
        GL_PERFMON_GLOBAL_MODE_QCOM
        GLEW_SGI_texture_color_table
        GL_PIXEL_MODE_BIT
        GL_BACK_RIGHT
        GL_NURBS_KNOT_VECTOR_SGIX
        GL_DEBUG_SOURCE_THIRD_PARTY_ARB
        GL_FONT_NUM_GLYPH_INDICES_BIT_NV
        GL_FLAT
        GL_EXT_texture_norm16
        GL_IBM_vertex_array_lists
        GL_RENDERBUFFER_SAMPLES_ANGLE
        GL_RGB9_E5_EXT
        GL_PHONG_HINT_WIN
        GL_RGBA
        GL_SGIX_vertex_preclip
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR
        GLEW_IBM_static_data
        GL_SLUMINANCE8
        GL_COMBINER_AB_DOT_PRODUCT_NV
        GL_EYE_SPACE_SGIX
        GL_COMPRESSED_RGBA_ASTC_10x8_KHR
        GL_EDGE_FLAG_ARRAY_POINTER_EXT
        GL_RELATIVE_LINE_TO_NV
        GL_ACTIVE_SUBROUTINES
        GL_CONVEX_HULL_NV
        GLEW_EXT_frag_depth
        GL_WRITE_ONLY
        GL_IMPLEMENTATION_COLOR_READ_TYPE
        GL_ALPHA_SNORM
        GL_OPERAND1_ALPHA_EXT
        GL_SAMPLE_MASK_VALUE_EXT
        GL_BGR
        GLEW_EXT_texture_env
        GL_QUAD_MESH_SUN
        GLEW_SGIX_blend_cmultiply
        GL_PACK_MAX_COMPRESSED_SIZE_SGIX
        GL_QUERY_BUFFER_BINDING
        GL_PERFQUERY_COUNTER_EVENT_INTEL
        GL_GREEN_INTEGER_EXT
        GL_SAMPLE_COUNT_NV
        GL_DST_COLOR
        GL_COMBINE4_NV
        GL_RGB16
        GL_MAP_INVALIDATE_RANGE_BIT
        GLEW_NV_draw_buffers
        GL_UNSIGNED_SHORT_5_6_5_REV
        GL_LIST_BIT
        GL_PERFQUERY_GPA_EXTENDED_COUNTERS_INTEL
        GL_PACK_ROW_LENGTH
        GLEW_NV_sample_locations
        GLEW_EXT_texture_snorm
        GL_ARB_shadow_ambient
        GL_OBJECT_DELETE_STATUS_ARB
        GL_SGIS_texture4D
        GL_RGB16F
        GL_CLIP_DISTANCE_NV
        GL_ATC_RGBA_INTERPOLATED_ALPHA_AMD
        GL_EXT_frag_depth
        GL_RGBA4
        GL_DRAW_BUFFER14
        GL_PROGRAM_LENGTH_ARB
        GL_MIRRORED_REPEAT_ARB
        GL_MAX_TEXTURE_COORDS_NV
        GL_MAX_INTEGER_SAMPLES
        GLEW_NVX_blend_equation_advanced_multi_draw_buffers
        GL_AMD_transform_feedback4
        GL_UNSIGNED_INT_2_10_10_10_REV
        GL_QUERY_BUFFER_BARRIER_BIT
        GL_CONSERVATIVE_RASTER_MODE_NV
        GL_VERTEX_ARRAY_RANGE_POINTER_APPLE
        GLEW_ARM_mali_shader_binary
        GLEW_ARB_program_interface_query
        GLEW_HP_convolution_border_modes
        GL_RENDERBUFFER_WIDTH_EXT
        GL_QUERY_RESULT_NV
        GL_TEXTURE_COORD_SET_INTERLEAVE_FACTOR_SGIS
        GL_DEPTH_STENCIL_EXT
        GL_PERFQUERY_WAIT_INTEL
        GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
        GL_COLOR_ATTACHMENT12
        GL_LUMINANCE12
        GL_UNSIGNED_INT_IMAGE_CUBE
        GL_SAMPLER_CUBE
        GL_SPIR_V_EXTENSIONS
        GLEW_ARB_robustness_share_group_isolation
        GL_QUERY_TARGET
        GL_NV_vertex_buffer_unified_memory
        GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT
        GL_INTENSITY12
        GL_MATRIX21_ARB
        GLEW_QCOM_shader_framebuffer_fetch_noncoherent
        GL_MATRIX1_NV
        GL_PERFQUERY_COUNTER_NAME_LENGTH_MAX_INTEL
        GL_ARRAY_ELEMENT_LOCK_COUNT_EXT
        GL_SGIX_pixel_texture_lod
        GL_RELATIVE_VERTICAL_LINE_TO_NV
        GLEW_EXT_memory_object
        GL_EXT_instanced_arrays
        GL_BLEND_SRC
        GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV
        GL_IMAGE_1D_ARRAY_EXT
        GL_COLOR_ATTACHMENT_EXT
        GL_MODELVIEW31_ARB
        GL_UUID_SIZE_EXT
        GLEW_EXT_clip_volume_hint
        GL_TRIANGLE_MESH_SUN
        GL_IGLOO_VIEWPORT_OFFSET_SGIX
        GLEW_SUN_global_alpha
        GL_RIGHT
        GL_PIXEL_UNPACK_BUFFER_BINDING_NV
        GLEW_APPLE_sync
        GL_INTENSITY_FLOAT16_APPLE
        GL_MAP1_VERTEX_ATTRIB6_4_NV
        GL_NV_platform_binary
        GL_IMAGE_2D_EXT
        GL_NV_3dvision_settings
        GL_PRESENT_TIME_NV
        GL_GEOMETRY_VERTICES_OUT_EXT
        GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE
        GL_MATRIX28_ARB
        GL_COMBINER7_NV
        GL_TEXTURE_INTENSITY_TYPE_ARB
        GL_LUMINANCE_ALPHA16UI_EXT
        GL_TEXTURE_RESIDENT
        GL_OR_REVERSE
        GL_COLOR_ARRAY_BUFFER_BINDING_ARB
        GL_NV_non_square_matrices
        GL_DEPTH_CLAMP
        GL_NV_framebuffer_mixed_samples
        GLEW_AMD_texture_texture4
        GL_HISTOGRAM_LUMINANCE_SIZE_EXT
        GL_CURRENT_SAMPLE_COUNT_QUERY_NV
        GL_MAX_PATCH_VERTICES
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER
        GL_PARAMETER_BUFFER_ARB
        GL_TEXTURE_MAX_ANISOTROPY
        GL_ITALIC_BIT_NV
        GL_COMPRESSED_RGB_S3TC_DXT1_NV
        GL_ARB_uniform_buffer_object
        GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS_EXT
        GL_DEBUG_TYPE_PUSH_GROUP
        GL_PRIMITIVES_SUBMITTED
        GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV
        GL_GPU_MEMORY_INFO_EVICTED_MEMORY_NVX
        GL_TEXTURE19
        GL_INT64_VEC4_NV
        GL_TEXTURE0
        GL_FRAGMENT_SHADER_FRAMEBUFFER_FETCH_MRT_ARM
        GL_VERTEX_ATTRIB_ARRAY_POINTER
        GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE
        GL_COMPLETION_STATUS_KHR
        MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB
        GL_QCOM_extended_get2
        GL_ARB_viewport_array
        GLEW_ARB_spirv_extensions
        GL_CLIP_DISTANCE4_APPLE
        GL_DRAW_BUFFER8_ARB
        GL_MAX_IMAGE_SAMPLES
        GL_RG16F
        GL_FOG_COORD_ARRAY
        GL_UNSIGNED_INT64_VEC2_ARB
        GL_SAMPLER_1D
        GL_FRAGMENT_COLOR_MATERIAL_EXT
        GLEW_EXT_shadow_samplers
        GLEW_SGIX_decimation
        GL_TRANSPOSE_TEXTURE_MATRIX_ARB
        GL_PROXY_TEXTURE_2D_EXT
        GL_STREAM_DRAW_ARB
        GL_LIGHTING
        GL_PROGRAM_POINT_SIZE_ARB
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_NV
        GL_RGB_SCALE_ARB
        GL_GEOMETRY_INPUT_TYPE_ARB
        GL_OUTPUT_COLOR1_EXT
        GL_ASYNC_DRAW_PIXELS_SGIX
        GL_MEDIUM_FLOAT
        GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_ARB
        GL_MIN_SPARSE_LEVEL_AMD
        GL_POST_COLOR_MATRIX_RED_SCALE_SGI
        GL_EDGE_FLAG_ARRAY_EXT
        GL_SAMPLER_BINDING
        GL_RGBA12_EXT
        GL_VERSION_4_5
        GL_DRAW_BUFFER14_EXT
        GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS
        GL_EXT_multisample
        GLEW_NV_gpu_multicast
        GL_INT64_VEC3_ARB
        GL_COLOR_ATTACHMENT7_NV
        GL_SGIX_texture_supersample
        GL_BUFFER_MAPPED_ARB
        GL_DS_SCALE_NV
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR
        GL_MODELVIEW1_MATRIX_EXT
        GLEW_APPLE_aux_depth_stencil
        GL_ENABLE_BIT
        GLEW_ARB_texture_compression_rgtc
        GLEW_ARB_copy_buffer
        GLEW_ARB_occlusion_query
        GL_NV_gpu_shader5
        GL_EXT_discard_framebuffer
        GL_UNPACK_SKIP_ROWS
        GL_TEXTURE_UPDATE_BARRIER_BIT
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
        GL_PATH_COORD_COUNT_NV
        GL_COLOR_TABLE_INTENSITY_SIZE
        GL_WIN_scene_markerXXX
        GLEW_ARB_texture_env_dot3
        GL_RGB_ICC_SGIX
        GL_PROGRAM_ATTRIB_COMPONENTS_NV
        GL_AMBIENT
        GL_BUFFER_UPDATE_BARRIER_BIT
        GLEW_ARB_sample_locations
        GL_VERTEX_ATTRIB_MAP2_COEFF_APPLE
        GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS
        GL_REGAL_ES1_0_compatibility
        GLEW_EXT_abgr
        GLEW_ERROR_GLX_VERSION_11_ONLY
        GL_MODELVIEW8_ARB
        GL_ATI_fragment_shader
        GL_PIXEL_COUNTER_BITS_NV
        GL_SHADER_STORAGE_BUFFER
        GL_TEXTURE_SWIZZLE_G_EXT
        GL_SAMPLE_MASK_INVERT_EXT
        GL_STREAM_COPY_ARB
        GL_MAX_FRAGMENT_UNIFORM_COMPONENTS
        GLEW_SGIS_texture_border_clamp
        GL_EXT_separate_specular_color
        GL_INTERNALFORMAT_ALPHA_TYPE
        GL_ARB_vertex_type_10f_11f_11f_rev
        GL_NV_primitive_restart
        GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB
        GL_VERTEX_ATTRIB_ARRAY4_NV
        GL_RENDERBUFFER_SAMPLES_NV
        GL_EXT_texture_compression_bptc
        GL_TEXTURE_SWIZZLE_G
        GL_FORMAT_SUBSAMPLE_24_24_OML
        GLEW_ARB_texture_cube_map_array
        GL_OBJECT_ACTIVE_UNIFORMS_ARB
        GL_VERTEX_ATTRIB_MAP1_ORDER_APPLE
        GL_FENCE_STATUS_NV
        GLEW_PGI_misc_hints
        GL_LUID_SIZE_EXT
        GL_ARB_clear_texture
        GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_ARB
        GLEW_ANGLE_translated_shader_source
        GLEW_SGIX_nurbs_eval
        GL_DOT3_RGB_EXT
        GL_VERTEX_PROGRAM_POINT_SIZE_ARB
        GL_NV_draw_vulkan_image
        GL_EXT_memory_object_fd
        GL_FONT_Y_MIN_BOUNDS_BIT_NV
        GL_EXT_subtexture
        GL_DRAW_FRAMEBUFFER_BINDING_NV
        GL_PATH_DASH_OFFSET_NV
        GL_TEXTURE13
        GL_DOUBLE_VEC3_EXT
        GL_TEXTURE_LO_SIZE_NV
        GL_STREAM_RASTERIZATION_AMD
        GL_GLYPH_HORIZONTAL_BEARING_Y_BIT_NV
        GL_SAMPLE_SHADING
        GLEW_APPLE_color_buffer_packed_float
        GL_RENDERBUFFER_SAMPLES_EXT
        GL_IMAGE_CLASS_2_X_16
        GL_ARB_ES3_2_compatibility
        GL_OPERAND0_RGB_ARB
        GL_FIRST_VERTEX_CONVENTION
        GL_ARB_shader_group_vote
        GL_ARB_texture_cube_map
        GL_TEXTURE_MIPMAP_ANISOTROPY_SGIX
        GL_OBJECT_TYPE_ARB
        GL_WEIGHT_ARRAY_STRIDE_ARB
        GL_UNSIGNED_INT_SAMPLER_2D_RECT
        GL_COMPUTE_TEXTURE
        GL_NAMED_STRING_TYPE_ARB
        GL_FONT_UNDERLINE_POSITION_BIT_NV
        GLEW_NV_light_max_exponent
        GLEW_NV_bindless_multi_draw_indirect_count
        GL_TEXTURE_MATRIX
        GL_ARRAY_OBJECT_BUFFER_ATI
        GL_BLEND_EQUATION_ALPHA_EXT
        GL_OR_INVERTED
        GL_DEBUG_SOURCE_THIRD_PARTY
        GL_REGISTER_COMBINERS_NV
        GL_DRAW_BUFFER5_EXT
        GL_COLOR_ATTACHMENT3_NV
        GL_COLOR_TABLE_BLUE_SIZE
        GL_FRAMEBUFFER_DEFAULT_LAYERS_EXT
        GL_Z4Y12Z4CB12Z4Y12Z4CR12_422_NV
        GL_VERTEX_ATTRIB_ARRAY12_NV
        GL_ARB_point_sprite
        GL_BLUE_BIAS
        GL_STENCIL_INDEX1
        GL_REFERENCED_BY_VERTEX_SHADER
        GL_TEXTURE_IMMUTABLE_FORMAT
        GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT
        GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB
        GL_CONTEXT_LOST
        GL_ARB_shader_atomic_counter_ops
        GL_UNSIGNED_INT16_VEC3_NV
        GL_TEXTURE_WRAP_T
        GL_DRAW_BUFFER1
        GL_OFFSET_TEXTURE_2D_BIAS_NV
        GL_DOUBLEBUFFER
        GL_INT_SAMPLER_1D
        GL_FLOAT_VEC4_ARB
        GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB
        GL_READ_BUFFER_EXT
        GL_VIRTUAL_PAGE_SIZE_INDEX_ARB
        GL_IMAGE_3D_EXT
        GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB
        GL_FONT_UNITS_PER_EM_BIT_NV
        GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB
        GL_UNIFORM_NAME_LENGTH
        GLEW_EXT_texture_view
        GL_PATH_FORMAT_SVG_NV
        GL_EYE_PLANE
        GL_FRAMEBUFFER_DEFAULT
        GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS
        GL_PN_TRIANGLES_NORMAL_MODE_ATI
        GL_ARB_multisample
        GL_CIRCULAR_CCW_ARC_TO_NV
        GL_TEXTURE21_ARB
        GLEW_NV_texture_compression_s3tc_update
        GL_UNSIGNED_INT_2_10_10_10_REV_EXT
        GL_MODULATE
        GL_FLOAT16_MAT3x4_AMD
        GL_IMAGE_CUBE
        GL_ATTENUATION_EXT
        GL_EXT_x11_sync_object
        GL_COMBINE_ALPHA_EXT
        GL_INTENSITY16_SNORM
        GLEW_NV_texture_rectangle
        GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE
        GLEW_EXT_secondary_color
        GL_NV_texture_npot_2D_mipmap
        GL_RGB_EXTENDED_RANGE_SGIX
        GL_STENCIL_PASS_DEPTH_FAIL
        GL_PIXEL_MAP_I_TO_A
        GL_LUMINANCE16_SIGNED_SGIX
        GL_DRAW_INDIRECT_UNIFIED_NV
        GL_VERTEX_SHADER_INSTRUCTIONS_EXT
        GL_CURRENT_RASTER_POSITION
        GL_TEXTURE_FREE_MEMORY_ATI
        GL_TEXTURE_FETCH_BARRIER_BIT_EXT
        GL_FRACTIONAL_ODD
        GL_PROGRAM_RESIDENT_NV
        GL_RGB_SCALE_EXT
        GL_VERTEX_BINDING_BUFFER
        GL_ANY_SAMPLES_PASSED_CONSERVATIVE
        GL_TRANSFORM_FEEDBACK_VARYINGS_NV
        GLEW_ARB_cl_event
        GL_TEXTURE_BORDER_COLOR_NV
        GL_COLOR_ATTACHMENT11_EXT
        GL_TEXTURE_HEIGHT_QCOM
        GL_LUMINANCE16_EXTENDED_RANGE_SGIX
        GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI
        GL_UNSIGNED_SHORT_1_5_5_5_REV
        GL_SCISSOR_BOX
        GLEW_SGIS_texture_edge_clamp
        GL_BALI_FRAGMENTS_GENERATED_INSTRUMENT
        GL_PIXEL_PACK_BUFFER_BINDING_EXT
        GL_CLAMP_READ_COLOR
        GL_DUP_FIRST_CUBIC_CURVE_TO_NV
        GL_TEXTURE_GATHER
        GL_SHADER_STORAGE_BUFFER_START
        GL_CURRENT_TANGENT_EXT
        GLEW_INGR_color_clamp
        GL_ARB_vertex_shader
        GL_PROGRAM_POINT_SIZE
        GL_STENCIL_PASS_DEPTH_PASS
        GL_VOLATILE_APPLE
        GL_FUNC_COMPLEX_MULTIPLY_EXT
        GL_PROXY_TEXTURE_2D_ARRAY_EXT
        GL_SGIX_spotlight_cutoff
        GL_DEBUG_SOURCE_OTHER
        GL_MODELVIEW18_ARB
        GL_INT_SAMPLER_CUBE_MAP_ARRAY_EXT
        GL_UNSIGNED_BYTE_3_3_2
        GL_POINT_SPRITE_CULL_MODE_ATIX
        GL_SAMPLER_2D_RECT_ARB
        GL_SUBSAMPLE_DISTANCE_AMD
        GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL
        GL_RGBA_FLOAT32_APPLE
        GL_TEXTURE_BINDING_BUFFER_EXT
        GL_ARB_get_program_binary
        GL_EXT_shared_texture_palette
        GL_SHARED_TEXTURE_PALETTE_EXT
        GL_UNPACK_ROW_BYTES_APPLE
        GL_CW
        GLEW_NV_blend_minmax_factor
        GL_EXT_blend_subtract
        GL_4PASS_0_SGIS
        GL_INVARIANT_EXT
        GLEW_EXT_multisample_compatibility
        GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB
        GL_KTX_BACK_REGION
        GL_VERSION_3_3
        GL_NEGATIVE_W_EXT
        GL_PIXEL_MAP_I_TO_G_SIZE
        GL_MIN_FRAGMENT_INTERPOLATION_OFFSET_NV
        GL_SAMPLE_MASK
        GL_INVERT
        GL_MAP_PERSISTENT_BIT_EXT
        GLEW_EXT_separate_shader_objects
        GLEW_EXT_gpu_shader5
        GL_CURRENT_MATRIX_STACK_DEPTH_NV
        GLEW_SGIS_shared_multisample
        GL_DRAW_BUFFER10
        GL_GPU_ADDRESS_NV
        GL_FOG_COORDINATE_SOURCE_EXT
        GL_VIEWPORT_INDEX_PROVOKING_VERTEX_NV
        GL_LOG_INFO_REGAL
        GL_MAX_DEBUG_LOGGED_MESSAGES_AMD
        GL_SCREEN_NV
        GL_RED_MAX_CLAMP_INGR
        GL_EXT_texture3D
        GLEW_SGIX_texture_supersample
        GL_PREVIOUS_ARB
        GL_PRIMITIVE_RESTART_FIXED_INDEX
        GL_COMBINER1_NV
        GL_BEVEL_NV
        GL_NV_video_capture
        GLEW_EXT_compiled_vertex_array
        GL_IMAGE_BINDING_FORMAT_EXT
        GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT
        GL_VIEWPORT_SWIZZLE_POSITIVE_W_NV
        GL_DEBUG_SEVERITY_HIGH_ARB
        GLEW_ATI_meminfo
        GL_SAMPLER_1D_ARB
        GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB
        GL_TEXTURE_EXTERNAL_OES
        GL_SOURCE2_RGB
        GL_OP_INDEX_EXT
        GLEW_ARB_blend_func_extended
        GL_LAYOUT_LINEAR_CPU_CACHED_INTEL
        GL_DOT3_RGB
        GL_MAX_LABEL_LENGTH
        GL_ARB_texture_env_add
        GL_LUMINANCE16_ALPHA8_ICC_SGIX
        GL_ARB_copy_buffer
        GL_FOG_COORD_ARRAY_STRIDE
        GL_UNSIGNED_INT64_NV
        GL_DEBUG_SEVERITY_LOW_AMD
        GL_ARB_conservative_depth
        GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT
        GL_DEPTH_BUFFER_BIT2_QCOM
        GL_MAX_PROGRAM_MATRICES_ARB
        GL_DEPTH_COMPONENT24_SGIX
        GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT
        GL_SAMPLER_2D_MULTISAMPLE
        GL_SGIX_ir_instrument1
        GL_REGAL_extension_query
        GL_TEXTURE10_ARB
        GL_EXT_sparse_texture2
        GL_TRUE
        GL_DYNAMIC_DRAW
        GLEW_EXT_texture_rectangle
        GL_QUERY_BUFFER
        GL_KHR_texture_compression_astc_hdr
        GLEW_ARB_shader_objects
        GL_SAMPLE_SHADING_ARB
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_NV
        GL_DEBUG_TYPE_POP_GROUP
        GL_EXPAND_NORMAL_NV
        GL_MIRROR_CLAMP_TO_BORDER_EXT
        GL_DEBUG_SOURCE_SHADER_COMPILER
        GL_MATRIX5_NV
        GL_BLUE_MAX_CLAMP_INGR
        GL_TEXTURE_OUTPUT_RGB_ATIX
        GL_3DC_XY_AMD
        GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT
        GLEW_ARB_shader_atomic_counter_ops
        GL_SECONDARY_INTERPOLATOR_ATI
        GL_DRAW_BUFFER4_NV
        GLEW_EXT_vertex_attrib_64bit
        GL_RENDERBUFFER_INTERNAL_FORMAT_EXT
        GL_DEBUG_TYPE_PORTABILITY
        GL_COLOR_TABLE_RED_SIZE
        GL_NV_occlusion_query
        GL_DEPTH_COMPONENT16_SGIX
        GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB
        GL_SOURCE2_ALPHA
        GL_DU8DV8_ATI
        GL_COMBINER2_NV
        GL_PREFER_DOUBLEBUFFER_HINT_PGI
        GL_TEXTURE_RED_TYPE
        GL_FRAMEBUFFER_BINDING_EXT
        GL_MAX_TEXTURE_BUFFER_SIZE
        GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER
        GL_READ_PIXEL_DATA_RANGE_LENGTH_NV
        GL_ARB_blend_func_extended
        GL_ARB_texture_env_dot3
        GL_ARB_color_buffer_float
        GL_VERTEX_ATTRIB_ARRAY8_NV
        GL_COPY_READ_BUFFER
        GL_EXT_pixel_buffer_object
        GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS
        GL_NUM_PASSES_ATI
        GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN
        GL_DRAW_BUFFER11
        GL_CLIENT_STORAGE_BIT
        GL_GREATER
        GL_PERFMON_RESULT_AMD
        GLEW_SUN_vertex
        GL_KEEP
        GL_RGBA8_SNORM
        GL_MAP_COHERENT_BIT
        GLEW_EXT_texture_filter_anisotropic
        GL_PIXEL_UNPACK_BUFFER_BINDING_EXT
        GL_FRAMEBUFFER_PROGRAMMABLE_SAMPLE_LOCATIONS_ARB
        GL_TEXTURE_COORD_ARRAY_SIZE
        GL_COMPRESSED_RGBA_ASTC_8x8_KHR
        GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV
        GL_MAP1_VERTEX_4_NURBS_SGIX
        GL_MULTISAMPLES_NV
        GL_CUBE_MAP_XP_SGIX
        GL_NV_fill_rectangle
        GLEW_NV_shader_buffer_load
        GL_SIGNED_NORMALIZED
        GL_VERSION_2_0
        GL_VIEWPORT_POSITION_W_SCALE_Y_COEFF_NV
        GL_FUNC_SUBTRACT_EXT
        GL_OUTPUT_TEXTURE_COORD22_EXT
        GL_COLORDODGE_NV
        GL_POLYGON_OFFSET_FACTOR
        GL_EMULATION_REGAL
        GLEW_ARB_tessellation_shader
        GL_ARRAY_BUFFER
        GL_INTERNALFORMAT_GREEN_SIZE
        GL_FRAMEBUFFER
        GLEW_EXT_color_buffer_float
        GL_QUADRATIC_CURVE_TO_NV
        GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB
        GL_DEPTH_COMPONENT32_SGIX
        GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_EXT
        GL_TEXTURE_BORDER
        GL_BOOL_VEC2_ARB
        GL_RGB2_EXT
        GL_MAX_PIXEL_MAP_TABLE
        GL_VERSION_4_0
        GLEW_SGIX_vertex_preclip
        GL_GLYPH_WIDTH_BIT_NV
        GL_NV_fog_distance
        GL_MEDIUM_INT
        GL_MODELVIEW0_EXT
        GL_HISTOGRAM_SINK_EXT
        GL_DISCARD_NV
        GL_ARB_shader_image_load_store
        GLEW_ARB_sparse_texture_clamp
        GLEW_EXT_texture_buffer_object
        GL_RG16
        GL_VIRTUAL_PAGE_SIZE_X_EXT
        GL_BLEND
        GL_ARB_sampler_objects
        GL_EXT_framebuffer_object
        GL_MAX_FFT_WIDTH_SGI
        GL_VIEW_CLASS_BPTC_UNORM
        GL_STENCIL_BACK_FAIL_ATI
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT
        GL_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS_EXT
        GL_DEBUG_TYPE_OTHER
        GL_PROXY_TEXTURE_1D_STACK_MESAX
        GL_KHR_texture_compression_astc_sliced_3d
        GL_RGBA32F_EXT
        GLEW_EXT_sRGB_write_control
        GL_PACK_COMPRESSED_BLOCK_SIZE
        GL_NV_fragment_program_option
        GL_FRAGMENT_INPUT_NV
        GL_PIXEL_MAP_R_TO_R_SIZE
        GL_RELATIVE_SMOOTH_QUADRATIC_CURVE_TO_NV
        GL_REFLECTION_MAP
        GL_CON_2_ATI
        GLEW_ARB_shader_subroutine
        GL_PERFQUERY_DONOT_FLUSH_INTEL
        GL_WEIGHT_ARRAY_BUFFER_BINDING
        GL_TEXTURE_PRIORITY_EXT
        GL_ALWAYS
        GL_NV_deep_texture3D
        GL_VERTEX_ATTRIB_MAP1_DOMAIN_APPLE
        GL_ATIX_texture_env_combine3
        GL_PATH_FOG_GEN_MODE_NV
        GL_EXT_shader_integer_mix
        GL_PROJECTION_MATRIX
        GL_COPY_WRITE_BUFFER
        GL_MAP1_BINORMAL_EXT
        GL_CURRENT_RASTER_COLOR
        GLEW_EXT_shadow_funcs
        GL_FOG_COORDINATE_ARRAY_LIST_IBM
        GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB
        GL_T2F_C4F_N3F_V3F
        GL_MODELVIEW13_ARB
        GL_REGAL_ES1_1_compatibility
        GL_ALPHA_EXTENDED_RANGE_SGIX
        GL_COLOR_TABLE_WIDTH
        GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS
        GL_ARB_texture_gather
        GL_SGIX_async
        GL_TEXTURE_COMPARE_MODE_EXT
        GLEW_SGIX_texture_mipmap_anisotropic
        GL_SR8_EXT
        GL_CONSTANT_COLOR1_NV
        GL_OPERAND1_RGB_ARB
        GL_UNPACK_SWAP_BYTES
        GL_RENDERBUFFER_ALPHA_SIZE_EXT
        GL_NV_image_formats
        GL_READ_WRITE
        GL_DEPTH_COMPONENT32_ARB
        GL_RG
        GL_DRAW_BUFFER3_ARB
        GL_NAMED_STRING_LENGTH_ARB
        GL_STENCIL_BITS
        GLEW_ARB_vertex_array_object
        GL_SGIX_pixel_tiles
        GL_AUTO_NORMAL
        GL_CONST_EYE_NV
        GL_PACK_SKIP_ROWS_NV
        GL_CURRENT_TIME_NV
        GLEW_SGIS_clip_band_hint
        GL_DRAW_BUFFER6_NV
        GL_SGIX_sprite
        GL_MAX_VERTEX_SHADER_VARIANTS_EXT
        GLEW_EXT_texture_shared_exponent
        GL_HALF_FLOAT
        GL_CURRENT_NORMAL
        GL_STENCIL_COMPONENTS
        GL_R8UI
        GL_COMPRESSED_SRGB_PVRTC_2BPPV1_EXT
        GLEW_EXT_timer_query
        GL_FLOAT_RGBA32_NV
        GL_ARM_shader_framebuffer_fetch_depth_stencil
        GL_TEXTURE_COORD_ARRAY_STRIDE_EXT
        GL_INTEL_parallel_arrays
        GL_SUPERSAMPLE_SCALE_X_NV
        GL_ARB_sparse_texture2
        GL_NV_framebuffer_multisample
        GL_SGI_texture_color_table
        GL_OUTPUT_TEXTURE_COORD21_EXT
        GL_SCREEN_COORDINATES_REND
        GLEW_AMD_transform_feedback4
        GL_INDEX_ARRAY_BUFFER_BINDING
        GL_SAMPLER_1D_ARRAY_EXT
        GL_HARDLIGHT_NV
        GL_EXCLUSION_NV
        GL_DEPENDENT_AR_TEXTURE_2D_NV
        GL_GEOMETRY_SUBROUTINE_UNIFORM
        GL_FLOAT16_MAT4x3_AMD
        GL_COMPRESSED_RGBA_ASTC_5x4_KHR
        GL_EXT_multisample_compatibility
        GLEW_QCOM_perfmon_global_mode
        GL_NV_shader_thread_group
        GL_INDEX
        GL_SGIX_mpeg2
        GL_MODELVIEW4_ARB
        GL_ARB_transform_feedback2
        GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB
        GLEW_EXT_gpu_shader4
        GL_CURRENT_MATRIX_NV
        GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN
        GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_NV
        GLEW_ARB_texture_multisample
        GL_CURRENT_FOG_COORDINATE_EXT
        GL_IMAGE_CLASS_10_10_10_2
        GL_AMD_stencil_operation_extended
        GL_DRAW_ELEMENTS_STRIP_COMMAND_NV
        GL_SWIZZLE_STR_DR_ATI
        GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_WRITE
        GL_SGIX_fragments_instrument
        GLEW_EGL_KHR_context_flush_control
        GL_READ_FRAMEBUFFER_APPLE
        GL_SHADER_IMAGE_ATOMIC
        GL_SAMPLE_ALPHA_TO_ONE_SGIS
        GL_SOFTLIGHT_NV
        GL_MAP2_VERTEX_ATTRIB15_4_NV
        GL_E_TIMES_F_NV
        GL_BOLD_BIT_NV
        GL_FOG_COORDINATE_ARRAY_STRIDE
        GL_TEXTURE_COORD_NV
        GL_SAMPLE_COUNT_BITS_NV
        GL_MULTISAMPLE_ARB
        GL_TRANSPOSE_MODELVIEW_MATRIX_ARB
        GL_PERFQUERY_COUNTER_RAW_INTEL
        GL_INDEX_ARRAY_POINTER_EXT
        GL_NORMAL_ARRAY_EXT
        GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT
        GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT
        GL_UNSIGNED_INT_SAMPLER_3D_EXT
        GLEW_AMD_interleaved_elements
        GL_MAX_TEXTURE_IMAGE_UNITS_NV
        GL_INTENSITY_SNORM
        GL_DISCARD_ATI
        GL_MAX_NUM_ACTIVE_VARIABLES
        GL_STENCIL_CLEAR_VALUE
        GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING
        GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT
        GL_MAP1_VERTEX_3_NURBS_SGIX
        GL_2X_BIT_ATI
        GL_IDENTITY_NV
        GL_MIN_GREEN_SGIS
        GLEW_ARB_transpose_matrix
        GL_TEXTURE_STACK_DEPTH
        GL_TEXTURE_MEMORY_LAYOUT_INTEL
        GL_RENDERBUFFER_DEPTH_SIZE_EXT
        GLEW_NV_depth_buffer_float
        GL_VERTEX_SHADER_INVOCATIONS
        GL_SM_COUNT_NV
        GL_REG_0_ATI
        GL_HILO8_NV
        GL_TEXTURE_GEN_R
        GL_CON_7_ATI
        GL_PIXEL_MAP_B_TO_B
        GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME
        GLEW_EXT_index_material
        GL_FLOAT_VEC3_ARB
        GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT
        GL_POST_COLOR_MATRIX_ALPHA_SCALE
        GL_ARB_sparse_texture
        GL_NV_geometry_program4
        GLEW_NV_shader_atomic_float
        GL_FOG_OFFSET_VALUE_SGIX
        GLEW_EXT_texture_mirror_clamp
        GL_EXT_blend_func_separate
        GL_COLOR_ATTACHMENT15_EXT
        GL_PROGRAMMABLE_SAMPLE_LOCATION_TABLE_SIZE_NV
        GL_PROGRAM_ERROR_POSITION_NV
        GL_NV_texture_shader3
        GL_ARB_compute_variable_group_size
        GL_INT_VEC2_ARB
        GLEW_EXT_texture_env_dot3
        GL_POLYGON_STIPPLE
        GL_AUX_BUFFERS
        GL_DRAW_BUFFER5_NV
        GL_DOT4_ATI
        GL_CLEAR_BUFFER
        GLEW_APPLE_clip_distance
        GL_ZERO_TO_ONE
        GL_APPLE_framebuffer_multisample
        GLEW_EXT_shader_image_load_formatted
        GL_COLOR_ATTACHMENT2_NV
        GLEW_APPLE_flush_buffer_range
        GLEW_NV_fbo_color_attachments
        GL_Z4Y12Z4CB12Z4A12Z4Y12Z4CR12Z4A12_4224_NV
        GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_EXT
        GL_SRG8_EXT
        GL_VERTEX_ATTRIB_ARRAY_DIVISOR_EXT
        GL_VECTOR_EXT
        GL_COMPILE
        GL_IMAGE_CUBE_MAP_ARRAY
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT
        GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI
        GLEW_EXT_misc_attribute
        GL_POINT_SIZE_MAX_EXT
        GL_SAMPLER_2D_RECT_SHADOW
        GL_RGBA32I_EXT
        GL_422_EXT
        GL_ARB_sparse_texture_clamp
        GL_EVAL_VERTEX_ATTRIB3_NV
        GLEW_VERSION_1_4
        GL_COLOR_ARRAY_STRIDE_EXT
        GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT
        GL_ATI_separate_stencil
        GLEW_EXT_semaphore_win32
        GL_NV_framebuffer_blit
        GL_LINES_ADJACENCY_ARB
        GL_GREEN_SCALE
        GL_VIEWPORT
        GL_VARIABLE_D_NV
        GL_SRGB8_NV
        GL_UNIFORM_MATRIX_STRIDE
        GL_EXT_texture_perturb_normal
        GL_UNSIGNED_INT_24_8
        GL_TRANSFORM_FEEDBACK_NV
        GL_CURRENT_INDEX
        GL_NV_multisample_coverage
        GL_POINT_FADE_THRESHOLD_SIZE_ARB
        GL_ATI_text_fragment_shader
        GL_SMOOTH_LINE_WIDTH_GRANULARITY
        GLEW_ARB_shading_language_100
        GL_CLIP_ORIGIN
        GL_SAMPLE_POSITION
        GL_Q
        GLEW_KHR_robustness
        GL_DEBUG_SEVERITY_HIGH
        GL_EXT_multisampled_render_to_texture
        GL_EXT_texture_filter_minmax
        GL_CUBE_MAP_ZP_SGIX
        GL_ARB_fragment_shader_interlock
        GL_INTERNALFORMAT_STENCIL_TYPE
        GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV
        GL_EXPAND_NEGATE_NV
        GL_DEPTH_BOUNDS_TEST_EXT
        GL_BLEND_DST
        GL_NUM_TILING_TYPES_EXT
        GL_CONSERVE_MEMORY_HINT_PGI
        GL_CLIP_DISTANCE1_EXT
        GL_EDGE_FLAG_ARRAY_ADDRESS_NV
        GL_KTX_FRONT_REGION
        GL_PATH_STROKE_COVER_MODE_NV
        GL_R1UI_N3F_V3F_SUN
        GL_MINMAX
        GL_SGIX_pixel_texture
        GL_COMBINER_MAPPING_NV
        GL_PIXEL_MAP_I_TO_R_SIZE
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR
        GL_COLOR_BUFFER_BIT4_QCOM
        GL_ALPHA8UI_EXT
        GLEW_AMD_sample_positions
        GL_R16I
        GL_RASTERIZER_DISCARD_EXT
        GL_RGB10_A2UI
        GL_MATRIX23_ARB
        GL_PACK_INVERT_MESA
        GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT_EXT
        GL_RELATIVE_ROUNDED_RECT8_NV
        GL_C4UB_V3F
        GL_INT_IMAGE_CUBE_MAP_ARRAY
        GL_SCALE_BY_FOUR_NV
        GLEW_NV_fragment_shader_interlock
        GLEW_EXT_gpu_program_parameters
        GL_DRAW_INDIRECT_LENGTH_NV
        GL_FRAMEBUFFER_DEFAULT_WIDTH
        GL_VERTEX_PROGRAM_POINT_SIZE_NV
        GL_FRAGMENT_PROGRAM_NV
        GL_COLOR_ATTACHMENT2_EXT
        GL_SYNC_OBJECT_APPLE
        GL_MAX_DEEP_3D_TEXTURE_WIDTH_HEIGHT_NV
        GL_COLOR_INDEX
        GL_PRIMARY_COLOR_EXT
        GL_LINE_STIPPLE
        GL_DRAW_FRAMEBUFFER_BINDING
        GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI
        GL_TRANSLATE_X_NV
        GL_STORAGE_CACHED_APPLE
        GL_MAP1_VERTEX_ATTRIB4_4_NV
        GL_DRAW_BUFFER6_ATI
        GL_MATRIX15_ARB
        GL_CLIP_DISTANCE0_EXT
        GL_SAMPLER_2D_RECT_SHADOW_ARB
        GL_TEXTURE_VIEW_MIN_LAYER
        GL_SUB_ATI
        GLEW_ATI_envmap_bumpmap
        GLEW_ARB_texture_storage_multisample
        GL_NOTEQUAL
        GL_ACCUM_GREEN_BITS
        GL_ACTIVE_UNIFORM_BLOCKS
        GL_LOG_REGAL
        GLEW_REGAL_error_string
        GL_VERSION_1_3
        GL_PIXEL_SUBSAMPLE_2424_SGIX
        GL_MINMAX_FORMAT
        GLEW_ARB_fragment_program_shadow
        GL_QUERY_BUFFER_BINDING_AMD
        GL_ARB_timer_query
        GL_TEXTURE_CUBE_MAP_ARRAY_ARB
        GL_SGIX_color_table_index_mode
        GL_LINE_WIDTH_COMMAND_NV
        GL_READ_FRAMEBUFFER_ANGLE
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_GEOMETRY_SHADER
        GL_TEXTURE_MIN_LOD_SGIS
        GL_SRC2_RGB
        GLEW_AMD_shader_stencil_value_export
        GL_MODELVIEW30_ARB
        GL_DOUBLE_VEC2_EXT
        GL_QUERY_STENCIL_FAIL_EVENT_BIT_AMD
        GL_DEBUG_TYPE_PERFORMANCE_ARB
        GL_YCRCBA_SGIX
        GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_CONTROL_SHADER
        GL_SAMPLE_PATTERN_EXT
        GL_SAMPLER_1D_SHADOW
        GL_DRAW_BUFFER8
        GL_MIN_LUMINANCE_SGIS
        GL_DOUBLE_EXT
        GL_MAX_GENERAL_COMBINERS_NV
        GL_INTEL_performance_query
        GL_EXT_index_func
        GL_VIEWPORT_SWIZZLE_NEGATIVE_Z_NV
        GL_MAX_DEBUG_MESSAGE_LENGTH
        GL_UNSIGNED_BYTE
        GL_NV_texture_multisample
        GL_BOUNDING_BOX_NV
        GL_VARIANT_EXT
        GL_DEBUG_SEVERITY_LOW_ARB
        GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED
        GL_VERTEX_ATTRIB_ARRAY7_NV
        GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT
        GL_ARB_occlusion_query
        GL_NV_EGL_stream_consumer_external
        GL_CON_5_ATI
        GL_MAX_FRAGMENT_INTERPOLATION_OFFSET_NV
        GL_ONE_MINUS_CONSTANT_COLOR
        GL_PROXY_TEXTURE_1D_ARRAY
        GL_ALPHA16_SIGNED_SGIX
        GL_OP_DOT4_EXT
        GL_IS_PER_PATCH
        GL_ARB_imaging
        GL_PROXY_TEXTURE_3D_EXT
        GL_VERTEX_SOURCE_ATI
        GL_INTERLACE_OML
        GL_UNCORRELATED_NV
        GL_RESAMPLE_ZERO_FILL_SGIX
        GL_TEXTURE_ENV_COORD_SET_SGIS
        GL_VERTEX_BINDING_DIVISOR
        GL_PERFMON_RESULT_AVAILABLE_AMD
        GL_MAX_PROGRAM_ENV_PARAMETERS_ARB
        GL_BUFFER_MAPPED
        GL_SAMPLER_3D
        GL_POINT_SIZE_MAX_ARB
        GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS
        GL_TRIANGLE_STRIP_ADJACENCY_EXT
        GL_UNIFORM_BUFFER_ADDRESS_NV
        GL_LUMINANCE_ALPHA16F_ARB
        GL_SAMPLES_PASSED
        GLEW_NVX_gpu_memory_info
        GL_BALI_DEPTH_PASS_INSTRUMENT
        GL_DARKEN_KHR
        GLEW_EXT_YUV_target
        GL_VIDEO_COLOR_CONVERSION_MATRIX_NV
        GLEW_REGAL_extension_query
        GL_INT_SAMPLER_1D_EXT
        GL_COLOR_TABLE_BIAS_SGI
        GLEW_AMD_texture_gather_bias_lod
        GL_MAX_VERTEX_OUTPUT_COMPONENTS
        GL_QUERY_RESULT_ANGLE
        GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB
        GL_COMPRESSED_TEXTURE_FORMATS_ARB
        GLEW_NV_shader_storage_buffer_object
        GL_LUMINANCE12_ALPHA4_EXT
        GLEW_EXT_draw_buffers_indexed
        GL_OUTPUT_TEXTURE_COORD10_EXT
        GL_OUTPUT_TEXTURE_COORD23_EXT
        GL_SIGNED_LUMINANCE8_ALPHA8_NV
        GL_PREVIOUS
        GL_VIEWPORT_SWIZZLE_NEGATIVE_X_NV
        GL_CURRENT_RASTER_INDEX
        GL_DOUBLE_MAT2x3_EXT
        GL_DEBUG_SOURCE_OTHER_ARB
        GL_EXT_texture_compression_s3tc
        GL_COLOR_BUFFER_BIT
        GLEW_ARB_multisample
        GL_LO_BIAS_NV
        GL_EXT_light_texture
        GL_PERFQUERY_COUNTER_DATA_DOUBLE_INTEL
        GL_ROUND_NV
        GLEW_EXT_texture_storage
        GL_TEXTURE_BINDING_CUBE_MAP
        GLEW_NV_EGL_stream_consumer_external
        GL_UNPACK_SKIP_IMAGES
        GLEW_ARB_shading_language_420pack
        GL_FLOAT_MAT4x3
        GL_DEPTH_BUFFER_BIT
        GLEW_KTX_buffer_region
        GL_NV_instanced_arrays
        GL_PROXY_HISTOGRAM_EXT
        GL_VERTEX_ATTRIB_ARRAY_DIVISOR_NV
        GL_PROGRAM_TEX_INSTRUCTIONS_ARB
        GLEW_NV_robustness_video_memory_purge
        GL_QCOM_shader_framebuffer_fetch_noncoherent
        GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS
        GL_EXT_color_buffer_half_float
        GL_PROGRAM_NATIVE_PARAMETERS_ARB
        GL_SYSTEM_FONT_NAME_NV
        GLEW_AMD_gpu_shader_half_float
        GL_SOURCE1_RGB
        GL_VERTEX_SHADER_INVOCATIONS_ARB
        GL_STATIC_READ
        GL_QUERY_WAIT_INVERTED
        GL_VERTEX23_BIT_PGI
        GL_INT_SAMPLER_CUBE_MAP_ARRAY
        GL_NV_framebuffer_multisample_coverage
        GL_BUFFER_ACCESS_FLAGS
        GL_DRAW_BUFFER9_ARB
        GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT
        GL_NV_sRGB_formats
        GL_TESS_EVALUATION_SUBROUTINE
        GL_TEXTURE_RED_SIZE_EXT
        GL_POST_CONVOLUTION_ALPHA_SCALE_EXT
        GL_MAX_GEOMETRY_OUTPUT_COMPONENTS_EXT
        GLEW_NV_fence
        GL_UNSIGNALED_APPLE
        GL_SGIX_list_priority
        GL_SGIS_clip_band_hint
        GL_OBJECT_LINE_SGIS
        GL_DSDT8_MAG8_INTENSITY8_NV
        GL_MAX_PROGRAM_IF_DEPTH_NV
        GL_LOGIC_OP
        GL_NV_geometry_shader4
        GL_EXTENDED_RANGE_SGIS
        GL_INTENSITY_FLOAT32_APPLE
        GL_NORMAL_MAP_ARB
        GL_TEXTURE29_ARB
        GL_CULL_VERTEX_EXT
        GL_OBJECT_VALIDATE_STATUS_ARB
        GL_WRITE_DISCARD_NV
        GL_INTERNALFORMAT_RED_SIZE
        GLEW_NV_vertex_program2_option
        GL_IMAGE_2D_MULTISAMPLE
        GL_SRGB8
        GL_RELATIVE_ROUNDED_RECT2_NV
        GL_EXT_pvrtc_sRGB
        GLEW_SGIX_ir_instrument1
        GL_OPERAND2_RGB_EXT
        GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT_ARB
        GL_FRAGMENT_SHADER
        GL_LUMINANCE12_ALPHA12_EXT
        GL_COMPUTE_SUBROUTINE
        GLEW_ARB_instanced_arrays
        GL_PRIMITIVE_RESTART_INDEX
        GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS
        GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV
        GL_INTENSITY32UI_EXT
        GL_DOUBLE_MAT4x3_EXT
        GL_SLUMINANCE_ALPHA
        GL_COLOR_ALPHA_PAIRING_ATI
        GLEW_EXT_convolution
        GL_MATRIX_MODE
        GLEW_ARB_shader_ballot
        GL_INT8_VEC2_NV
        GL_SAMPLER_CUBE_ARB
        GL_DRAW_BUFFER10_NV
        GL_RGB12
        GL_REG_2_ATI
        GL_DRAW_FRAMEBUFFER_ANGLE
        GL_CLIENT_ACTIVE_TEXTURE_ARB
        GL_DEPTH_ATTACHMENT
        GL_EXT_geometry_point_size
        GL_OUTPUT_TEXTURE_COORD14_EXT
        GL_TESS_EVALUATION_SHADER_INVOCATIONS
        GL_POINT_SPRITE_CULL_CENTER_ATIX
        GL_ARB_direct_state_access
        GL_GET_TEXTURE_IMAGE_FORMAT
        GLEW_EXT_stencil_clear_tag
        GL_MULTISAMPLE_BUFFER_BIT7_QCOM
        GL_NV_stereo_view_rendering
        GL_YCBYCR8_422_NV
        GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS
        GLEW_OVR_multiview_multisampled_render_to_texture
        GL_VERSION
        GL_OBJECT_SUBTYPE_ARB
        GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB
        GLEW_KHR_context_flush_control
        GL_DRAW_BUFFER4_ARB
        GLEW_EXT_Cg_shader
        GL_COLOR_TABLE_ALPHA_SIZE_EXT
        GL_COLOR_ATTACHMENT13
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_VERTEX_SHADER
        GL_RESCALE_NORMAL
        GL_OUTPUT_TEXTURE_COORD6_EXT
        GLEW_ARB_vertex_blend
        GLEW_ARB_post_depth_coverage
        GLEW_EXT_shader_image_load_store
        GLEW_SGIS_detail_texture
        GL_VIEW_CLASS_64_BITS
        GL_TEXTURE_MIN_FILTER
        GL_TEXTURE_DS_SIZE_NV
        GL_QCOM_framebuffer_foveated
        GL_UNSIGNED_INT_VEC2
        GL_ALL_BARRIER_BITS_EXT
        GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB
        GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
        GL_TEXTURE_BINDING_RECTANGLE_EXT
        GLEW_EXT_texture_array
        GL_RECT_NV
        GL_SIGNALED
        GL_TEXTURE_PRIORITY
        GL_VERTEX_ATTRIB_MAP2_SIZE_APPLE
        GL_INDEX_ARRAY_BUFFER_BINDING_ARB
        GL_RGB_SNORM
        GL_LIGHT3
        GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX
        GL_STENCIL_BACK_FAIL
        GL_REFERENCED_BY_GEOMETRY_SHADER_EXT
        GL_BALI_NUM_PRIMS_REJECT_INSTRUMENT
        GL_GEOMETRY_BIT
        GL_ARB_internalformat_query
        GLEW_EXT_base_instance
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT
        GL_ARB_vertex_type_2_10_10_10_rev
        GL_SRC_ALPHA_SATURATE_EXT
        GLEW_INGR_interlace_read
        GL_VIEWPORT_SWIZZLE_POSITIVE_Y_NV
        GL_DRAW_BUFFER
        GL_FONT_X_MAX_BOUNDS_BIT_NV
        GL_SGIX_shadow_ambient
        GL_TEXTURE_SWIZZLE_A
        GL_IMAGE_PIXEL_FORMAT
        GL_RGB_RAW_422_APPLE
        GL_COLOR_ATTACHMENT14
        GL_INTEL_texture_scissor
        GL_PATH_FILL_MODE_NV
        GLEW_ARB_ES3_2_compatibility
        GL_NORMAL_ARRAY_ADDRESS_NV
        GL_UNSIGNED_SHORT_8_8_MESA
        GL_AMD_pinned_memory
        GL_COMBINER6_NV
        GL_DEBUG_SEVERITY_HIGH_AMD
        GL_TEXTURE1
        GLEW_SGIX_shadow
        GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV
        GL_COMPRESSED_SIGNED_RG_RGTC2
        GL_POLYGON
        GL_OUTPUT_TEXTURE_COORD26_EXT
        GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT
        GL_DEPTH_COMPONENT24
        GL_ELEMENT_ARRAY_ADDRESS_NV
        GL_MATRIX20_ARB
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR
        GLEW_EXT_buffer_storage
        GL_LINE_RESET_TOKEN
        GL_RENDERBUFFER_GREEN_SIZE_EXT
        GL_DEPTH_COMPONENT16_ARB
        GL_VIEWPORT_BOUNDS_RANGE_NV
        GL_TEXTURE_HI_SIZE_NV
        GL_PATH_MITER_LIMIT_NV
        GL_LOCAL_CONSTANT_VALUE_EXT
        GL_VERTEX_ATTRIB_ARRAY_DIVISOR
        GL_TEXTURE_SHARED_SIZE_EXT
        GL_STENCIL_BACK_OP_VALUE_AMD
        GL_REG_3_ATI
        GLEW_ARB_texture_mirrored_repeat
        GL_GREEN
        GL_ARB_vertex_program
        GL_R11F_G11F_B10F
        GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV
        GL_TRIANGULAR_NV
        GL_TEXTURE_BINDING_2D
        GL_FILL_RECTANGLE_NV
        GL_ARB_pixel_buffer_object
        GL_ARB_draw_instanced
        GLEW_ARB_draw_elements_base_vertex
        GL_MAX_ASYNC_TEX_IMAGE_SGIX
        GL_MAX_NAME_LENGTH
        GL_BYTE
        GL_SGIS_texture_lod
        GL_SOURCE2_ALPHA_EXT
        GL_RELATIVE_QUADRATIC_CURVE_TO_NV
        GL_MULTISAMPLE
        GL_MAP_TESSELLATION_NV
        GL_SLUMINANCE8_ALPHA8_EXT
        GL_COLOR_ARRAY_TYPE_EXT
        GL_MATRIX17_ARB
        GL_ARM_mali_shader_binary
        GL_SCALED_RESOLVE_NICEST_EXT
        GL_SAMPLE_POSITION_NV
        GL_NV_transform_feedback
        GL_ATC_RGB_AMD
        GL_COLOR_MATERIAL
        GL_BALI_NUM_PRIMS_CLIPPED_INSTRUMENT
        GL_CONSERVATIVE_RASTER_DILATE_GRANULARITY_NV
        GL_FLOAT16_MAT2x4_AMD
        GL_AMD_name_gen_delete
        GL_OUT_OF_MEMORY
        GL_CONSTANT_BORDER
        GL_RED
        GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB
        GL_INDEX_WRITEMASK
        GL_2PASS_0_SGIS
        GL_VERTEX_STREAM1_ATI
        GLEW_ARB_texture_env_combine
        GLEW_EXT_separate_specular_color
        GL_SOURCE0_ALPHA_EXT
        GL_PATH_INITIAL_END_CAP_NV
        GL_MATRIX24_ARB
        GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT
        GL_DEPTH_STENCIL_TO_BGRA_NV
        GL_PATH_END_CAPS_NV
        GL_TESS_CONTROL_PROGRAM_NV
        GL_POLYGON_OFFSET_FACTOR_EXT
        GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS
        GL_MATRIX18_ARB
        GL_ARB_texture_env_crossbar
        GL_FLOAT16_VEC3_NV
        GL_SGIX_tag_sample_buffer
        GLEW_EXT_pixel_buffer_object
        GLEW_REND_screen_coordinates
        GL_INT_VEC2
        GLEW_EXT_paletted_texture
        GL_INTERPOLATE_ARB
        GL_DRAW_BUFFER12_NV
        GL_READ_ONLY_ARB
        GL_IGLOO_TMESHMODE_SGIX
        GL_TESS_EVALUATION_TEXTURE
        GL_COMBINER0_NV
        GL_TEXTURE_DEPTH_SIZE
        GL_ARB_shader_storage_buffer_object
        GL_BUFFER_OBJECT_EXT
        GL_PROGRAM_INPUT
        GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS
        GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT
        GL_AFFINE_3D_NV
        GL_ARB_debug_output
        GL_TESS_EVALUATION_SUBROUTINE_UNIFORM
        GL_MAX_FRAMEBUFFER_HEIGHT
        GL_COLOR_MATRIX_SGI
        GL_PROGRAM_PIPELINE_BINDING
        GL_DOT_PRODUCT_NV
        GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB
        GLEW_ARB_depth_clamp
        GLEW_ARB_fragment_coord_conventions
        GL_NORMAL_ARRAY_TYPE
        GL_NORMAL_ARRAY_POINTER
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT
        GL_STENCIL_INDEX16
        GL_DEBUG_CATEGORY_DEPRECATION_AMD
        GL_COVERAGE_MODULATION_TABLE_NV
        GLEW_SGIX_clipmap
        GL_OPERAND3_ALPHA_NV
        GL_FLOAT_MAT3x4
        GL_EQUIV
        GL_ACTIVE_ATOMIC_COUNTER_BUFFERS
        GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB
        GL_INNOCENT_CONTEXT_RESET_ARB
        GLEW_ATI_vertex_attrib_array_object
        GL_BINORMAL_ARRAY_TYPE_EXT
        GLEW_NV_conservative_raster_pre_snap_triangles
        GL_TEXTURE11_ARB
        GL_CONSERVATIVE_RASTER_DILATE_RANGE_NV
        GL_MULTISAMPLE_BIT_EXT
        GL_SGIX_calligraphic_fragment
        GLEW_MESA_window_pos
        GLEW_ARB_timer_query
        GL_BUFFER_IMMUTABLE_STORAGE_EXT
        GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ARB
        GL_KHR_no_error
        GL_SCISSOR_COMMAND_NV
        GL_SGIX_complex_polar
        GL_UNSIGNED_SHORT_5_5_5_1_EXT
        GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS
        GL_MAX_PROGRAM_CALL_DEPTH_NV
        GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI
        GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT
        GL_MIRROR_CLAMP_EXT
        GL_PROXY_TEXTURE_1D
        GL_CURRENT_ATTRIB_NV
        GLEW_SGIS_texture_lod
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR
        GL_DEPTH_RENDERABLE
        GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET
        GL_NATIVE_GRAPHICS_END_HINT_PGI
        GLEW_AMD_draw_buffers_blend
        GL_DRAW_FRAMEBUFFER_NV
        GL_CLIP_DISTANCE4_EXT
        GL_CLIPPING_OUTPUT_PRIMITIVES_ARB
        GLEW_APPLE_texture_2D_limited_npot
        GL_LOG_INTERNAL_REGAL
        GL_DEBUG_TYPE_OTHER_ARB
        GL_ATI_meminfo
        GL_DSDT_MAG_NV
        GLEW_NV_non_square_matrices
        GL_PROGRAM_NATIVE_ATTRIBS_ARB
        GL_FRAGMENT_DEPTH
        GL_EXT_buffer_storage
        GL_PROXY_TEXTURE_2D
        GL_RGBA_MODE
        GL_VERTEX_ATTRIB_ARRAY_TYPE
        GL_LOG_APP_REGAL
        GL_VIEWPORT_SWIZZLE_POSITIVE_Z_NV
        GL_ALL_BARRIER_BITS
        GLEW_REGAL_ES1_0_compatibility
        GLEW_EXT_copy_image
        GL_TEXTURE_RESIDENT_EXT
        GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT
        GL_LUMINANCE6_ALPHA2_EXT
        GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB
        GL_POST_CONVOLUTION_ALPHA_BIAS_EXT
        GL_CLAMP
        GL_MULTIPLY_KHR
        GL_ARB_texture_filter_minmax
        GLEW_ARB_base_instance
        GL_TEXTURE_MAG_SIZE_NV
        GL_PRESENT_DURATION_NV
        GL_MAX_PROGRAM_LOOP_DEPTH_NV
        GL_SAMPLE_LOCATION_SUBPIXEL_BITS_ARB
        GL_SHADER_STORAGE_BUFFER_OFFSET_ALIGNMENT
        GL_POINT_SPRITE_ARB
        GL_OPERAND1_ALPHA_ARB
        GL_ORDER
        GL_SMOOTH_POINT_SIZE_GRANULARITY
        GL_MAP2_INDEX
        GL_EYE_LINE_SGIS
        GL_PROVOKING_VERTEX
        GL_READ_FRAMEBUFFER_BINDING_EXT
        GL_VERTEX_ATTRIB_MAP1_APPLE
        GLEW_SGIX_datapipe
        GL_TEXTURE_GEQUAL_R_SGIX
        GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED_ARB
        GL_ARB_transpose_matrix
        GL_INT16_VEC2_NV
        GL_SRGB8_EXT
        GL_NV_shader_storage_buffer_object
        GL_PACK_COMPRESSED_BLOCK_DEPTH
        GL_SYNC_GPU_COMMANDS_COMPLETE
        GLEW_NV_fragment_program_option
        GL_INDEX_SHIFT
        GL_FILL_NV
        GL_LIGHT6
        GL_ARB_vertex_array_object
        GL_S
        GLEW_QCOM_extended_get
        GL_TOP_LEVEL_ARRAY_STRIDE
        GL_VARIANT_ARRAY_TYPE_EXT
        GL_MAX_DRAW_BUFFERS_ATI
        GLEW_EXT_blend_subtract
        GL_ATOMIC_COUNTER_BUFFER_SIZE
        GL_MAX_COMBINED_SHADER_OUTPUT_RESOURCES
        GL_MAP1_VERTEX_ATTRIB10_4_NV
        GLEW_AMD_transform_feedback3_lines_triangles
        GL_GPU_MEMORY_INFO_DEDICATED_VIDMEM_NVX
        GL_LINEARDODGE_NV
        GLEW_ATI_map_object_buffer
        GL_VERTEX_PRECLIP_SGIX
        GL_TRANSFORM_FEEDBACK_BINDING
        GLEW_NV_register_combiners
        GL_EXT_pixel_transform_color_table
        GL_NV_vertex_array_range
        GLEW_KHR_blend_equation_advanced_coherent
        GL_MULT
        GL_MAP1_NORMAL_NURBS_SGIX
        GL_LUMINANCE16_SNORM
        GL_TRANSFORM_FEEDBACK
        GL_TEXTURE_WRAP_R
        GL_MULTISAMPLE_BUFFER_BIT2_QCOM
        GL_PACK_REVERSE_ROW_ORDER_ANGLE
        GL_ACTIVE_TEXTURE
        GLEW_ARB_framebuffer_sRGB
        GL_BUFFER_SERIALIZED_MODIFY_APPLE
        GL_VIDEO_CAPTURE_FIELD_UPPER_HEIGHT_NV
        GL_NVIDIA_PLATFORM_BINARY_NV
        GL_APPLE_transform_hint
        GL_TEXTURE_BUFFER_OFFSET
        GLEW_SGIX_subdiv_patch
        GL_SHADER_STORAGE_BLOCK
        GLEW_EXT_framebuffer_object
        GL_MAX_DEBUG_MESSAGE_LENGTH_ARB
        GL_CND_ATI
        GL_TEXTURE_IMAGE_VALID_QCOM
        GL_SGIX_shadow
        GLEW_SGIX_tag_sample_buffer
        GL_SCALEBIAS_HINT_SGIX
        GL_RGBA12
        GL_EXCLUSIVE_EXT
        GL_FRAGMENT_SHADER_DERIVATIVE_HINT
        GLEW_EXT_fog_coord
        GL_MOVE_TO_CONTINUES_NV
        GLEW_ARB_invalidate_subdata
        GL_VIVIDLIGHT_NV
        GLEW_IBM_texture_mirrored_repeat
        GL_OFFSET_TEXTURE_RECTANGLE_NV
        GL_CULL_VERTEX_OBJECT_POSITION_EXT
        GL_EXT_texture_env_combine
        GL_POST_CONVOLUTION_GREEN_SCALE
        GL_SGIX_fog_texture
        GL_DISPATCH_INDIRECT_BUFFER_BINDING
        GL_PURGED_CONTEXT_RESET_NV
        GL_COMPRESSED_RGBA_ASTC_10x5_KHR
        GL_DSDT_MAG_INTENSITY_NV
        GL_SRC1_ALPHA
        GL_FRAGMENT_SHADER_BIT
        GL_TEXTURE_1D
        GL_UNSIGNED_INT_IMAGE_2D
        GL_NORMAL_ARRAY_BUFFER_BINDING_ARB
        GL_UNIFORM_BLOCK_BINDING
        GL_UNSIGNED_INT16_VEC4_NV
        GL_DRAW_BUFFER10_ARB
        GL_ARB_shadow
        GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT
        GL_DRAW_ARRAYS_COMMAND_NV
        GL_ARB_geometry_shader4
        GL_MAP1_VERTEX_ATTRIB1_4_NV
        GL_REPLICATE_BORDER
        GLEW_OK
        GL_TEXTURE_BINDING_RECTANGLE_NV
        GL_RELATIVE_SMALL_CW_ARC_TO_NV
        GL_TEXTURE_STORAGE_HINT_APPLE
        GL_FOG_GROUND_PLANE_SGIX
        GLEW_NV_gpu_program5_mem_extended
        GLEW_ARB_explicit_uniform_location
        GL_SIGNED_HILO16_NV
        GL_ACCUM_BUFFER_BIT
        GL_SGIX_nurbs_eval
        GL_MIN_PROGRAM_TEXEL_OFFSET
        GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL
        GL_LINE_STIPPLE_REPEAT
        GL_BACK_NORMALS_HINT_PGI
        GLEW_ARB_arrays_of_arrays
        GL_DST_ALPHA
        GL_ARB_post_depth_coverage
        GL_TEXTURE_COMPRESSED_BLOCK_WIDTH
        GL_VERTEX_STREAM5_ATI
        GL_POST_CONVOLUTION_RED_BIAS_EXT
        GL_UNIFORM_OFFSET
        GL_DEBUG_CALLBACK_USER_PARAM_ARB
        GL_RGB_FLOAT32_ATI
        GL_TEXTURE_MATERIAL_PARAMETER_EXT
        GL_UNSIGNED_INT64_VEC4_NV
        GL_OVR_multiview2
        GL_AUX3
        GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_NV
        GL_DRAW_FRAMEBUFFER_BINDING_APPLE
        GL_ANGLE_instanced_arrays
        GL_UNIFORM_BUFFER_BINDING
        GL_DRAW_BUFFER13_ATI
        GL_RG16_SNORM
        GL_PROXY_TEXTURE_CUBE_MAP_ARRAY
        GL_LAYER_PROVOKING_VERTEX
        GL_COLOR_ATTACHMENT0
        GL_TRANSFORM_FEEDBACK_BUFFER_MODE
        GL_R32F
        GL_MAX_RECTANGLE_TEXTURE_SIZE_NV
        GL_4X_BIT_ATI
        GL_READ_FRAMEBUFFER
        GL_NUM_INSTRUCTIONS_PER_PASS_ATI
        GL_DRAW_BUFFER13_EXT
        GL_BUFFER_IMMUTABLE_STORAGE
        GLEW_SGIS_multitexture
        GL_MAX_CONVOLUTION_HEIGHT_EXT
        GL_DYNAMIC_READ_ARB
        GL_NORMAL_ARRAY_BUFFER_BINDING
        GL_LINE_SMOOTH
        GL_NEGATIVE_ONE_EXT
        GL_EXT_texture_sRGB_RG8
        GL_SAMPLER_CUBE_SHADOW_NV
        GL_CG_VERTEX_SHADER_EXT
        GLEW_OVR_multiview2
        GL_R11F_G11F_B10F_EXT
        GL_BLUE_INTEGER_EXT
        GL_PERFQUERY_COUNTER_DATA_BOOL32_INTEL
        GL_COMPRESSED_SRGB_ALPHA
        GL_MODELVIEW0_STACK_DEPTH_EXT
        GL_MAX_VARYING_FLOATS_ARB
        GL_UNPACK_SKIP_ROWS_EXT
        GL_OES_byte_coordinates
        GLEW_EXT_raster_multisample
        GL_TEXTURE_RECTANGLE_ARB
        GL_FOG_BLEND_COLOR_SGIX
        GL_ARB_seamless_cubemap_per_texture
        GL_MATRIX1_ARB
        GL_TRANSFORM_FEEDBACK_ACTIVE
        GL_ARB_shader_objects
        GLEW_INTEL_conservative_rasterization
        GL_MAX_PROGRAM_TEXEL_OFFSET
        GL_MULTISAMPLE_EXT
        GLEW_ARB_vertex_array_bgra
        GL_PIXEL_MAP_I_TO_B_SIZE
        GL_MODELVIEW5_ARB
        GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT
        GL_MINMAX_SINK
        GL_FLOAT_RGB_NV
        GLEW_SGI_complex
        GL_QUERY_NO_WAIT_NV
        GL_ALPHA_SCALE
        GL_EIGHTH_BIT_ATI
        GL_ONE_MINUS_CONSTANT_ALPHA_EXT
        GL_SEPARABLE_2D_EXT
        GLEW_SGIX_fog_offset
        GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI
        GL_COMPARE_R_TO_TEXTURE_ARB
        GL_RED_BITS
        GL_COLOR_TABLE_LUMINANCE_SIZE_EXT
        GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS
        GL_UNSIGNED_INT_IMAGE_3D_EXT
        GL_MAX_TEXTURE_UNITS_ARB
        GL_READ_FRAMEBUFFER_EXT
        GLEW_KHR_debug
        GL_TRIANGLE_LIST_SUN
        GL_FRAMEBUFFER_COMPLETE_EXT
        GL_COMPRESSED_SRGB_S3TC_DXT1_NV
        GLEW_SGIX_fog_patchy
        GL_PROXY_POST_CONVOLUTION_COLOR_TABLE
        GL_CLIP_DISTANCE7_EXT
        GL_SGIS_sharpen_texture
        GL_SURFACE_MAPPED_NV
        GL_MAP1_VERTEX_ATTRIB3_4_NV
        GL_VIEW_CLASS_S3TC_DXT5_RGBA
        GL_NORMAL_BIT_PGI
        GLEW_ARB_query_buffer_object
        GLEW_EXT_compressed_ETC1_RGB8_sub_texture
        GL_SRGB8_ALPHA8_EXT
        GL_ARB_arrays_of_arrays
        GL_VARIABLE_A_NV
        GL_TEXTURE_CUBE_MAP_POSITIVE_Y
        GLEW_SGIX_ycrcb
        GL_TEXTURE2_ARB
        GLEW_INTEL_performance_query
        GL_ALPHA32F_EXT
        GL_HSL_COLOR_KHR
        GL_PIXEL_PACK_BUFFER_ARB
        GL_MOVE_TO_RESETS_NV
        GL_TEXTURE20
        GL_UNSIGNED_INT_IMAGE_1D_EXT
        GLEW_APPLE_object_purgeable
        GL_ARB_instanced_arrays
        GLEW_ATI_texture_env_combine3
        GL_MULTISAMPLE_COVERAGE_MODES_NV
        GL_INTERNALFORMAT_DEPTH_TYPE
        GL_UNSIGNED_INT_SAMPLER_2D
        GL_APPLE_texture_range
        GL_DISCRETE_AMD
        GL_CURRENT_MATRIX_INDEX_ARB
        GLEW_NV_present_video
        GLEW_ARB_shader_texture_image_samples
        GL_COMBINE_ALPHA_ARB
        GL_SPOT_DIRECTION
        GL_COMPRESSED_RGB_FXT1_3DFX
        GL_CONVOLUTION_1D
        GL_BLOCK_INDEX
        GL_COLOR_WRITEMASK
        GL_OBJECT_ATTACHED_OBJECTS_ARB
        GL_UNIFORM_BUFFER
        GL_VERTEX_ARRAY_RANGE_POINTER_NV
        GL_RGB32F_EXT
        GL_RGBA_INTEGER
        GL_DEPTH_STENCIL_NV
        GL_ARB_texture_stencil8
        GLEW_IBM_rasterpos_clip
        GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT
        GL_COMPLEX_INT_SGI
        GL_SAMPLER_BUFFER_EXT
        GL_ARB_enhanced_layouts
        GL_BUFFER_ACCESS
        GL_3_BYTES
        GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX
        GL_HALF_BIT_ATI
        GL_CONVOLUTION_FILTER_SCALE_EXT
        GL_QUERY_BY_REGION_WAIT_NV
        GL_TEXTURE7
        GLEW_SGIX_bali_g_instruments
        GL_SOURCE0_RGB
        GL_TEXTURE2
        GL_TANGENT_SPACE_SGIX
        GL_ARB_gpu_shader_fp64
        GL_GEOMETRY_SHADER_INVOCATIONS_EXT
        GL_EVAL_VERTEX_ATTRIB9_NV
        GL_ONE_MINUS_SRC1_ALPHA
        GLEW_ANGLE_texture_compression_dxt5
        GL_FRAGMENT_LIGHT_MODEL_AMBIENT_EXT
        GLEW_ATI_texture_float
        GL_CLIP_DISTANCE3_EXT
        GL_COMPRESSED_SRGB_ALPHA_PVRTC_4BPPV1_EXT
        GL_ALPHA_TO_COVERAGE_DITHER_DEFAULT_NV
        GL_TEXTURE_OBJECT_VALID_QCOM
        GL_SMALL_CCW_ARC_TO_NV
        GL_SCALE_BY_TWO_NV
        GL_POINT_FADE_THRESHOLD_SIZE_EXT
        GL_CLIPPING_INPUT_PRIMITIVES
        GL_DRAW_PIXEL_TOKEN
        GLEW_NV_copy_depth_to_color
        GLEW_AMD_seamless_cubemap_per_texture
        GL_MAX_GEOMETRY_ATOMIC_COUNTERS_EXT
        GL_INTENSITY12_EXT
        GL_QUERY_RESULT_AVAILABLE_ARB
        GLEW_EXT_texture_sRGB
        GL_PROGRAM_BINARY_ANGLE
        GL_VIEWPORT_SWIZZLE_X_NV
        GL_ARRAY_OBJECT_OFFSET_ATI
        GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM
        GL_FLOAT_RG32_NV
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SAMPLES_EXT
        GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT
        GL_FLOAT_R32_NV
        GLEW_SGIX_scalebias_hint
        GL_DEVICE_LUID_EXT
        GL_IMAGE_CLASS_11_11_10
        GL_UNSIGNED_INT_10_10_10_2
        GL_INTENSITY16
        GL_ARB_texture_float
        GL_MODELVIEW0_MATRIX_EXT
        GL_MAX_CUBE_MAP_TEXTURE_SIZE
        GL_R32UI
        GLEW_NV_tessellation_program5
        GLEW_MESA_resize_buffers
        GL_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS_ARB
        GL_ARB_shading_language_420pack
        GLEW_NV_path_rendering_shared_edge
        GL_ARB_half_float_pixel
        GL_ETC1_SRGB8_NV
        GL_PATH_TERMINAL_END_CAP_NV
        GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT
        GL_FRAMEBUFFER_SAMPLE_LOCATION_PIXEL_GRID_NV
        GL_SGIS_multisample
        GLEW_ARB_texture_buffer_object
        GL_EXT_debug_marker
        GL_FRAGMENT_LIGHT7_EXT
        GLEW_VERSION
        GL_INVERSE_NV
        GL_GEOMETRY_SHADER_ARB
        GL_FOG_COORD_ARRAY_POINTER
        GLEW_ARB_texture_stencil8
        GL_PINLIGHT_NV
        GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV
        GL_NUM_DEVICE_UUIDS_EXT
        GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_NV
        GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_TEST
        GL_ZERO
        GLEW_SGIX_shadow_ambient
        GLEW_SGIX_convolution_accuracy
        GL_REFLECTION_MAP_EXT
        GL_TEXTURE19_ARB
        GL_TEXTURE9_ARB
        GL_CLIP_FAR_HINT_PGI
        GLEW_ARB_depth_texture
        GL_TEXTURE_BINDING_2D_ARRAY_NV
        GL_IMAGE_PIXEL_TYPE
        GL_OPERAND1_ALPHA
        GL_RENDER
        GL_NV_texture_array
        GL_COLOR_ARRAY_POINTER
        GLEW_EXT_draw_instanced
        GL_MAX_DEBUG_LOGGED_MESSAGES_ARB
        GL_MODELVIEW29_ARB
        GL_DST_IN_NV
        GLEW_ARB_transform_feedback3
        GL_MULTISAMPLE_BIT_ARB
        GL_TEXTURE_BUFFER_EXT
        GL_MESA_resize_buffers
        GL_VERTEX_ATTRIB_ARRAY3_NV
        GL_TANGENT_ARRAY_STRIDE_EXT
        GL_TEXTURE_VIEW
        GL_ARB_shading_language_include
        GL_SGIX_fog_layers
        GL_NICEST
        GL_TEXTURE_DEFORMATION_BIT_SGIX
        GL_PIXELS_PER_SAMPLE_PATTERN_X_AMD
        GL_RESTART_SUN
        GL_SAMPLES_EXT
        GL_ADD_SIGNED_ARB
        GL_NV_viewport_swizzle
        GL_OVR_multiview_multisampled_render_to_texture
        GL_MAX_TEXTURE_IMAGE_UNITS_ARB
        GL_TEXTURE_COORD_ARRAY_ADDRESS_NV
        GL_TYPE
        GL_ARB_transform_feedback_overflow_query
        GL_OP_ROUND_EXT
        GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB
        GL_TEXTURE_COORD_ARRAY_EXT
        GL_MAX_GEOMETRY_IMAGE_UNIFORMS
        GL_STENCIL_ATTACHMENT
        GL_NV_blend_equation_advanced_coherent
        GL_QCOM_binning_control
        GLEW_NV_vertex_attrib_integer_64bit
        GL_OUTPUT_TEXTURE_COORD31_EXT
        GL_UNSIGNED_INT_IMAGE_BUFFER_EXT
        GL_CLIENT_ACTIVE_TEXTURE
        GL_LUMINANCE_FLOAT16_APPLE
        GL_PACK_SKIP_PIXELS_NV
        GL_MAX_IMAGE_UNITS_EXT
        GLEW_AMD_blend_minmax_factor
        GL_MAX_PALETTE_MATRICES_ARB
        GL_BLUE
        GL_RGB8
        GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV
        GL_NVX_linked_gpu_multicast
        GL_TEXTURE_BINDING_CUBE_MAP_ARRAY
        GL_FUNC_COMPLEX_ADD_EXT
        GL_RENDERBUFFER_BINDING
        GL_LOG_DRIVER_REGAL
        GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES
        GL_DEPTH_EXT
        GL_COLOR4_BIT_PGI
        GL_MAX_TESS_PATCH_COMPONENTS
        GL_MESA_window_pos
        GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS
        GLEW_SGIX_subsample
        GL_STATISTICS_REGAL
        GL_DOUBLE_MAT4x2_EXT
        GL_STENCIL_TAG_BITS_EXT
        GL_SRC1_COLOR_EXT
        GL_TESS_CONTROL_SHADER
        GL_FOG_SCALE_SGIX
        GL_MITER_REVERT_NV
        GL_EXCLUSION_KHR
        GL_CULL_MODES_NV
        GL_CLIP_PLANE3
        GL_CURRENT_QUERY
        GL_CONSTANT_ATTENUATION
        GL_LUMINANCE16_ALPHA16_SIGNED_SGIX
        GL_CLIP_DISTANCE4
        GL_VERTEX_ATTRIB_MAP1_COEFF_APPLE
        GL_RGBA_FLOAT16_APPLE
        GL_NVX_gpu_memory_info
        GL_FRAMEBUFFER_ATTACHMENT_LAYERED
        GL_INTENSITY16_SIGNED_SGIX
        GL_LOCATION_INDEX
        GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT
        GL_ALREADY_SIGNALED_APPLE
        GL_COLOR_TABLE_RED_SIZE_EXT
        GLEW_NV_conservative_raster
        GL_MAP2_TEXTURE_COORD_1_NURBS_SGIX
        GL_ALPHA32I_EXT
        GL_SEPARATE_ATTRIBS_EXT
        GL_INDEX_ARRAY_COUNT_EXT
        GL_422_AVERAGE_EXT
        GL_EXT_vertex_attrib_64bit
        GL_MATRIX_PALETTE_ARB
        GLEW_EXT_clear_texture
        GL_SHADER_IMAGE_LOAD
        GL_TRANSFORM_FEEDBACK_VARYINGS
        GL_PROGRAM_PARAMETER_NV
        GL_RGBA_INTEGER_EXT
        GL_R8_SNORM
        GL_PERFQUERY_COUNTER_DATA_UINT64_INTEL
        GL_ALPHA8_EXT
        GL_NV_gpu_program4
        GL_VIEW_CLASS_32_BITS
        GL_SIGNED_LUMINANCE_ALPHA_NV
        GL_TEXTURE_FORMAT_QCOM
        GL_SGIX_depth_texture
        GL_RGBA16UI
        GL_SELECTION_BUFFER_SIZE
        GL_COUNT_UP_NV
        GLEW_NV_packed_float
        GL_TEXTURE_MULTI_BUFFER_HINT_SGIX
        GL_ALPHA16F_ARB
        GL_COMPRESSED_RGBA_ASTC_6x5_KHR
        GL_ARB_ES3_compatibility
        GL_MAP2_VERTEX_3_NURBS_SGIX
        GL_DEVICE_UUID_EXT
        GL_LUMINANCE16_ICC_SGIX
        GL_DRAW_BUFFER8_EXT
        GL_FRAMEBUFFER_UNSUPPORTED_EXT
        GL_TRANSPOSE_CURRENT_MATRIX_ARB
        GL_GREEN_MAX_CLAMP_INGR
        GL_POST_CONVOLUTION_GREEN_BIAS_EXT
        GL_NVX_blend_equation_advanced_multi_draw_buffers
        GL_NV_vertex_program
        GL_OPERAND0_ALPHA
        GL_PATH_STROKE_WIDTH_NV
        GL_IMAGE_BINDING_LAYER
        GL_FIELD_UPPER_NV
        GL_INTENSITY_FLOAT16_ATI
        GL_UNSIGNED_SHORT_8_8_REV_MESA
        GL_TEXTURE_NORMAL_EXT
        GL_MAX_COMPUTE_SHADER_STORAGE_BLOCKS
        GL_ALPHA_INTEGER
        GL_INT_VEC4_ARB
        GL_FLOAT_RGBA16_NV
        GL_NOOP
        GL_SGIS_pixel_texture
        GL_SAMPLE_ALPHA_TO_ONE
        GL_CLIENT_STORAGE_BIT_EXT
        GLEW_EXT_vertex_array
        GL_MAX_PROGRAM_TEXTURE_GATHER_COMPONENTS_ARB
        GL_SHADER_IMAGE_ACCESS_BARRIER_BIT_EXT
        GL_AMD_shader_stencil_export
        GL_NV_texgen_reflection
        GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT
        GLEW_NV_gpu_shader5
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT
        GLEW_NV_gpu_program5
        GL_LUMINANCE8_SNORM
        GL_GEOMETRY_LINKED_INPUT_TYPE_EXT
        GL_COLOR_ATTACHMENT4
        GL_COMPUTE_PROGRAM_NV
        GL_SAMPLER_2D_ARRAY
        GL_PGI_vertex_hints
        GL_CURRENT_FOG_COORD
        GL_SAMPLE_LOCATION_SUBPIXEL_BITS_NV
        GL_COMPUTE_SHADER_BIT
        GL_EMBOSS_LIGHT_NV
        GL_VERSION_4_4
        GL_AMD_shader_explicit_vertex_parameter
        GL_CLAMP_FRAGMENT_COLOR
        GL_TEXTURE_COMPRESSED_BLOCK_SIZE
        GL_EXT_texture_storage
        GL_TRANSFORM_FEEDBACK_BUFFER_SIZE
        GL_APPLE_object_purgeable
        GL_INT_IMAGE_2D_ARRAY_EXT
        GL_MAP1_VERTEX_3
        GL_FEEDBACK
        GL_TEXTURE_INTENSITY_SIZE
        GL_COMBINER4_NV
        GL_SWIZZLE_STR_ATI
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_EVALUATION_SHADER
        GLEW_NV_sRGB_formats
        GL_TEXTURE_IMMUTABLE_LEVELS
        GL_LIGHT5
        GL_POLYGON_OFFSET_POINT
        GL_NORMAL_ARRAY_LENGTH_NV
        GL_VERTEX_ATTRIB_ARRAY13_NV
        GL_STATIC_VERTEX_ARRAY_IBM
        GL_TEXTURE_BASE_LEVEL
        GL_MAX_ELEMENTS_INDICES
        GL_MAP2_VERTEX_ATTRIB0_4_NV
        GLEW_VERSION_4_3
        GL_BGRA_EXT
        GL_TEXTURE_COLOR_TABLE_SGI
        GL_ARB_texture_filter_anisotropic
        GLEW_NV_internalformat_sample_query
        GL_STATIC_DRAW
        GL_INTENSITY16_ICC_SGIX
        GL_SYNC_FLUSH_COMMANDS_BIT
        GL_ENV_MAP_SGIX
        GL_SRC_NV
        GL_T4F_V4F
        GL_PROXY_TEXTURE_1D_ARRAY_EXT
        GL_CLIP_DISTANCE6_EXT
        GL_SGIS_point_line_texgen
        GL_TRANSPOSE_NV
        GL_NV_bindless_multi_draw_indirect
        GL_LUMINANCE32F_ARB
        GL_LIGHT2
        GLEW_NV_deep_texture3D
        GL_MAP1_VERTEX_ATTRIB15_4_NV
        GL_MESA_shader_integer_functions
        GL_IS_ROW_MAJOR
        GL_HISTOGRAM_GREEN_SIZE_EXT
        GL_SKIP_DECODE_EXT
        GL_VERSION_2_1
        GL_CONTEXT_COMPATIBILITY_PROFILE_BIT
        GL_INT_IMAGE_BUFFER
        GL_ARB_sample_locations
        GL_SGIX_quad_mesh
        GL_MULTISAMPLE_BIT
        GL_COMPRESSED_SIGNED_RED_RGTC1
        GL_MAX_TEXTURE_SIZE
        GL_SAMPLE_LOCATION_PIXEL_GRID_WIDTH_NV
        GLEW_SGIS_sharpen_texture
        GL_COLOR_LOGIC_OP
        GL_VIEWPORT_SWIZZLE_Z_NV
        GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING
        GL_COMPRESSED_LUMINANCE
        GL_MIN_PBUFFER_VIEWPORT_DIMS_APPLE
        GL_DEPTH_BUFFER_BIT5_QCOM
        GL_DOT2_ADD_ATI
        GL_PIXEL_UNPACK_BUFFER_ARB
        GLEW_APPLE_transform_hint
        GL_CONVOLUTION_2D_EXT
        GLEW_SGIX_line_quality_hint
        GL_EXT_texture_swizzle
        GL_MAX_COMPUTE_FIXED_GROUP_SIZE_ARB
        GL_POINT_SIZE_GRANULARITY
        GLEW_EXT_discard_framebuffer
        GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW
        GL_HISTOGRAM_ALPHA_SIZE_EXT
        GL_EQUAL
        GL_VERTEX_ATTRIB_ARRAY_SIZE
        GL_BUFFER_UPDATE_BARRIER_BIT_EXT
        GL_DOT3_RGBA_ARB
        GL_ARB_draw_buffers_blend
        GL_COLOR_COMPONENTS
        GL_RETURN
        GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY_EXT
        GL_BIAS_BIT_ATI
        GL_BLEND_SRC_ALPHA_EXT
        GL_DISPLAY_LIST
        GL_BGRA_INTEGER_EXT
        GLEW_NV_stereo_view_rendering
        GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB
        GL_ADD
        GL_UNIFORM_BLOCK
        GL_EXT_point_parameters
        GL_ASYNC_TEX_IMAGE_SGIX
        GL_CONSTANT_EXT
        GL_ZOOM_Y
        GLEW_SGIS_color_range
        GL_TEXTURE_BINDING_BUFFER
        GL_DT_SCALE_NV
        GL_WINDOW_RECTANGLE_MODE_EXT
        GL_ELEMENT_ARRAY_TYPE_ATI
        GL_CON_1_ATI
        GL_BITMAP_TOKEN
        GLEW_EXT_win32_keyed_mutex
        GL_MAX_VERTEX_UNITS_ARB
        GL_TEXTURE11
        GL_INTERNALFORMAT_BLUE_TYPE
        GL_ALPHA_TO_COVERAGE_DITHER_MODE_NV
        GL_GENERATE_MIPMAP_HINT_SGIS
        GL_COLOR_ARRAY_LENGTH_NV
        GL_PROGRAM_ATTRIBS_ARB
        GL_RENDER_DIRECT_TO_FRAMEBUFFER_QCOM
        GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_EXT
        GL_MINMAX_EXT
        GL_UNIFORM_BUFFER_START
        GL_DEBUG_SOURCE_API
        GL_READ_PIXELS_TYPE
        GL_ATOMIC_COUNTER_BARRIER_BIT_EXT
        GLEW_MESAX_texture_stack
        GL_NV_blend_square
        GL_ARB_provoking_vertex
        GLEW_SGIX_fog_scale
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT
        GL_LOCAL_EXT
        GL_Z6Y10Z6CB10Z6A10Z6Y10Z6CR10Z6A10_4224_NV
        GL_MAT_AMBIENT_BIT_PGI
        GL_AMD_sparse_texture
        GL_BGRA
        GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS
        GL_COMPRESSED_LUMINANCE_LATC1_EXT
        GLEW_ARB_provoking_vertex
        GL_SUCCESS_NV
        GLEW_NV_blend_equation_advanced
        GL_VERTEX_ELEMENT_SWIZZLE_AMD
        GL_3DFX_tbuffer
        GL_LOCAL_CONSTANT_EXT
        GL_OUTPUT_TEXTURE_COORD2_EXT
        GL_EXP2
        GL_CONVOLUTION_FILTER_BIAS
        GL_MULTISAMPLE_RASTERIZATION_ALLOWED_EXT
        GL_KHR_context_flush_control
        GL_EXT_blend_color
        GL_RESTART_PATH_NV
        GL_R16_SNORM_EXT
        GLEW_NV_texture_multisample
        GLEW_AMD_shader_explicit_vertex_parameter
        GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS_EXT
        GLEW_NV_geometry_shader4
        GLEW_SGIX_list_priority
        GL_BLEND_DST_ALPHA
        GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB
        GL_SHADER_OPERATION_NV
        GL_VERTEX_ARRAY_STRIDE
        GL_MAP1_VERTEX_ATTRIB14_4_NV
        GL_NORMAL_ARRAY_LIST_IBM
        GL_DRAW_BUFFER3
        GL_AMD_sample_positions
        GL_OPERAND2_ALPHA_ARB
        GL_NV_evaluators
        GL_LUMINANCE_EXTENDED_RANGE_SGIX
        GL_QUARTER_BIT_ATI
        GL_TEXTURE_GATHER_SHADOW
        GL_GPU_MEMORY_INFO_CURRENT_AVAILABLE_VIDMEM_NVX
        GL_COMPILE_STATUS
        GL_EGL_NV_robustness_video_memory_purge
        GL_NEGATIVE_Z_EXT
        GLEW_ARB_texture_view
        GLEW_REGAL_ES1_1_compatibility
        GL_PRIMITIVES_GENERATED_EXT
        GL_ARB_shader_subroutine
        GLEW_EXT_map_buffer_range
        GL_POINT_SIZE_MIN
        GL_UNSIGNED_INT_ATOMIC_COUNTER
        GL_MAX_COMPUTE_IMAGE_UNIFORMS
        GL_STANDARD_FONT_FORMAT_NV
        GL_INFO_LOG_LENGTH
        GL_VERTEX_ARRAY_OBJECT_EXT
        GL_MAP_WRITE_BIT_EXT
        GL_REQUIRED_TEXTURE_IMAGE_UNITS_OES
        GL_IGLOO_IRISGL_MODE_SGIX
        GL_INT_SAMPLER_BUFFER
        GL_STACK_OVERFLOW
        GL_GREEN_MIN_CLAMP_INGR
        GL_TEXTURE0_ARB
        GL_RGB16I_EXT
        GL_MAP2_VERTEX_3
        GLEW_SGIX_cylinder_texgen
        GL_PATH_GEN_MODE_NV
        GL_PERFQUERY_COUNTER_THROUGHPUT_INTEL
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS
        GL_NAME_STACK_DEPTH
        GLEW_EXT_framebuffer_multisample
        GL_TEXTURE_RENDERBUFFER_NV
        GL_MAX_CONVOLUTION_HEIGHT
        GL_TEXTURE_MAX_LEVEL_APPLE
        GL_POLYGON_OFFSET_LINE
        GL_UNSIGNED_INT_8_8_8_8_EXT
        GL_GEOMETRY_LINKED_VERTICES_OUT_EXT
        GL_LUMINANCE16UI_EXT
        GL_HALF_BIAS_NEGATE_NV
        GL_INT8_NV
        GLEW_SGIX_fog_blend
        GLEW_EXT_pvrtc_sRGB
        GL_TEXTURE_2D_ARRAY_NV
        GL_IMAGE_2D
        GL_INT64_ARB
        GL_ARB_fragment_shader
        GL_TEXTURE_LIGHT_EXT
        GLEW_ATIX_texture_env_combine3
        GL_MIRRORED_REPEAT
        GL_PATH_FILL_BOUNDING_BOX_NV
        GL_PER_GPU_STORAGE_BIT_NV
        GL_TEXTURE_COMPARE_FAIL_VALUE_ARB
        GL_RGBA_DXT5_S3TC
        GLEW_EXT_color_buffer_half_float
        GL_PERFQUERY_COUNTER_DATA_FLOAT_INTEL
        GL_EXT_shader_pixel_local_storage2
        GL_TRANSPOSE_COLOR_MATRIX_ARB
        GL_MODELVIEW25_ARB
        GL_IMAGE_FORMAT_COMPATIBILITY_TYPE
        GL_CON_0_ATI
        GL_COLOR_BUFFER_BIT3_QCOM
        GL_MATRIX6_ARB
        GLEW_ARB_map_buffer_alignment
        GLEW_SGI_color_table
        GL_SHADER_BINARY_FORMAT_SPIR_V
        GL_SAMPLER_OBJECT_AMD
        GL_MAX_ELEMENT_INDEX
        GL_BOOL_ARB
        GL_RED_BIT_ATI
        GL_ARM_rgba8
        GL_RG8_EXT
        GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE
        GL_MAX_DEPTH
        GL_TRIANGLE_FAN
        GL_CULL_VERTEX_EYE_POSITION_EXT
        GL_ARB_fragment_program_shadow
        GL_PRIMITIVES_GENERATED_NV
        GL_WIN_phong_shading
        GL_VARIANT_ARRAY_EXT
        GLEW_SGIX_instrument_error
        GL_FRAMEBUFFER_BARRIER_BIT
        GL_LINE_STRIP_ADJACENCY_EXT
        GLEW_ANGLE_pack_reverse_row_order
        GLEW_EXT_conservative_depth
        GL_QCOM_driver_control
        GL_POST_COLOR_MATRIX_ALPHA_BIAS
        GL_SGIS_generate_mipmap
        GL_SEPARATE_SPECULAR_COLOR_EXT
        GL_MAP2_TEXTURE_COORD_3
        GL_MAX_GEOMETRY_SHADER_INVOCATIONS_EXT
        GL_MULTISAMPLE_BUFFER_BIT6_QCOM
        GLEW_SUN_convolution_border_modes
        GL_STENCIL_VALUE_MASK
        GL_MAX_GEOMETRY_UNIFORM_BLOCKS
        GL_FUNC_SUBTRACT
        GL_TRIANGLE_STRIP_ADJACENCY
        GL_COMPRESSED_RGB_ARB
        GL_VERTEX_CONSISTENT_HINT_PGI
        GL_TEXTURE_VIEW_NUM_LAYERS_EXT
        GL_REPLACEMENT_CODE_ARRAY_SUN
        GLEW_ARB_shader_group_vote
        GLEW_NV_clip_space_w_scaling
        GLEW_NV_framebuffer_mixed_samples
        GL_FIRST_TO_REST_NV
        GL_OCCLUSION_QUERY_EVENT_MASK_AMD
        GL_COMBINER_MUX_SUM_NV
        GL_MAX_HEIGHT
        GL_RGB8_SNORM
        GL_VERTEX_SHADER_INVARIANTS_EXT
        GL_APPLE_vertex_array_range
        GL_COMBINE_RGB_ARB
        GL_MAD_ATI
        GL_MAX_COLOR_TEXTURE_SAMPLES
        GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT
        GL_RESAMPLE_REPLICATE_OML
        GL_ATI_vertex_streams
        GL_SGIX_framezoom
        GL_DRAW_BUFFER12_ATI
        GLEW_ARB_get_program_binary
        GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI
        GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV
        GLEW_EXT_texture3D
        GL_TRIANGLE_STRIP
        GL_MAX_COLOR_ATTACHMENTS_EXT
        GL_AMBIENT_AND_DIFFUSE
        GL_UNSIGNED_INT_10F_11F_11F_REV
        GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT
        GL_NUM_LOOPBACK_COMPONENTS_ATI
        GL_DRAW_BUFFER7
        GL_CUBE_MAP_BINDING_SGIX
        GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB
        GL_NV_texture_shader
        GL_MVP_MATRIX_EXT
        GL_ARB_multi_bind
        GL_INTEL_framebuffer_CMAA
        GL_FRAMEBUFFER_DEFAULT_HEIGHT
        GL_AMD_gpu_shader_int64
        GL_DRAW_BUFFER13_NV
        GL_PROGRAM_FORMAT_ASCII_ARB
        GL_INT_IMAGE_1D_ARRAY
        GL_EXT_texture_cube_map
        GL_MAX_ASYNC_HISTOGRAM_SGIX
        GL_EXT_texture_env_add
        GL_MAX_DEEP_3D_TEXTURE_DEPTH_NV
        GL_SAMPLER_2D
        GL_IMAGE_BINDING_NAME
        GL_FLOAT_MAT4x3_NV
        GL_R3_G3_B2
        GL_PERFQUERY_COUNTER_DURATION_RAW_INTEL
        GL_STENCIL_BUFFER_BIT2_QCOM
        GL_WEIGHT_ARRAY_ARB
        GLEW_SUN_read_video_pixels
        GLEW_NV_command_list
        GL_MOV_ATI
        GL_DRIVER_UUID_EXT
        GL_FOG_COORDINATE_ARRAY
        GL_OFFSET_TEXTURE_2D_NV
        GL_TEXTURE_REDUCTION_MODE_ARB
        GL_GET_TEXTURE_IMAGE_TYPE
        GL_STENCIL_CLEAR_TAG_VALUE_EXT
        GL_UNIFORM_BARRIER_BIT_EXT
        GLEW_EXT_provoking_vertex
        GL_INDEX_MODE
        GL_CONVOLUTION_2D
        GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT
        GL_DEBUG_SOURCE_WINDOW_SYSTEM
        GL_PIXEL_SUBSAMPLE_4444_SGIX
        GL_FRAGMENT_SHADER_INVOCATIONS_ARB
        GL_VIEWPORT_SWIZZLE_Y_NV
        GL_QUERY_BY_REGION_WAIT
        GL_SGIX_color_matrix_accuracy
        GL_POST_CONVOLUTION_RED_SCALE
        GL_MAX_TESS_CONTROL_INPUT_COMPONENTS
        GL_COMPRESSED_INTENSITY_ARB
        GL_OVERLAY_KHR
        GL_RGB9_E5
        GL_DRAW_BUFFER0_EXT
        GL_SAMPLER_2D_ARRAY_EXT
        GL_MAP_WRITE_BIT
        GL_SGIX_pixel_texture_bits
        GL_SELECT
        GL_VARIANT_ARRAY_STRIDE_EXT
        GL_3DFX_multisample
        GL_SAMPLE_ALPHA_TO_COVERAGE
        GL_MAP2_VERTEX_4_NURBS_SGIX
        GLEW_QCOM_extended_get2
        GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING
        GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB
        GL_COMPRESSED_SIGNED_RED_RGTC1_EXT
        GLEW_ATI_vertex_array_object
        GL_VERTEX_STREAM0_ATI
        GL_MAP1_TEXTURE_COORD_1
        GL_DRAW_BUFFER3_ATI
        GL_MAP1_VERTEX_ATTRIB7_4_NV
        GL_COLOR_CLEAR_VALUE
        GL_LIGHT4
        GL_MAX_FRAMEBUFFER_WIDTH
        GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS
        GL_MAX_TEXTURES_SGIS
        GL_MAGNITUDE_BIAS_NV
        GL_BLEND_OVERLAP_NV
        GLEW_SGIX_ycrcb_subsample
        GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS
        GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT_EXT
        GL_COLOR_ATTACHMENT6_EXT
        GL_STREAM_DRAW
        GL_VERTEX_ATTRIB_ARRAY_LONG
        GL_TEXTURE_VIEW_NUM_LEVELS_EXT
        GL_SRC_IN_NV
        GL_INT_IMAGE_3D
        GL_AMD_texture_texture4
        GL_UNSIGNED_INT_IMAGE_1D
        GL_EXT_color_buffer_float
        GL_MAX_FRAGMENT_UNIFORM_BLOCKS
        GL_ACTIVE_SUBROUTINE_UNIFORMS
        GL_PROGRAMMABLE_SAMPLE_LOCATION_NV
        GL_DEPTH_COMPONENT16
        GL_SKIP_MISSING_GLYPH_NV
        GL_ARB_sample_shading
        GL_RENDERBUFFER_WIDTH
        GL_UNSIGNED_INT_VEC3_EXT
        GL_CUBE_MAP_YN_SGIX
        GL_SECONDARY_COLOR_ARRAY_STRIDE
        GL_FLOAT16_MAT3_AMD
        GL_ACTIVE_VERTEX_UNITS_ARB
        GLEW_NV_framebuffer_blit
        GL_NV_depth_range_unclamped
        GL_RENDERBUFFER_RED_SIZE
        GL_FRAMEBUFFER_RENDERABLE
        GLEW_ARB_draw_buffers_blend
        GL_NV_texture_compression_s3tc_update
        GLEW_EXT_multisampled_render_to_texture
        GLEW_SGIS_multisample
        GL_STENCIL_TEST
        GL_EXT_texture_integer
        GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM
        GL_MATRIX4_NV
        GL_ARB_ES3_1_compatibility
        GL_SGIX_subdiv_patch
        GL_DOUBLE_MAT3x4
        GL_BLEND_COLOR_EXT
        GL_TEXTURE22_ARB
        GL_ARB_matrix_palette
        GL_TEXTURE_GREEN_SIZE
        GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB
        GL_NORMAL_MAP_EXT
        GL_APPLE_client_storage
        GL_RGB5_A1
        GL_MIXED_DEPTH_SAMPLES_SUPPORTED_NV
        GL_RGBA16UI_EXT
        GL_EXT_unpack_subimage
        GL_MAP_ATTRIB_U_ORDER_NV
        GL_COLOR_BUFFER_BIT0_QCOM
        GL_FALSE
        GL_QUERY_DEPTH_PASS_EVENT_BIT_AMD
        GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE
        GL_CONSTANT_COLOR0_NV
        GL_NV_sample_mask_override_coverage
        GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI
        GL_PROXY_COLOR_TABLE
        GL_UNSIGNED_INT64_VEC3_NV
        GL_CURRENT_TEXTURE_COORDS
        GL_INTERPOLATE
        GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB
        GL_UNPACK_COMPRESSED_BLOCK_SIZE
        GL_TEXTURE_COORD_ARRAY_COUNT_EXT
        GL_EXT_YUV_target
        GL_TEXTURE_DEPTH_TYPE_ARB
        GL_RG32UI
        GL_COLORBURN_NV
        GLEW_ATI_draw_buffers
        GL_UNSIGNED_INVERT_NV
        GL_DRAW_BUFFER6_EXT
        GL_PARAMETER_BUFFER
        GL_FLOAT_MAT4_ARB
        GL_UNSIGNED_INT
        GL_MAX_FRAGMENT_IMAGE_UNIFORMS
        GL_FASTEST
        GL_COMPRESSED_RGBA
        GL_PROXY_TEXTURE_CUBE_MAP_ARB
        GL_RELATIVE_ROUNDED_RECT_NV
        GL_ELEMENT_ARRAY_UNIFIED_NV
        GL_TEXTURE_RANGE_LENGTH_APPLE
        GL_HP_convolution_border_modes
        GL_TEXTURE_MAX_LOD_SGIS
        GLEW_NV_framebuffer_multisample_coverage
        GL_TEXTURE_SWIZZLE_B_EXT
        GL_ARB_texture_compression_rgtc
        GL_COLOR_ARRAY_TYPE
        GL_DEPTH
        GL_FLOAT_VEC2
        GL_SGIX_async_histogram
        GL_COLOR_MATRIX_STACK_DEPTH_SGI
        GLEW_NV_fog_distance
        GLEW_SGIS_texture_select
        GL_SLICE_ACCUM_SUN
        GL_UNSIGNED_INT_SAMPLER_1D
        GL_LINE_NV
        GL_VERTEX_ARRAY_RANGE_LENGTH_NV
        GL_EXT_gpu_shader5
        GLEW_NV_3dvision_settings
        GLEW_EXT_shader_implicit_conversions
        GL_COLOR_ARRAY_LIST_STRIDE_IBM
        GL_TEXTURE_BINDING_2D_MULTISAMPLE
        GL_HANDLE_TYPE_D3D12_RESOURCE_EXT
        GL_RESAMPLE_DECIMATE_OML
        GLEW_ARB_texture_query_levels
        GL_BOOL
        GL_BLEND_ADVANCED_COHERENT_KHR
        GL_EXT_packed_depth_stencil
        GL_EXT_scene_marker
        GL_INTENSITY16I_EXT
        GL_MAP_PERSISTENT_BIT
        GL_COLOR_TABLE_GREEN_SIZE_SGI
        GL_SLUMINANCE_ALPHA_NV
        GLEW_EXT_render_snorm
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR
        GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX
        GL_SLUMINANCE_EXT
        GL_PERFQUERY_QUERY_NAME_LENGTH_MAX_INTEL
        GL_PROXY_TEXTURE_RECTANGLE_ARB
        GL_TEXTURE_COLOR_SAMPLES_NV
        GL_CLEAR_TEXTURE
        GL_DOT_PRODUCT_DEPTH_REPLACE_NV
        GL_RENDERBUFFER_INTERNAL_FORMAT
        GLEW_ARB_ES2_compatibility
        GL_CONTEXT_FLAG_NO_ERROR_BIT
        GL_SAMPLER_CUBE_MAP_ARRAY_ARB
        GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV
        GL_RENDERER
        GL_FIRST_VERTEX_CONVENTION_EXT
        GL_FLOAT_MAT2_ARB
        GL_PRIMITIVES_SUBMITTED_ARB
        GL_STATIC_DRAW_ARB
        GL_BGRA_INTEGER
        GL_TEXTURE_WIDTH_QCOM
        GL_VERTEX_STREAM3_ATI
        GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI
        GL_STENCIL_BUFFER
        GL_MAX_GEOMETRY_UNIFORM_BLOCKS_EXT
        GLEW_AMD_vertex_shader_viewport_index
        GL_EXT_index_array_formats
        GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV
        GL_MOVE_TO_NV
        GL_ARM_mali_program_binary
        GL_MODELVIEW20_ARB
        GL_NV_generate_mipmap_sRGB
        GL_MAX_CLIP_PLANES
        GL_SGIX_slim
        GL_SRGB_ALPHA_EXT
        GLEW_EXT_unpack_subimage
        GL_FOG_SCALE_VALUE_SGIX
        GL_GENERATE_MIPMAP
        GL_BUFFER_USAGE
        GL_ATOMIC_COUNTER_BUFFER_BINDING
        GL_PACK_COMPRESSED_BLOCK_WIDTH
        GL_ROUNDED_RECT_NV
        GLEW_EXT_texture_cube_map
        GLEW_ARB_texture_filter_anisotropic
        GL_BUFFER_STORAGE_FLAGS_EXT
        GL_EXT_separate_shader_objects
        GL_EVAL_VERTEX_ATTRIB6_NV
        GL_COLOR_ATTACHMENT12_NV
        GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE
        GL_OBJECT_COMPILE_STATUS_ARB
        GL_ATI_texture_env_combine3
        GL_NV_packed_float
        GL_MAX_DUAL_SOURCE_DRAW_BUFFERS
        GL_WEIGHT_ARRAY_POINTER_ARB
        GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT
        GLEW_EXT_texture_swizzle
        GL_CLIENT_ATTRIB_STACK_DEPTH
        GL_FOG_COORDINATE_ARRAY_POINTER
        GL_VIDEO_COLOR_CONVERSION_OFFSET_NV
        GL_BINORMAL_ARRAY_STRIDE_EXT
        GLEW_VERSION_4_4
        GLEW_ARB_separate_shader_objects
        GL_RENDERBUFFER_BLUE_SIZE
        GL_COMMAND_BARRIER_BIT_EXT
        GLEW_VERSION_2_1
        GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT
        GL_MIN_SAMPLE_SHADING_VALUE
        GL_MODELVIEW11_ARB
        GL_MAX_PROGRAM_TEMPORARIES_ARB
        GL_NORMAL_ARRAY_STRIDE_EXT
        GL_ACTIVE_STENCIL_FACE_EXT
        GL_SECONDARY_COLOR_ARRAY_TYPE_EXT
        GLEW_NV_texgen_reflection
        GL_UNSIGNED_NORMALIZED_ARB
        GL_FRAGMENT_SHADER_INVOCATIONS
        GL_FOG_COORD_SRC
        GL_ARB_robustness_share_group_isolation
        GL_2_BYTES
        GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS
        GL_EDGE_FLAG_ARRAY_STRIDE_EXT
        GL_ALPHA8_SNORM
        GL_VERSION_3_2
        GL_FRAMEBUFFER_SRGB_EXT
        GL_DRAW_ELEMENTS_COMMAND_NV
        GL_MAX_DEPTH_TEXTURE_SAMPLES
        GL_INT_IMAGE_BUFFER_EXT
        GL_TEXTURE_VIEW_MIN_LEVEL_EXT
        GL_INT_IMAGE_CUBE_MAP_ARRAY_EXT
        GL_EXT_422_pixels
        GL_VIEWPORT_SUBPIXEL_BITS_NV
        GL_CONTEXT_ROBUST_ACCESS
        GL_TEXTURE13_ARB
        GL_VIRTUAL_PAGE_SIZE_INDEX_EXT
        GL_STENCIL_ATTACHMENT_EXT
        GL_BACK
        GLEW_VERSION_3_0
        GLEW_HP_texture_lighting
        GL_MESAX_texture_stack
        GL_FUNC_REVERSE_SUBTRACT_EXT
        GL_VERTEX_ARRAY_STRIDE_EXT
        GL_ALPHA_TO_COVERAGE_DITHER_ENABLE_NV
        GL_COLOR_ATTACHMENT14_NV
        GL_VIDEO_BUFFER_PITCH_NV
        GL_NV_half_float
        GL_STENCIL_BUFFER_BIT4_QCOM
        GL_TRANSFORM_FEEDBACK_STREAM_OVERFLOW
        GL_EXT_texture_lod_bias
        GL_INTEL_map_texture
        GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI
        GL_OP_SET_LT_EXT
        GLEW_EXT_multi_draw_indirect
        GLEW_SUNX_constant_data
        GL_UNPACK_IMAGE_HEIGHT_EXT
        GL_SGIX_occlusion_instrument
        GLEW_NV_read_depth_stencil
        GL_VERTEX_ARRAY_RANGE_NV
        GL_TEXTURE_SWIZZLE_RGBA_EXT
        GL_WEIGHT_ARRAY_TYPE_ARB
        GLEW_SGIX_impact_pixel_texture
        GL_EXP
        GL_TEXTURE6
        GL_PIXEL_COUNT_AVAILABLE_NV
        GL_RGB
        GL_HANDLE_TYPE_OPAQUE_FD_EXT
        GL_EXT_texture_sRGB
        GLEW_EXT_texture_rg
        GL_ALL_PIXELS_AMD
        GL_IMAGE_BINDING_LAYERED
        GL_NV_texture_compression_s3tc
        GL_MAP2_VERTEX_ATTRIB13_4_NV
        GLEW_NV_point_sprite
        GL_CONSERVATIVE_RASTERIZATION_NV
        GL_MAX_CLIP_DISTANCES
        GL_CONDITION_SATISFIED
        GLEW_EXT_multisample
        GL_RGB4
        GL_ARB_shader_ballot
        GL_ARB_texture_env_combine
        GL_RGB_422_APPLE
        GL_DEBUG_TYPE_ERROR_ARB
        GL_LUMINANCE_FLOAT32_APPLE
        GL_SGIX_polynomial_ffd
        GL_DEPTH_TEST
        GL_MAX_VIEWPORTS
        GL_MAX_SUBPIXEL_PRECISION_BIAS_BITS_NV
        GL_CLIP_DISTANCE2
        GLEW_ATI_separate_stencil
        GL_TEXTURE_CUBE_MAP_POSITIVE_X
        GLEW_EXT_vertex_shader
        GL_ANY_SAMPLES_PASSED
        GLEW_NV_geometry_program4
        GLEW_SGIX_reference_plane
        GL_OP_ADD_EXT
        GL_ADJACENT_PAIRS_NV
        GL_PROGRAM_MATRIX_STACK_DEPTH_EXT
        GL_UNIFORM_BUFFER_LENGTH_NV
        GL_TEXTURE16
        GL_UNPACK_COMPRESSED_BLOCK_WIDTH
        GL_SGIX_texture_scale_bias
        GL_SAMPLE_COVERAGE
        GL_TEXTURE_BUFFER_SIZE
        GLEW_NO_ERROR
        GL_UNPACK_RESAMPLE_SGIX
        GL_EXT_blend_logic_op
        GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT
        GL_QUERY_WAIT
        GL_DRAW_BUFFER3_EXT
        GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB
        GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL
        GL_EXT_vertex_array_bgra
        GL_COMBINE_ALPHA
        GL_FLOAT_CLEAR_COLOR_VALUE_NV
        GLEW_ARB_draw_buffers
        GL_PERFQUERY_SINGLE_CONTEXT_INTEL
        GL_WIN_swap_hint
        GL_FLOAT_MAT2
        GL_POINT_SMOOTH_HINT
        GL_UNKNOWN_CONTEXT_RESET
        GL_STENCIL_REF_COMMAND_NV
        GL_T
        GL_TEXTURE_BINDING_1D_ARRAY_EXT
        GL_DEBUG_TYPE_PORTABILITY_ARB
        GL_AMD_transform_feedback3_lines_triangles
        GL_AMD_query_buffer_object
        GL_ACCUM
        GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT
        GLEW_ARB_shader_storage_buffer_object
        GL_VIEWPORT_SWIZZLE_W_NV
        GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB
        GL_UNPACK_CMYK_HINT_EXT
        GL_SGIX_fog_offset
        GL_ONE_MINUS_SRC1_COLOR
        GL_LINES
        GLEW_NV_read_stencil
        GL_COLOR_ATTACHMENT1
        GL_ARB_depth_buffer_float
        GL_DRAW_BUFFER0_NV
        GL_ATC_RGBA_EXPLICIT_ALPHA_AMD
        GLEW_SGIX_complex_polar
        GLEW_ARB_internalformat_query
        GL_REFERENCED_BY_FRAGMENT_SHADER
        GL_MAX_SAMPLE_MASK_WORDS
        GL_NEVER
        GL_HISTOGRAM_ALPHA_SIZE
        GL_SYNC_STATUS
        GL_DEBUG_SEVERITY_NOTIFICATION
        GL_MAX_SPARSE_3D_TEXTURE_SIZE_AMD
        GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV
        GL_HSL_SATURATION_NV
        GL_UNSIGNED_INT_SAMPLER_1D_ARRAY
        GL_MAX_TEXTURE_BUFFER_SIZE_EXT
        GL_RENDER_GPU_MASK_NV
        GL_IMPLEMENTATION_COLOR_READ_FORMAT
        GL_UNPACK_SKIP_PIXELS_EXT
        GLEW_NV_parameter_buffer_object2
        GL_REFERENCED_BY_TESS_CONTROL_SHADER
        GL_EXT_memory_object
        GL_PRIMITIVE_RESTART
        GL_ARB_shader_precision
        GL_ATTRIB_ARRAY_TYPE_NV
        GLEW_SGIX_ycrcba
        GL_FRAMEBUFFER_UNSUPPORTED
        GL_EDGE_FLAG_ARRAY_STRIDE
        GL_NV_pixel_buffer_object
        GL_VIRTUAL_PAGE_SIZE_X_AMD
        GL_SHADER_PIXEL_LOCAL_STORAGE_EXT
        GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT
        GL_ACTIVE_SUBROUTINE_UNIFORM_LOCATIONS
        GL_CURRENT_SECONDARY_COLOR_EXT
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
        GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT
        GL_UTF16_NV
        GL_STENCIL_BUFFER_BIT3_QCOM
        GL_SRGB_DECODE_ARB
        GL_MODELVIEW24_ARB
        GL_DOT_PRODUCT_PASS_THROUGH_NV
        GL_EXT_texture_mirror_clamp
        GL_MAP1_VERTEX_4
        GLEW_EXT_fragment_lighting
        GL_TEXTURE_POINT_ONE_COORD_ATIX
        GL_RGB12_EXT
        GL_SRC_ALPHA_SATURATE
        GLEW_INTEL_framebuffer_CMAA
        GL_SHARED_EDGE_NV
        GLEW_NV_fragment_program
        GLEW_SGIX_pixel_texture_lod
        GL_IMAGE_2D_MULTISAMPLE_ARRAY_EXT
        GL_GREMEDY_string_marker
        GL_TRANSFORM_FEEDBACK_OVERFLOW
        GL_MAX_VERTEX_ATTRIB_BINDINGS
        GL_UNPACK_IMAGE_HEIGHT_NV
        GL_POST_COLOR_MATRIX_BLUE_SCALE
        GL_EXT_pixel_transform
        GL_SLUMINANCE_ALPHA_EXT
        GL_LUMINANCE8UI_EXT
        GL_SEPARATE_ATTRIBS_NV
        GL_VERTEX_ATTRIB_MAP2_DOMAIN_APPLE
        GL_PIXEL_MAP_I_TO_A_SIZE
        GL_COMPRESSED_RGBA_BPTC_UNORM_ARB
        GLEW_SGIS_fog_function
        GL_POLYGON_OFFSET_UNITS
        GL_LOW_INT
        GL_TEXTURE_COORD_ARRAY_LIST_IBM
        GL_GLYPH_HAS_KERNING_BIT_NV
        GL_MAX_ELEMENTS_VERTICES
        GL_DRIVER_REGAL
        GL_APPLE_texture_2D_limited_npot
        GL_NO_RESET_NOTIFICATION
        GL_NUM_SHADER_BINARY_FORMATS
        GL_REND_screen_coordinates
        GL_NORMAL_ARRAY_POINTER_EXT
        GL_TEXTURE3_SGIS
        GL_COLOR_TABLE_GREEN_SIZE
        GL_FLOAT16_NV
        GL_SGIX_blend_cmultiply
        GL_DEBUG_CALLBACK_FUNCTION
        GL_TEXTURE4_ARB
        GL_CIRCULAR_TANGENT_ARC_TO_NV
        GL_FRAMEBUFFER_SRGB
        GL_MODELVIEW16_ARB
        GL_VERTEX_ATTRIB_MAP2_APPLE
        GL_PATH_TERMINAL_DASH_CAP_NV
        GL_SHININESS
        GL_INTERPOLATE_EXT
        GL_POST_CONVOLUTION_BLUE_BIAS_EXT
        GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE
        GL_MAP2_VERTEX_4
        GL_STENCIL_INDEX4
        GLEW_KHR_no_error
        GL_POST_CONVOLUTION_ALPHA_BIAS
        GL_RGBA8
        GL_EMBOSS_CONSTANT_NV
        GLEW_ARB_texture_compression_bptc
        GL_PACK_ROW_BYTES_APPLE
        GLEW_OML_subsample
        GL_VARIABLE_G_NV
        GL_TEXTURE_COMPARE_SGIX
        GL_TEXTURE_STENCIL_SIZE
        GL_COPY_WRITE_BUFFER_NV
        GL_CURRENT_VERTEX_EXT
        GL_ANDROID_extension_pack_es31a
        GL_PIXEL_PACK_BUFFER_NV
        GL_PROXY_TEXTURE_2D_MULTISAMPLE
        GL_X_EXT
        GL_CULL_FRAGMENT_NV
        GL_DEPTH_BUFFER_BIT0_QCOM
        GL_COMP_BIT_ATI
        GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_FRAGMENT_SHADER
        GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY_ARB
        GL_PERFMON_RESULT_SIZE_AMD
        GL_LUMINANCE_ALPHA_SIGNED_SGIX
        GL_OUTPUT_TEXTURE_COORD25_EXT
        GL_BUMP_ROT_MATRIX_SIZE_ATI
        GL_NUM_EXTENSIONS
        GL_DEPTH_WRITEMASK
        GL_IMAGE_BINDING_LEVEL
        GL_QUERY_BY_REGION_NO_WAIT_INVERTED
        GL_DRAW_BUFFER8_NV
        GL_AMD_draw_buffers_blend
        GL_BLEND_COLOR_COMMAND_NV
        GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS
        GL_AMD_program_binary_Z400
        GLEW_ARB_multi_draw_indirect
        GL_LUMINANCE16I_EXT
        GL_RESAMPLE_REPLICATE_SGIX
        GL_RGBA_SIGNED_SGIX
        GL_STREAM_COPY
        GL_COMPLEX_UNSIGNED_SHORT_SGI
        GL_TEXTURE_CONSTANT_DATA_SUNX
        GL_COLOR_ATTACHMENT7_EXT
        GL_PROXY_HISTOGRAM
        GL_NAND
        GL_TRANSPOSE_PROJECTION_MATRIX_ARB
        GL_FRAME_NV
        GL_MATERIAL_SIDE_HINT_PGI
        GL_POST_CONVOLUTION_BLUE_SCALE
        GL_COMPRESSED_SRGB
        GL_SGIX_interlace
        GL_CONTEXT_CORE_PROFILE_BIT
        GL_BLUE_BIT_ATI
        GL_ARRAY_ELEMENT_LOCK_FIRST_EXT
        GLEW_SGIX_sprite
        GL_INT_IMAGE_1D_ARRAY_EXT
        GL_DEPTH_ATTACHMENT_EXT
        GL_INTERNALFORMAT_SHARED_SIZE
        GL_PROTECTED_MEMORY_OBJECT_EXT
        GL_ARB_fragment_program
        GLEW_ARB_window_pos
        GL_INTENSITY4
        GL_SGIX_texture_range
        GL_EXT_copy_texture
        GL_COLOR_ARRAY
        GL_COLOR_TABLE_ALPHA_SIZE
        GL_OFFSET_TEXTURE_2D_MATRIX_NV
        GL_AMD_occlusion_query_event
        GL_TRANSFORM_FEEDBACK_BUFFER_BINDING
        GL_COMMAND_BARRIER_BIT
        GL_AMD_gpu_shader_half_float
        GLEW_SGIX_color_table_index_mode
        GL_SGIX_texture_multi_buffer
        GL_RGB32UI_EXT
        GLEW_ARB_shader_stencil_export
        GL_ARB_texture_rgb10_a2ui
        GL_BLEND_SRC_ALPHA
        GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT
        GL_AMD_shader_trinary_minmax
        GL_SOURCE1_RGB_EXT
        GL_UNSIGNED_SHORT_8_8_APPLE
        GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS
        GL_SGIX_texture_lod_bias
        GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB
        GL_RGB565
        GL_LINE_STRIP_ADJACENCY
        GL_TEXTURE_FIXED_SAMPLE_LOCATIONS
        GL_RESAMPLE_AVERAGE_OML
        GL_INVARIANT_DATATYPE_EXT
        GL_REGAL_proc_address
        GL_CONSTANT_COLOR
        GL_OUTPUT_FOG_EXT
        GL_LUMINANCE_ALPHA16I_EXT
        GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB
        GL_FLOAT_MAT4x2
        GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE
        GLEW_APPLE_specular_vector
        GL_EXT_Cg_shader
        GL_OUTPUT_TEXTURE_COORD9_EXT
        GL_CLIENT_MAPPED_BUFFER_BARRIER_BIT
        GL_UNSIGNED_INT_IMAGE_2D_EXT
        GL_RGBA16_SNORM_EXT
        GL_RGB_INTEGER
        GLEW_ARB_gpu_shader_int64
        GLEW_EXT_bgra
        GLEW_ARB_transform_feedback_overflow_query
        GL_DEBUG_SEVERITY_MEDIUM_AMD
        GL_RGB16_EXTENDED_RANGE_SGIX
        GL_USE_MISSING_GLYPH_NV
        GL_FLOAT_VEC2_ARB
        GL_PROXY_TEXTURE_1D_EXT
        GL_NV_geometry_shader_passthrough
        GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES
        GLEW_SGIX_icc_texture
        GLEW_SGIX_texture_multi_buffer
        GL_COLORDODGE_KHR
        GL_POINT_TOKEN
        GL_CONSERVATIVE_RASTERIZATION_INTEL
        GL_INT_SAMPLER_CUBE_EXT
        GL_PERFQUERY_COUNTER_DATA_UINT32_INTEL
        GL_RENDERBUFFER_SAMPLES_APPLE
        GL_COLOR_EXT
        GL_MATRIX25_ARB
        GL_VERTEX_PROGRAM_TWO_SIDE_NV
        GL_EGL_KHR_context_flush_control
        GL_RENDERBUFFER_RED_SIZE_EXT
        GLEW_ARB_fragment_layer_viewport
        GL_COMPRESSED_RGBA_FXT1_3DFX
        GL_EXT_render_snorm
        GL_STENCIL_INDEX16_EXT
        GL_COLORBURN_KHR
        GL_NORMAL_ARRAY_TYPE_EXT
        GL_LINEAR_MIPMAP_LINEAR
        GL_DOUBLE_VEC4
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_NV
        GL_ALPHA_MIN_CLAMP_INGR
        GL_Z_EXT
        GL_LIGHTING_BIT
        GL_GEOMETRY_SHADER
        GL_ALPHA_TEST
        GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS
        GL_SEPARABLE_2D
        GL_MAX_VIEWS_OVR
        GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI
        GL_SGIX_texture_coordinate_clamp
        GL_INT64_VEC3_NV
        GL_COMPRESSED_LUMINANCE_LATC1_NV
        GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_APPLE
        GL_ARB_query_buffer_object
        GL_OPERAND3_RGB_NV
        GL_ARB_shader_draw_parameters
        GL_VIEWPORT_COMMAND_NV
        GL_COMPRESSED_RGBA_S3TC_DXT1_EXT
        GLEW_ARB_compute_variable_group_size
        GLEW_EXT_index_func
        GL_INDEX_ARRAY_STRIDE
        GL_CONTEXT_FLAG_DEBUG_BIT
        GL_RGB5_A1_EXT
        GL_TEXTURE_TYPE_QCOM
        GL_NV_clip_space_w_scaling
        GL_COMPRESSED_RG
        GL_ARB_texture_barrier
        GL_MAX_GEOMETRY_IMAGE_UNIFORMS_EXT
        GL_WEIGHTED_AVERAGE_ARB
        GL_STENCIL_BUFFER_BIT1_QCOM
        GL_MATRIX_EXT
        GL_ARB_texture_cube_map_array
        GLEW_SGIX_bali_timer_instruments
        GL_BLUE_NV
        GL_LUMINANCE_ALPHA_SNORM
        GL_QUERY_BY_REGION_NO_WAIT
        GL_INDEX_ARRAY_ADDRESS_NV
        GL_TANGENT_ARRAY_POINTER_EXT
        GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT
        GL_CLIP_PLANE2
        GL_OML_interlace
        GL_SAMPLE_BUFFERS_3DFX
        GL_DOUBLE_MAT4x3
        GL_MAX_COMBINED_SHADER_STORAGE_BLOCKS
        GLEW_VERSION_1_5
        GL_MAX_SHADER_COMPILER_THREADS_ARB
        GLEW_AMD_program_binary_Z400
        GL_INTENSITY_ICC_SGIX
        GL_EXT_draw_elements_base_vertex
        GLEW_VERSION_MICRO
        GL_OP_LOG_BASE_2_EXT
        GL_FILL
        GL_NV_conservative_raster_pre_snap_triangles
        GL_VIEW_CLASS_BPTC_FLOAT
        GL_REPLACE_MIDDLE_SUN
        GLEW_SGIX_fog_layers
        GL_ALPHA16
        GL_POST_CONVOLUTION_COLOR_TABLE_SGI
        GL_SGIX_vertex_preclip_hint
        GL_ACTIVE_RESOURCES
        GL_COLOR_ATTACHMENT10_NV
        GL_NV_tessellation_program5
        GL_CPU_OPTIMIZED_QCOM
        GL_EXT_framebuffer_multisample_blit_scaled
        GL_CLAMP_TO_BORDER_NV
        GL_TEXTURE_CUBE_MAP
        GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS
        GL_STANDARD_FONT_NAME_NV
        GLEW_SGIX_texture_add_env
        GL_NV_fragment_coverage_to_color
        GL_MAX_MAP_TESSELLATION_NV
        GL_INTERLEAVED_ATTRIBS
        GL_FLOAT_MAT2x3
        GL_APPLE_fence
        GL_DEBUG_LOGGED_MESSAGES_AMD
        GL_ARB_clear_buffer_object
        GL_NV_transform_feedback2
        GLEW_ARB_draw_instanced
        GL_COMBINE_RGB
        GL_FOG_COORDINATE_SOURCE
        GL_RGBA_FLOAT32_ATI
        GLEW_SGIX_cube_map
        GL_SAMPLE_COVERAGE_VALUE_ARB
        GL_LINE_SMOOTH_HINT
        GL_FOG_MODE
        GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN
        GL_CONSTANT_COLOR_EXT
        GL_INVERT_OVG_NV
        GL_VERTEX_ATTRIB_ARRAY9_NV
        GL_ARRAY_SIZE
        GL_READ_FRAMEBUFFER_BINDING
        GLEW_ARB_ES3_compatibility
        GL_FUNC_ADD
        GLEW_ARB_compressed_texture_pixel_storage
        GL_VERTEX_SHADER_VARIANTS_EXT
        GL_KHR_debug
        GL_MAX_UNIFORM_BUFFER_BINDINGS
        GL_INTERNALFORMAT_RED_TYPE
        GL_SCISSOR_BIT
        GL_CONVOLUTION_HEIGHT
        GL_SGIX_blend_cadd
        GL_LUMINANCE
        GL_MODULATE_SUBTRACT_ATI
        GL_ALPHA16I_EXT
        GL_TEXTURE30
        GL_NV_register_combiners
        GL_CONJOINT_NV
        GL_SHADER_OBJECT_EXT
        GL_REFLECTION_MAP_NV
        GL_TEXTURE_MAX_ANISOTROPY_EXT
        GL_INVERTED_SCREEN_W_REND
        GL_INT64_VEC2_NV
        GLEW_EXT_sRGB
        GL_BACK_SECONDARY_COLOR_NV
        GLEW_APPLE_client_storage
        GL_SAMPLER_2D_ARRAY_SHADOW_EXT
        GL_LOW_FLOAT
        GL_AMD_compressed_3DC_texture
        GL_STENCIL_BACK_FUNC
        GL_INTENSITY16_EXT
        GL_ACTIVE_TEXTURE_ARB
        GL_NV_path_rendering_shared_edge
        GL_ARB_texture_mirrored_repeat
        GLEW_EXT_rescale_normal
        GL_PLUS_NV
        GL_RG32F_EXT
        GL_SGIS_color_range
        GLEW_ARB_shader_clock
        GL_LUMINANCE_ALPHA_FLOAT16_APPLE
        GL_POST_CONVOLUTION_COLOR_TABLE
        GL_TEXTURE12_ARB
        GL_STENCIL
        GL_SAMPLE_BUFFERS
        GL_PROGRAM_MATRIX_EXT
        GL_MODELVIEW15_ARB
        GL_COLOR_SAMPLES_NV
        GLEW_ANGLE_timer_query
        GL_VARIANT_ARRAY_POINTER_EXT
        GLEW_EXT_texture_perturb_normal
        GL_MIN_EXT
        GL_INT16_NV
        GL_NV_robustness_video_memory_purge
        GL_CLIP_VOLUME_CLIPPING_HINT_EXT
        GL_DRAW_BUFFER9
        GL_COMPRESSED_LUMINANCE_ARB
        GL_STENCIL_WRITEMASK
        GL_RGB32F
        GL_R8_EXT
        GL_MAX_SHADER_STORAGE_BUFFER_BINDINGS
        GLEW_EXT_texture
        GL_COLOR_FLOAT_APPLE
        GL_TEXTURE_BUFFER_BINDING
        GL_SIGNED_INTENSITY8_NV
        GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE
        GL_MODELVIEW1_STACK_DEPTH_EXT
        GL_YCRCB_SGIX
        GL_LAYOUT_DEPTH_STENCIL_READ_ONLY_EXT
        GL_LUMINANCE12_EXT
        GL_UNPACK_ROW_LENGTH_EXT
        GL_ATTACHED_SHADERS
        GL_TEXTURE_TARGET
        GL_MAX_FRAMEBUFFER_LAYERS_EXT
        GL_EXT_base_instance
        GL_TIMEOUT_EXPIRED
        GL_TEXTURE_BUFFER
        GLEW_ARB_shading_language_packing
        GL_PROGRAM_RESULT_COMPONENTS_NV
        GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS
        GLEW_NV_depth_clamp
        GL_EXT_texture_snorm
        GL_NV_shader_atomic_float64
        GL_MAP2_TEXTURE_COORD_2_NURBS_SGIX
        GL_MAT_SPECULAR_BIT_PGI
        GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT
        GL_ANGLE_texture_usage
        GL_TEXTURE_COMPRESSION_HINT
        GL_RGBA16
        GL_PROXY_TEXTURE_RECTANGLE_NV
        GL_SLIM10U_SGIX
        GL_C3F_V3F
        GL_PHONG_WIN
        GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE
        GL_MATRIX_INDEX_ARRAY_POINTER_ARB
        GLEW_REGAL_log
        GL_MAX_COMPUTE_UNIFORM_COMPONENTS
        GL_MATRIX_STRIDE
        GLEW_SGIX_interlace
        GL_PERFQUERY_COUNTER_TIMESTAMP_INTEL
        GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS_EXT
        GL_RGBA16_EXTENDED_RANGE_SGIX
        GL_INDEX_CLEAR_VALUE
        GL_SHADING_LANGUAGE_VERSION_ARB
        GL_FEEDBACK_BUFFER_TYPE
        GL_DEPTH_BIAS
        GLEW_EXT_framebuffer_multisample_blit_scaled
        GL_NORMAL_ARRAY_LIST_STRIDE_IBM
        GL_LINES_ADJACENCY_EXT
        GL_ARB_texture_view
        GL_UNIFORM_ADDRESS_COMMAND_NV
        GL_TESS_CONTROL_SHADER_PATCHES
        GL_MAX_FRAGMENT_ATOMIC_COUNTERS
        GL_MAX_TRACK_MATRICES_NV
        GL_UNSIGNED_INT_8_8_S8_S8_REV_NV
        GL_DRAW_BUFFER6
        GL_COLOR_BUFFER_BIT7_QCOM
        GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED
        GL_DRAW_BUFFER10_ATI
        GL_WRITE_PIXEL_DATA_RANGE_NV
        GL_UNIFORM_BUFFER_UNIFIED_NV
        GLEW_SGIS_pixel_texture
        GL_CACHE_REGAL
        GL_MAX_EVAL_ORDER
        GL_MAX_SPARSE_3D_TEXTURE_SIZE_EXT
        GL_FONT_ASCENDER_BIT_NV
        GL_TEXTURE_BUFFER_FORMAT_EXT
        GLEW_NV_alpha_to_coverage_dither_control
        GL_POLYGON_MODE_NV
        GL_MAX_ALPHA_SGIS
        GL_SWIZZLE_STQ_DQ_ATI
        GL_POINT_SIZE_MIN_EXT
        GL_RENDERBUFFER_COVERAGE_SAMPLES_NV
        GLEW_EXT_copy_texture
        GL_DEPTH_COMPONENT24_ARB
        GL_HIGH_INT
        GL_MAX_LUMINANCE_SGIS
        GL_INT16_VEC4_NV
        GL_BUFFER_BINDING
        GLEW_EXT_framebuffer_sRGB
        GL_COMPRESSED_INTENSITY
        GL_RELATIVE_SMOOTH_CUBIC_CURVE_TO_NV
        GLEW_AMD_occlusion_query_event
        GL_STATIC_COPY_ARB
        GL_NV_texture_compression_latc
        GL_MAP1_TEXTURE_COORD_4_NURBS_SGIX
        GL_SIGNED_RGB_UNSIGNED_ALPHA_NV
        GLEW_EXT_blend_minmax
        GL_ANGLE_texture_compression_dxt1
        GL_PROGRAM_ERROR_STRING_NV
        GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER
        GL_SOURCE2_ALPHA_ARB
        GL_VERTEX_SHADER_ARB
        GL_MULTISAMPLE_FILTER_HINT_NV
        GL_NV_polygon_mode
        GL_OP_RECIP_EXT
        GL_EVAL_VERTEX_ATTRIB7_NV
        GLEW_SGIX_async_histogram
        GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV
        GL_DYNAMIC_COPY
        GL_BINORMAL_ARRAY_EXT
        GL_OCCLUSION_INSTRUMENT_SGIX
        GL_VIRTUAL_PAGE_SIZE_Y_ARB
        GL_TRACE_REGAL
        GL_MAX_GEOMETRY_OUTPUT_COMPONENTS
        GL_TEXTURE22
        GL_MAX_3D_TEXTURE_SIZE
        GL_TESS_CONTROL_SHADER_BIT
        GL_TESS_CONTROL_SHADER_PATCHES_ARB
        GL_GLYPH_VERTICAL_BEARING_Y_BIT_NV
        GL_ATOMIC_COUNTER_BUFFER_INDEX
        GLEW_SUN_mesh_array
        GL_EVAL_FRACTIONAL_TESSELLATION_NV
        GL_MODELVIEW27_ARB
        GL_CLEAR
        GL_MODELVIEW26_ARB
        GL_RGBA_SNORM
        GL_LOCATION_COMPONENT
        GL_LIGHT_MODEL_AMBIENT
        GL_BUFFER_STORAGE_FLAGS
        GL_COLOR_TABLE_ALPHA_SIZE_SGI
        GL_CG_FRAGMENT_SHADER_EXT
        GL_SET
        GL_CLIP_DISTANCE2_APPLE
        GL_1PASS_EXT
        GL_CLIENT_ALL_ATTRIB_BITS
        GLEW_EXT_shader_pixel_local_storage
        GL_PROGRAM_ERROR_STRING_ARB
        GL_MIPMAP
        GL_NV_compute_program5
        GL_ARB_stencil_texturing
        GL_SGIX_clipmap
        GL_FOG_COORDINATE_ARRAY_TYPE
        GL_TEXTURE_SWIZZLE_B
        GL_NV_shader_noperspective_interpolation
        GLEW_ARM_mali_program_binary
        GL_REFERENCED_BY_TESS_EVALUATION_SHADER
        GLEW_ARB_shader_texture_lod
        GL_ALLOW_DRAW_OBJ_HINT_PGI
        GL_HISTOGRAM_GREEN_SIZE
        GL_POLYGON_BIT
        GL_SGIX_ycrcb_subsample
        GL_PATCH_DEFAULT_OUTER_LEVEL
        GL_RGBA8I_EXT
        GL_DYNAMIC_STORAGE_BIT
        GL_ANGLE_timer_query
        GL_TEXTURE_INTENSITY_SIZE_EXT
        GL_DRAW_ELEMENTS_INSTANCED_COMMAND_NV
        GL_MAP_STENCIL
        GL_COMPRESSED_LUMINANCE_ALPHA
        GL_TRANSFORM_FEEDBACK_BUFFER_INDEX
        GL_RED_INTEGER_EXT
        GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV
        GL_VERTEX_ARRAY_STORAGE_HINT_APPLE
        GLEW_ARB_sparse_buffer
        GL_HALF_FLOAT_NV
        GL_STENCIL_BUFFER_BIT7_QCOM
        GL_SINGLE_COLOR
        GLEW_ARB_clip_control
        GL_APPLE_pixel_buffer
        GL_DOT_PRODUCT_TEXTURE_2D_NV
        GL_APPLE_vertex_array_object
        GLEW_ARB_geometry_shader4
        GL_SAMPLER_RENDERBUFFER_NV
        GL_SUN_slice_accum
        GL_LUMINANCE_ICC_SGIX
        GL_NV_bindless_multi_draw_indirect_count
        GL_TRANSPOSE_MODELVIEW_MATRIX
        GL_MATRIX2_ARB
        GL_DRAW_FRAMEBUFFER_APPLE
        GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC
        GL_MAP2_TANGENT_EXT
        GL_FULL_STIPPLE_HINT_PGI
        GL_STREAM_READ_ARB
        GL_INDEX_ARRAY_TYPE_EXT
        GL_DEPTH_STENCIL_ATTACHMENT
        GL_RGB_S3TC
        GL_VERTEX_ATTRIB_ARRAY14_NV
        GLEW_ARB_texture_filter_minmax
        GL_TEXTURE_BUFFER_FORMAT
        GL_RGBA_S3TC
        GL_ARB_vertex_buffer_object
        GL_CLAMP_TO_BORDER
        GL_TRANSFORM_FEEDBACK_ATTRIBS_NV
        GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT
        GL_EXT_bgra
        GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB
        GL_UNSIGNED_SHORT_5_6_5
        GL_TOP_LEVEL_ARRAY_SIZE
        GLEW_EXT_index_texture
        GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_NV
        GL_NUM_VIDEO_CAPTURE_STREAMS_NV
        GL_ALPHA_TEST_QCOM
        GL_COLOR_ATTACHMENT5_EXT
        GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB
        GL_VERTEX_PROGRAM_ARB
        GL_OP_SUB_EXT
        GL_DEBUG_SOURCE_APPLICATION_ARB
        GL_MANUAL_GENERATE_MIPMAP
        GL_TRANSFORM_BIT
        GL_TEXTURE_COORD_ARRAY_SIZE_EXT
        GL_RG16UI
        GL_CUBE_MAP_YP_SGIX
        GL_DEBUG_OUTPUT_SYNCHRONOUS
        GL_COLOR_TABLE_RED_SIZE_SGI
        GL_RGB4_S3TC
        GL_TESS_CONTROL_SUBROUTINE_UNIFORM
        GL_BOOL_VEC3
        GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB
        GLEW_ARB_direct_state_access
        GL_AUTO_GENERATE_MIPMAP
        GLEW_ARB_framebuffer_no_attachments
        GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI
        GL_COMPUTE_PROGRAM_PARAMETER_BUFFER_NV
        GL_ARB_texture_buffer_object
        GL_MAX_PROGRAM_INSTRUCTIONS_ARB
        GL_TEXTURE_IMMUTABLE_FORMAT_EXT
        GL_CLAMP_FRAGMENT_COLOR_ARB
        GL_UNSIGNED_INT_SAMPLER_2D_ARRAY
        GL_EXT_packed_float
        GLEW_NV_shader_atomic_int64
        GLEW_EXT_texture_env_combine
        GL_VIEW_CLASS_S3TC_DXT3_RGBA
        GL_MUL_ATI
        GL_MAX_ATTRIB_STACK_DEPTH
        GL_KHR_blend_equation_advanced
        GLEW_EXT_shader_integer_mix
        GLEW_AMD_shader_trinary_minmax
        GL_OUTPUT_TEXTURE_COORD24_EXT
        GL_ARB_pipeline_statistics_query
        GL_INTERNALFORMAT_DEPTH_SIZE
        GL_PIXELS_PER_SAMPLE_PATTERN_Y_AMD
        GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV
        GL_POINT_SIZE_RANGE
        GL_MAX_NUM_COMPATIBLE_SUBROUTINES
        GLEW_EXT_texture_compression_rgtc
        GLEW_AMD_gpu_shader_int64
        GL_MATRIX26_ARB
        GLEW_NV_bindless_texture
        GL_TEXTURE_LUMINANCE_TYPE
        GL_COLOR_ATTACHMENT8
        GL_FRAGMENT_LIGHTING_EXT
        GL_REPLACE_OLDEST_SUN
        GLEW_EXT_debug_label
        GL_SHADER_STORAGE_BARRIER_BIT
        GL_UNSIGNED_INT_SAMPLER_3D
        GL_AVERAGE_EXT
        GL_INT16_VEC3_NV
        GL_SAMPLER_2D_ARRAY_SHADOW_NV
        GL_HORIZONTAL_LINE_TO_NV
        GLEW_NV_packed_float_linear
        GL_QUERY_NO_WAIT
        GL_TEXTURE15
        GLEW_ARB_debug_output
        GL_MAP2_NORMAL
        GL_EYE_RADIAL_NV
        GL_SAMPLER_3D_ARB
        GL_PLUS_CLAMPED_NV
        GL_MODELVIEW3_ARB
        GL_IMAGE_CLASS_2_X_8
        GL_TEXT_FRAGMENT_SHADER_ATI
        GL_TEXTURE_DEPTH_SIZE_ARB
        GL_LERP_ATI
        GL_REPEAT
        GL_IBM_multimode_draw_arrays
        GL_FRAGMENT_PROGRAM_BINDING_NV
        GL_NV_alpha_to_coverage_dither_control
        GL_VIRTUAL_PAGE_SIZE_Z_ARB
        GL_MAX_VERTEX_SHADER_STORAGE_BLOCKS
        GL_INT_SAMPLER_RENDERBUFFER_NV
        GL_UNIFORM_SIZE
        GL_ARB_cull_distance
        GL_COLOR_TABLE
        GL_PATH_STENCIL_VALUE_MASK_NV
        GL_GEOMETRY_SHADER_BIT
        GL_RGB_SCALE
        GL_RGBA8UI_EXT
        GL_FLOAT_MAT2x3_NV
        GL_VERTEX_ATTRIB_ARRAY10_NV
        GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2
        GL_QUERY_OBJECT_EXT
        GL_MODELVIEW17_ARB
        GL_TIME_ELAPSED_ANGLE
        GL_TEXTURE_ALPHA_SIZE_EXT
        GL_COMPRESSED_RGBA_ASTC_12x10_KHR
        GL_CURRENT_WEIGHT_ARB
        GL_RGB9_E5_APPLE
        GL_MAX_VERTEX_ATTRIBS_ARB
        GLEW_EXT_coordinate_frame
        GLEW_EXT_texture_compression_s3tc
        GLEW_NV_pixel_buffer_object
        GLEW_SGIS_point_line_texgen
        GL_VERTICES_SUBMITTED
        GLEW_NV_parameter_buffer_object
        GL_TEXTURE_BLUE_SIZE
        GL_NV_shader_atomic_int64
        GL_FLOAT_MAT3x2_NV
        GL_SYNC_GPU_COMMANDS_COMPLETE_APPLE
        GL_COMBINER_SUM_OUTPUT_NV
        GL_MULTISAMPLE_SGIS
        GL_NEXT_VIDEO_CAPTURE_BUFFER_STATUS_NV
        GL_DRAW_BUFFER14_ATI
        GL_R1UI_T2F_V3F_SUN
        GL_SAMPLE_COVERAGE_INVERT
        GL_COMPARE_R_TO_TEXTURE
        GLEW_VERSION_4_0
        GLEW_NV_sample_mask_override_coverage
        GL_CMYK_EXT
        GL_TEXTURE_1D_ARRAY_EXT
        GL_DOUBLE_MAT2_EXT
        GL_COMPRESSED_RGB8_ETC2
        GL_SINGLE_COLOR_EXT
        GL_UNIFORM_BUFFER_SIZE
        GL_BLEND_COLOR
        GLEW_ATI_texture_compression_3dc
        GL_EDGE_FLAG_ARRAY
        GL_RENDERBUFFER_HEIGHT_EXT
        GL_SAMPLE_BUFFERS_SGIS
        GL_GEOMETRY_SUBROUTINE
        GL_SYNC_FENCE
        GL_COLOR_ARRAY_SIZE
        GL_COMPRESSED_SLUMINANCE_ALPHA_EXT
        GL_MAX_VERTEX_HINT_PGI
        GL_IMAGE_BINDING_LAYER_EXT
        GLEW_NV_vertex_buffer_unified_memory
        GL_PROGRAM_ADDRESS_REGISTERS_ARB
        GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS
        GL_RED_BIAS
        GL_COMBINE_EXT
        GL_TEXTURE_BINDING_RECTANGLE
        GL_MAP2_VERTEX_ATTRIB5_4_NV
        GL_VERTEX4_BIT_PGI
        GLEW_EXT_vertex_weighting
        GL_UNSIGNED_SHORT_4_4_4_4_EXT
        GLEW_ARB_parallel_shader_compile
        GL_OPERAND0_RGB
        GL_EXT_read_format_bgra
        GL_GENERIC_ATTRIB_NV
        GL_POINT_SMOOTH
        GLEW_NV_draw_vulkan_image
        GL_MAP2_VERTEX_ATTRIB3_4_NV
        GL_RED_MIN_CLAMP_INGR
        GLEW_SGIX_image_compression
        GL_MAP2_VERTEX_ATTRIB11_4_NV
        GL_SPOT_CUTOFF_DELTA_SGIX
        GL_ASYNC_HISTOGRAM_SGIX
        GL_MAP2_GRID_DOMAIN
        GL_TANGENT_ARRAY_TYPE_EXT
        GL_SECONDARY_COLOR_ARRAY_POINTER_EXT
        GLEW_NV_shader_atomic_float64
        GL_CUBE_MAP_SGIX
        GL_TEXTURE_INTERNAL_FORMAT_QCOM
        GL_QUADS
        GL_GEOMETRY_VERTICES_OUT
        GL_VERTEX_ARRAY
        GL_OPERAND2_ALPHA_EXT
        GL_SECONDARY_COLOR_ARRAY_SIZE_EXT
        GL_STENCIL_FUNC
        GL_EYE_PLANE_ABSOLUTE_NV
        GL_DRAW_BUFFER7_NV
        GL_SAMPLE_ALPHA_TO_MASK_EXT
        GL_VIEW_CLASS_RGTC1_RED
        GL_MAX_SHADER_STORAGE_BLOCK_SIZE
        GL_SGIX_convolution_accuracy
        GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_NV
        GL_MAP2_VERTEX_ATTRIB9_4_NV
        GL_DEPTH_CLAMP_NV
        GL_RG8_SNORM
        GL_PACK_COMPRESSED_BLOCK_HEIGHT
        GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB
        GL_INVARIANT_VALUE_EXT
        GL_COLOR_INDEX1_EXT
        GL_OBJECT_DISTANCE_TO_POINT_SGIS
        GLEW_NV_pixel_data_range
        GL_DEPTH_BUFFER
        GL_RELATIVE_RECT_NV
        GL_PROGRAM_STRING_NV
        GL_MAX_CLIP_DISTANCES_APPLE
        GL_ALPHA32UI_EXT
        GL_EXT_blend_func_extended
        GLEW_EXT_draw_buffers
        GLEW_ARB_viewport_array
        GL_PIXEL_MAP_I_TO_G
        GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR
        GL_SELECTED_TEXTURE_TRANSFORM_SGIS
        GLEW_EXT_shader_texture_lod
        GL_RED_INTEGER
        GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT
        GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB
        GL_SGIS_texture_select
        GL_ARB_framebuffer_sRGB
        GL_RGBA32F_ARB
        GL_ARB_compatibility
        GL_MATRIX_INDEX_ARRAY_ARB
        GL_ALPHA16F_EXT
        GL_MAP2_TEXTURE_COORD_2
        GL_LIGHT1
        GL_LUMINANCE12_ALPHA4
        GL_SRGB_READ
        GL_EXT_window_rectangles
        GL_FLOAT_RG_NV
        GL_MAP1_TEXTURE_COORD_4
        GL_NEAREST_MIPMAP_LINEAR
        GL_FRAMEBUFFER_BLEND
        GL_DSDT_NV
        GL_HARDLIGHT_KHR
        GL_MAP2_BINORMAL_EXT
        GL_VERTEX_STREAM4_ATI
        GL_QUERY_RESULT
        GL_DRAW_BUFFER4
        GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV
        GL_DRAW_BUFFER3_NV
        GLEW_IBM_cull_vertex
        GL_ARB_half_float_vertex
        GL_PATCHES
        GL_VERTEX_ARRAY_OBJECT_AMD
        GLEW_SGIX_depth_pass_instrument
        GL_ALLOW_DRAW_MEM_HINT_PGI
        GL_NV_uniform_buffer_unified_memory
        GL_POINT_SPRITE_COORD_ORIGIN
        GL_PIXEL_TRANSFORM_OPERATOR_SGI
        GL_INTENSITY4_EXT
        GL_PROGRAM_POINT_SIZE_EXT
        GL_RENDERBUFFER_HEIGHT
        GL_DEPTH_TEXTURE_MODE
        GL_ELEMENT_ARRAY_POINTER_ATI
        GL_COMPRESSED_RGBA_ASTC_12x12_KHR
        GL_NV_draw_buffers
        GL_LAST_VIDEO_CAPTURE_STATUS_NV
        GL_WIN_specular_fog
        GL_COLOR_ARRAY_EXT
        GL_TEXTURE_ALPHA_TYPE_ARB
        GL_HILO_NV
        GL_QUERY_BY_REGION_WAIT_INVERTED
        GL_CURRENT_SECONDARY_COLOR
        GL_EXT_texture_env
        GL_EVAL_VERTEX_ATTRIB1_NV
        GLEW_WIN_scene_markerXXX
        GLEW_NV_polygon_mode
        GL_TEXTURE_COMPRESSED_ARB
        GL_BIAS_BY_NEGATIVE_ONE_HALF_NV
        GLEW_3DFX_tbuffer
        GL_NV_gpu_program_fp64
        GL_POST_COLOR_MATRIX_BLUE_BIAS
        GL_STENCIL_INDEX8_EXT
        GL_HANDLE_TYPE_D3D11_IMAGE_EXT
        GL_ARB_window_pos
        GL_MAX_COLOR_MATRIX_STACK_DEPTH
        GL_SAMPLE_PATTERN_SGIS
        GL_VIRTUAL_PAGE_SIZE_Y_EXT
        GLEW_EXT_texture_filter_minmax
        GL_COMPLEX_SHORT_SGI
        GL_ARB_buffer_storage
        GL_ARB_texture_mirror_clamp_to_edge
        GL_DECR
        GL_LUMINANCE16F_ARB
        GL_MODELVIEW14_ARB
        GLEW_APPLE_framebuffer_multisample
        GL_COLOR_INDEX4_EXT
        GL_MAP_READ_BIT_EXT
        GL_RGBA32UI_EXT
        GL_TEXTURE5
        GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE
        GL_OBJECT_SPACE_SGIX
        GL_COLOR_ATTACHMENT15_NV
        GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS_EXT
        GL_UNSIGNED_INT_IMAGE_3D
        GLEW_NV_evaluators
        GL_LOG_ERROR_REGAL
        GL_DOT3_RGB_ARB
        GL_ZOOM_X
        GL_NV_texture_rectangle
        GL_LUMINANCE_FLOAT32_ATI
        GL_SGIS_texture_filter4
        GL_OUTPUT_TEXTURE_COORD4_EXT
        GL_UNSIGNED_INT_8_8_8_8_REV
        GL_MAP2_NORMAL_NURBS_SGIX
        GLEW_OVR_multiview
        GL_STENCIL_BACK_WRITEMASK
        GL_TEXTURE3
        GL_DOUBLE_VEC2
        GL_VIEWPORT_BIT
        GL_AUX_DEPTH_STENCIL_APPLE
        GL_DEPTH_BUFFER_BIT7_QCOM
        GL_PROGRAM_STRING_ARB
        GL_D3D12_FENCE_VALUE_EXT
        GL_QUERY_DEPTH_BOUNDS_FAIL_EVENT_BIT_AMD
        GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_EXT
        GL_COMPRESSED_RGBA_ASTC_10x10_KHR
        GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV
        GL_MATRIX9_ARB
        GLEW_EXT_subtexture
        GL_SUN_vertex
        GL_EXT_raster_multisample
        GL_NV_blend_minmax_factor
        GL_GEOMETRY_INPUT_TYPE_EXT
        GL_FRAGMENTS_INSTRUMENT_COUNTERS_SGIX
        GL_PATH_JOIN_STYLE_NV
        GL_COLOR_ATTACHMENT7
        GL_LUMINANCE16
        GLEW_AMD_performance_monitor
        GL_INDEX_ARRAY_POINTER
        GL_APPLE_element_array
        GL_MAX_COMPUTE_FIXED_GROUP_INVOCATIONS_ARB
        GL_FOG_LAYERS_POINTS_SGIX
        GLEW_KHR_blend_equation_advanced
        GL_PROXY_TEXTURE_RECTANGLE_EXT
        GL_MODULATE_ADD_ATI
        GL_ROUNDED_RECT8_NV
        GLEW_ATI_texture_mirror_once
        GL_SAMPLE_ALPHA_TO_ONE_ARB
        GLEW_ARB_shader_atomic_counters
        GL_MIN_SAMPLE_SHADING_VALUE_ARB
        GL_QCOM_extended_get
        GL_WEIGHT_ARRAY_SIZE_ARB
        GL_UNPACK_COMPRESSED_BLOCK_DEPTH
        GLEW_ARB_fragment_shader
        GL_MAX_TEXTURE_LOD_BIAS
        GL_TEXTURE_COMPRESSED_IMAGE_SIZE
        GL_MAX_PROGRAM_TEXTURE_GATHER_COMPONENTS
        GL_FOG_COORDINATE
        GL_TEXTURE7_ARB
        GL_LINEAR
        GL_MAX_PROGRAM_TEXEL_OFFSET_NV
        GL_MAX_SHININESS_NV
        GL_VERTEX_ATTRIB_ARRAY6_NV
        GL_MAP1_VERTEX_ATTRIB11_4_NV
        GL_ATIX_point_sprites
        GL_NV_viewport_array
        GL_BUMP_NUM_TEX_UNITS_ATI
        GL_PIXEL_PACK_BUFFER
        GLEW_ARB_texture_mirror_clamp_to_edge
        GL_SHADER_IMAGE_ACCESS_BARRIER_BIT
        GLEW_SGIX_packed_6bytes
        GL_MATRIX7_ARB
        GL_FONT_MAX_ADVANCE_WIDTH_BIT_NV
        GL_DST_NV
        GL_ADD_SIGNED_EXT
        GL_CLIP_DISTANCE5_APPLE
        GL_MAX_SAMPLES_ANGLE
        GL_SGIX_ycrcb
        GL_FRONT_FACE_COMMAND_NV
        GLEW_APPLE_ycbcr_422
        GL_BOOL_VEC4
        GL_UNSIGNED_INT_IMAGE_1D_ARRAY
        GL_DEBUG_TYPE_PERFORMANCE
        GL_DEBUG_CALLBACK_FUNCTION_ARB
        GL_SAMPLES_PASSED_ARB
        GL_MAP1_VERTEX_ATTRIB0_4_NV
        GL_HALF_BIAS_NORMAL_NV
        GLEW_ARB_vertex_attrib_64bit
        GL_TEXTURE26
        GL_NV_read_stencil
        GL_LAST_VERTEX_CONVENTION_EXT
        GL_BUMP_TEX_UNITS_ATI
        GL_LOWER_LEFT
        GL_FRONT_AND_BACK
        GL_INT_SAMPLER_2D_RECT
        GL_EXT_color_subtable
        GL_PACK_SKIP_IMAGES
        GL_SLUMINANCE
        GL_FLOAT_MAT4x2_NV
        GLEW_EXT_multiple_textures
        GL_OUTPUT_TEXTURE_COORD18_EXT
        GL_MAX_DEBUG_GROUP_STACK_DEPTH
        GL_FILE_NAME_NV
        GL_CURRENT_RASTER_TEXTURE_COORDS
        GL_PACK_IMAGE_HEIGHT
        GL_MAX_DRAW_BUFFERS_ARB
        GL_DEPENDENT_GB_TEXTURE_2D_NV
        GL_SAMPLER_1D_ARRAY_SHADOW_EXT
        GL_OUTPUT_TEXTURE_COORD28_EXT
        GLEW_OML_resample
        GL_RGBA_FLOAT_MODE_ARB
        GL_SELECTED_TEXTURE_COORD_SET_SGIS
        GL_ALLOW_DRAW_FRG_HINT_PGI
        GL_DECODE_EXT
        GL_EVAL_VERTEX_ATTRIB5_NV
        GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB
        GL_IMAGE_FORMAT_COMPATIBILITY_BY_CLASS
        GL_PIXEL_MAP_S_TO_S_SIZE
        GL_COLOR_ATTACHMENT0_EXT
        GL_NV_read_depth
        GL_RGBA8UI
        GL_TRANSPOSE_COLOR_MATRIX
        GL_SGIS_texture_border_clamp
        GL_FOG
        GL_2PASS_1_SGIS
        GL_R1UI_T2F_C4F_N3F_V3F_SUN
        GLEW_EXT_shader_io_blocks
        GLEW_ARB_vertex_type_2_10_10_10_rev
        GL_VIRTUAL_PAGE_SIZE_Y_AMD
        GL_VIDEO_BUFFER_INTERNAL_FORMAT_NV
        GL_LAYOUT_DEPTH_STENCIL_ATTACHMENT_EXT
        GL_VERTEX_SHADER_OPTIMIZED_EXT
        GLEW_ARB_vertex_type_10f_11f_11f_rev
        GL_ATOMIC_COUNTER_BARRIER_BIT
        GL_T2F_C4UB_V3F
        GLEW_SGIS_generate_mipmap
        GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM_EXT
        GL_ATI_draw_buffers
        GL_MAT_DIFFUSE_BIT_PGI
        GL_SGIX_line_quality_hint
        GL_OP_MAX_EXT
        GLEW_NV_texture_env_combine4
        GL_4PASS_2_SGIS
        GL_LARGE_CW_ARC_TO_NV
        GL_VIDEO_CAPTURE_SURFACE_ORIGIN_NV
        GL_FRONT_LEFT
        GLEW_ATI_pn_triangles
        GL_PATH_COMPUTED_LENGTH_NV
        GL_MATRIX0_ARB
        GL_DEPTH_CLAMP_NEAR_AMD
        GL_TEXTURE_COORD_ARRAY_TYPE
        GL_TEXTURE_COMPARE_MODE_ARB
        GL_QCOM_alpha_test
        GLEW_NVX_conditional_render
        GL_ARB_shader_viewport_layer_array
        GL_COMBINER_INPUT_NV
        GL_APPLE_texture_packed_float
        GL_PIXEL_UNPACK_BUFFER
        GL_DOUBLE_VEC4_EXT
        GL_DUDV_ATI
        GL_RASTER_MULTISAMPLE_EXT
        GL_DEBUG_CATEGORY_APPLICATION_AMD
        GL_ARB_vertex_blend
        GLEW_VERSION_MAJOR
        GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
        GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR
        GL_COMBINER_SCALE_NV
        GLEW_QCOM_tiled_rendering
        GL_NUM_SPARSE_LEVELS_EXT
        GLEW_EXT_texture_compression_astc_decode_mode
        GL_WAIT_FAILED_APPLE
        GL_PROGRAM_TEX_INDIRECTIONS_ARB
        GL_FONT_UNAVAILABLE_NV
        GL_COMPRESSED_RGBA_S3TC_DXT3_ANGLE
        GL_GEOMETRY_VERTICES_OUT_ARB
        GL_RGB16I
        GL_INT_IMAGE_2D_MULTISAMPLE
        GL_QUERY_DEPTH_FAIL_EVENT_BIT_AMD
        GL_ALPHA16_EXTENDED_RANGE_SGIX
        GL_ARRAY_BUFFER_ARB
        GL_SMOOTH_POINT_SIZE_RANGE
        GL_COLOR_INDEX16_EXT
        GL_TEXTURE23_ARB
        GL_EXT_shader_pixel_local_storage
        GL_BALI_R_CHIP_COUNT
        GL_SAMPLE_LOCATION_ARB
        GLEW_NV_explicit_attrib_location
        GL_SIGNALED_APPLE
        GL_RGBA2
        GL_ALPHA8I_EXT
        GL_TRANSLATE_Y_NV
        GLEW_ANGLE_depth_texture
        GL_OUTPUT_TEXTURE_COORD30_EXT
        GL_4D_COLOR_TEXTURE
        GL_EXT_texture_compression_dxt1
        GL_TEXTURE_BUFFER_DATA_STORE_BINDING
        GL_VALIDATE_STATUS
        GLEW_EXT_post_depth_coverage
        GL_T4F_C4F_N3F_V4F
        GL_TEXTURE_1D_BINDING_EXT
        GL_PROGRAMMABLE_SAMPLE_LOCATION_TABLE_SIZE_ARB
        GLEW_EGL_NV_robustness_video_memory_purge
        GL_ALL_ATTRIB_BITS
        GLEW_REGAL_proc_address
        GL_COPY_WRITE_BUFFER_BINDING
        GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_EXT
        GL_TEXTURE_2D_MULTISAMPLE_ARRAY
        GL_VERTEX_ARRAY_LIST_IBM
        GLEW_NV_texture_shader
        GLEW_EXT_bindable_uniform
        GL_PACK_SWAP_BYTES
        GL_MAX_IMAGE_SAMPLES_EXT
        GL_DRAW_BUFFER9_ATI
        GL_TEXTURE_SPARSE_ARB
        GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
        GL_NUM_GENERAL_COMBINERS_NV
        GLEW_NVX_linked_gpu_multicast
        GLEW_NV_multisample_filter_hint
        GL_MAX_VERTEX_SHADER_INVARIANTS_EXT
        GL_VERTEX_SHADER_BINDING_EXT
        GL_COMPRESSED_R11_EAC
        GL_MIRROR_CLAMP_ATI
        GLEW_EXT_multisampled_render_to_texture2
        GL_FRAGMENT_COVERAGE_COLOR_NV
        GL_TEXTURE16_ARB
        GL_SYNC_STATUS_APPLE
        GL_COPY
        GL_PROGRAM_PARAMETERS_ARB
        GL_INT8_VEC4_NV
        GL_COUNTER_RANGE_AMD
        GLEW_AMD_debug_output
        GL_STENCIL_INDEX1_EXT
        GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB
        GL_MAX_BINDABLE_UNIFORM_SIZE_EXT
        GL_COMPRESSED_SLUMINANCE_EXT
        GLEW_VERSION_4_1
        GLEW_APPLE_row_bytes
        GL_SECONDARY_COLOR_ARRAY
        GL_LUMINANCE6_ALPHA2
        GL_OPERAND0_RGB_EXT
        GL_IGNORE_BORDER
        GL_APPLE_rgb_422
        GL_ATI_envmap_bumpmap
        GL_MULTISAMPLE_BUFFER_BIT0_QCOM
        GL_CONVOLUTION_WIDTH
        GL_WARP_SIZE_NV
        GL_LINEAR_TILING_EXT
        GLEW_EXT_texture_compression_dxt1
        GL_OP_FLOOR_EXT
        GL_COLOR_ATTACHMENT11_NV
        GL_BLEND_SRC_RGB
        GL_RGB4_EXT
        GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT
        GL_STENCIL_BACK_FUNC_ATI
        GL_INT8_VEC3_NV
        GL_EXT_paletted_texture
        GL_EXT_vertex_weighting
        GL_STENCIL_FAIL
        GL_MAX_PROGRAM_LOOP_COUNT_NV
        GL_DRAW_BUFFER11_ATI
        GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM
        GL_NV_vertex_array_range2
        GL_TRANSLATED_SHADER_SOURCE_LENGTH_ANGLE
        GL_ATI_vertex_array_object
        GL_DEBUG_CATEGORY_API_ERROR_AMD
        GLEW_EXT_depth_bounds_test
        GL_MAX_COMBINED_DIMENSIONS
        GL_EXT_transform_feedback
        GL_CUBIC_CURVE_TO_NV
        GL_SHADE_MODEL
        GL_COMPRESSED_ALPHA
        GL_KHR_parallel_shader_compile
        GL_LINE_TO_NV
        GL_SIGNED_RGBA8_NV
        GL_SGIX_vector_ops
        GL_COLOR_ATTACHMENT1_NV
        GL_FFT_1D_SGI
        GL_FONT_Y_MAX_BOUNDS_BIT_NV
        GL_SUN_mesh_array
        GL_WARPS_PER_SM_NV
        GL_UNKNOWN_CONTEXT_RESET_ARB
        GL_EVAL_VERTEX_ATTRIB0_NV
        GL_PATH_OBJECT_BOUNDING_BOX_NV
        GL_POLYGON_OFFSET_EXT
        GL_SGIS_fog_function
        GL_SMALL_CW_ARC_TO_NV
        GL_QUERY_RESULT_AVAILABLE
        GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB
        GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION_EXT
        GL_MAX_ARRAY_TEXTURE_LAYERS_EXT
        GL_LOG_DEBUG_REGAL
        GL_COLOR_ENCODING
        GL_GENERATE_MIPMAP_SGIS
        GL_EVAL_VERTEX_ATTRIB15_NV
        GL_LUMINANCE32UI_EXT
        GL_UNSIGNED_INT_SAMPLER_CUBE_EXT
        GL_ALPHA_TO_COVERAGE_DITHER_DISABLE_NV
        GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV
        GLEW_PGI_vertex_hints
        GL_PROGRAM_BINARY_RETRIEVABLE_HINT
        GL_PRIMITIVE_BOUNDING_BOX_ARB
        GL_DEBUG_TYPE_ERROR
        GL_PURGEABLE_APPLE
        GL_BLEND_ADVANCED_COHERENT_NV
        GL_EXT_map_buffer_range
        GL_SAMPLE_MASK_VALUE_SGIS
        GLEW_NV_viewport_array2
        GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS
        GL_TEXTURE_DEPTH_EXT
        GL_TEXTURE_ENV_MODE
        GLEW_NV_blend_equation_advanced_coherent
        GL_R11F_G11F_B10F_NV
        GL_SECONDARY_COLOR_ARRAY_EXT
        GL_ARB_indirect_parameters
        GL_REDUCE_EXT
        GL_8X_BIT_ATI
        GL_NV_sample_locations
        GL_IMAGE_CLASS_4_X_16
        GL_TEXTURE_2D_ARRAY
        GL_ARB_texture_query_lod
        GL_BLEND_EQUATION
        GL_EXT_compiled_vertex_array
        GL_FONT_DESCENDER_BIT_NV
        GL_LIGHT31
        GL_IMAGE_BINDING_LAYERED_EXT
        GL_LINK_STATUS
        GLEW_EXT_external_buffer
        GL_PERTURB_EXT
        GL_YCBAYCR8A_4224_NV
        GL_NUM_FILL_STREAMS_NV
        GL_ARB_shader_atomic_counters
        GL_MAP_COLOR
        GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT
        GL_BACK_LEFT
        GL_SYNC_CONDITION_APPLE
        GLEW_ARB_depth_buffer_float
        GL_GEOMETRY_SHADER_INVOCATIONS
        GL_SGIX_fog_blend
        GL_SECONDARY_COLOR_ATIX
        GLEW_NV_copy_image
        GL_COLOR_ATTACHMENT15
        GL_PROXY_TEXTURE_2D_ARRAY
        GL_3D_COLOR
        GL_ONE_MINUS_CONSTANT_ALPHA
        GL_DRAW_BUFFER12_ARB
        GL_HISTOGRAM_BLUE_SIZE
        GL_UNIFORM_TYPE
        GL_VERTEX_ARRAY_BUFFER_BINDING_ARB
        GL_FRAGMENT_COLOR_MATERIAL_FACE_EXT
        GL_OUTPUT_TEXTURE_COORD1_EXT
        GL_VIBRANCE_SCALE_NV
        GL_BALI_NUM_TRIS_CULLED_INSTRUMENT
        GL_ALPHA_MIN_SGIX
        GLEW_EXT_semaphore
        GL_LINE_STRIP
        GLEW_AMD_gcn_shader
        GL_MODULATE_SUBTRACT_ATIX
        GL_FEEDBACK_BUFFER_SIZE
        GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE
        GL_MINOR_VERSION
        GL_LINE_STIPPLE_PATTERN
        GL_BGR_INTEGER_EXT
        GL_APPLE_aux_depth_stencil
        GL_2D
        GL_HANDLE_TYPE_D3D11_IMAGE_KMT_EXT
        GL_ACTIVE_SUBROUTINE_UNIFORM_MAX_LENGTH
        GL_FLOAT_R16_NV
        GL_IMAGE_CLASS_4_X_32
        GL_SYNC_CL_EVENT_COMPLETE_ARB
        GL_NV_texture_rectangle_compressed
        GL_COORD_REPLACE_ARB
        GL_LOCATION_INDEX_EXT
        GLEW_NV_texture_border_clamp
        GL_AND
        GL_FULL_RANGE_EXT
        GL_SRC1_RGB
        GL_RELATIVE_CONIC_CURVE_TO_NV
        GL_NONE
        GL_INT_2_10_10_10_REV
        GL_MIN_LOD_WARNING_AMD
        GL_PATH_STENCIL_FUNC_NV
        GL_POLYGON_MODE
        GL_RGB32UI
        GL_EXT_shader_io_blocks
        GL_DOT3_ATI
        GL_SCALE_BY_ONE_HALF_NV
        GL_RELATIVE_CUBIC_CURVE_TO_NV
        GL_ATI_vertex_attrib_array_object
        GL_BLEND_EQUATION_RGB
        GL_COLOR_TABLE_INTENSITY_SIZE_SGI
        GLEW_APPLE_texture_format_BGRA8888
        GL_SGIX_flush_raster
        GL_MAX_GREEN_SGIS
        GL_T2F_C3F_V3F
        GLEW_AMD_vertex_shader_layer
        GL_RGBA_FLOAT16_ATI
        GL_POSITION
        GL_RGBA4_DXT5_S3TC
        GL_TESS_GEN_POINT_MODE
        GL_VIEW_COMPATIBILITY_CLASS
        GL_OUTPUT_TEXTURE_COORD17_EXT
        GL_CLIP_PLANE4
        GL_MAX_SHADER_PIXEL_LOCAL_STORAGE_SIZE_EXT
        GL_ABGR_EXT
        GL_TEXTURE_2D
        GL_FLOAT16_MAT4_AMD
        GLEW_NV_texture_compression_s3tc
        GL_UNIFORM
        GL_RGB10_EXT
        GL_CULL_VERTEX_IBM
        GL_ALWAYS_FAST_HINT_PGI
        GL_COMPRESSED_RGBA8_ETC2_EAC
        GLEW_NV_fill_rectangle
        GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_EXT
        GL_FRACTIONAL_EVEN
        GL_LAYOUT_TRANSFER_SRC_EXT
        GL_ARB_fragment_layer_viewport
        GL_CON_3_ATI
        GL_NV_copy_buffer
        GL_AMD_gcn_shader
        GL_TEXTURE_BORDER_COLOR
        GL_COMPRESSED_RGBA_ASTC_8x6_KHR
        GL_VARIANT_VALUE_EXT
        GL_PATH_FORMAT_PS_NV
        GL_INT_SAMPLER_2D_MULTISAMPLE
        GL_NV_conservative_raster
        GL_COMBINER_AB_OUTPUT_NV
        GL_COMPRESSED_RG_RGTC2
        GL_TEXTURE_DEPTH_TYPE
        GL_PROXY_TEXTURE_3D
        GL_NV_copy_image
        GL_VERTEX_SHADER_EXT
        GLEW_APPLE_texture_max_level
        GL_FOG_COORDINATE_ARRAY_TYPE_EXT
        GLEW_HP_occlusion_test
        GL_ATTRIB_ARRAY_STRIDE_NV
        GL_TRANSFORM_FEEDBACK_STREAM_OVERFLOW_ARB
        GL_FRAGMENT_SUBROUTINE
        GL_COPY_READ_BUFFER_NV
        GL_EXT_texture_compression_latc
        GL_Z400_BINARY_AMD
        GL_VERTEX_BLEND_ARB
        GL_TEXTURE0_SGIS
        GL_FRAGMENT_SHADER_ARB
        GL_NV_bindless_texture
        GL_OFFSET_TEXTURE_BIAS_NV
        GL_PROGRAM_SEPARABLE
        GL_TEXTURE18
        GL_FRAMEBUFFER_PROGRAMMABLE_SAMPLE_LOCATIONS_NV
        GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT
        GL_NORMALIZED_RANGE_EXT
        GL_SAMPLER
        GL_MODELVIEW1_EXT
        GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT
        GL_RGB8I
        GL_COMPRESSED_RGBA_S3TC_DXT5_NV
        GL_EXT_stencil_two_side
        GL_PATH_COMMAND_COUNT_NV
        GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV
        GL_SAMPLER_BUFFER
        GLEW_ANDROID_extension_pack_es31a
        GLEW_ARB_shadow
        GL_FLOAT_MAT3_ARB
        GLEW_ARB_shader_precision
        GLEW_ERROR_GL_VERSION_10_ONLY
        GL_FRONT_RIGHT
        GL_RG32I
        GL_MATRIX29_ARB
        GL_SAMPLER_2D_SHADOW_ARB
        GL_SLUMINANCE8_NV
        GL_MIN
        GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB
        GL_WRAP_BORDER_SUN
        GL_STORAGE_SHARED_APPLE
        GL_POLYGON_OFFSET_FILL
        GL_ALPHA_TEST_FUNC_QCOM
        GL_LINE_BIT
        GL_MAX_ARRAY_TEXTURE_LAYERS_NV
        GL_MODELVIEW1_ARB
        GL_BLEND_DST_ALPHA_EXT
        GL_LARGE_CCW_ARC_TO_NV
        GL_TILING_TYPES_EXT
        GL_RENDERBUFFER_EXT
        GLEW_EXT_texture_norm16
        GL_RETAINED_APPLE
        GL_INTENSITY32F_ARB
        GL_T2F_N3F_V3F
        GL_UNSIGNED_INT_IMAGE_BUFFER
        GL_EXT_fragment_lighting
        GL_MIN_RED_SGIS
        GL_NUM_FRAGMENT_CONSTANTS_ATI
        GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
        GL_NV_texgen_emboss
        GLEW_ARB_vertex_buffer_object
        GLEW_WIN_phong_shading
        GL_EXT_secondary_color
        GL_PREVIOUS_TEXTURE_INPUT_NV
        GL_ONE_MINUS_SRC1_COLOR_EXT
        GLEW_HP_image_transform
        GL_CLIENT_VERTEX_ARRAY_BIT
        GL_FORCE_BLUE_TO_ONE_NV
        GL_UNSIGNED_INT64_ARB
        GL_SAMPLER_BUFFER_AMD
        GL_DEBUG_CATEGORY_WINDOW_SYSTEM_AMD
        GL_LUMINANCE8I_EXT
        GL_EXT_semaphore_fd
        GL_LEQUAL
        GL_DRAW_FRAMEBUFFER_EXT
        GL_STENCIL_BUFFER_BIT6_QCOM
        GL_DRAW_BUFFER2_EXT
        GL_FILTER
        GL_COMPRESSED_RGBA_BPTC_UNORM
        GL_UNIFORM_BUFFER_EXT
        GL_UNIFORM_BARRIER_BIT
        GL_UNSIGNED_INT_10F_11F_11F_REV_NV
        GL_SHADER_BINARY_FORMATS
        GL_HI_SCALE_NV
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR
        GL_DRAW_BUFFER9_NV
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR
        GL_EXT_draw_range_elements
        GL_PIXEL_PACK_BUFFER_BINDING_NV
        GL_MATRIX30_ARB
        GL_VIEW_CLASS_24_BITS
        GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM_ARB
        GL_TEXTURE_DT_SIZE_NV
        GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY
        GL_POINT_DISTANCE_ATTENUATION_ARB
        GL_DEBUG_LOGGED_MESSAGES
        GL_ACTIVE_PROGRAM
        GL_COLOR_TABLE_SCALE
        GLEW_QCOM_writeonly_rendering
        GL_EXT_rescale_normal
        GL_VIDEO_BUFFER_BINDING_NV
        GL_IMAGE_CLASS_2_X_32
        GL_SGIS_texture_edge_clamp
        GL_STENCIL_BUFFER_BIT5_QCOM
        GLEW_SGIX_vector_ops
        GL_VARIABLE_F_NV
        GLEW_NV_vertex_program
        GL_MAP1_VERTEX_ATTRIB8_4_NV
        GLEW_SGIX_quad_mesh
        GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_ARB
        GL_FLOAT_RG16_NV
        GL_TEXTURE_INTERNAL_FORMAT
        GL_COPY_INVERTED
        GL_POST_COLOR_MATRIX_RED_BIAS
        GL_TEXTURE_LUMINANCE_SIZE_EXT
        GL_ARB_shader_texture_lod
        GL_INT_SAMPLER_CUBE_MAP_ARRAY_ARB
        GL_COLOR_ATTACHMENT6_NV
        GL_CLIP_DISTANCE6_APPLE
        GL_STENCIL_RENDERABLE
        GL_DEPTH_STENCIL_TEXTURE_MODE
        GL_MODULATE_SIGNED_ADD_ATIX
        GL_MAX_INTENSITY_SGIS
        GL_PROGRAM_TEMPORARIES_ARB
        GL_TEXTURE_FILTER_CONTROL
        GL_SRC_OUT_NV
        GL_ONE
        GL_ACTIVE_VARYING_MAX_LENGTH_NV
        GL_READ_ONLY
        GL_COLOR_TABLE_LUMINANCE_SIZE
        GL_RENDERBUFFER_FREE_MEMORY_ATI
        GL_EXT_depth_bounds_test
        GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS
        GL_MAP2_VERTEX_ATTRIB10_4_NV
        GL_FOG_COORD_ARRAY_LENGTH_NV
        GL_PROGRAM_BINARY_LENGTH
        GL_EVAL_TRIANGULAR_2D_NV
        GL_EXT_multisampled_render_to_texture2
        GL_APPLE_float_pixels
        GL_INT_SAMPLER_2D
        GL_TEXTURE1_SGIS
        GL_QCOM_tiled_rendering
        GL_SUN_global_alpha
        GLEW_EXT_blend_func_separate
        GL_TEXTURE_WRAP_S
        GL_MAX_UNIFORM_LOCATIONS
        GL_PATH_GEN_COLOR_FORMAT_NV
        GL_MULTISAMPLE_3DFX
        GL_RGB8_EXT
        GL_INT_SAMPLER_BUFFER_AMD
        GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI
        GL_ARB_multitexture
        GL_LUMINANCE8_ALPHA8_EXT
        GL_EXT_framebuffer_multisample
        GL_GEOMETRY_LINKED_OUTPUT_TYPE_EXT
        GLEW_EXT_draw_elements_base_vertex
        GL_DIFFUSE
        GL_REGAL_error_string
        GLEW_AMD_compressed_3DC_texture
        GL_POST_CONVOLUTION_BLUE_SCALE_EXT
        GLEW_SGIX_fragment_specular_lighting
        GL_DEPTH_FUNC
        GL_PIXEL_UNPACK_BUFFER_EXT
        GL_MATRIX31_ARB
        GL_VERTEX_ARRAY_RANGE_APPLE
        GL_UNSIGNED_INT_SAMPLER_1D_EXT
        GL_DST_OUT_NV
        GL_FACTOR_MAX_AMD
        GL_TEXCOORD3_BIT_PGI
        GL_DRAW_ARRAYS_STRIP_COMMAND_NV
        GLEW_VERSION_MINOR
        GL_TEXTURE_BINDING_CUBE_MAP_EXT
        GL_PACK_SKIP_ROWS
        GL_PRIMARY_COLOR_ARB
        GL_VERTEX_SHADER_BIT
        GL_TEXTURE_BUFFER_FORMAT_ARB
        GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI
        GL_TESSELLATION_FACTOR_AMD
        GL_NV_conditional_render
        GL_EXT_packed_pixels
        GL_RELATIVE_ARC_TO_NV
        GL_SUBTRACT
        GL_COUNTER_TYPE_AMD
        GL_ARB_texture_buffer_range
        GL_NEGATE_BIT_ATI
        GL_DEBUG_GROUP_STACK_DEPTH
        GL_FRAMEBUFFER_SRGB_CAPABLE_EXT
        GL_BLUE_BITS
        GL_INDEX_OFFSET
        GL_ARB_transform_feedback3
        GL_POST_CONVOLUTION_RED_SCALE_EXT
        GL_MESA_pack_invert
        GL_INTENSITY_SIGNED_SGIX
        GL_MAX_SHADER_PIXEL_LOCAL_STORAGE_FAST_SIZE_EXT
        GL_CONTRAST_NV
        GL_PRIMITIVE_RESTART_FOR_PATCHES_SUPPORTED
        GLEW_EXT_blend_color
        GL_DRAW_FRAMEBUFFER_BINDING_ANGLE
        GL_SURFACE_REGISTERED_NV
        GL_SATURATE_BIT_ATI
        GL_DRAW_BUFFER11_EXT
        GL_LIGHT0
        GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS
        GL_EXT_index_texture
        GL_R11F_G11F_B10F_APPLE
        GL_QUERY_NO_WAIT_INVERTED
        GL_BGR_EXT
        GL_RENDERBUFFER_SAMPLES
        GL_COLOR_TABLE_FORMAT
        GL_OPERAND0_ALPHA_EXT
        GL_EXT_texture_type_2_10_10_10_REV
        GL_ARRAY_STRIDE
        GL_PATH_GEN_COMPONENTS_NV
        GL_STENCIL_BUFFER_BIT
        GL_BLUE_INTEGER
        GL_DISJOINT_NV
        GL_COMPRESSED_RGBA_S3TC_DXT5_ANGLE
        GL_COMPUTE_SUBROUTINE_UNIFORM
        GLEW_NV_texture_expand_normal
        GL_EXT_draw_instanced
        GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS
        GL_TEXTURE_VIEW_NUM_LEVELS
        GL_IMAGE_FORMAT_COMPATIBILITY_BY_SIZE
        GL_PIXEL_SUBSAMPLE_4242_SGIX
        GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV
        GL_READ_FRAMEBUFFER_NV
        GL_ALPHA12_EXT
        GL_EVAL_VERTEX_ATTRIB13_NV
        GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR
        GL_SHADER
        GL_OBJECT_LINK_STATUS_ARB
        GL_MAP_INVALIDATE_BUFFER_BIT
        GL_MAX_VERTEX_ATOMIC_COUNTERS
        GL_MODELVIEW7_ARB
        GL_UNSIGNED_INT_IMAGE_2D_ARRAY
        GL_EVAL_VERTEX_ATTRIB14_NV
        GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH
        GL_LIGHT_MODEL_TWO_SIDE
        GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT
        GL_COLOR_ATTACHMENT10_EXT
        GL_R8I
        GLEW_NV_path_rendering
        GL_READ_PIXEL_DATA_RANGE_POINTER_NV
        GL_DRAW_FRAMEBUFFER_BINDING_EXT
        GL_PROGRAM_NATIVE_TEMPORARIES_ARB
        GL_SOURCE2_RGB_EXT
        GL_OUTPUT_TEXTURE_COORD12_EXT
        GL_NV_gpu_multicast
        GL_DRAW_BUFFER2_ATI
        GL_SIGNED_IDENTITY_NV
        GL_PASS_THROUGH_TOKEN
        GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV
        GL_ATTRIB_ARRAY_POINTER_NV
        GL_TEXTURE27_ARB
        GL_MAX_EXT
        GL_COLOR_ARRAY_POINTER_EXT
        GLEW_ARB_sparse_texture2
        GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB
        GL_DEPTH24_STENCIL8_EXT
        GL_LUMINANCE4
        GL_TEXTURE_COMPARE_MODE
        GL_FOG_DENSITY
        GL_UNPACK_LSB_FIRST
        GL_RESCALE_NORMAL_EXT
        GL_DRAW_BUFFER13_ARB
        GL_TEXTURE_SHARED_SIZE
        GL_ARB_framebuffer_object
        GL_CONVOLUTION_SGI
        GLEW_EXT_texture_sRGB_RG8
        GL_DEPTH_BITS
        GL_MAX_3D_TEXTURE_SIZE_EXT
        GL_DEPTH_BUFFER_BIT4_QCOM
        GL_INTERNALFORMAT_GREEN_TYPE
        GL_RGBA16_SIGNED_SGIX
        GL_DEBUG_CALLBACK_USER_PARAM
        GL_MAP2_TEXTURE_COORD_4_NURBS_SGIX
        GL_COMPILE_AND_EXECUTE

    void glCopyTexSubImage3D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, void *indices)
    void glTexImage3D(GLenum target, GLint level, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, void *pixels)
    void glTexSubImage3D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    void glActiveTexture(GLenum texture)
    void glClientActiveTexture(GLenum texture)
    void glCompressedTexImage1D(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexImage2D(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexImage3D(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexSubImage1D(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTexSubImage3D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glGetCompressedTexImage(GLenum target, GLint lod, void *img)
    void glLoadTransposeMatrixd(GLdouble m[16])
    void glLoadTransposeMatrixf(GLfloat m[16])
    void glMultTransposeMatrixd(GLdouble m[16])
    void glMultTransposeMatrixf(GLfloat m[16])
    void glMultiTexCoord1d(GLenum target, GLdouble s)
    void glMultiTexCoord1dv(GLenum target, GLdouble *v)
    void glMultiTexCoord1f(GLenum target, GLfloat s)
    void glMultiTexCoord1fv(GLenum target, GLfloat *v)
    void glMultiTexCoord1i(GLenum target, GLint s)
    void glMultiTexCoord1iv(GLenum target, GLint *v)
    void glMultiTexCoord1s(GLenum target, GLshort s)
    void glMultiTexCoord1sv(GLenum target, GLshort *v)
    void glMultiTexCoord2d(GLenum target, GLdouble s, GLdouble t)
    void glMultiTexCoord2dv(GLenum target, GLdouble *v)
    void glMultiTexCoord2f(GLenum target, GLfloat s, GLfloat t)
    void glMultiTexCoord2fv(GLenum target, GLfloat *v)
    void glMultiTexCoord2i(GLenum target, GLint s, GLint t)
    void glMultiTexCoord2iv(GLenum target, GLint *v)
    void glMultiTexCoord2s(GLenum target, GLshort s, GLshort t)
    void glMultiTexCoord2sv(GLenum target, GLshort *v)
    void glMultiTexCoord3d(GLenum target, GLdouble s, GLdouble t, GLdouble r)
    void glMultiTexCoord3dv(GLenum target, GLdouble *v)
    void glMultiTexCoord3f(GLenum target, GLfloat s, GLfloat t, GLfloat r)
    void glMultiTexCoord3fv(GLenum target, GLfloat *v)
    void glMultiTexCoord3i(GLenum target, GLint s, GLint t, GLint r)
    void glMultiTexCoord3iv(GLenum target, GLint *v)
    void glMultiTexCoord3s(GLenum target, GLshort s, GLshort t, GLshort r)
    void glMultiTexCoord3sv(GLenum target, GLshort *v)
    void glMultiTexCoord4d(GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q)
    void glMultiTexCoord4dv(GLenum target, GLdouble *v)
    void glMultiTexCoord4f(GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q)
    void glMultiTexCoord4fv(GLenum target, GLfloat *v)
    void glMultiTexCoord4i(GLenum target, GLint s, GLint t, GLint r, GLint q)
    void glMultiTexCoord4iv(GLenum target, GLint *v)
    void glMultiTexCoord4s(GLenum target, GLshort s, GLshort t, GLshort r, GLshort q)
    void glMultiTexCoord4sv(GLenum target, GLshort *v)
    void glSampleCoverage(GLclampf value, GLboolean invert)
    void glBlendColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    void glBlendEquation(GLenum mode)
    void glBlendFuncSeparate(GLenum sfactorRGB, GLenum dfactorRGB, GLenum sfactorAlpha, GLenum dfactorAlpha)
    void glFogCoordPointer(GLenum type, GLsizei stride, void *pointer)
    void glFogCoordd(GLdouble coord)
    void glFogCoorddv(GLdouble *coord)
    void glFogCoordf(GLfloat coord)
    void glFogCoordfv(GLfloat *coord)
    void glMultiDrawArrays(GLenum mode, GLint *first, GLsizei *count, GLsizei drawcount)
    void glMultiDrawElements(GLenum mode, GLsizei *count, GLenum type, void **indices, GLsizei drawcount)
    void glPointParameterf(GLenum pname, GLfloat param)
    void glPointParameterfv(GLenum pname, GLfloat *params)
    void glPointParameteri(GLenum pname, GLint param)
    void glPointParameteriv(GLenum pname, GLint *params)
    void glSecondaryColor3b(GLbyte red, GLbyte green, GLbyte blue)
    void glSecondaryColor3bv(GLbyte *v)
    void glSecondaryColor3d(GLdouble red, GLdouble green, GLdouble blue)
    void glSecondaryColor3dv(GLdouble *v)
    void glSecondaryColor3f(GLfloat red, GLfloat green, GLfloat blue)
    void glSecondaryColor3fv(GLfloat *v)
    void glSecondaryColor3i(GLint red, GLint green, GLint blue)
    void glSecondaryColor3iv(GLint *v)
    void glSecondaryColor3s(GLshort red, GLshort green, GLshort blue)
    void glSecondaryColor3sv(GLshort *v)
    void glSecondaryColor3ub(GLubyte red, GLubyte green, GLubyte blue)
    void glSecondaryColor3ubv(GLubyte *v)
    void glSecondaryColor3ui(GLuint red, GLuint green, GLuint blue)
    void glSecondaryColor3uiv(GLuint *v)
    void glSecondaryColor3us(GLushort red, GLushort green, GLushort blue)
    void glSecondaryColor3usv(GLushort *v)
    void glSecondaryColorPointer(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glWindowPos2d(GLdouble x, GLdouble y)
    void glWindowPos2dv(GLdouble *p)
    void glWindowPos2f(GLfloat x, GLfloat y)
    void glWindowPos2fv(GLfloat *p)
    void glWindowPos2i(GLint x, GLint y)
    void glWindowPos2iv(GLint *p)
    void glWindowPos2s(GLshort x, GLshort y)
    void glWindowPos2sv(GLshort *p)
    void glWindowPos3d(GLdouble x, GLdouble y, GLdouble z)
    void glWindowPos3dv(GLdouble *p)
    void glWindowPos3f(GLfloat x, GLfloat y, GLfloat z)
    void glWindowPos3fv(GLfloat *p)
    void glWindowPos3i(GLint x, GLint y, GLint z)
    void glWindowPos3iv(GLint *p)
    void glWindowPos3s(GLshort x, GLshort y, GLshort z)
    void glWindowPos3sv(GLshort *p)
    void glBeginQuery(GLenum target, GLuint id)
    void glBindBuffer(GLenum target, GLuint buffer)
    void glBufferData(GLenum target, GLsizeiptr size, void *data, GLenum usage)
    void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, void *data)
    void glDeleteBuffers(GLsizei n, GLuint *buffers)
    void glDeleteQueries(GLsizei n, GLuint *ids)
    void glEndQuery(GLenum target)
    void glGenBuffers(GLsizei n, GLuint *buffers)
    void glGenQueries(GLsizei n, GLuint *ids)
    void glGetBufferParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetBufferPointerv(GLenum target, GLenum pname, void **params)
    void glGetBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, void *data)
    void glGetQueryObjectiv(GLuint id, GLenum pname, GLint *params)
    void glGetQueryObjectuiv(GLuint id, GLenum pname, GLuint *params)
    void glGetQueryiv(GLenum target, GLenum pname, GLint *params)
    GLboolean glIsBuffer(GLuint buffer)
    GLboolean glIsQuery(GLuint id)
    void *glMapBuffer(GLenum target, GLenum access)
    GLboolean glUnmapBuffer(GLenum target)
    void glAttachShader(GLuint program, GLuint shader)
    void glBindAttribLocation(GLuint program, GLuint index, GLchar *name)
    void glBlendEquationSeparate(GLenum modeRGB, GLenum modeAlpha)
    void glCompileShader(GLuint shader)
    GLuint glCreateProgram()
    GLuint glCreateShader(GLenum type)
    void glDeleteProgram(GLuint program)
    void glDeleteShader(GLuint shader)
    void glDetachShader(GLuint program, GLuint shader)
    void glDisableVertexAttribArray(GLuint index)
    void glDrawBuffers(GLsizei n, GLenum *bufs)
    void glEnableVertexAttribArray(GLuint index)
    void glGetActiveAttrib(GLuint program, GLuint index, GLsizei maxLength, GLsizei *length, GLint *size, GLenum *type, GLchar *name)
    void glGetActiveUniform(GLuint program, GLuint index, GLsizei maxLength, GLsizei *length, GLint *size, GLenum *type, GLchar *name)
    void glGetAttachedShaders(GLuint program, GLsizei maxCount, GLsizei *count, GLuint *shaders)
    GLint glGetAttribLocation(GLuint program, GLchar *name)
    void glGetProgramInfoLog(GLuint program, GLsizei bufSize, GLsizei *length, GLchar *infoLog)
    void glGetProgramiv(GLuint program, GLenum pname, GLint *param)
    void glGetShaderInfoLog(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *infoLog)
    void glGetShaderSource(GLuint obj, GLsizei maxLength, GLsizei *length, GLchar *source)
    void glGetShaderiv(GLuint shader, GLenum pname, GLint *param)
    GLint glGetUniformLocation(GLuint program, GLchar *name)
    void glGetUniformfv(GLuint program, GLint location, GLfloat *params)
    void glGetUniformiv(GLuint program, GLint location, GLint *params)
    void glGetVertexAttribPointerv(GLuint index, GLenum pname, void **pointer)
    void glGetVertexAttribdv(GLuint index, GLenum pname, GLdouble *params)
    void glGetVertexAttribfv(GLuint index, GLenum pname, GLfloat *params)
    void glGetVertexAttribiv(GLuint index, GLenum pname, GLint *params)
    GLboolean glIsProgram(GLuint program)
    GLboolean glIsShader(GLuint shader)
    void glLinkProgram(GLuint program)
    void glShaderSource(GLuint shader, GLsizei count, GLchar **string, GLint *length)
    void glStencilFuncSeparate(GLenum frontfunc, GLenum backfunc, GLint ref, GLuint mask)
    void glStencilMaskSeparate(GLenum face, GLuint mask)
    void glStencilOpSeparate(GLenum face, GLenum sfail, GLenum dpfail, GLenum dppass)
    void glUniform1f(GLint location, GLfloat v0)
    void glUniform1fv(GLint location, GLsizei count, GLfloat *value)
    void glUniform1i(GLint location, GLint v0)
    void glUniform1iv(GLint location, GLsizei count, GLint *value)
    void glUniform2f(GLint location, GLfloat v0, GLfloat v1)
    void glUniform2fv(GLint location, GLsizei count, GLfloat *value)
    void glUniform2i(GLint location, GLint v0, GLint v1)
    void glUniform2iv(GLint location, GLsizei count, GLint *value)
    void glUniform3f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2)
    void glUniform3fv(GLint location, GLsizei count, GLfloat *value)
    void glUniform3i(GLint location, GLint v0, GLint v1, GLint v2)
    void glUniform3iv(GLint location, GLsizei count, GLint *value)
    void glUniform4f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
    void glUniform4fv(GLint location, GLsizei count, GLfloat *value)
    void glUniform4i(GLint location, GLint v0, GLint v1, GLint v2, GLint v3)
    void glUniform4iv(GLint location, GLsizei count, GLint *value)
    void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUseProgram(GLuint program)
    void glValidateProgram(GLuint program)
    void glVertexAttrib1d(GLuint index, GLdouble x)
    void glVertexAttrib1dv(GLuint index, GLdouble *v)
    void glVertexAttrib1f(GLuint index, GLfloat x)
    void glVertexAttrib1fv(GLuint index, GLfloat *v)
    void glVertexAttrib1s(GLuint index, GLshort x)
    void glVertexAttrib1sv(GLuint index, GLshort *v)
    void glVertexAttrib2d(GLuint index, GLdouble x, GLdouble y)
    void glVertexAttrib2dv(GLuint index, GLdouble *v)
    void glVertexAttrib2f(GLuint index, GLfloat x, GLfloat y)
    void glVertexAttrib2fv(GLuint index, GLfloat *v)
    void glVertexAttrib2s(GLuint index, GLshort x, GLshort y)
    void glVertexAttrib2sv(GLuint index, GLshort *v)
    void glVertexAttrib3d(GLuint index, GLdouble x, GLdouble y, GLdouble z)
    void glVertexAttrib3dv(GLuint index, GLdouble *v)
    void glVertexAttrib3f(GLuint index, GLfloat x, GLfloat y, GLfloat z)
    void glVertexAttrib3fv(GLuint index, GLfloat *v)
    void glVertexAttrib3s(GLuint index, GLshort x, GLshort y, GLshort z)
    void glVertexAttrib3sv(GLuint index, GLshort *v)
    void glVertexAttrib4Nbv(GLuint index, GLbyte *v)
    void glVertexAttrib4Niv(GLuint index, GLint *v)
    void glVertexAttrib4Nsv(GLuint index, GLshort *v)
    void glVertexAttrib4Nub(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w)
    void glVertexAttrib4Nubv(GLuint index, GLubyte *v)
    void glVertexAttrib4Nuiv(GLuint index, GLuint *v)
    void glVertexAttrib4Nusv(GLuint index, GLushort *v)
    void glVertexAttrib4bv(GLuint index, GLbyte *v)
    void glVertexAttrib4d(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexAttrib4dv(GLuint index, GLdouble *v)
    void glVertexAttrib4f(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glVertexAttrib4fv(GLuint index, GLfloat *v)
    void glVertexAttrib4iv(GLuint index, GLint *v)
    void glVertexAttrib4s(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w)
    void glVertexAttrib4sv(GLuint index, GLshort *v)
    void glVertexAttrib4ubv(GLuint index, GLubyte *v)
    void glVertexAttrib4uiv(GLuint index, GLuint *v)
    void glVertexAttrib4usv(GLuint index, GLushort *v)
    void glVertexAttribPointer(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, void *pointer)
    void glUniformMatrix2x3fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix2x4fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3x2fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3x4fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4x2fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4x3fv(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glBeginConditionalRender(GLuint id, GLenum mode)
    void glBeginTransformFeedback(GLenum primitiveMode)
    void glBindFragDataLocation(GLuint program, GLuint colorNumber, GLchar *name)
    void glClampColor(GLenum target, GLenum clamp)
    void glClearBufferfi(GLenum buffer, GLint drawBuffer, GLfloat depth, GLint stencil)
    void glClearBufferfv(GLenum buffer, GLint drawBuffer, GLfloat *value)
    void glClearBufferiv(GLenum buffer, GLint drawBuffer, GLint *value)
    void glClearBufferuiv(GLenum buffer, GLint drawBuffer, GLuint *value)
    void glColorMaski(GLuint buf, GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha)
    void glDisablei(GLenum cap, GLuint index)
    void glEnablei(GLenum cap, GLuint index)
    void glEndConditionalRender()
    void glEndTransformFeedback()
    void glGetBooleani_v(GLenum pname, GLuint index, GLboolean *data)
    GLint glGetFragDataLocation(GLuint program, GLchar *name)
    GLubyte *glGetStringi(GLenum name, GLuint index)
    void glGetTexParameterIiv(GLenum target, GLenum pname, GLint *params)
    void glGetTexParameterIuiv(GLenum target, GLenum pname, GLuint *params)
    void glGetTransformFeedbackVarying(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name)
    void glGetUniformuiv(GLuint program, GLint location, GLuint *params)
    void glGetVertexAttribIiv(GLuint index, GLenum pname, GLint *params)
    void glGetVertexAttribIuiv(GLuint index, GLenum pname, GLuint *params)
    GLboolean glIsEnabledi(GLenum cap, GLuint index)
    void glTexParameterIiv(GLenum target, GLenum pname, GLint *params)
    void glTexParameterIuiv(GLenum target, GLenum pname, GLuint *params)
    void glTransformFeedbackVaryings(GLuint program, GLsizei count, GLchar **varyings, GLenum bufferMode)
    void glUniform1ui(GLint location, GLuint v0)
    void glUniform1uiv(GLint location, GLsizei count, GLuint *value)
    void glUniform2ui(GLint location, GLuint v0, GLuint v1)
    void glUniform2uiv(GLint location, GLsizei count, GLuint *value)
    void glUniform3ui(GLint location, GLuint v0, GLuint v1, GLuint v2)
    void glUniform3uiv(GLint location, GLsizei count, GLuint *value)
    void glUniform4ui(GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3)
    void glUniform4uiv(GLint location, GLsizei count, GLuint *value)
    void glVertexAttribI1i(GLuint index, GLint v0)
    void glVertexAttribI1iv(GLuint index, GLint *v0)
    void glVertexAttribI1ui(GLuint index, GLuint v0)
    void glVertexAttribI1uiv(GLuint index, GLuint *v0)
    void glVertexAttribI2i(GLuint index, GLint v0, GLint v1)
    void glVertexAttribI2iv(GLuint index, GLint *v0)
    void glVertexAttribI2ui(GLuint index, GLuint v0, GLuint v1)
    void glVertexAttribI2uiv(GLuint index, GLuint *v0)
    void glVertexAttribI3i(GLuint index, GLint v0, GLint v1, GLint v2)
    void glVertexAttribI3iv(GLuint index, GLint *v0)
    void glVertexAttribI3ui(GLuint index, GLuint v0, GLuint v1, GLuint v2)
    void glVertexAttribI3uiv(GLuint index, GLuint *v0)
    void glVertexAttribI4bv(GLuint index, GLbyte *v0)
    void glVertexAttribI4i(GLuint index, GLint v0, GLint v1, GLint v2, GLint v3)
    void glVertexAttribI4iv(GLuint index, GLint *v0)
    void glVertexAttribI4sv(GLuint index, GLshort *v0)
    void glVertexAttribI4ubv(GLuint index, GLubyte *v0)
    void glVertexAttribI4ui(GLuint index, GLuint v0, GLuint v1, GLuint v2, GLuint v3)
    void glVertexAttribI4uiv(GLuint index, GLuint *v0)
    void glVertexAttribI4usv(GLuint index, GLushort *v0)
    void glVertexAttribIPointer(GLuint index, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glDrawArraysInstanced(GLenum mode, GLint first, GLsizei count, GLsizei primcount)
    void glDrawElementsInstanced(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount)
    void glPrimitiveRestartIndex(GLuint buffer)
    void glTexBuffer(GLenum target, GLenum internalFormat, GLuint buffer)
    void glFramebufferTexture(GLenum target, GLenum attachment, GLuint texture, GLint level)
    void glGetBufferParameteri64v(GLenum target, GLenum value, GLint64 *data)
    void glGetInteger64i_v(GLenum pname, GLuint index, GLint64 *data)
    void glVertexAttribDivisor(GLuint index, GLuint divisor)
    void glBlendEquationSeparatei(GLuint buf, GLenum modeRGB, GLenum modeAlpha)
    void glBlendEquationi(GLuint buf, GLenum mode)
    void glBlendFuncSeparatei(GLuint buf, GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha)
    void glBlendFunci(GLuint buf, GLenum src, GLenum dst)
    void glMinSampleShading(GLclampf value)
    GLenum glGetGraphicsResetStatus()
    void glGetnCompressedTexImage(GLenum target, GLint lod, GLsizei bufSize, GLvoid *pixels)
    void glGetnTexImage(GLenum tex, GLint level, GLenum format, GLenum type, GLsizei bufSize, GLvoid *pixels)
    void glGetnUniformdv(GLuint program, GLint location, GLsizei bufSize, GLdouble *params)
    void glMultiDrawArraysIndirectCount(GLenum mode, GLvoid *indirect, GLintptr drawcount, GLsizei maxdrawcount, GLsizei stride)
    void glMultiDrawElementsIndirectCount(GLenum mode, GLenum type, GLvoid *indirect, GLintptr drawcount, GLsizei maxdrawcount, GLsizei stride)
    void glSpecializeShader(GLuint shader, GLchar *pEntryPoint, GLuint numSpecializationConstants, GLuint *pConstantIndex, GLuint *pConstantValue)
    void glTbufferMask3DFX(GLuint mask)
    void glDebugMessageCallbackAMD(GLDEBUGPROCAMD callback, void *userParam)
    void glDebugMessageEnableAMD(GLenum category, GLenum severity, GLsizei count, GLuint *ids, GLboolean enabled)
    void glDebugMessageInsertAMD(GLenum category, GLenum severity, GLuint id, GLsizei length, GLchar *buf)
    GLuint glGetDebugMessageLogAMD(GLuint count, GLsizei bufsize, GLenum *categories, GLuint *severities, GLuint *ids, GLsizei *lengths, GLchar *message)
    void glBlendEquationIndexedAMD(GLuint buf, GLenum mode)
    void glBlendEquationSeparateIndexedAMD(GLuint buf, GLenum modeRGB, GLenum modeAlpha)
    void glBlendFuncIndexedAMD(GLuint buf, GLenum src, GLenum dst)
    void glBlendFuncSeparateIndexedAMD(GLuint buf, GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha)
    void glFramebufferSamplePositionsfvAMD(GLenum target, GLuint numsamples, GLuint pixelindex, GLfloat *values)
    void glGetFramebufferParameterfvAMD(GLenum target, GLenum pname, GLuint numsamples, GLuint pixelindex, GLsizei size, GLfloat *values)
    void glGetNamedFramebufferParameterfvAMD(GLuint framebuffer, GLenum pname, GLuint numsamples, GLuint pixelindex, GLsizei size, GLfloat *values)
    void glNamedFramebufferSamplePositionsfvAMD(GLuint framebuffer, GLuint numsamples, GLuint pixelindex, GLfloat *values)
    void glVertexAttribParameteriAMD(GLuint index, GLenum pname, GLint param)
    void glMultiDrawArraysIndirectAMD(GLenum mode, void *indirect, GLsizei primcount, GLsizei stride)
    void glMultiDrawElementsIndirectAMD(GLenum mode, GLenum type, void *indirect, GLsizei primcount, GLsizei stride)
    void glDeleteNamesAMD(GLenum identifier, GLuint num, GLuint *names)
    void glGenNamesAMD(GLenum identifier, GLuint num, GLuint *names)
    GLboolean glIsNameAMD(GLenum identifier, GLuint name)
    void glQueryObjectParameteruiAMD(GLenum target, GLuint id, GLenum pname, GLuint param)
    void glBeginPerfMonitorAMD(GLuint monitor)
    void glDeletePerfMonitorsAMD(GLsizei n, GLuint *monitors)
    void glEndPerfMonitorAMD(GLuint monitor)
    void glGenPerfMonitorsAMD(GLsizei n, GLuint *monitors)
    void glGetPerfMonitorCounterDataAMD(GLuint monitor, GLenum pname, GLsizei dataSize, GLuint *data, GLint *bytesWritten)
    void glGetPerfMonitorCounterInfoAMD(GLuint group, GLuint counter, GLenum pname, void *data)
    void glGetPerfMonitorCounterStringAMD(GLuint group, GLuint counter, GLsizei bufSize, GLsizei *length, GLchar *counterString)
    void glGetPerfMonitorCountersAMD(GLuint group, GLint *numCounters, GLint *maxActiveCounters, GLsizei countersSize, GLuint *counters)
    void glGetPerfMonitorGroupStringAMD(GLuint group, GLsizei bufSize, GLsizei *length, GLchar *groupString)
    void glGetPerfMonitorGroupsAMD(GLint *numGroups, GLsizei groupsSize, GLuint *groups)
    void glSelectPerfMonitorCountersAMD(GLuint monitor, GLboolean enable, GLuint group, GLint numCounters, GLuint *counterList)
    void glSetMultisamplefvAMD(GLenum pname, GLuint index, GLfloat *val)
    void glTexStorageSparseAMD(GLenum target, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLsizei layers, GLbitfield flags)
    void glTextureStorageSparseAMD(GLuint texture, GLenum target, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLsizei layers, GLbitfield flags)
    void glStencilOpValueAMD(GLenum face, GLuint value)
    void glTessellationFactorAMD(GLfloat factor)
    void glTessellationModeAMD(GLenum mode)
    void glBlitFramebufferANGLE(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    void glRenderbufferStorageMultisampleANGLE(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glDrawArraysInstancedANGLE(GLenum mode, GLint first, GLsizei count, GLsizei primcount)
    void glDrawElementsInstancedANGLE(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount)
    void glVertexAttribDivisorANGLE(GLuint index, GLuint divisor)
    void glBeginQueryANGLE(GLenum target, GLuint id)
    void glDeleteQueriesANGLE(GLsizei n, GLuint *ids)
    void glEndQueryANGLE(GLenum target)
    void glGenQueriesANGLE(GLsizei n, GLuint *ids)
    void glGetQueryObjecti64vANGLE(GLuint id, GLenum pname, GLint64 *params)
    void glGetQueryObjectivANGLE(GLuint id, GLenum pname, GLint *params)
    void glGetQueryObjectui64vANGLE(GLuint id, GLenum pname, GLuint64 *params)
    void glGetQueryObjectuivANGLE(GLuint id, GLenum pname, GLuint *params)
    void glGetQueryivANGLE(GLenum target, GLenum pname, GLint *params)
    GLboolean glIsQueryANGLE(GLuint id)
    void glQueryCounterANGLE(GLuint id, GLenum target)
    void glGetTranslatedShaderSourceANGLE(GLuint shader, GLsizei bufsize, GLsizei *length, GLchar *source)
    void glCopyTextureLevelsAPPLE(GLuint destinationTexture, GLuint sourceTexture, GLint sourceBaseLevel, GLsizei sourceLevelCount)
    void glDrawElementArrayAPPLE(GLenum mode, GLint first, GLsizei count)
    void glDrawRangeElementArrayAPPLE(GLenum mode, GLuint start, GLuint end, GLint first, GLsizei count)
    void glElementPointerAPPLE(GLenum type, void *pointer)
    void glMultiDrawElementArrayAPPLE(GLenum mode, GLint *first, GLsizei *count, GLsizei primcount)
    void glMultiDrawRangeElementArrayAPPLE(GLenum mode, GLuint start, GLuint end, GLint *first, GLsizei *count, GLsizei primcount)
    void glDeleteFencesAPPLE(GLsizei n, GLuint *fences)
    void glFinishFenceAPPLE(GLuint fence)
    void glFinishObjectAPPLE(GLenum object, GLint name)
    void glGenFencesAPPLE(GLsizei n, GLuint *fences)
    GLboolean glIsFenceAPPLE(GLuint fence)
    void glSetFenceAPPLE(GLuint fence)
    GLboolean glTestFenceAPPLE(GLuint fence)
    GLboolean glTestObjectAPPLE(GLenum object, GLuint name)
    void glBufferParameteriAPPLE(GLenum target, GLenum pname, GLint param)
    void glFlushMappedBufferRangeAPPLE(GLenum target, GLintptr offset, GLsizeiptr size)
    void glRenderbufferStorageMultisampleAPPLE(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glResolveMultisampleFramebufferAPPLE()
    void glGetObjectParameterivAPPLE(GLenum objectType, GLuint name, GLenum pname, GLint *params)
    GLenum glObjectPurgeableAPPLE(GLenum objectType, GLuint name, GLenum option)
    GLenum glObjectUnpurgeableAPPLE(GLenum objectType, GLuint name, GLenum option)
    GLenum glClientWaitSyncAPPLE(GLsync GLsync, GLbitfield flags, GLuint64 timeout)
    void glDeleteSyncAPPLE(GLsync GLsync)
    GLsync glFenceSyncAPPLE(GLenum condition, GLbitfield flags)
    void glGetInteger64vAPPLE(GLenum pname, GLint64 *params)
    void glGetSyncivAPPLE(GLsync GLsync, GLenum pname, GLsizei bufSize, GLsizei *length, GLint *values)
    GLboolean glIsSyncAPPLE(GLsync GLsync)
    void glWaitSyncAPPLE(GLsync GLsync, GLbitfield flags, GLuint64 timeout)
    void glGetTexParameterPointervAPPLE(GLenum target, GLenum pname, void **params)
    void glTextureRangeAPPLE(GLenum target, GLsizei length, void *pointer)
    void glBindVertexArrayAPPLE(GLuint array)
    void glDeleteVertexArraysAPPLE(GLsizei n, GLuint *arrays)
    void glGenVertexArraysAPPLE(GLsizei n, GLuint *arrays)
    GLboolean glIsVertexArrayAPPLE(GLuint array)
    void glFlushVertexArrayRangeAPPLE(GLsizei length, void *pointer)
    void glVertexArrayParameteriAPPLE(GLenum pname, GLint param)
    void glVertexArrayRangeAPPLE(GLsizei length, void *pointer)
    void glDisableVertexAttribAPPLE(GLuint index, GLenum pname)
    void glEnableVertexAttribAPPLE(GLuint index, GLenum pname)
    GLboolean glIsVertexAttribEnabledAPPLE(GLuint index, GLenum pname)
    void glMapVertexAttrib1dAPPLE(GLuint index, GLuint size, GLdouble u1, GLdouble u2, GLint stride, GLint order, GLdouble *points)
    void glMapVertexAttrib1fAPPLE(GLuint index, GLuint size, GLfloat u1, GLfloat u2, GLint stride, GLint order, GLfloat *points)
    void glMapVertexAttrib2dAPPLE(GLuint index, GLuint size, GLdouble u1, GLdouble u2, GLint ustride, GLint uorder, GLdouble v1, GLdouble v2, GLint vstride, GLint vorder, GLdouble *points)
    void glMapVertexAttrib2fAPPLE(GLuint index, GLuint size, GLfloat u1, GLfloat u2, GLint ustride, GLint uorder, GLfloat v1, GLfloat v2, GLint vstride, GLint vorder, GLfloat *points)
    void glClearDepthf(GLclampf d)
    void glDepthRangef(GLclampf n, GLclampf f)
    void glGetShaderPrecisionFormat(GLenum shadertype, GLenum precisiontype, GLint *range, GLint *precision)
    void glReleaseShaderCompiler()
    void glShaderBinary(GLsizei count, GLuint *shaders, GLenum binaryformat, void *binary, GLsizei length)
    void glMemoryBarrierByRegion(GLbitfield barriers)
    void glPrimitiveBoundingBoxARB(GLfloat minX, GLfloat minY, GLfloat minZ, GLfloat minW, GLfloat maxX, GLfloat maxY, GLfloat maxZ, GLfloat maxW)
    void glDrawArraysInstancedBaseInstance(GLenum mode, GLint first, GLsizei count, GLsizei primcount, GLuint baseinstance)
    void glDrawElementsInstancedBaseInstance(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount, GLuint baseinstance)
    void glDrawElementsInstancedBaseVertexBaseInstance(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount, GLint basevertex, GLuint baseinstance)
    GLuint64 glGetImageHandleARB(GLuint texture, GLint level, GLboolean layered, GLint layer, GLenum format)
    GLuint64 glGetTextureHandleARB(GLuint texture)
    GLuint64 glGetTextureSamplerHandleARB(GLuint texture, GLuint sampler)
    void glGetVertexAttribLui64vARB(GLuint index, GLenum pname, GLuint64EXT *params)
    GLboolean glIsImageHandleResidentARB(GLuint64 handle)
    GLboolean glIsTextureHandleResidentARB(GLuint64 handle)
    void glMakeImageHandleNonResidentARB(GLuint64 handle)
    void glMakeImageHandleResidentARB(GLuint64 handle, GLenum access)
    void glMakeTextureHandleNonResidentARB(GLuint64 handle)
    void glMakeTextureHandleResidentARB(GLuint64 handle)
    void glProgramUniformHandleui64ARB(GLuint program, GLint location, GLuint64 value)
    void glProgramUniformHandleui64vARB(GLuint program, GLint location, GLsizei count, GLuint64 *values)
    void glUniformHandleui64ARB(GLint location, GLuint64 value)
    void glUniformHandleui64vARB(GLint location, GLsizei count, GLuint64 *value)
    void glVertexAttribL1ui64ARB(GLuint index, GLuint64EXT x)
    void glVertexAttribL1ui64vARB(GLuint index, GLuint64EXT *v)
    void glBindFragDataLocationIndexed(GLuint program, GLuint colorNumber, GLuint index, GLchar *name)
    GLint glGetFragDataIndex(GLuint program, GLchar *name)
    void glBufferStorage(GLenum target, GLsizeiptr size, void *data, GLbitfield flags)
    GLsync glCreateSyncFromCLeventARB(cl_context context, cl_event event, GLbitfield flags)
    void glClearBufferData(GLenum target, GLenum internalformat, GLenum format, GLenum type, void *data)
    void glClearBufferSubData(GLenum target, GLenum internalformat, GLintptr offset, GLsizeiptr size, GLenum format, GLenum type, void *data)
    void glClearNamedBufferDataEXT(GLuint buffer, GLenum internalformat, GLenum format, GLenum type, void *data)
    void glClearNamedBufferSubDataEXT(GLuint buffer, GLenum internalformat, GLintptr offset, GLsizeiptr size, GLenum format, GLenum type, void *data)
    void glClearTexImage(GLuint texture, GLint level, GLenum format, GLenum type, void *data)
    void glClearTexSubImage(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *data)
    void glClipControl(GLenum origin, GLenum depth)
    void glClampColorARB(GLenum target, GLenum clamp)
    void glDispatchCompute(GLuint num_groups_x, GLuint num_groups_y, GLuint num_groups_z)
    void glDispatchComputeIndirect(GLintptr indirect)
    void glDispatchComputeGroupSizeARB(GLuint num_groups_x, GLuint num_groups_y, GLuint num_groups_z, GLuint group_size_x, GLuint group_size_y, GLuint group_size_z)
    void glCopyBufferSubData(GLenum readtarget, GLenum writetarget, GLintptr readoffset, GLintptr writeoffset, GLsizeiptr size)
    void glCopyImageSubData(GLuint srcName, GLenum srcTarget, GLint srcLevel, GLint srcX, GLint srcY, GLint srcZ, GLuint dstName, GLenum dstTarget, GLint dstLevel, GLint dstX, GLint dstY, GLint dstZ, GLsizei srcWidth, GLsizei srcHeight, GLsizei srcDepth)
    void glDebugMessageCallbackARB(GLDEBUGPROCARB callback, void *userParam)
    void glDebugMessageControlARB(GLenum source, GLenum type, GLenum severity, GLsizei count, GLuint *ids, GLboolean enabled)
    void glDebugMessageInsertARB(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, GLchar *buf)
    GLuint glGetDebugMessageLogARB(GLuint count, GLsizei bufSize, GLenum *sources, GLenum *types, GLuint *ids, GLenum *severities, GLsizei *lengths, GLchar *messageLog)
    void glBindTextureUnit(GLuint unit, GLuint texture)
    void glBlitNamedFramebuffer(GLuint readFramebuffer, GLuint drawFramebuffer, GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    GLenum glCheckNamedFramebufferStatus(GLuint framebuffer, GLenum target)
    void glClearNamedBufferData(GLuint buffer, GLenum internalformat, GLenum format, GLenum type, void *data)
    void glClearNamedBufferSubData(GLuint buffer, GLenum internalformat, GLintptr offset, GLsizeiptr size, GLenum format, GLenum type, void *data)
    void glClearNamedFramebufferfi(GLuint framebuffer, GLenum buffer, GLint drawbuffer, GLfloat depth, GLint stencil)
    void glClearNamedFramebufferfv(GLuint framebuffer, GLenum buffer, GLint drawbuffer, GLfloat *value)
    void glClearNamedFramebufferiv(GLuint framebuffer, GLenum buffer, GLint drawbuffer, GLint *value)
    void glClearNamedFramebufferuiv(GLuint framebuffer, GLenum buffer, GLint drawbuffer, GLuint *value)
    void glCompressedTextureSubImage1D(GLuint texture, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTextureSubImage2D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTextureSubImage3D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glCopyNamedBufferSubData(GLuint readBuffer, GLuint writeBuffer, GLintptr readOffset, GLintptr writeOffset, GLsizeiptr size)
    void glCopyTextureSubImage1D(GLuint texture, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width)
    void glCopyTextureSubImage2D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCopyTextureSubImage3D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCreateBuffers(GLsizei n, GLuint *buffers)
    void glCreateFramebuffers(GLsizei n, GLuint *framebuffers)
    void glCreateProgramPipelines(GLsizei n, GLuint *pipelines)
    void glCreateQueries(GLenum target, GLsizei n, GLuint *ids)
    void glCreateRenderbuffers(GLsizei n, GLuint *renderbuffers)
    void glCreateSamplers(GLsizei n, GLuint *samplers)
    void glCreateTextures(GLenum target, GLsizei n, GLuint *textures)
    void glCreateTransformFeedbacks(GLsizei n, GLuint *ids)
    void glCreateVertexArrays(GLsizei n, GLuint *arrays)
    void glDisableVertexArrayAttrib(GLuint vaobj, GLuint index)
    void glEnableVertexArrayAttrib(GLuint vaobj, GLuint index)
    void glFlushMappedNamedBufferRange(GLuint buffer, GLintptr offset, GLsizeiptr length)
    void glGenerateTextureMipmap(GLuint texture)
    void glGetCompressedTextureImage(GLuint texture, GLint level, GLsizei bufSize, void *pixels)
    void glGetNamedBufferParameteri64v(GLuint buffer, GLenum pname, GLint64 *params)
    void glGetNamedBufferParameteriv(GLuint buffer, GLenum pname, GLint *params)
    void glGetNamedBufferPointerv(GLuint buffer, GLenum pname, void **params)
    void glGetNamedBufferSubData(GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glGetNamedFramebufferAttachmentParameteriv(GLuint framebuffer, GLenum attachment, GLenum pname, GLint *params)
    void glGetNamedFramebufferParameteriv(GLuint framebuffer, GLenum pname, GLint *param)
    void glGetNamedRenderbufferParameteriv(GLuint renderbuffer, GLenum pname, GLint *params)
    void glGetQueryBufferObjecti64v(GLuint id, GLuint buffer, GLenum pname, GLintptr offset)
    void glGetQueryBufferObjectiv(GLuint id, GLuint buffer, GLenum pname, GLintptr offset)
    void glGetQueryBufferObjectui64v(GLuint id, GLuint buffer, GLenum pname, GLintptr offset)
    void glGetQueryBufferObjectuiv(GLuint id, GLuint buffer, GLenum pname, GLintptr offset)
    void glGetTextureImage(GLuint texture, GLint level, GLenum format, GLenum type, GLsizei bufSize, void *pixels)
    void glGetTextureLevelParameterfv(GLuint texture, GLint level, GLenum pname, GLfloat *params)
    void glGetTextureLevelParameteriv(GLuint texture, GLint level, GLenum pname, GLint *params)
    void glGetTextureParameterIiv(GLuint texture, GLenum pname, GLint *params)
    void glGetTextureParameterIuiv(GLuint texture, GLenum pname, GLuint *params)
    void glGetTextureParameterfv(GLuint texture, GLenum pname, GLfloat *params)
    void glGetTextureParameteriv(GLuint texture, GLenum pname, GLint *params)
    void glGetTransformFeedbacki64_v(GLuint xfb, GLenum pname, GLuint index, GLint64 *param)
    void glGetTransformFeedbacki_v(GLuint xfb, GLenum pname, GLuint index, GLint *param)
    void glGetTransformFeedbackiv(GLuint xfb, GLenum pname, GLint *param)
    void glGetVertexArrayIndexed64iv(GLuint vaobj, GLuint index, GLenum pname, GLint64 *param)
    void glGetVertexArrayIndexediv(GLuint vaobj, GLuint index, GLenum pname, GLint *param)
    void glGetVertexArrayiv(GLuint vaobj, GLenum pname, GLint *param)
    void glInvalidateNamedFramebufferData(GLuint framebuffer, GLsizei numAttachments, GLenum *attachments)
    void glInvalidateNamedFramebufferSubData(GLuint framebuffer, GLsizei numAttachments, GLenum *attachments, GLint x, GLint y, GLsizei width, GLsizei height)
    void *glMapNamedBuffer(GLuint buffer, GLenum access)
    void *glMapNamedBufferRange(GLuint buffer, GLintptr offset, GLsizeiptr length, GLbitfield access)
    void glNamedBufferData(GLuint buffer, GLsizeiptr size, void *data, GLenum usage)
    void glNamedBufferStorage(GLuint buffer, GLsizeiptr size, void *data, GLbitfield flags)
    void glNamedBufferSubData(GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glNamedFramebufferDrawBuffer(GLuint framebuffer, GLenum mode)
    void glNamedFramebufferDrawBuffers(GLuint framebuffer, GLsizei n, GLenum *bufs)
    void glNamedFramebufferParameteri(GLuint framebuffer, GLenum pname, GLint param)
    void glNamedFramebufferReadBuffer(GLuint framebuffer, GLenum mode)
    void glNamedFramebufferRenderbuffer(GLuint framebuffer, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
    void glNamedFramebufferTexture(GLuint framebuffer, GLenum attachment, GLuint texture, GLint level)
    void glNamedFramebufferTextureLayer(GLuint framebuffer, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glNamedRenderbufferStorage(GLuint renderbuffer, GLenum internalformat, GLsizei width, GLsizei height)
    void glNamedRenderbufferStorageMultisample(GLuint renderbuffer, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glTextureBuffer(GLuint texture, GLenum internalformat, GLuint buffer)
    void glTextureBufferRange(GLuint texture, GLenum internalformat, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glTextureParameterIiv(GLuint texture, GLenum pname, GLint *params)
    void glTextureParameterIuiv(GLuint texture, GLenum pname, GLuint *params)
    void glTextureParameterf(GLuint texture, GLenum pname, GLfloat param)
    void glTextureParameterfv(GLuint texture, GLenum pname, GLfloat *param)
    void glTextureParameteri(GLuint texture, GLenum pname, GLint param)
    void glTextureParameteriv(GLuint texture, GLenum pname, GLint *param)
    void glTextureStorage1D(GLuint texture, GLsizei levels, GLenum internalformat, GLsizei width)
    void glTextureStorage2D(GLuint texture, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height)
    void glTextureStorage2DMultisample(GLuint texture, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLboolean fixedsamplelocations)
    void glTextureStorage3D(GLuint texture, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth)
    void glTextureStorage3DMultisample(GLuint texture, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedsamplelocations)
    void glTextureSubImage1D(GLuint texture, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, void *pixels)
    void glTextureSubImage2D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glTextureSubImage3D(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    void glTransformFeedbackBufferBase(GLuint xfb, GLuint index, GLuint buffer)
    void glTransformFeedbackBufferRange(GLuint xfb, GLuint index, GLuint buffer, GLintptr offset, GLsizeiptr size)
    GLboolean glUnmapNamedBuffer(GLuint buffer)
    void glVertexArrayAttribBinding(GLuint vaobj, GLuint attribindex, GLuint bindingindex)
    void glVertexArrayAttribFormat(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLboolean normalized, GLuint relativeoffset)
    void glVertexArrayAttribIFormat(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexArrayAttribLFormat(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexArrayBindingDivisor(GLuint vaobj, GLuint bindingindex, GLuint divisor)
    void glVertexArrayElementBuffer(GLuint vaobj, GLuint buffer)
    void glVertexArrayVertexBuffer(GLuint vaobj, GLuint bindingindex, GLuint buffer, GLintptr offset, GLsizei stride)
    void glVertexArrayVertexBuffers(GLuint vaobj, GLuint first, GLsizei count, GLuint *buffers, GLintptr *offsets, GLsizei *strides)
    void glDrawBuffersARB(GLsizei n, GLenum *bufs)
    void glBlendEquationSeparateiARB(GLuint buf, GLenum modeRGB, GLenum modeAlpha)
    void glBlendEquationiARB(GLuint buf, GLenum mode)
    void glBlendFuncSeparateiARB(GLuint buf, GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha)
    void glBlendFunciARB(GLuint buf, GLenum src, GLenum dst)
    void glDrawElementsBaseVertex(GLenum mode, GLsizei count, GLenum type, void *indices, GLint basevertex)
    void glDrawElementsInstancedBaseVertex(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount, GLint basevertex)
    void glDrawRangeElementsBaseVertex(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, void *indices, GLint basevertex)
    void glMultiDrawElementsBaseVertex(GLenum mode, GLsizei *count, GLenum type, void **indices, GLsizei primcount, GLint *basevertex)
    void glDrawArraysIndirect(GLenum mode, void *indirect)
    void glDrawElementsIndirect(GLenum mode, GLenum type, void *indirect)
    void glFramebufferParameteri(GLenum target, GLenum pname, GLint param)
    void glGetFramebufferParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetNamedFramebufferParameterivEXT(GLuint framebuffer, GLenum pname, GLint *params)
    void glNamedFramebufferParameteriEXT(GLuint framebuffer, GLenum pname, GLint param)
    void glBindFramebuffer(GLenum target, GLuint framebuffer)
    void glBindRenderbuffer(GLenum target, GLuint renderbuffer)
    void glBlitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    GLenum glCheckFramebufferStatus(GLenum target)
    void glDeleteFramebuffers(GLsizei n, GLuint *framebuffers)
    void glDeleteRenderbuffers(GLsizei n, GLuint *renderbuffers)
    void glFramebufferRenderbuffer(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
    void glFramebufferTexture1D(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glFramebufferTexture2D(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glFramebufferTexture3D(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint layer)
    void glFramebufferTextureLayer(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glGenFramebuffers(GLsizei n, GLuint *framebuffers)
    void glGenRenderbuffers(GLsizei n, GLuint *renderbuffers)
    void glGenerateMipmap(GLenum target)
    void glGetFramebufferAttachmentParameteriv(GLenum target, GLenum attachment, GLenum pname, GLint *params)
    void glGetRenderbufferParameteriv(GLenum target, GLenum pname, GLint *params)
    GLboolean glIsFramebuffer(GLuint framebuffer)
    GLboolean glIsRenderbuffer(GLuint renderbuffer)
    void glRenderbufferStorage(GLenum target, GLenum internalformat, GLsizei width, GLsizei height)
    void glRenderbufferStorageMultisample(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glFramebufferTextureARB(GLenum target, GLenum attachment, GLuint texture, GLint level)
    void glFramebufferTextureFaceARB(GLenum target, GLenum attachment, GLuint texture, GLint level, GLenum face)
    void glFramebufferTextureLayerARB(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glProgramParameteriARB(GLuint program, GLenum pname, GLint value)
    void glGetProgramBinary(GLuint program, GLsizei bufSize, GLsizei *length, GLenum *binaryFormat, void *binary)
    void glProgramBinary(GLuint program, GLenum binaryFormat, void *binary, GLsizei length)
    void glProgramParameteri(GLuint program, GLenum pname, GLint value)
    void glGetCompressedTextureSubImage(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLsizei bufSize, void *pixels)
    void glGetTextureSubImage(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, GLsizei bufSize, void *pixels)
    void glSpecializeShaderARB(GLuint shader, GLchar *pEntryPoint, GLuint numSpecializationConstants, GLuint *pConstantIndex, GLuint *pConstantValue)
    void glGetUniformdv(GLuint program, GLint location, GLdouble *params)
    void glUniform1d(GLint location, GLdouble x)
    void glUniform1dv(GLint location, GLsizei count, GLdouble *value)
    void glUniform2d(GLint location, GLdouble x, GLdouble y)
    void glUniform2dv(GLint location, GLsizei count, GLdouble *value)
    void glUniform3d(GLint location, GLdouble x, GLdouble y, GLdouble z)
    void glUniform3dv(GLint location, GLsizei count, GLdouble *value)
    void glUniform4d(GLint location, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glUniform4dv(GLint location, GLsizei count, GLdouble *value)
    void glUniformMatrix2dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix2x3dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix2x4dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix3dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix3x2dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix3x4dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix4dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix4x2dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glUniformMatrix4x3dv(GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glGetUniformi64vARB(GLuint program, GLint location, GLint64 *params)
    void glGetUniformui64vARB(GLuint program, GLint location, GLuint64 *params)
    void glGetnUniformi64vARB(GLuint program, GLint location, GLsizei bufSize, GLint64 *params)
    void glGetnUniformui64vARB(GLuint program, GLint location, GLsizei bufSize, GLuint64 *params)
    void glProgramUniform1i64ARB(GLuint program, GLint location, GLint64 x)
    void glProgramUniform1i64vARB(GLuint program, GLint location, GLsizei count, GLint64 *value)
    void glProgramUniform1ui64ARB(GLuint program, GLint location, GLuint64 x)
    void glProgramUniform1ui64vARB(GLuint program, GLint location, GLsizei count, GLuint64 *value)
    void glProgramUniform2i64ARB(GLuint program, GLint location, GLint64 x, GLint64 y)
    void glProgramUniform2i64vARB(GLuint program, GLint location, GLsizei count, GLint64 *value)
    void glProgramUniform2ui64ARB(GLuint program, GLint location, GLuint64 x, GLuint64 y)
    void glProgramUniform2ui64vARB(GLuint program, GLint location, GLsizei count, GLuint64 *value)
    void glProgramUniform3i64ARB(GLuint program, GLint location, GLint64 x, GLint64 y, GLint64 z)
    void glProgramUniform3i64vARB(GLuint program, GLint location, GLsizei count, GLint64 *value)
    void glProgramUniform3ui64ARB(GLuint program, GLint location, GLuint64 x, GLuint64 y, GLuint64 z)
    void glProgramUniform3ui64vARB(GLuint program, GLint location, GLsizei count, GLuint64 *value)
    void glProgramUniform4i64ARB(GLuint program, GLint location, GLint64 x, GLint64 y, GLint64 z, GLint64 w)
    void glProgramUniform4i64vARB(GLuint program, GLint location, GLsizei count, GLint64 *value)
    void glProgramUniform4ui64ARB(GLuint program, GLint location, GLuint64 x, GLuint64 y, GLuint64 z, GLuint64 w)
    void glProgramUniform4ui64vARB(GLuint program, GLint location, GLsizei count, GLuint64 *value)
    void glUniform1i64ARB(GLint location, GLint64 x)
    void glUniform1i64vARB(GLint location, GLsizei count, GLint64 *value)
    void glUniform1ui64ARB(GLint location, GLuint64 x)
    void glUniform1ui64vARB(GLint location, GLsizei count, GLuint64 *value)
    void glUniform2i64ARB(GLint location, GLint64 x, GLint64 y)
    void glUniform2i64vARB(GLint location, GLsizei count, GLint64 *value)
    void glUniform2ui64ARB(GLint location, GLuint64 x, GLuint64 y)
    void glUniform2ui64vARB(GLint location, GLsizei count, GLuint64 *value)
    void glUniform3i64ARB(GLint location, GLint64 x, GLint64 y, GLint64 z)
    void glUniform3i64vARB(GLint location, GLsizei count, GLint64 *value)
    void glUniform3ui64ARB(GLint location, GLuint64 x, GLuint64 y, GLuint64 z)
    void glUniform3ui64vARB(GLint location, GLsizei count, GLuint64 *value)
    void glUniform4i64ARB(GLint location, GLint64 x, GLint64 y, GLint64 z, GLint64 w)
    void glUniform4i64vARB(GLint location, GLsizei count, GLint64 *value)
    void glUniform4ui64ARB(GLint location, GLuint64 x, GLuint64 y, GLuint64 z, GLuint64 w)
    void glUniform4ui64vARB(GLint location, GLsizei count, GLuint64 *value)
    void glColorSubTable(GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, void *data)
    void glColorTable(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, void *table)
    void glColorTableParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glColorTableParameteriv(GLenum target, GLenum pname, GLint *params)
    void glConvolutionFilter1D(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, void *image)
    void glConvolutionFilter2D(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, void *image)
    void glConvolutionParameterf(GLenum target, GLenum pname, GLfloat params)
    void glConvolutionParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glConvolutionParameteri(GLenum target, GLenum pname, GLint params)
    void glConvolutionParameteriv(GLenum target, GLenum pname, GLint *params)
    void glCopyColorSubTable(GLenum target, GLsizei start, GLint x, GLint y, GLsizei width)
    void glCopyColorTable(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width)
    void glCopyConvolutionFilter1D(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width)
    void glCopyConvolutionFilter2D(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height)
    void glGetColorTable(GLenum target, GLenum format, GLenum type, void *table)
    void glGetColorTableParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetColorTableParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetConvolutionFilter(GLenum target, GLenum format, GLenum type, void *image)
    void glGetConvolutionParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetConvolutionParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetHistogram(GLenum target, GLboolean reset, GLenum format, GLenum type, void *values)
    void glGetHistogramParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetHistogramParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetMinmax(GLenum target, GLboolean reset, GLenum format, GLenum types, void *values)
    void glGetMinmaxParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetMinmaxParameteriv(GLenum target, GLenum pname, GLint *params)
    void glGetSeparableFilter(GLenum target, GLenum format, GLenum type, void *row, void *column, void *span)
    void glHistogram(GLenum target, GLsizei width, GLenum internalformat, GLboolean sink)
    void glMinmax(GLenum target, GLenum internalformat, GLboolean sink)
    void glResetHistogram(GLenum target)
    void glResetMinmax(GLenum target)
    void glSeparableFilter2D(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, void *row, void *column)
    void glMultiDrawArraysIndirectCountARB(GLenum mode, void *indirect, GLintptr drawcount, GLsizei maxdrawcount, GLsizei stride)
    void glMultiDrawElementsIndirectCountARB(GLenum mode, GLenum type, void *indirect, GLintptr drawcount, GLsizei maxdrawcount, GLsizei stride)
    void glDrawArraysInstancedARB(GLenum mode, GLint first, GLsizei count, GLsizei primcount)
    void glDrawElementsInstancedARB(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount)
    void glVertexAttribDivisorARB(GLuint index, GLuint divisor)
    void glGetInternalformativ(GLenum target, GLenum internalformat, GLenum pname, GLsizei bufSize, GLint *params)
    void glGetInternalformati64v(GLenum target, GLenum internalformat, GLenum pname, GLsizei bufSize, GLint64 *params)
    void glInvalidateBufferData(GLuint buffer)
    void glInvalidateBufferSubData(GLuint buffer, GLintptr offset, GLsizeiptr length)
    void glInvalidateFramebuffer(GLenum target, GLsizei numAttachments, GLenum *attachments)
    void glInvalidateSubFramebuffer(GLenum target, GLsizei numAttachments, GLenum *attachments, GLint x, GLint y, GLsizei width, GLsizei height)
    void glInvalidateTexImage(GLuint texture, GLint level)
    void glInvalidateTexSubImage(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth)
    void glFlushMappedBufferRange(GLenum target, GLintptr offset, GLsizeiptr length)
    void *glMapBufferRange(GLenum target, GLintptr offset, GLsizeiptr length, GLbitfield access)
    void glCurrentPaletteMatrixARB(GLint index)
    void glMatrixIndexPointerARB(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glMatrixIndexubvARB(GLint size, GLubyte *indices)
    void glMatrixIndexuivARB(GLint size, GLuint *indices)
    void glMatrixIndexusvARB(GLint size, GLushort *indices)
    void glBindBuffersBase(GLenum target, GLuint first, GLsizei count, GLuint *buffers)
    void glBindBuffersRange(GLenum target, GLuint first, GLsizei count, GLuint *buffers, GLintptr *offsets, GLsizeiptr *sizes)
    void glBindImageTextures(GLuint first, GLsizei count, GLuint *textures)
    void glBindSamplers(GLuint first, GLsizei count, GLuint *samplers)
    void glBindTextures(GLuint first, GLsizei count, GLuint *textures)
    void glBindVertexBuffers(GLuint first, GLsizei count, GLuint *buffers, GLintptr *offsets, GLsizei *strides)
    void glMultiDrawArraysIndirect(GLenum mode, void *indirect, GLsizei primcount, GLsizei stride)
    void glMultiDrawElementsIndirect(GLenum mode, GLenum type, void *indirect, GLsizei primcount, GLsizei stride)
    void glSampleCoverageARB(GLclampf value, GLboolean invert)
    void glActiveTextureARB(GLenum texture)
    void glClientActiveTextureARB(GLenum texture)
    void glMultiTexCoord1dARB(GLenum target, GLdouble s)
    void glMultiTexCoord1dvARB(GLenum target, GLdouble *v)
    void glMultiTexCoord1fARB(GLenum target, GLfloat s)
    void glMultiTexCoord1fvARB(GLenum target, GLfloat *v)
    void glMultiTexCoord1iARB(GLenum target, GLint s)
    void glMultiTexCoord1ivARB(GLenum target, GLint *v)
    void glMultiTexCoord1sARB(GLenum target, GLshort s)
    void glMultiTexCoord1svARB(GLenum target, GLshort *v)
    void glMultiTexCoord2dARB(GLenum target, GLdouble s, GLdouble t)
    void glMultiTexCoord2dvARB(GLenum target, GLdouble *v)
    void glMultiTexCoord2fARB(GLenum target, GLfloat s, GLfloat t)
    void glMultiTexCoord2fvARB(GLenum target, GLfloat *v)
    void glMultiTexCoord2iARB(GLenum target, GLint s, GLint t)
    void glMultiTexCoord2ivARB(GLenum target, GLint *v)
    void glMultiTexCoord2sARB(GLenum target, GLshort s, GLshort t)
    void glMultiTexCoord2svARB(GLenum target, GLshort *v)
    void glMultiTexCoord3dARB(GLenum target, GLdouble s, GLdouble t, GLdouble r)
    void glMultiTexCoord3dvARB(GLenum target, GLdouble *v)
    void glMultiTexCoord3fARB(GLenum target, GLfloat s, GLfloat t, GLfloat r)
    void glMultiTexCoord3fvARB(GLenum target, GLfloat *v)
    void glMultiTexCoord3iARB(GLenum target, GLint s, GLint t, GLint r)
    void glMultiTexCoord3ivARB(GLenum target, GLint *v)
    void glMultiTexCoord3sARB(GLenum target, GLshort s, GLshort t, GLshort r)
    void glMultiTexCoord3svARB(GLenum target, GLshort *v)
    void glMultiTexCoord4dARB(GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q)
    void glMultiTexCoord4dvARB(GLenum target, GLdouble *v)
    void glMultiTexCoord4fARB(GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q)
    void glMultiTexCoord4fvARB(GLenum target, GLfloat *v)
    void glMultiTexCoord4iARB(GLenum target, GLint s, GLint t, GLint r, GLint q)
    void glMultiTexCoord4ivARB(GLenum target, GLint *v)
    void glMultiTexCoord4sARB(GLenum target, GLshort s, GLshort t, GLshort r, GLshort q)
    void glMultiTexCoord4svARB(GLenum target, GLshort *v)
    void glBeginQueryARB(GLenum target, GLuint id)
    void glDeleteQueriesARB(GLsizei n, GLuint *ids)
    void glEndQueryARB(GLenum target)
    void glGenQueriesARB(GLsizei n, GLuint *ids)
    void glGetQueryObjectivARB(GLuint id, GLenum pname, GLint *params)
    void glGetQueryObjectuivARB(GLuint id, GLenum pname, GLuint *params)
    void glGetQueryivARB(GLenum target, GLenum pname, GLint *params)
    GLboolean glIsQueryARB(GLuint id)
    void glMaxShaderCompilerThreadsARB(GLuint count)
    void glPointParameterfARB(GLenum pname, GLfloat param)
    void glPointParameterfvARB(GLenum pname, GLfloat *params)
    void glPolygonOffsetClamp(GLfloat factor, GLfloat units, GLfloat clamp)
    void glGetProgramInterfaceiv(GLuint program, GLenum programInterface, GLenum pname, GLint *params)
    GLuint glGetProgramResourceIndex(GLuint program, GLenum programInterface, GLchar *name)
    GLint glGetProgramResourceLocation(GLuint program, GLenum programInterface, GLchar *name)
    GLint glGetProgramResourceLocationIndex(GLuint program, GLenum programInterface, GLchar *name)
    void glGetProgramResourceName(GLuint program, GLenum programInterface, GLuint index, GLsizei bufSize, GLsizei *length, GLchar *name)
    void glGetProgramResourceiv(GLuint program, GLenum programInterface, GLuint index, GLsizei propCount, GLenum *props, GLsizei bufSize, GLsizei *length, GLint *params)
    void glProvokingVertex(GLenum mode)
    GLenum glGetGraphicsResetStatusARB()
    void glGetnColorTableARB(GLenum target, GLenum format, GLenum type, GLsizei bufSize, void *table)
    void glGetnCompressedTexImageARB(GLenum target, GLint lod, GLsizei bufSize, void *img)
    void glGetnConvolutionFilterARB(GLenum target, GLenum format, GLenum type, GLsizei bufSize, void *image)
    void glGetnHistogramARB(GLenum target, GLboolean reset, GLenum format, GLenum type, GLsizei bufSize, void *values)
    void glGetnMapdvARB(GLenum target, GLenum query, GLsizei bufSize, GLdouble *v)
    void glGetnMapfvARB(GLenum target, GLenum query, GLsizei bufSize, GLfloat *v)
    void glGetnMapivARB(GLenum target, GLenum query, GLsizei bufSize, GLint *v)
    void glGetnMinmaxARB(GLenum target, GLboolean reset, GLenum format, GLenum type, GLsizei bufSize, void *values)
    void glGetnPixelMapfvARB(GLenum map, GLsizei bufSize, GLfloat *values)
    void glGetnPixelMapuivARB(GLenum map, GLsizei bufSize, GLuint *values)
    void glGetnPixelMapusvARB(GLenum map, GLsizei bufSize, GLushort *values)
    void glGetnPolygonStippleARB(GLsizei bufSize, GLubyte *pattern)
    void glGetnSeparableFilterARB(GLenum target, GLenum format, GLenum type, GLsizei rowBufSize, void *row, GLsizei columnBufSize, void *column, void *span)
    void glGetnTexImageARB(GLenum target, GLint level, GLenum format, GLenum type, GLsizei bufSize, void *img)
    void glGetnUniformdvARB(GLuint program, GLint location, GLsizei bufSize, GLdouble *params)
    void glGetnUniformfvARB(GLuint program, GLint location, GLsizei bufSize, GLfloat *params)
    void glGetnUniformivARB(GLuint program, GLint location, GLsizei bufSize, GLint *params)
    void glGetnUniformuivARB(GLuint program, GLint location, GLsizei bufSize, GLuint *params)
    void glReadnPixelsARB(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLsizei bufSize, void *data)
    void glFramebufferSampleLocationsfvARB(GLenum target, GLuint start, GLsizei count, GLfloat *v)
    void glNamedFramebufferSampleLocationsfvARB(GLuint framebuffer, GLuint start, GLsizei count, GLfloat *v)
    void glMinSampleShadingARB(GLclampf value)
    void glBindSampler(GLuint unit, GLuint sampler)
    void glDeleteSamplers(GLsizei count, GLuint *samplers)
    void glGenSamplers(GLsizei count, GLuint *samplers)
    void glGetSamplerParameterIiv(GLuint sampler, GLenum pname, GLint *params)
    void glGetSamplerParameterIuiv(GLuint sampler, GLenum pname, GLuint *params)
    void glGetSamplerParameterfv(GLuint sampler, GLenum pname, GLfloat *params)
    void glGetSamplerParameteriv(GLuint sampler, GLenum pname, GLint *params)
    GLboolean glIsSampler(GLuint sampler)
    void glSamplerParameterIiv(GLuint sampler, GLenum pname, GLint *params)
    void glSamplerParameterIuiv(GLuint sampler, GLenum pname, GLuint *params)
    void glSamplerParameterf(GLuint sampler, GLenum pname, GLfloat param)
    void glSamplerParameterfv(GLuint sampler, GLenum pname, GLfloat *params)
    void glSamplerParameteri(GLuint sampler, GLenum pname, GLint param)
    void glSamplerParameteriv(GLuint sampler, GLenum pname, GLint *params)
    void glActiveShaderProgram(GLuint pipeline, GLuint program)
    void glBindProgramPipeline(GLuint pipeline)
    GLuint glCreateShaderProgramv(GLenum type, GLsizei count, GLchar * *strings)
    void glDeleteProgramPipelines(GLsizei n, GLuint *pipelines)
    void glGenProgramPipelines(GLsizei n, GLuint *pipelines)
    void glGetProgramPipelineInfoLog(GLuint pipeline, GLsizei bufSize, GLsizei *length, GLchar *infoLog)
    void glGetProgramPipelineiv(GLuint pipeline, GLenum pname, GLint *params)
    GLboolean glIsProgramPipeline(GLuint pipeline)
    void glProgramUniform1d(GLuint program, GLint location, GLdouble x)
    void glProgramUniform1dv(GLuint program, GLint location, GLsizei count, GLdouble *value)
    void glProgramUniform1f(GLuint program, GLint location, GLfloat x)
    void glProgramUniform1fv(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform1i(GLuint program, GLint location, GLint x)
    void glProgramUniform1iv(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform1ui(GLuint program, GLint location, GLuint x)
    void glProgramUniform1uiv(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform2d(GLuint program, GLint location, GLdouble x, GLdouble y)
    void glProgramUniform2dv(GLuint program, GLint location, GLsizei count, GLdouble *value)
    void glProgramUniform2f(GLuint program, GLint location, GLfloat x, GLfloat y)
    void glProgramUniform2fv(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform2i(GLuint program, GLint location, GLint x, GLint y)
    void glProgramUniform2iv(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform2ui(GLuint program, GLint location, GLuint x, GLuint y)
    void glProgramUniform2uiv(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform3d(GLuint program, GLint location, GLdouble x, GLdouble y, GLdouble z)
    void glProgramUniform3dv(GLuint program, GLint location, GLsizei count, GLdouble *value)
    void glProgramUniform3f(GLuint program, GLint location, GLfloat x, GLfloat y, GLfloat z)
    void glProgramUniform3fv(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform3i(GLuint program, GLint location, GLint x, GLint y, GLint z)
    void glProgramUniform3iv(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform3ui(GLuint program, GLint location, GLuint x, GLuint y, GLuint z)
    void glProgramUniform3uiv(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform4d(GLuint program, GLint location, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glProgramUniform4dv(GLuint program, GLint location, GLsizei count, GLdouble *value)
    void glProgramUniform4f(GLuint program, GLint location, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glProgramUniform4fv(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform4i(GLuint program, GLint location, GLint x, GLint y, GLint z, GLint w)
    void glProgramUniform4iv(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform4ui(GLuint program, GLint location, GLuint x, GLuint y, GLuint z, GLuint w)
    void glProgramUniform4uiv(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniformMatrix2dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix2fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix2x3dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix2x3fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix2x4dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix2x4fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix3fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3x2dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix3x2fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3x4dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix3x4fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix4fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4x2dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix4x2fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4x3dv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLdouble *value)
    void glProgramUniformMatrix4x3fv(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUseProgramStages(GLuint pipeline, GLbitfield stages, GLuint program)
    void glValidateProgramPipeline(GLuint pipeline)
    void glGetActiveAtomicCounterBufferiv(GLuint program, GLuint bufferIndex, GLenum pname, GLint *params)
    void glBindImageTexture(GLuint unit, GLuint texture, GLint level, GLboolean layered, GLint layer, GLenum access, GLenum format)
    void glMemoryBarrier(GLbitfield barriers)
    void glAttachObjectARB(GLhandleARB containerObj, GLhandleARB obj)
    void glCompileShaderARB(GLhandleARB shaderObj)
    GLhandleARB glCreateProgramObjectARB()
    GLhandleARB glCreateShaderObjectARB(GLenum shaderType)
    void glDeleteObjectARB(GLhandleARB obj)
    void glDetachObjectARB(GLhandleARB containerObj, GLhandleARB attachedObj)
    void glGetActiveUniformARB(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, GLint *size, GLenum *type, GLcharARB *name)
    void glGetAttachedObjectsARB(GLhandleARB containerObj, GLsizei maxCount, GLsizei *count, GLhandleARB *obj)
    GLhandleARB glGetHandleARB(GLenum pname)
    void glGetInfoLogARB(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *infoLog)
    void glGetObjectParameterfvARB(GLhandleARB obj, GLenum pname, GLfloat *params)
    void glGetObjectParameterivARB(GLhandleARB obj, GLenum pname, GLint *params)
    void glGetShaderSourceARB(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *source)
    GLint glGetUniformLocationARB(GLhandleARB programObj, GLcharARB *name)
    void glGetUniformfvARB(GLhandleARB programObj, GLint location, GLfloat *params)
    void glGetUniformivARB(GLhandleARB programObj, GLint location, GLint *params)
    void glLinkProgramARB(GLhandleARB programObj)
    void glShaderSourceARB(GLhandleARB shaderObj, GLsizei count, GLcharARB **string, GLint *length)
    void glUniform1fARB(GLint location, GLfloat v0)
    void glUniform1fvARB(GLint location, GLsizei count, GLfloat *value)
    void glUniform1iARB(GLint location, GLint v0)
    void glUniform1ivARB(GLint location, GLsizei count, GLint *value)
    void glUniform2fARB(GLint location, GLfloat v0, GLfloat v1)
    void glUniform2fvARB(GLint location, GLsizei count, GLfloat *value)
    void glUniform2iARB(GLint location, GLint v0, GLint v1)
    void glUniform2ivARB(GLint location, GLsizei count, GLint *value)
    void glUniform3fARB(GLint location, GLfloat v0, GLfloat v1, GLfloat v2)
    void glUniform3fvARB(GLint location, GLsizei count, GLfloat *value)
    void glUniform3iARB(GLint location, GLint v0, GLint v1, GLint v2)
    void glUniform3ivARB(GLint location, GLsizei count, GLint *value)
    void glUniform4fARB(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
    void glUniform4fvARB(GLint location, GLsizei count, GLfloat *value)
    void glUniform4iARB(GLint location, GLint v0, GLint v1, GLint v2, GLint v3)
    void glUniform4ivARB(GLint location, GLsizei count, GLint *value)
    void glUniformMatrix2fvARB(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3fvARB(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4fvARB(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUseProgramObjectARB(GLhandleARB programObj)
    void glValidateProgramARB(GLhandleARB programObj)
    void glShaderStorageBlockBinding(GLuint program, GLuint storageBlockIndex, GLuint storageBlockBinding)
    void glGetActiveSubroutineName(GLuint program, GLenum shadertype, GLuint index, GLsizei bufsize, GLsizei *length, GLchar *name)
    void glGetActiveSubroutineUniformName(GLuint program, GLenum shadertype, GLuint index, GLsizei bufsize, GLsizei *length, GLchar *name)
    void glGetActiveSubroutineUniformiv(GLuint program, GLenum shadertype, GLuint index, GLenum pname, GLint *values)
    void glGetProgramStageiv(GLuint program, GLenum shadertype, GLenum pname, GLint *values)
    GLuint glGetSubroutineIndex(GLuint program, GLenum shadertype, GLchar *name)
    GLint glGetSubroutineUniformLocation(GLuint program, GLenum shadertype, GLchar *name)
    void glGetUniformSubroutineuiv(GLenum shadertype, GLint location, GLuint *params)
    void glUniformSubroutinesuiv(GLenum shadertype, GLsizei count, GLuint *indices)
    void glCompileShaderIncludeARB(GLuint shader, GLsizei count, GLchar * *path, GLint *length)
    void glDeleteNamedStringARB(GLint namelen, GLchar *name)
    void glGetNamedStringARB(GLint namelen, GLchar *name, GLsizei bufSize, GLint *stringlen, GLchar *string)
    void glGetNamedStringivARB(GLint namelen, GLchar *name, GLenum pname, GLint *params)
    GLboolean glIsNamedStringARB(GLint namelen, GLchar *name)
    void glNamedStringARB(GLenum type, GLint namelen, GLchar *name, GLint stringlen, GLchar *string)
    void glBufferPageCommitmentARB(GLenum target, GLintptr offset, GLsizeiptr size, GLboolean commit)
    void glTexPageCommitmentARB(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLboolean commit)
    GLenum glClientWaitSync(GLsync GLsync, GLbitfield flags, GLuint64 timeout)
    void glDeleteSync(GLsync GLsync)
    GLsync glFenceSync(GLenum condition, GLbitfield flags)
    void glGetInteger64v(GLenum pname, GLint64 *params)
    void glGetSynciv(GLsync GLsync, GLenum pname, GLsizei bufSize, GLsizei *length, GLint *values)
    GLboolean glIsSync(GLsync GLsync)
    void glWaitSync(GLsync GLsync, GLbitfield flags, GLuint64 timeout)
    void glPatchParameterfv(GLenum pname, GLfloat *values)
    void glPatchParameteri(GLenum pname, GLint value)
    void glTextureBarrier()
    void glTexBufferARB(GLenum target, GLenum internalformat, GLuint buffer)
    void glTexBufferRange(GLenum target, GLenum internalformat, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glTextureBufferRangeEXT(GLuint texture, GLenum target, GLenum internalformat, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glCompressedTexImage1DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexImage2DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexImage3DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexSubImage1DARB(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTexSubImage2DARB(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTexSubImage3DARB(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glGetCompressedTexImageARB(GLenum target, GLint lod, void *img)
    void glGetMultisamplefv(GLenum pname, GLuint index, GLfloat *val)
    void glSampleMaski(GLuint index, GLbitfield mask)
    void glTexImage2DMultisample(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLboolean fixedsamplelocations)
    void glTexImage3DMultisample(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedsamplelocations)
    void glTexStorage1D(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width)
    void glTexStorage2D(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height)
    void glTexStorage3D(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth)
    void glTexStorage2DMultisample(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLboolean fixedsamplelocations)
    void glTexStorage3DMultisample(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedsamplelocations)
    void glTextureStorage2DMultisampleEXT(GLuint texture, GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLboolean fixedsamplelocations)
    void glTextureStorage3DMultisampleEXT(GLuint texture, GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedsamplelocations)
    void glTextureView(GLuint texture, GLenum target, GLuint origtexture, GLenum internalformat, GLuint minlevel, GLuint numlevels, GLuint minlayer, GLuint numlayers)
    void glGetQueryObjecti64v(GLuint id, GLenum pname, GLint64 *params)
    void glGetQueryObjectui64v(GLuint id, GLenum pname, GLuint64 *params)
    void glQueryCounter(GLuint id, GLenum target)
    void glBindTransformFeedback(GLenum target, GLuint id)
    void glDeleteTransformFeedbacks(GLsizei n, GLuint *ids)
    void glDrawTransformFeedback(GLenum mode, GLuint id)
    void glGenTransformFeedbacks(GLsizei n, GLuint *ids)
    GLboolean glIsTransformFeedback(GLuint id)
    void glPauseTransformFeedback()
    void glResumeTransformFeedback()
    void glBeginQueryIndexed(GLenum target, GLuint index, GLuint id)
    void glDrawTransformFeedbackStream(GLenum mode, GLuint id, GLuint stream)
    void glEndQueryIndexed(GLenum target, GLuint index)
    void glGetQueryIndexediv(GLenum target, GLuint index, GLenum pname, GLint *params)
    void glDrawTransformFeedbackInstanced(GLenum mode, GLuint id, GLsizei primcount)
    void glDrawTransformFeedbackStreamInstanced(GLenum mode, GLuint id, GLuint stream, GLsizei primcount)
    void glLoadTransposeMatrixdARB(GLdouble m[16])
    void glLoadTransposeMatrixfARB(GLfloat m[16])
    void glMultTransposeMatrixdARB(GLdouble m[16])
    void glMultTransposeMatrixfARB(GLfloat m[16])
    void glBindBufferBase(GLenum target, GLuint index, GLuint buffer)
    void glBindBufferRange(GLenum target, GLuint index, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glGetActiveUniformBlockName(GLuint program, GLuint uniformBlockIndex, GLsizei bufSize, GLsizei *length, GLchar *uniformBlockName)
    void glGetActiveUniformBlockiv(GLuint program, GLuint uniformBlockIndex, GLenum pname, GLint *params)
    void glGetActiveUniformName(GLuint program, GLuint uniformIndex, GLsizei bufSize, GLsizei *length, GLchar *uniformName)
    void glGetActiveUniformsiv(GLuint program, GLsizei uniformCount, GLuint *uniformIndices, GLenum pname, GLint *params)
    void glGetIntegeri_v(GLenum target, GLuint index, GLint *data)
    GLuint glGetUniformBlockIndex(GLuint program, GLchar *uniformBlockName)
    void glGetUniformIndices(GLuint program, GLsizei uniformCount, GLchar * *uniformNames, GLuint *uniformIndices)
    void glUniformBlockBinding(GLuint program, GLuint uniformBlockIndex, GLuint uniformBlockBinding)
    void glBindVertexArray(GLuint array)
    void glDeleteVertexArrays(GLsizei n, GLuint *arrays)
    void glGenVertexArrays(GLsizei n, GLuint *arrays)
    GLboolean glIsVertexArray(GLuint array)
    void glGetVertexAttribLdv(GLuint index, GLenum pname, GLdouble *params)
    void glVertexAttribL1d(GLuint index, GLdouble x)
    void glVertexAttribL1dv(GLuint index, GLdouble *v)
    void glVertexAttribL2d(GLuint index, GLdouble x, GLdouble y)
    void glVertexAttribL2dv(GLuint index, GLdouble *v)
    void glVertexAttribL3d(GLuint index, GLdouble x, GLdouble y, GLdouble z)
    void glVertexAttribL3dv(GLuint index, GLdouble *v)
    void glVertexAttribL4d(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexAttribL4dv(GLuint index, GLdouble *v)
    void glVertexAttribLPointer(GLuint index, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glBindVertexBuffer(GLuint bindingindex, GLuint buffer, GLintptr offset, GLsizei stride)
    void glVertexArrayBindVertexBufferEXT(GLuint vaobj, GLuint bindingindex, GLuint buffer, GLintptr offset, GLsizei stride)
    void glVertexArrayVertexAttribBindingEXT(GLuint vaobj, GLuint attribindex, GLuint bindingindex)
    void glVertexArrayVertexAttribFormatEXT(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLboolean normalized, GLuint relativeoffset)
    void glVertexArrayVertexAttribIFormatEXT(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexArrayVertexAttribLFormatEXT(GLuint vaobj, GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexArrayVertexBindingDivisorEXT(GLuint vaobj, GLuint bindingindex, GLuint divisor)
    void glVertexAttribBinding(GLuint attribindex, GLuint bindingindex)
    void glVertexAttribFormat(GLuint attribindex, GLint size, GLenum type, GLboolean normalized, GLuint relativeoffset)
    void glVertexAttribIFormat(GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexAttribLFormat(GLuint attribindex, GLint size, GLenum type, GLuint relativeoffset)
    void glVertexBindingDivisor(GLuint bindingindex, GLuint divisor)
    void glVertexBlendARB(GLint count)
    void glWeightPointerARB(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glWeightbvARB(GLint size, GLbyte *weights)
    void glWeightdvARB(GLint size, GLdouble *weights)
    void glWeightfvARB(GLint size, GLfloat *weights)
    void glWeightivARB(GLint size, GLint *weights)
    void glWeightsvARB(GLint size, GLshort *weights)
    void glWeightubvARB(GLint size, GLubyte *weights)
    void glWeightuivARB(GLint size, GLuint *weights)
    void glWeightusvARB(GLint size, GLushort *weights)
    void glBindBufferARB(GLenum target, GLuint buffer)
    void glBufferDataARB(GLenum target, GLsizeiptrARB size, void *data, GLenum usage)
    void glBufferSubDataARB(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void *data)
    void glDeleteBuffersARB(GLsizei n, GLuint *buffers)
    void glGenBuffersARB(GLsizei n, GLuint *buffers)
    void glGetBufferParameterivARB(GLenum target, GLenum pname, GLint *params)
    void glGetBufferPointervARB(GLenum target, GLenum pname, void **params)
    void glGetBufferSubDataARB(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void *data)
    GLboolean glIsBufferARB(GLuint buffer)
    void *glMapBufferARB(GLenum target, GLenum access)
    GLboolean glUnmapBufferARB(GLenum target)
    void glBindProgramARB(GLenum target, GLuint program)
    void glDeleteProgramsARB(GLsizei n, GLuint *programs)
    void glDisableVertexAttribArrayARB(GLuint index)
    void glEnableVertexAttribArrayARB(GLuint index)
    void glGenProgramsARB(GLsizei n, GLuint *programs)
    void glGetProgramEnvParameterdvARB(GLenum target, GLuint index, GLdouble *params)
    void glGetProgramEnvParameterfvARB(GLenum target, GLuint index, GLfloat *params)
    void glGetProgramLocalParameterdvARB(GLenum target, GLuint index, GLdouble *params)
    void glGetProgramLocalParameterfvARB(GLenum target, GLuint index, GLfloat *params)
    void glGetProgramStringARB(GLenum target, GLenum pname, void *string)
    void glGetProgramivARB(GLenum target, GLenum pname, GLint *params)
    void glGetVertexAttribPointervARB(GLuint index, GLenum pname, void **pointer)
    void glGetVertexAttribdvARB(GLuint index, GLenum pname, GLdouble *params)
    void glGetVertexAttribfvARB(GLuint index, GLenum pname, GLfloat *params)
    void glGetVertexAttribivARB(GLuint index, GLenum pname, GLint *params)
    GLboolean glIsProgramARB(GLuint program)
    void glProgramEnvParameter4dARB(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glProgramEnvParameter4dvARB(GLenum target, GLuint index, GLdouble *params)
    void glProgramEnvParameter4fARB(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glProgramEnvParameter4fvARB(GLenum target, GLuint index, GLfloat *params)
    void glProgramLocalParameter4dARB(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glProgramLocalParameter4dvARB(GLenum target, GLuint index, GLdouble *params)
    void glProgramLocalParameter4fARB(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glProgramLocalParameter4fvARB(GLenum target, GLuint index, GLfloat *params)
    void glProgramStringARB(GLenum target, GLenum format, GLsizei len, void *string)
    void glVertexAttrib1dARB(GLuint index, GLdouble x)
    void glVertexAttrib1dvARB(GLuint index, GLdouble *v)
    void glVertexAttrib1fARB(GLuint index, GLfloat x)
    void glVertexAttrib1fvARB(GLuint index, GLfloat *v)
    void glVertexAttrib1sARB(GLuint index, GLshort x)
    void glVertexAttrib1svARB(GLuint index, GLshort *v)
    void glVertexAttrib2dARB(GLuint index, GLdouble x, GLdouble y)
    void glVertexAttrib2dvARB(GLuint index, GLdouble *v)
    void glVertexAttrib2fARB(GLuint index, GLfloat x, GLfloat y)
    void glVertexAttrib2fvARB(GLuint index, GLfloat *v)
    void glVertexAttrib2sARB(GLuint index, GLshort x, GLshort y)
    void glVertexAttrib2svARB(GLuint index, GLshort *v)
    void glVertexAttrib3dARB(GLuint index, GLdouble x, GLdouble y, GLdouble z)
    void glVertexAttrib3dvARB(GLuint index, GLdouble *v)
    void glVertexAttrib3fARB(GLuint index, GLfloat x, GLfloat y, GLfloat z)
    void glVertexAttrib3fvARB(GLuint index, GLfloat *v)
    void glVertexAttrib3sARB(GLuint index, GLshort x, GLshort y, GLshort z)
    void glVertexAttrib3svARB(GLuint index, GLshort *v)
    void glVertexAttrib4NbvARB(GLuint index, GLbyte *v)
    void glVertexAttrib4NivARB(GLuint index, GLint *v)
    void glVertexAttrib4NsvARB(GLuint index, GLshort *v)
    void glVertexAttrib4NubARB(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w)
    void glVertexAttrib4NubvARB(GLuint index, GLubyte *v)
    void glVertexAttrib4NuivARB(GLuint index, GLuint *v)
    void glVertexAttrib4NusvARB(GLuint index, GLushort *v)
    void glVertexAttrib4bvARB(GLuint index, GLbyte *v)
    void glVertexAttrib4dARB(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexAttrib4dvARB(GLuint index, GLdouble *v)
    void glVertexAttrib4fARB(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glVertexAttrib4fvARB(GLuint index, GLfloat *v)
    void glVertexAttrib4ivARB(GLuint index, GLint *v)
    void glVertexAttrib4sARB(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w)
    void glVertexAttrib4svARB(GLuint index, GLshort *v)
    void glVertexAttrib4ubvARB(GLuint index, GLubyte *v)
    void glVertexAttrib4uivARB(GLuint index, GLuint *v)
    void glVertexAttrib4usvARB(GLuint index, GLushort *v)
    void glVertexAttribPointerARB(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, void *pointer)
    void glBindAttribLocationARB(GLhandleARB programObj, GLuint index, GLcharARB *name)
    void glGetActiveAttribARB(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, GLint *size, GLenum *type, GLcharARB *name)
    GLint glGetAttribLocationARB(GLhandleARB programObj, GLcharARB *name)
    void glColorP3ui(GLenum type, GLuint color)
    void glColorP3uiv(GLenum type, GLuint *color)
    void glColorP4ui(GLenum type, GLuint color)
    void glColorP4uiv(GLenum type, GLuint *color)
    void glMultiTexCoordP1ui(GLenum texture, GLenum type, GLuint coords)
    void glMultiTexCoordP1uiv(GLenum texture, GLenum type, GLuint *coords)
    void glMultiTexCoordP2ui(GLenum texture, GLenum type, GLuint coords)
    void glMultiTexCoordP2uiv(GLenum texture, GLenum type, GLuint *coords)
    void glMultiTexCoordP3ui(GLenum texture, GLenum type, GLuint coords)
    void glMultiTexCoordP3uiv(GLenum texture, GLenum type, GLuint *coords)
    void glMultiTexCoordP4ui(GLenum texture, GLenum type, GLuint coords)
    void glMultiTexCoordP4uiv(GLenum texture, GLenum type, GLuint *coords)
    void glNormalP3ui(GLenum type, GLuint coords)
    void glNormalP3uiv(GLenum type, GLuint *coords)
    void glSecondaryColorP3ui(GLenum type, GLuint color)
    void glSecondaryColorP3uiv(GLenum type, GLuint *color)
    void glTexCoordP1ui(GLenum type, GLuint coords)
    void glTexCoordP1uiv(GLenum type, GLuint *coords)
    void glTexCoordP2ui(GLenum type, GLuint coords)
    void glTexCoordP2uiv(GLenum type, GLuint *coords)
    void glTexCoordP3ui(GLenum type, GLuint coords)
    void glTexCoordP3uiv(GLenum type, GLuint *coords)
    void glTexCoordP4ui(GLenum type, GLuint coords)
    void glTexCoordP4uiv(GLenum type, GLuint *coords)
    void glVertexAttribP1ui(GLuint index, GLenum type, GLboolean normalized, GLuint value)
    void glVertexAttribP1uiv(GLuint index, GLenum type, GLboolean normalized, GLuint *value)
    void glVertexAttribP2ui(GLuint index, GLenum type, GLboolean normalized, GLuint value)
    void glVertexAttribP2uiv(GLuint index, GLenum type, GLboolean normalized, GLuint *value)
    void glVertexAttribP3ui(GLuint index, GLenum type, GLboolean normalized, GLuint value)
    void glVertexAttribP3uiv(GLuint index, GLenum type, GLboolean normalized, GLuint *value)
    void glVertexAttribP4ui(GLuint index, GLenum type, GLboolean normalized, GLuint value)
    void glVertexAttribP4uiv(GLuint index, GLenum type, GLboolean normalized, GLuint *value)
    void glVertexP2ui(GLenum type, GLuint value)
    void glVertexP2uiv(GLenum type, GLuint *value)
    void glVertexP3ui(GLenum type, GLuint value)
    void glVertexP3uiv(GLenum type, GLuint *value)
    void glVertexP4ui(GLenum type, GLuint value)
    void glVertexP4uiv(GLenum type, GLuint *value)
    void glDepthRangeArrayv(GLuint first, GLsizei count, GLclampd *v)
    void glDepthRangeIndexed(GLuint index, GLclampd n, GLclampd f)
    void glGetDoublei_v(GLenum target, GLuint index, GLdouble *data)
    void glGetFloati_v(GLenum target, GLuint index, GLfloat *data)
    void glScissorArrayv(GLuint first, GLsizei count, GLint *v)
    void glScissorIndexed(GLuint index, GLint left, GLint bottom, GLsizei width, GLsizei height)
    void glScissorIndexedv(GLuint index, GLint *v)
    void glViewportArrayv(GLuint first, GLsizei count, GLfloat *v)
    void glViewportIndexedf(GLuint index, GLfloat x, GLfloat y, GLfloat w, GLfloat h)
    void glViewportIndexedfv(GLuint index, GLfloat *v)
    void glWindowPos2dARB(GLdouble x, GLdouble y)
    void glWindowPos2dvARB(GLdouble *p)
    void glWindowPos2fARB(GLfloat x, GLfloat y)
    void glWindowPos2fvARB(GLfloat *p)
    void glWindowPos2iARB(GLint x, GLint y)
    void glWindowPos2ivARB(GLint *p)
    void glWindowPos2sARB(GLshort x, GLshort y)
    void glWindowPos2svARB(GLshort *p)
    void glWindowPos3dARB(GLdouble x, GLdouble y, GLdouble z)
    void glWindowPos3dvARB(GLdouble *p)
    void glWindowPos3fARB(GLfloat x, GLfloat y, GLfloat z)
    void glWindowPos3fvARB(GLfloat *p)
    void glWindowPos3iARB(GLint x, GLint y, GLint z)
    void glWindowPos3ivARB(GLint *p)
    void glWindowPos3sARB(GLshort x, GLshort y, GLshort z)
    void glWindowPos3svARB(GLshort *p)
    void glDrawBuffersATI(GLsizei n, GLenum *bufs)
    void glDrawElementArrayATI(GLenum mode, GLsizei count)
    void glDrawRangeElementArrayATI(GLenum mode, GLuint start, GLuint end, GLsizei count)
    void glElementPointerATI(GLenum type, void *pointer)
    void glGetTexBumpParameterfvATI(GLenum pname, GLfloat *param)
    void glGetTexBumpParameterivATI(GLenum pname, GLint *param)
    void glTexBumpParameterfvATI(GLenum pname, GLfloat *param)
    void glTexBumpParameterivATI(GLenum pname, GLint *param)
    void glAlphaFragmentOp1ATI(GLenum op, GLuint dst, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod)
    void glAlphaFragmentOp2ATI(GLenum op, GLuint dst, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod, GLuint arg2, GLuint arg2Rep, GLuint arg2Mod)
    void glAlphaFragmentOp3ATI(GLenum op, GLuint dst, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod, GLuint arg2, GLuint arg2Rep, GLuint arg2Mod, GLuint arg3, GLuint arg3Rep, GLuint arg3Mod)
    void glBeginFragmentShaderATI()
    void glBindFragmentShaderATI(GLuint id)
    void glColorFragmentOp1ATI(GLenum op, GLuint dst, GLuint dstMask, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod)
    void glColorFragmentOp2ATI(GLenum op, GLuint dst, GLuint dstMask, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod, GLuint arg2, GLuint arg2Rep, GLuint arg2Mod)
    void glColorFragmentOp3ATI(GLenum op, GLuint dst, GLuint dstMask, GLuint dstMod, GLuint arg1, GLuint arg1Rep, GLuint arg1Mod, GLuint arg2, GLuint arg2Rep, GLuint arg2Mod, GLuint arg3, GLuint arg3Rep, GLuint arg3Mod)
    void glDeleteFragmentShaderATI(GLuint id)
    void glEndFragmentShaderATI()
    GLuint glGenFragmentShadersATI(GLuint range)
    void glPassTexCoordATI(GLuint dst, GLuint coord, GLenum swizzle)
    void glSampleMapATI(GLuint dst, GLuint interp, GLenum swizzle)
    void glSetFragmentShaderConstantATI(GLuint dst, GLfloat *value)
    void *glMapObjectBufferATI(GLuint buffer)
    void glUnmapObjectBufferATI(GLuint buffer)
    void glPNTrianglesfATI(GLenum pname, GLfloat param)
    void glPNTrianglesiATI(GLenum pname, GLint param)
    void glStencilFuncSeparateATI(GLenum frontfunc, GLenum backfunc, GLint ref, GLuint mask)
    void glStencilOpSeparateATI(GLenum face, GLenum sfail, GLenum dpfail, GLenum dppass)
    void glArrayObjectATI(GLenum array, GLint size, GLenum type, GLsizei stride, GLuint buffer, GLuint offset)
    void glFreeObjectBufferATI(GLuint buffer)
    void glGetArrayObjectfvATI(GLenum array, GLenum pname, GLfloat *params)
    void glGetArrayObjectivATI(GLenum array, GLenum pname, GLint *params)
    void glGetObjectBufferfvATI(GLuint buffer, GLenum pname, GLfloat *params)
    void glGetObjectBufferivATI(GLuint buffer, GLenum pname, GLint *params)
    void glGetVariantArrayObjectfvATI(GLuint id, GLenum pname, GLfloat *params)
    void glGetVariantArrayObjectivATI(GLuint id, GLenum pname, GLint *params)
    GLboolean glIsObjectBufferATI(GLuint buffer)
    GLuint glNewObjectBufferATI(GLsizei size, void *pointer, GLenum usage)
    void glUpdateObjectBufferATI(GLuint buffer, GLuint offset, GLsizei size, void *pointer, GLenum preserve)
    void glVariantArrayObjectATI(GLuint id, GLenum type, GLsizei stride, GLuint buffer, GLuint offset)
    void glGetVertexAttribArrayObjectfvATI(GLuint index, GLenum pname, GLfloat *params)
    void glGetVertexAttribArrayObjectivATI(GLuint index, GLenum pname, GLint *params)
    void glVertexAttribArrayObjectATI(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, GLuint buffer, GLuint offset)
    void glClientActiveVertexStreamATI(GLenum stream)
    void glNormalStream3bATI(GLenum stream, GLbyte x, GLbyte y, GLbyte z)
    void glNormalStream3bvATI(GLenum stream, GLbyte *coords)
    void glNormalStream3dATI(GLenum stream, GLdouble x, GLdouble y, GLdouble z)
    void glNormalStream3dvATI(GLenum stream, GLdouble *coords)
    void glNormalStream3fATI(GLenum stream, GLfloat x, GLfloat y, GLfloat z)
    void glNormalStream3fvATI(GLenum stream, GLfloat *coords)
    void glNormalStream3iATI(GLenum stream, GLint x, GLint y, GLint z)
    void glNormalStream3ivATI(GLenum stream, GLint *coords)
    void glNormalStream3sATI(GLenum stream, GLshort x, GLshort y, GLshort z)
    void glNormalStream3svATI(GLenum stream, GLshort *coords)
    void glVertexBlendEnvfATI(GLenum pname, GLfloat param)
    void glVertexBlendEnviATI(GLenum pname, GLint param)
    void glVertexStream1dATI(GLenum stream, GLdouble x)
    void glVertexStream1dvATI(GLenum stream, GLdouble *coords)
    void glVertexStream1fATI(GLenum stream, GLfloat x)
    void glVertexStream1fvATI(GLenum stream, GLfloat *coords)
    void glVertexStream1iATI(GLenum stream, GLint x)
    void glVertexStream1ivATI(GLenum stream, GLint *coords)
    void glVertexStream1sATI(GLenum stream, GLshort x)
    void glVertexStream1svATI(GLenum stream, GLshort *coords)
    void glVertexStream2dATI(GLenum stream, GLdouble x, GLdouble y)
    void glVertexStream2dvATI(GLenum stream, GLdouble *coords)
    void glVertexStream2fATI(GLenum stream, GLfloat x, GLfloat y)
    void glVertexStream2fvATI(GLenum stream, GLfloat *coords)
    void glVertexStream2iATI(GLenum stream, GLint x, GLint y)
    void glVertexStream2ivATI(GLenum stream, GLint *coords)
    void glVertexStream2sATI(GLenum stream, GLshort x, GLshort y)
    void glVertexStream2svATI(GLenum stream, GLshort *coords)
    void glVertexStream3dATI(GLenum stream, GLdouble x, GLdouble y, GLdouble z)
    void glVertexStream3dvATI(GLenum stream, GLdouble *coords)
    void glVertexStream3fATI(GLenum stream, GLfloat x, GLfloat y, GLfloat z)
    void glVertexStream3fvATI(GLenum stream, GLfloat *coords)
    void glVertexStream3iATI(GLenum stream, GLint x, GLint y, GLint z)
    void glVertexStream3ivATI(GLenum stream, GLint *coords)
    void glVertexStream3sATI(GLenum stream, GLshort x, GLshort y, GLshort z)
    void glVertexStream3svATI(GLenum stream, GLshort *coords)
    void glVertexStream4dATI(GLenum stream, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexStream4dvATI(GLenum stream, GLdouble *coords)
    void glVertexStream4fATI(GLenum stream, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glVertexStream4fvATI(GLenum stream, GLfloat *coords)
    void glVertexStream4iATI(GLenum stream, GLint x, GLint y, GLint z, GLint w)
    void glVertexStream4ivATI(GLenum stream, GLint *coords)
    void glVertexStream4sATI(GLenum stream, GLshort x, GLshort y, GLshort z, GLshort w)
    void glVertexStream4svATI(GLenum stream, GLshort *coords)
    void glDrawArraysInstancedBaseInstanceEXT(GLenum mode, GLint first, GLsizei count, GLsizei instancecount, GLuint baseinstance)
    void glDrawElementsInstancedBaseInstanceEXT(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei instancecount, GLuint baseinstance)
    void glDrawElementsInstancedBaseVertexBaseInstanceEXT(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei instancecount, GLint basevertex, GLuint baseinstance)
    GLint glGetUniformBufferSizeEXT(GLuint program, GLint location)
    GLintptr glGetUniformOffsetEXT(GLuint program, GLint location)
    void glUniformBufferEXT(GLuint program, GLint location, GLuint buffer)
    void glBlendColorEXT(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    void glBlendEquationSeparateEXT(GLenum modeRGB, GLenum modeAlpha)
    void glBindFragDataLocationIndexedEXT(GLuint program, GLuint colorNumber, GLuint index, GLchar *name)
    GLint glGetFragDataIndexEXT(GLuint program, GLchar *name)
    GLint glGetProgramResourceLocationIndexEXT(GLuint program, GLenum programInterface, GLchar *name)
    void glBlendFuncSeparateEXT(GLenum sfactorRGB, GLenum dfactorRGB, GLenum sfactorAlpha, GLenum dfactorAlpha)
    void glBlendEquationEXT(GLenum mode)
    void glBufferStorageEXT(GLenum target, GLsizeiptr size, void *data, GLbitfield flags)
    void glNamedBufferStorageEXT(GLuint buffer, GLsizeiptr size, void *data, GLbitfield flags)
    void glClearTexImageEXT(GLuint texture, GLint level, GLenum format, GLenum type, void *data)
    void glClearTexSubImageEXT(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *data)
    void glColorSubTableEXT(GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, void *data)
    void glCopyColorSubTableEXT(GLenum target, GLsizei start, GLint x, GLint y, GLsizei width)
    void glLockArraysEXT(GLint first, GLsizei count)
    void glUnlockArraysEXT()
    void glConvolutionFilter1DEXT(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, void *image)
    void glConvolutionFilter2DEXT(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, void *image)
    void glConvolutionParameterfEXT(GLenum target, GLenum pname, GLfloat param)
    void glConvolutionParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glConvolutionParameteriEXT(GLenum target, GLenum pname, GLint param)
    void glConvolutionParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glCopyConvolutionFilter1DEXT(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width)
    void glCopyConvolutionFilter2DEXT(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height)
    void glGetConvolutionFilterEXT(GLenum target, GLenum format, GLenum type, void *image)
    void glGetConvolutionParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glGetConvolutionParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glGetSeparableFilterEXT(GLenum target, GLenum format, GLenum type, void *row, void *column, void *span)
    void glSeparableFilter2DEXT(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, void *row, void *column)
    void glBinormalPointerEXT(GLenum type, GLsizei stride, void *pointer)
    void glTangentPointerEXT(GLenum type, GLsizei stride, void *pointer)
    void glCopyImageSubDataEXT(GLuint srcName, GLenum srcTarget, GLint srcLevel, GLint srcX, GLint srcY, GLint srcZ, GLuint dstName, GLenum dstTarget, GLint dstLevel, GLint dstX, GLint dstY, GLint dstZ, GLsizei srcWidth, GLsizei srcHeight, GLsizei srcDepth)
    void glCopyTexImage1DEXT(GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border)
    void glCopyTexImage2DEXT(GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border)
    void glCopyTexSubImage1DEXT(GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width)
    void glCopyTexSubImage2DEXT(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCopyTexSubImage3DEXT(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCullParameterdvEXT(GLenum pname, GLdouble *params)
    void glCullParameterfvEXT(GLenum pname, GLfloat *params)
    void glGetObjectLabelEXT(GLenum type, GLuint object, GLsizei bufSize, GLsizei *length, GLchar *label)
    void glLabelObjectEXT(GLenum type, GLuint object, GLsizei length, GLchar *label)
    void glInsertEventMarkerEXT(GLsizei length, GLchar *marker)
    void glPopGroupMarkerEXT()
    void glPushGroupMarkerEXT(GLsizei length, GLchar *marker)
    void glDepthBoundsEXT(GLclampd zmin, GLclampd zmax)
    void glBindMultiTextureEXT(GLenum texunit, GLenum target, GLuint texture)
    GLenum glCheckNamedFramebufferStatusEXT(GLuint framebuffer, GLenum target)
    void glClientAttribDefaultEXT(GLbitfield mask)
    void glCompressedMultiTexImage1DEXT(GLenum texunit, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, void *data)
    void glCompressedMultiTexImage2DEXT(GLenum texunit, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, void *data)
    void glCompressedMultiTexImage3DEXT(GLenum texunit, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, void *data)
    void glCompressedMultiTexSubImage1DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, void *data)
    void glCompressedMultiTexSubImage2DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, void *data)
    void glCompressedMultiTexSubImage3DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTextureImage1DEXT(GLuint texture, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, void *data)
    void glCompressedTextureImage2DEXT(GLuint texture, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, void *data)
    void glCompressedTextureImage3DEXT(GLuint texture, GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, void *data)
    void glCompressedTextureSubImage1DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTextureSubImage2DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, void *data)
    void glCompressedTextureSubImage3DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glCopyMultiTexImage1DEXT(GLenum texunit, GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border)
    void glCopyMultiTexImage2DEXT(GLenum texunit, GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border)
    void glCopyMultiTexSubImage1DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width)
    void glCopyMultiTexSubImage2DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCopyMultiTexSubImage3DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCopyTextureImage1DEXT(GLuint texture, GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border)
    void glCopyTextureImage2DEXT(GLuint texture, GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border)
    void glCopyTextureSubImage1DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width)
    void glCopyTextureSubImage2DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCopyTextureSubImage3DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glDisableClientStateIndexedEXT(GLenum array, GLuint index)
    void glDisableClientStateiEXT(GLenum array, GLuint index)
    void glDisableVertexArrayAttribEXT(GLuint vaobj, GLuint index)
    void glDisableVertexArrayEXT(GLuint vaobj, GLenum array)
    void glEnableClientStateIndexedEXT(GLenum array, GLuint index)
    void glEnableClientStateiEXT(GLenum array, GLuint index)
    void glEnableVertexArrayAttribEXT(GLuint vaobj, GLuint index)
    void glEnableVertexArrayEXT(GLuint vaobj, GLenum array)
    void glFlushMappedNamedBufferRangeEXT(GLuint buffer, GLintptr offset, GLsizeiptr length)
    void glFramebufferDrawBufferEXT(GLuint framebuffer, GLenum mode)
    void glFramebufferDrawBuffersEXT(GLuint framebuffer, GLsizei n, GLenum *bufs)
    void glFramebufferReadBufferEXT(GLuint framebuffer, GLenum mode)
    void glGenerateMultiTexMipmapEXT(GLenum texunit, GLenum target)
    void glGenerateTextureMipmapEXT(GLuint texture, GLenum target)
    void glGetCompressedMultiTexImageEXT(GLenum texunit, GLenum target, GLint level, void *img)
    void glGetCompressedTextureImageEXT(GLuint texture, GLenum target, GLint level, void *img)
    void glGetDoubleIndexedvEXT(GLenum target, GLuint index, GLdouble *params)
    void glGetDoublei_vEXT(GLenum pname, GLuint index, GLdouble *params)
    void glGetFloatIndexedvEXT(GLenum target, GLuint index, GLfloat *params)
    void glGetFloati_vEXT(GLenum pname, GLuint index, GLfloat *params)
    void glGetFramebufferParameterivEXT(GLuint framebuffer, GLenum pname, GLint *param)
    void glGetMultiTexEnvfvEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat *params)
    void glGetMultiTexEnvivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *params)
    void glGetMultiTexGendvEXT(GLenum texunit, GLenum coord, GLenum pname, GLdouble *params)
    void glGetMultiTexGenfvEXT(GLenum texunit, GLenum coord, GLenum pname, GLfloat *params)
    void glGetMultiTexGenivEXT(GLenum texunit, GLenum coord, GLenum pname, GLint *params)
    void glGetMultiTexImageEXT(GLenum texunit, GLenum target, GLint level, GLenum format, GLenum type, void *pixels)
    void glGetMultiTexLevelParameterfvEXT(GLenum texunit, GLenum target, GLint level, GLenum pname, GLfloat *params)
    void glGetMultiTexLevelParameterivEXT(GLenum texunit, GLenum target, GLint level, GLenum pname, GLint *params)
    void glGetMultiTexParameterIivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *params)
    void glGetMultiTexParameterIuivEXT(GLenum texunit, GLenum target, GLenum pname, GLuint *params)
    void glGetMultiTexParameterfvEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat *params)
    void glGetMultiTexParameterivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *params)
    void glGetNamedBufferParameterivEXT(GLuint buffer, GLenum pname, GLint *params)
    void glGetNamedBufferPointervEXT(GLuint buffer, GLenum pname, void **params)
    void glGetNamedBufferSubDataEXT(GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glGetNamedFramebufferAttachmentParameterivEXT(GLuint framebuffer, GLenum attachment, GLenum pname, GLint *params)
    void glGetNamedProgramLocalParameterIivEXT(GLuint program, GLenum target, GLuint index, GLint *params)
    void glGetNamedProgramLocalParameterIuivEXT(GLuint program, GLenum target, GLuint index, GLuint *params)
    void glGetNamedProgramLocalParameterdvEXT(GLuint program, GLenum target, GLuint index, GLdouble *params)
    void glGetNamedProgramLocalParameterfvEXT(GLuint program, GLenum target, GLuint index, GLfloat *params)
    void glGetNamedProgramStringEXT(GLuint program, GLenum target, GLenum pname, void *string)
    void glGetNamedProgramivEXT(GLuint program, GLenum target, GLenum pname, GLint *params)
    void glGetNamedRenderbufferParameterivEXT(GLuint renderbuffer, GLenum pname, GLint *params)
    void glGetPointerIndexedvEXT(GLenum target, GLuint index, void **params)
    void glGetPointeri_vEXT(GLenum pname, GLuint index, void **params)
    void glGetTextureImageEXT(GLuint texture, GLenum target, GLint level, GLenum format, GLenum type, void *pixels)
    void glGetTextureLevelParameterfvEXT(GLuint texture, GLenum target, GLint level, GLenum pname, GLfloat *params)
    void glGetTextureLevelParameterivEXT(GLuint texture, GLenum target, GLint level, GLenum pname, GLint *params)
    void glGetTextureParameterIivEXT(GLuint texture, GLenum target, GLenum pname, GLint *params)
    void glGetTextureParameterIuivEXT(GLuint texture, GLenum target, GLenum pname, GLuint *params)
    void glGetTextureParameterfvEXT(GLuint texture, GLenum target, GLenum pname, GLfloat *params)
    void glGetTextureParameterivEXT(GLuint texture, GLenum target, GLenum pname, GLint *params)
    void glGetVertexArrayIntegeri_vEXT(GLuint vaobj, GLuint index, GLenum pname, GLint *param)
    void glGetVertexArrayIntegervEXT(GLuint vaobj, GLenum pname, GLint *param)
    void glGetVertexArrayPointeri_vEXT(GLuint vaobj, GLuint index, GLenum pname, void **param)
    void glGetVertexArrayPointervEXT(GLuint vaobj, GLenum pname, void **param)
    void *glMapNamedBufferEXT(GLuint buffer, GLenum access)
    void *glMapNamedBufferRangeEXT(GLuint buffer, GLintptr offset, GLsizeiptr length, GLbitfield access)
    void glMatrixFrustumEXT(GLenum matrixMode, GLdouble l, GLdouble r, GLdouble b, GLdouble t, GLdouble n, GLdouble f)
    void glMatrixLoadIdentityEXT(GLenum matrixMode)
    void glMatrixLoadTransposedEXT(GLenum matrixMode, GLdouble *m)
    void glMatrixLoadTransposefEXT(GLenum matrixMode, GLfloat *m)
    void glMatrixLoaddEXT(GLenum matrixMode, GLdouble *m)
    void glMatrixLoadfEXT(GLenum matrixMode, GLfloat *m)
    void glMatrixMultTransposedEXT(GLenum matrixMode, GLdouble *m)
    void glMatrixMultTransposefEXT(GLenum matrixMode, GLfloat *m)
    void glMatrixMultdEXT(GLenum matrixMode, GLdouble *m)
    void glMatrixMultfEXT(GLenum matrixMode, GLfloat *m)
    void glMatrixOrthoEXT(GLenum matrixMode, GLdouble l, GLdouble r, GLdouble b, GLdouble t, GLdouble n, GLdouble f)
    void glMatrixPopEXT(GLenum matrixMode)
    void glMatrixPushEXT(GLenum matrixMode)
    void glMatrixRotatedEXT(GLenum matrixMode, GLdouble angle, GLdouble x, GLdouble y, GLdouble z)
    void glMatrixRotatefEXT(GLenum matrixMode, GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
    void glMatrixScaledEXT(GLenum matrixMode, GLdouble x, GLdouble y, GLdouble z)
    void glMatrixScalefEXT(GLenum matrixMode, GLfloat x, GLfloat y, GLfloat z)
    void glMatrixTranslatedEXT(GLenum matrixMode, GLdouble x, GLdouble y, GLdouble z)
    void glMatrixTranslatefEXT(GLenum matrixMode, GLfloat x, GLfloat y, GLfloat z)
    void glMultiTexBufferEXT(GLenum texunit, GLenum target, GLenum internalformat, GLuint buffer)
    void glMultiTexCoordPointerEXT(GLenum texunit, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glMultiTexEnvfEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat param)
    void glMultiTexEnvfvEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat *params)
    void glMultiTexEnviEXT(GLenum texunit, GLenum target, GLenum pname, GLint param)
    void glMultiTexEnvivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *params)
    void glMultiTexGendEXT(GLenum texunit, GLenum coord, GLenum pname, GLdouble param)
    void glMultiTexGendvEXT(GLenum texunit, GLenum coord, GLenum pname, GLdouble *params)
    void glMultiTexGenfEXT(GLenum texunit, GLenum coord, GLenum pname, GLfloat param)
    void glMultiTexGenfvEXT(GLenum texunit, GLenum coord, GLenum pname, GLfloat *params)
    void glMultiTexGeniEXT(GLenum texunit, GLenum coord, GLenum pname, GLint param)
    void glMultiTexGenivEXT(GLenum texunit, GLenum coord, GLenum pname, GLint *params)
    void glMultiTexImage1DEXT(GLenum texunit, GLenum target, GLint level, GLint internalformat, GLsizei width, GLint border, GLenum format, GLenum type, void *pixels)
    void glMultiTexImage2DEXT(GLenum texunit, GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, void *pixels)
    void glMultiTexImage3DEXT(GLenum texunit, GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, void *pixels)
    void glMultiTexParameterIivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *params)
    void glMultiTexParameterIuivEXT(GLenum texunit, GLenum target, GLenum pname, GLuint *params)
    void glMultiTexParameterfEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat param)
    void glMultiTexParameterfvEXT(GLenum texunit, GLenum target, GLenum pname, GLfloat *param)
    void glMultiTexParameteriEXT(GLenum texunit, GLenum target, GLenum pname, GLint param)
    void glMultiTexParameterivEXT(GLenum texunit, GLenum target, GLenum pname, GLint *param)
    void glMultiTexRenderbufferEXT(GLenum texunit, GLenum target, GLuint renderbuffer)
    void glMultiTexSubImage1DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, void *pixels)
    void glMultiTexSubImage2DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glMultiTexSubImage3DEXT(GLenum texunit, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    void glNamedBufferDataEXT(GLuint buffer, GLsizeiptr size, void *data, GLenum usage)
    void glNamedBufferSubDataEXT(GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glNamedCopyBufferSubDataEXT(GLuint readBuffer, GLuint writeBuffer, GLintptr readOffset, GLintptr writeOffset, GLsizeiptr size)
    void glNamedFramebufferRenderbufferEXT(GLuint framebuffer, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
    void glNamedFramebufferTexture1DEXT(GLuint framebuffer, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glNamedFramebufferTexture2DEXT(GLuint framebuffer, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glNamedFramebufferTexture3DEXT(GLuint framebuffer, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset)
    void glNamedFramebufferTextureEXT(GLuint framebuffer, GLenum attachment, GLuint texture, GLint level)
    void glNamedFramebufferTextureFaceEXT(GLuint framebuffer, GLenum attachment, GLuint texture, GLint level, GLenum face)
    void glNamedFramebufferTextureLayerEXT(GLuint framebuffer, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glNamedProgramLocalParameter4dEXT(GLuint program, GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glNamedProgramLocalParameter4dvEXT(GLuint program, GLenum target, GLuint index, GLdouble *params)
    void glNamedProgramLocalParameter4fEXT(GLuint program, GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glNamedProgramLocalParameter4fvEXT(GLuint program, GLenum target, GLuint index, GLfloat *params)
    void glNamedProgramLocalParameterI4iEXT(GLuint program, GLenum target, GLuint index, GLint x, GLint y, GLint z, GLint w)
    void glNamedProgramLocalParameterI4ivEXT(GLuint program, GLenum target, GLuint index, GLint *params)
    void glNamedProgramLocalParameterI4uiEXT(GLuint program, GLenum target, GLuint index, GLuint x, GLuint y, GLuint z, GLuint w)
    void glNamedProgramLocalParameterI4uivEXT(GLuint program, GLenum target, GLuint index, GLuint *params)
    void glNamedProgramLocalParameters4fvEXT(GLuint program, GLenum target, GLuint index, GLsizei count, GLfloat *params)
    void glNamedProgramLocalParametersI4ivEXT(GLuint program, GLenum target, GLuint index, GLsizei count, GLint *params)
    void glNamedProgramLocalParametersI4uivEXT(GLuint program, GLenum target, GLuint index, GLsizei count, GLuint *params)
    void glNamedProgramStringEXT(GLuint program, GLenum target, GLenum format, GLsizei len, void *string)
    void glNamedRenderbufferStorageEXT(GLuint renderbuffer, GLenum internalformat, GLsizei width, GLsizei height)
    void glNamedRenderbufferStorageMultisampleCoverageEXT(GLuint renderbuffer, GLsizei coverageSamples, GLsizei colorSamples, GLenum internalformat, GLsizei width, GLsizei height)
    void glNamedRenderbufferStorageMultisampleEXT(GLuint renderbuffer, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glProgramUniform1fEXT(GLuint program, GLint location, GLfloat v0)
    void glProgramUniform1fvEXT(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform1iEXT(GLuint program, GLint location, GLint v0)
    void glProgramUniform1ivEXT(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform1uiEXT(GLuint program, GLint location, GLuint v0)
    void glProgramUniform1uivEXT(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform2fEXT(GLuint program, GLint location, GLfloat v0, GLfloat v1)
    void glProgramUniform2fvEXT(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform2iEXT(GLuint program, GLint location, GLint v0, GLint v1)
    void glProgramUniform2ivEXT(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform2uiEXT(GLuint program, GLint location, GLuint v0, GLuint v1)
    void glProgramUniform2uivEXT(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform3fEXT(GLuint program, GLint location, GLfloat v0, GLfloat v1, GLfloat v2)
    void glProgramUniform3fvEXT(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform3iEXT(GLuint program, GLint location, GLint v0, GLint v1, GLint v2)
    void glProgramUniform3ivEXT(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform3uiEXT(GLuint program, GLint location, GLuint v0, GLuint v1, GLuint v2)
    void glProgramUniform3uivEXT(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniform4fEXT(GLuint program, GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
    void glProgramUniform4fvEXT(GLuint program, GLint location, GLsizei count, GLfloat *value)
    void glProgramUniform4iEXT(GLuint program, GLint location, GLint v0, GLint v1, GLint v2, GLint v3)
    void glProgramUniform4ivEXT(GLuint program, GLint location, GLsizei count, GLint *value)
    void glProgramUniform4uiEXT(GLuint program, GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3)
    void glProgramUniform4uivEXT(GLuint program, GLint location, GLsizei count, GLuint *value)
    void glProgramUniformMatrix2fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix2x3fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix2x4fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3x2fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix3x4fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4x2fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glProgramUniformMatrix4x3fvEXT(GLuint program, GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glPushClientAttribDefaultEXT(GLbitfield mask)
    void glTextureBufferEXT(GLuint texture, GLenum target, GLenum internalformat, GLuint buffer)
    void glTextureImage1DEXT(GLuint texture, GLenum target, GLint level, GLint internalformat, GLsizei width, GLint border, GLenum format, GLenum type, void *pixels)
    void glTextureImage2DEXT(GLuint texture, GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, void *pixels)
    void glTextureImage3DEXT(GLuint texture, GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, void *pixels)
    void glTextureParameterIivEXT(GLuint texture, GLenum target, GLenum pname, GLint *params)
    void glTextureParameterIuivEXT(GLuint texture, GLenum target, GLenum pname, GLuint *params)
    void glTextureParameterfEXT(GLuint texture, GLenum target, GLenum pname, GLfloat param)
    void glTextureParameterfvEXT(GLuint texture, GLenum target, GLenum pname, GLfloat *param)
    void glTextureParameteriEXT(GLuint texture, GLenum target, GLenum pname, GLint param)
    void glTextureParameterivEXT(GLuint texture, GLenum target, GLenum pname, GLint *param)
    void glTextureRenderbufferEXT(GLuint texture, GLenum target, GLuint renderbuffer)
    void glTextureSubImage1DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, void *pixels)
    void glTextureSubImage2DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glTextureSubImage3DEXT(GLuint texture, GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    GLboolean glUnmapNamedBufferEXT(GLuint buffer)
    void glVertexArrayColorOffsetEXT(GLuint vaobj, GLuint buffer, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayEdgeFlagOffsetEXT(GLuint vaobj, GLuint buffer, GLsizei stride, GLintptr offset)
    void glVertexArrayFogCoordOffsetEXT(GLuint vaobj, GLuint buffer, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayIndexOffsetEXT(GLuint vaobj, GLuint buffer, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayMultiTexCoordOffsetEXT(GLuint vaobj, GLuint buffer, GLenum texunit, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayNormalOffsetEXT(GLuint vaobj, GLuint buffer, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArraySecondaryColorOffsetEXT(GLuint vaobj, GLuint buffer, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayTexCoordOffsetEXT(GLuint vaobj, GLuint buffer, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayVertexAttribDivisorEXT(GLuint vaobj, GLuint index, GLuint divisor)
    void glVertexArrayVertexAttribIOffsetEXT(GLuint vaobj, GLuint buffer, GLuint index, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexArrayVertexAttribOffsetEXT(GLuint vaobj, GLuint buffer, GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, GLintptr offset)
    void glVertexArrayVertexOffsetEXT(GLuint vaobj, GLuint buffer, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glDiscardFramebufferEXT(GLenum target, GLsizei numAttachments, GLenum *attachments)
    void glDrawBuffersEXT(GLsizei n, GLenum *bufs)
    void glColorMaskIndexedEXT(GLuint buf, GLboolean r, GLboolean g, GLboolean b, GLboolean a)
    void glDisableIndexedEXT(GLenum target, GLuint index)
    void glEnableIndexedEXT(GLenum target, GLuint index)
    void glGetBooleanIndexedvEXT(GLenum value, GLuint index, GLboolean *data)
    void glGetIntegerIndexedvEXT(GLenum value, GLuint index, GLint *data)
    GLboolean glIsEnabledIndexedEXT(GLenum target, GLuint index)
    void glBlendEquationSeparateiEXT(GLuint buf, GLenum modeRGB, GLenum modeAlpha)
    void glBlendEquationiEXT(GLuint buf, GLenum mode)
    void glBlendFuncSeparateiEXT(GLuint buf, GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha)
    void glBlendFunciEXT(GLuint buf, GLenum src, GLenum dst)
    void glColorMaskiEXT(GLuint buf, GLboolean r, GLboolean g, GLboolean b, GLboolean a)
    void glDisableiEXT(GLenum target, GLuint index)
    void glEnableiEXT(GLenum target, GLuint index)
    GLboolean glIsEnablediEXT(GLenum target, GLuint index)
    void glDrawElementsBaseVertexEXT(GLenum mode, GLsizei count, GLenum type, void *indices, GLint basevertex)
    void glDrawElementsInstancedBaseVertexEXT(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei instancecount, GLint basevertex)
    void glDrawRangeElementsBaseVertexEXT(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, void *indices, GLint basevertex)
    void glMultiDrawElementsBaseVertexEXT(GLenum mode, GLsizei *count, GLenum type, void * *indices, GLsizei primcount, GLint *basevertex)
    void glDrawArraysInstancedEXT(GLenum mode, GLint start, GLsizei count, GLsizei primcount)
    void glDrawElementsInstancedEXT(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount)
    void glDrawRangeElementsEXT(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, void *indices)
    void glBufferStorageExternalEXT(GLenum target, GLintptr offset, GLsizeiptr size, GLeglClientBufferEXT clientBuffer, GLbitfield flags)
    void glNamedBufferStorageExternalEXT(GLuint buffer, GLintptr offset, GLsizeiptr size, GLeglClientBufferEXT clientBuffer, GLbitfield flags)
    void glFogCoordPointerEXT(GLenum type, GLsizei stride, void *pointer)
    void glFogCoorddEXT(GLdouble coord)
    void glFogCoorddvEXT(GLdouble *coord)
    void glFogCoordfEXT(GLfloat coord)
    void glFogCoordfvEXT(GLfloat *coord)
    void glFragmentColorMaterialEXT(GLenum face, GLenum mode)
    void glFragmentLightModelfEXT(GLenum pname, GLfloat param)
    void glFragmentLightModelfvEXT(GLenum pname, GLfloat *params)
    void glFragmentLightModeliEXT(GLenum pname, GLint param)
    void glFragmentLightModelivEXT(GLenum pname, GLint *params)
    void glFragmentLightfEXT(GLenum light, GLenum pname, GLfloat param)
    void glFragmentLightfvEXT(GLenum light, GLenum pname, GLfloat *params)
    void glFragmentLightiEXT(GLenum light, GLenum pname, GLint param)
    void glFragmentLightivEXT(GLenum light, GLenum pname, GLint *params)
    void glFragmentMaterialfEXT(GLenum face, GLenum pname, GLfloat param)
    void glFragmentMaterialfvEXT(GLenum face, GLenum pname, GLfloat *params)
    void glFragmentMaterialiEXT(GLenum face, GLenum pname, GLint param)
    void glFragmentMaterialivEXT(GLenum face, GLenum pname, GLint *params)
    void glGetFragmentLightfvEXT(GLenum light, GLenum pname, GLfloat *params)
    void glGetFragmentLightivEXT(GLenum light, GLenum pname, GLint *params)
    void glGetFragmentMaterialfvEXT(GLenum face, GLenum pname, GLfloat *params)
    void glGetFragmentMaterialivEXT(GLenum face, GLenum pname, GLint *params)
    void glLightEnviEXT(GLenum pname, GLint param)
    void glBlitFramebufferEXT(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    void glRenderbufferStorageMultisampleEXT(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glBindFramebufferEXT(GLenum target, GLuint framebuffer)
    void glBindRenderbufferEXT(GLenum target, GLuint renderbuffer)
    GLenum glCheckFramebufferStatusEXT(GLenum target)
    void glDeleteFramebuffersEXT(GLsizei n, GLuint *framebuffers)
    void glDeleteRenderbuffersEXT(GLsizei n, GLuint *renderbuffers)
    void glFramebufferRenderbufferEXT(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
    void glFramebufferTexture1DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glFramebufferTexture2DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
    void glFramebufferTexture3DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset)
    void glGenFramebuffersEXT(GLsizei n, GLuint *framebuffers)
    void glGenRenderbuffersEXT(GLsizei n, GLuint *renderbuffers)
    void glGenerateMipmapEXT(GLenum target)
    void glGetFramebufferAttachmentParameterivEXT(GLenum target, GLenum attachment, GLenum pname, GLint *params)
    void glGetRenderbufferParameterivEXT(GLenum target, GLenum pname, GLint *params)
    GLboolean glIsFramebufferEXT(GLuint framebuffer)
    GLboolean glIsRenderbufferEXT(GLuint renderbuffer)
    void glRenderbufferStorageEXT(GLenum target, GLenum internalformat, GLsizei width, GLsizei height)
    void glFramebufferTextureEXT(GLenum target, GLenum attachment, GLuint texture, GLint level)
    void glFramebufferTextureFaceEXT(GLenum target, GLenum attachment, GLuint texture, GLint level, GLenum face)
    void glProgramParameteriEXT(GLuint program, GLenum pname, GLint value)
    void glProgramEnvParameters4fvEXT(GLenum target, GLuint index, GLsizei count, GLfloat *params)
    void glProgramLocalParameters4fvEXT(GLenum target, GLuint index, GLsizei count, GLfloat *params)
    void glBindFragDataLocationEXT(GLuint program, GLuint color, GLchar *name)
    GLint glGetFragDataLocationEXT(GLuint program, GLchar *name)
    void glGetUniformuivEXT(GLuint program, GLint location, GLuint *params)
    void glGetVertexAttribIivEXT(GLuint index, GLenum pname, GLint *params)
    void glGetVertexAttribIuivEXT(GLuint index, GLenum pname, GLuint *params)
    void glUniform1uiEXT(GLint location, GLuint v0)
    void glUniform1uivEXT(GLint location, GLsizei count, GLuint *value)
    void glUniform2uiEXT(GLint location, GLuint v0, GLuint v1)
    void glUniform2uivEXT(GLint location, GLsizei count, GLuint *value)
    void glUniform3uiEXT(GLint location, GLuint v0, GLuint v1, GLuint v2)
    void glUniform3uivEXT(GLint location, GLsizei count, GLuint *value)
    void glUniform4uiEXT(GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3)
    void glUniform4uivEXT(GLint location, GLsizei count, GLuint *value)
    void glVertexAttribI1iEXT(GLuint index, GLint x)
    void glVertexAttribI1ivEXT(GLuint index, GLint *v)
    void glVertexAttribI1uiEXT(GLuint index, GLuint x)
    void glVertexAttribI1uivEXT(GLuint index, GLuint *v)
    void glVertexAttribI2iEXT(GLuint index, GLint x, GLint y)
    void glVertexAttribI2ivEXT(GLuint index, GLint *v)
    void glVertexAttribI2uiEXT(GLuint index, GLuint x, GLuint y)
    void glVertexAttribI2uivEXT(GLuint index, GLuint *v)
    void glVertexAttribI3iEXT(GLuint index, GLint x, GLint y, GLint z)
    void glVertexAttribI3ivEXT(GLuint index, GLint *v)
    void glVertexAttribI3uiEXT(GLuint index, GLuint x, GLuint y, GLuint z)
    void glVertexAttribI3uivEXT(GLuint index, GLuint *v)
    void glVertexAttribI4bvEXT(GLuint index, GLbyte *v)
    void glVertexAttribI4iEXT(GLuint index, GLint x, GLint y, GLint z, GLint w)
    void glVertexAttribI4ivEXT(GLuint index, GLint *v)
    void glVertexAttribI4svEXT(GLuint index, GLshort *v)
    void glVertexAttribI4ubvEXT(GLuint index, GLubyte *v)
    void glVertexAttribI4uiEXT(GLuint index, GLuint x, GLuint y, GLuint z, GLuint w)
    void glVertexAttribI4uivEXT(GLuint index, GLuint *v)
    void glVertexAttribI4usvEXT(GLuint index, GLushort *v)
    void glVertexAttribIPointerEXT(GLuint index, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glGetHistogramEXT(GLenum target, GLboolean reset, GLenum format, GLenum type, void *values)
    void glGetHistogramParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glGetHistogramParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glGetMinmaxEXT(GLenum target, GLboolean reset, GLenum format, GLenum type, void *values)
    void glGetMinmaxParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glGetMinmaxParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glHistogramEXT(GLenum target, GLsizei width, GLenum internalformat, GLboolean sink)
    void glMinmaxEXT(GLenum target, GLenum internalformat, GLboolean sink)
    void glResetHistogramEXT(GLenum target)
    void glResetMinmaxEXT(GLenum target)
    void glIndexFuncEXT(GLenum func, GLfloat ref)
    void glIndexMaterialEXT(GLenum face, GLenum mode)
    void glVertexAttribDivisorEXT(GLuint index, GLuint divisor)
    void glApplyTextureEXT(GLenum mode)
    void glTextureLightEXT(GLenum pname)
    void glTextureMaterialEXT(GLenum face, GLenum mode)
    void glFlushMappedBufferRangeEXT(GLenum target, GLintptr offset, GLsizeiptr length)
    void *glMapBufferRangeEXT(GLenum target, GLintptr offset, GLsizeiptr length, GLbitfield access)
    void glBufferStorageMemEXT(GLenum target, GLsizeiptr size, GLuint memory, GLuint64 offset)
    void glCreateMemoryObjectsEXT(GLsizei n, GLuint *memoryObjects)
    void glDeleteMemoryObjectsEXT(GLsizei n, GLuint *memoryObjects)
    void glGetMemoryObjectParameterivEXT(GLuint memoryObject, GLenum pname, GLint *params)
    void glGetUnsignedBytei_vEXT(GLenum target, GLuint index, GLubyte *data)
    void glGetUnsignedBytevEXT(GLenum pname, GLubyte *data)
    GLboolean glIsMemoryObjectEXT(GLuint memoryObject)
    void glMemoryObjectParameterivEXT(GLuint memoryObject, GLenum pname, GLint *params)
    void glNamedBufferStorageMemEXT(GLuint buffer, GLsizeiptr size, GLuint memory, GLuint64 offset)
    void glTexStorageMem1DEXT(GLenum target, GLsizei levels, GLenum internalFormat, GLsizei width, GLuint memory, GLuint64 offset)
    void glTexStorageMem2DEXT(GLenum target, GLsizei levels, GLenum internalFormat, GLsizei width, GLsizei height, GLuint memory, GLuint64 offset)
    void glTexStorageMem2DMultisampleEXT(GLenum target, GLsizei samples, GLenum internalFormat, GLsizei width, GLsizei height, GLboolean fixedSampleLocations, GLuint memory, GLuint64 offset)
    void glTexStorageMem3DEXT(GLenum target, GLsizei levels, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLuint memory, GLuint64 offset)
    void glTexStorageMem3DMultisampleEXT(GLenum target, GLsizei samples, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedSampleLocations, GLuint memory, GLuint64 offset)
    void glTextureStorageMem1DEXT(GLuint texture, GLsizei levels, GLenum internalFormat, GLsizei width, GLuint memory, GLuint64 offset)
    void glTextureStorageMem2DEXT(GLuint texture, GLsizei levels, GLenum internalFormat, GLsizei width, GLsizei height, GLuint memory, GLuint64 offset)
    void glTextureStorageMem2DMultisampleEXT(GLuint texture, GLsizei samples, GLenum internalFormat, GLsizei width, GLsizei height, GLboolean fixedSampleLocations, GLuint memory, GLuint64 offset)
    void glTextureStorageMem3DEXT(GLuint texture, GLsizei levels, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLuint memory, GLuint64 offset)
    void glTextureStorageMem3DMultisampleEXT(GLuint texture, GLsizei samples, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedSampleLocations, GLuint memory, GLuint64 offset)
    void glImportMemoryFdEXT(GLuint memory, GLuint64 size, GLenum handleType, GLint fd)
    void glImportMemoryWin32HandleEXT(GLuint memory, GLuint64 size, GLenum handleType, void *handle)
    void glImportMemoryWin32NameEXT(GLuint memory, GLuint64 size, GLenum handleType, void *name)
    void glMultiDrawArraysEXT(GLenum mode, GLint *first, GLsizei *count, GLsizei primcount)
    void glMultiDrawElementsEXT(GLenum mode, GLsizei *count, GLenum type, void * *indices, GLsizei primcount)
    void glMultiDrawArraysIndirectEXT(GLenum mode, void *indirect, GLsizei drawcount, GLsizei stride)
    void glMultiDrawElementsIndirectEXT(GLenum mode, GLenum type, void *indirect, GLsizei drawcount, GLsizei stride)
    void glSampleMaskEXT(GLclampf value, GLboolean invert)
    void glSamplePatternEXT(GLenum pattern)
    void glFramebufferTexture2DMultisampleEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLsizei samples)
    void glDrawBuffersIndexedEXT(GLint n, GLenum *location, GLint *indices)
    void glGetIntegeri_vEXT(GLenum target, GLuint index, GLint *data)
    void glReadBufferIndexedEXT(GLenum src, GLint index)
    void glColorTableEXT(GLenum target, GLenum internalFormat, GLsizei width, GLenum format, GLenum type, void *data)
    void glGetColorTableEXT(GLenum target, GLenum format, GLenum type, void *data)
    void glGetColorTableParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glGetColorTableParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glGetPixelTransformParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glGetPixelTransformParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glPixelTransformParameterfEXT(GLenum target, GLenum pname, GLfloat param)
    void glPixelTransformParameterfvEXT(GLenum target, GLenum pname, GLfloat *params)
    void glPixelTransformParameteriEXT(GLenum target, GLenum pname, GLint param)
    void glPixelTransformParameterivEXT(GLenum target, GLenum pname, GLint *params)
    void glPointParameterfEXT(GLenum pname, GLfloat param)
    void glPointParameterfvEXT(GLenum pname, GLfloat *params)
    void glPolygonOffsetEXT(GLfloat factor, GLfloat bias)
    void glPolygonOffsetClampEXT(GLfloat factor, GLfloat units, GLfloat clamp)
    void glProvokingVertexEXT(GLenum mode)
    void glCoverageModulationNV(GLenum components)
    void glCoverageModulationTableNV(GLsizei n, GLfloat *v)
    void glGetCoverageModulationTableNV(GLsizei bufsize, GLfloat *v)
    void glRasterSamplesEXT(GLuint samples, GLboolean fixedsamplelocations)
    void glBeginSceneEXT()
    void glEndSceneEXT()
    void glSecondaryColor3bEXT(GLbyte red, GLbyte green, GLbyte blue)
    void glSecondaryColor3bvEXT(GLbyte *v)
    void glSecondaryColor3dEXT(GLdouble red, GLdouble green, GLdouble blue)
    void glSecondaryColor3dvEXT(GLdouble *v)
    void glSecondaryColor3fEXT(GLfloat red, GLfloat green, GLfloat blue)
    void glSecondaryColor3fvEXT(GLfloat *v)
    void glSecondaryColor3iEXT(GLint red, GLint green, GLint blue)
    void glSecondaryColor3ivEXT(GLint *v)
    void glSecondaryColor3sEXT(GLshort red, GLshort green, GLshort blue)
    void glSecondaryColor3svEXT(GLshort *v)
    void glSecondaryColor3ubEXT(GLubyte red, GLubyte green, GLubyte blue)
    void glSecondaryColor3ubvEXT(GLubyte *v)
    void glSecondaryColor3uiEXT(GLuint red, GLuint green, GLuint blue)
    void glSecondaryColor3uivEXT(GLuint *v)
    void glSecondaryColor3usEXT(GLushort red, GLushort green, GLushort blue)
    void glSecondaryColor3usvEXT(GLushort *v)
    void glSecondaryColorPointerEXT(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glDeleteSemaphoresEXT(GLsizei n, GLuint *semaphores)
    void glGenSemaphoresEXT(GLsizei n, GLuint *semaphores)
    void glGetSemaphoreParameterui64vEXT(GLuint semaphore, GLenum pname, GLuint64 *params)
    GLboolean glIsSemaphoreEXT(GLuint semaphore)
    void glSemaphoreParameterui64vEXT(GLuint semaphore, GLenum pname, GLuint64 *params)
    void glSignalSemaphoreEXT(GLuint semaphore, GLuint numBufferBarriers, GLuint *buffers, GLuint numTextureBarriers, GLuint *textures, GLenum *dstLayouts)
    void glWaitSemaphoreEXT(GLuint semaphore, GLuint numBufferBarriers, GLuint *buffers, GLuint numTextureBarriers, GLuint *textures, GLenum *srcLayouts)
    void glImportSemaphoreFdEXT(GLuint semaphore, GLenum handleType, GLint fd)
    void glImportSemaphoreWin32HandleEXT(GLuint semaphore, GLenum handleType, void *handle)
    void glImportSemaphoreWin32NameEXT(GLuint semaphore, GLenum handleType, void *name)
    void glActiveProgramEXT(GLuint program)
    GLuint glCreateShaderProgramEXT(GLenum type, GLchar *string)
    void glUseShaderProgramEXT(GLenum type, GLuint program)
    void glBindImageTextureEXT(GLuint index, GLuint texture, GLint level, GLboolean layered, GLint layer, GLenum access, GLint format)
    void glMemoryBarrierEXT(GLbitfield barriers)
    void glClearPixelLocalStorageuiEXT(GLsizei offset, GLsizei n, GLuint *values)
    void glFramebufferPixelLocalStorageSizeEXT(GLuint target, GLsizei size)
    GLsizei glGetFramebufferPixelLocalStorageSizeEXT(GLuint target)
    void glTexPageCommitmentEXT(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLboolean commit)
    void glTexturePageCommitmentEXT(GLuint texture, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLboolean commit)
    void glActiveStencilFaceEXT(GLenum face)
    void glTexSubImage1DEXT(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, void *pixels)
    void glTexSubImage2DEXT(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glTexSubImage3DEXT(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    void glTexImage3DEXT(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, void *pixels)
    void glFramebufferTextureLayerEXT(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glTexBufferEXT(GLenum target, GLenum internalformat, GLuint buffer)
    void glClearColorIiEXT(GLint red, GLint green, GLint blue, GLint alpha)
    void glClearColorIuiEXT(GLuint red, GLuint green, GLuint blue, GLuint alpha)
    void glGetTexParameterIivEXT(GLenum target, GLenum pname, GLint *params)
    void glGetTexParameterIuivEXT(GLenum target, GLenum pname, GLuint *params)
    void glTexParameterIivEXT(GLenum target, GLenum pname, GLint *params)
    void glTexParameterIuivEXT(GLenum target, GLenum pname, GLuint *params)
    GLboolean glAreTexturesResidentEXT(GLsizei n, GLuint *textures, GLboolean *residences)
    void glBindTextureEXT(GLenum target, GLuint texture)
    void glDeleteTexturesEXT(GLsizei n, GLuint *textures)
    void glGenTexturesEXT(GLsizei n, GLuint *textures)
    GLboolean glIsTextureEXT(GLuint texture)
    void glPrioritizeTexturesEXT(GLsizei n, GLuint *textures, GLclampf *priorities)
    void glTextureNormalEXT(GLenum mode)
    void glTexStorage1DEXT(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width)
    void glTexStorage2DEXT(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height)
    void glTexStorage3DEXT(GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth)
    void glTextureStorage1DEXT(GLuint texture, GLenum target, GLsizei levels, GLenum internalformat, GLsizei width)
    void glTextureStorage2DEXT(GLuint texture, GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height)
    void glTextureStorage3DEXT(GLuint texture, GLenum target, GLsizei levels, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth)
    void glTextureViewEXT(GLuint texture, GLenum target, GLuint origtexture, GLenum internalformat, GLuint minlevel, GLuint numlevels, GLuint minlayer, GLuint numlayers)
    void glGetQueryObjecti64vEXT(GLuint id, GLenum pname, GLint64EXT *params)
    void glGetQueryObjectui64vEXT(GLuint id, GLenum pname, GLuint64EXT *params)
    void glBeginTransformFeedbackEXT(GLenum primitiveMode)
    void glBindBufferBaseEXT(GLenum target, GLuint index, GLuint buffer)
    void glBindBufferOffsetEXT(GLenum target, GLuint index, GLuint buffer, GLintptr offset)
    void glBindBufferRangeEXT(GLenum target, GLuint index, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glEndTransformFeedbackEXT()
    void glGetTransformFeedbackVaryingEXT(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name)
    void glTransformFeedbackVaryingsEXT(GLuint program, GLsizei count, GLchar * *varyings, GLenum bufferMode)
    void glArrayElementEXT(GLint i)
    void glColorPointerEXT(GLint size, GLenum type, GLsizei stride, GLsizei count, void *pointer)
    void glDrawArraysEXT(GLenum mode, GLint first, GLsizei count)
    void glEdgeFlagPointerEXT(GLsizei stride, GLsizei count, GLboolean *pointer)
    void glIndexPointerEXT(GLenum type, GLsizei stride, GLsizei count, void *pointer)
    void glNormalPointerEXT(GLenum type, GLsizei stride, GLsizei count, void *pointer)
    void glTexCoordPointerEXT(GLint size, GLenum type, GLsizei stride, GLsizei count, void *pointer)
    void glVertexPointerEXT(GLint size, GLenum type, GLsizei stride, GLsizei count, void *pointer)
    void glBindArraySetEXT(void *arrayset)
    void *glCreateArraySetExt()
    void glDeleteArraySetsEXT(GLsizei n, void *arrayset[])
    void glGetVertexAttribLdvEXT(GLuint index, GLenum pname, GLdouble *params)
    void glVertexArrayVertexAttribLOffsetEXT(GLuint vaobj, GLuint buffer, GLuint index, GLint size, GLenum type, GLsizei stride, GLintptr offset)
    void glVertexAttribL1dEXT(GLuint index, GLdouble x)
    void glVertexAttribL1dvEXT(GLuint index, GLdouble *v)
    void glVertexAttribL2dEXT(GLuint index, GLdouble x, GLdouble y)
    void glVertexAttribL2dvEXT(GLuint index, GLdouble *v)
    void glVertexAttribL3dEXT(GLuint index, GLdouble x, GLdouble y, GLdouble z)
    void glVertexAttribL3dvEXT(GLuint index, GLdouble *v)
    void glVertexAttribL4dEXT(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexAttribL4dvEXT(GLuint index, GLdouble *v)
    void glVertexAttribLPointerEXT(GLuint index, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glBeginVertexShaderEXT()
    GLuint glBindLightParameterEXT(GLenum light, GLenum value)
    GLuint glBindMaterialParameterEXT(GLenum face, GLenum value)
    GLuint glBindParameterEXT(GLenum value)
    GLuint glBindTexGenParameterEXT(GLenum unit, GLenum coord, GLenum value)
    GLuint glBindTextureUnitParameterEXT(GLenum unit, GLenum value)
    void glBindVertexShaderEXT(GLuint id)
    void glDeleteVertexShaderEXT(GLuint id)
    void glDisableVariantClientStateEXT(GLuint id)
    void glEnableVariantClientStateEXT(GLuint id)
    void glEndVertexShaderEXT()
    void glExtractComponentEXT(GLuint res, GLuint src, GLuint num)
    GLuint glGenSymbolsEXT(GLenum dataType, GLenum storageType, GLenum range, GLuint components)
    GLuint glGenVertexShadersEXT(GLuint range)
    void glGetInvariantBooleanvEXT(GLuint id, GLenum value, GLboolean *data)
    void glGetInvariantFloatvEXT(GLuint id, GLenum value, GLfloat *data)
    void glGetInvariantIntegervEXT(GLuint id, GLenum value, GLint *data)
    void glGetLocalConstantBooleanvEXT(GLuint id, GLenum value, GLboolean *data)
    void glGetLocalConstantFloatvEXT(GLuint id, GLenum value, GLfloat *data)
    void glGetLocalConstantIntegervEXT(GLuint id, GLenum value, GLint *data)
    void glGetVariantBooleanvEXT(GLuint id, GLenum value, GLboolean *data)
    void glGetVariantFloatvEXT(GLuint id, GLenum value, GLfloat *data)
    void glGetVariantIntegervEXT(GLuint id, GLenum value, GLint *data)
    void glGetVariantPointervEXT(GLuint id, GLenum value, void **data)
    void glInsertComponentEXT(GLuint res, GLuint src, GLuint num)
    GLboolean glIsVariantEnabledEXT(GLuint id, GLenum cap)
    void glSetInvariantEXT(GLuint id, GLenum type, void *addr)
    void glSetLocalConstantEXT(GLuint id, GLenum type, void *addr)
    void glShaderOp1EXT(GLenum op, GLuint res, GLuint arg1)
    void glShaderOp2EXT(GLenum op, GLuint res, GLuint arg1, GLuint arg2)
    void glShaderOp3EXT(GLenum op, GLuint res, GLuint arg1, GLuint arg2, GLuint arg3)
    void glSwizzleEXT(GLuint res, GLuint in_, GLenum outX, GLenum outY, GLenum outZ, GLenum outW)
    void glVariantPointerEXT(GLuint id, GLenum type, GLuint stride, void *addr)
    void glVariantbvEXT(GLuint id, GLbyte *addr)
    void glVariantdvEXT(GLuint id, GLdouble *addr)
    void glVariantfvEXT(GLuint id, GLfloat *addr)
    void glVariantivEXT(GLuint id, GLint *addr)
    void glVariantsvEXT(GLuint id, GLshort *addr)
    void glVariantubvEXT(GLuint id, GLubyte *addr)
    void glVariantuivEXT(GLuint id, GLuint *addr)
    void glVariantusvEXT(GLuint id, GLushort *addr)
    void glWriteMaskEXT(GLuint res, GLuint in_, GLenum outX, GLenum outY, GLenum outZ, GLenum outW)
    void glVertexWeightPointerEXT(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glVertexWeightfEXT(GLfloat weight)
    void glVertexWeightfvEXT(GLfloat *weight)
    GLboolean glAcquireKeyedMutexWin32EXT(GLuint memory, GLuint64 key, GLuint timeout)
    GLboolean glReleaseKeyedMutexWin32EXT(GLuint memory, GLuint64 key)
    void glWindowRectanglesEXT(GLenum mode, GLsizei count, GLint box[])
    GLsync glImportSyncEXT(GLenum external_sync_type, GLintptr external_sync, GLbitfield flags)
    void glFrameTerminatorGREMEDY()
    void glStringMarkerGREMEDY(GLsizei len, void *string)
    void glGetImageTransformParameterfvHP(GLenum target, GLenum pname, GLfloat *params)
    void glGetImageTransformParameterivHP(GLenum target, GLenum pname, GLint *params)
    void glImageTransformParameterfHP(GLenum target, GLenum pname, GLfloat param)
    void glImageTransformParameterfvHP(GLenum target, GLenum pname, GLfloat *params)
    void glImageTransformParameteriHP(GLenum target, GLenum pname, GLint param)
    void glImageTransformParameterivHP(GLenum target, GLenum pname, GLint *params)
    void glMultiModeDrawArraysIBM(GLenum *mode, GLint *first, GLsizei *count, GLsizei primcount, GLint modestride)
    void glMultiModeDrawElementsIBM(GLenum *mode, GLsizei *count, GLenum type, void * *indices, GLsizei primcount, GLint modestride)
    void glColorPointerListIBM(GLint size, GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glEdgeFlagPointerListIBM(GLint stride, GLboolean **pointer, GLint ptrstride)
    void glFogCoordPointerListIBM(GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glIndexPointerListIBM(GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glNormalPointerListIBM(GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glSecondaryColorPointerListIBM(GLint size, GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glTexCoordPointerListIBM(GLint size, GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void glVertexPointerListIBM(GLint size, GLenum type, GLint stride, void **pointer, GLint ptrstride)
    void *glMapTexture2DINTEL(GLuint texture, GLint level, GLbitfield access, GLint *stride, GLenum *layout)
    void glSyncTextureINTEL(GLuint texture)
    void glUnmapTexture2DINTEL(GLuint texture, GLint level)
    void glColorPointervINTEL(GLint size, GLenum type, void **pointer)
    void glNormalPointervINTEL(GLenum type, void **pointer)
    void glTexCoordPointervINTEL(GLint size, GLenum type, void **pointer)
    void glVertexPointervINTEL(GLint size, GLenum type, void **pointer)
    void glBeginPerfQueryINTEL(GLuint queryHandle)
    void glCreatePerfQueryINTEL(GLuint queryId, GLuint *queryHandle)
    void glDeletePerfQueryINTEL(GLuint queryHandle)
    void glEndPerfQueryINTEL(GLuint queryHandle)
    void glGetFirstPerfQueryIdINTEL(GLuint *queryId)
    void glGetNextPerfQueryIdINTEL(GLuint queryId, GLuint *nextQueryId)
    void glGetPerfCounterInfoINTEL(GLuint queryId, GLuint counterId, GLuint counterNameLength, GLchar *counterName, GLuint counterDescLength, GLchar *counterDesc, GLuint *counterOffset, GLuint *counterDataSize, GLuint *counterTypeEnum, GLuint *counterDataTypeEnum, GLuint64 *rawCounterMaxValue)
    void glGetPerfQueryDataINTEL(GLuint queryHandle, GLuint flags, GLsizei dataSize, void *data, GLuint *bytesWritten)
    void glGetPerfQueryIdByNameINTEL(GLchar *queryName, GLuint *queryId)
    void glGetPerfQueryInfoINTEL(GLuint queryId, GLuint queryNameLength, GLchar *queryName, GLuint *dataSize, GLuint *noCounters, GLuint *noInstances, GLuint *capsMask)
    void glTexScissorFuncINTEL(GLenum target, GLenum lfunc, GLenum hfunc)
    void glTexScissorINTEL(GLenum target, GLclampf tlow, GLclampf thigh)
    void glBlendBarrierKHR()
    void glDebugMessageCallback(GLDEBUGPROC callback, void *userParam)
    void glDebugMessageControl(GLenum source, GLenum type, GLenum severity, GLsizei count, GLuint *ids, GLboolean enabled)
    void glDebugMessageInsert(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, GLchar *buf)
    GLuint glGetDebugMessageLog(GLuint count, GLsizei bufSize, GLenum *sources, GLenum *types, GLuint *ids, GLenum *severities, GLsizei *lengths, GLchar *messageLog)
    void glGetObjectLabel(GLenum identifier, GLuint name, GLsizei bufSize, GLsizei *length, GLchar *label)
    void glGetObjectPtrLabel(void *ptr, GLsizei bufSize, GLsizei *length, GLchar *label)
    void glObjectLabel(GLenum identifier, GLuint name, GLsizei length, GLchar *label)
    void glObjectPtrLabel(void *ptr, GLsizei length, GLchar *label)
    void glPopDebugGroup()
    void glPushDebugGroup(GLenum source, GLuint id, GLsizei length, GLchar *message)
    void glMaxShaderCompilerThreadsKHR(GLuint count)
    void glGetnUniformfv(GLuint program, GLint location, GLsizei bufSize, GLfloat *params)
    void glGetnUniformiv(GLuint program, GLint location, GLsizei bufSize, GLint *params)
    void glGetnUniformuiv(GLuint program, GLint location, GLsizei bufSize, GLuint *params)
    void glReadnPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLsizei bufSize, void *data)
    GLuint glBufferRegionEnabled()
    void glDeleteBufferRegion(GLenum region)
    void glDrawBufferRegion(GLuint region, GLint x, GLint y, GLsizei width, GLsizei height, GLint xDest, GLint yDest)
    GLuint glNewBufferRegion(GLenum region)
    void glReadBufferRegion(GLuint region, GLint x, GLint y, GLsizei width, GLsizei height)
    void glResizeBuffersMESA()
    void glWindowPos2dMESA(GLdouble x, GLdouble y)
    void glWindowPos2dvMESA(GLdouble *p)
    void glWindowPos2fMESA(GLfloat x, GLfloat y)
    void glWindowPos2fvMESA(GLfloat *p)
    void glWindowPos2iMESA(GLint x, GLint y)
    void glWindowPos2ivMESA(GLint *p)
    void glWindowPos2sMESA(GLshort x, GLshort y)
    void glWindowPos2svMESA(GLshort *p)
    void glWindowPos3dMESA(GLdouble x, GLdouble y, GLdouble z)
    void glWindowPos3dvMESA(GLdouble *p)
    void glWindowPos3fMESA(GLfloat x, GLfloat y, GLfloat z)
    void glWindowPos3fvMESA(GLfloat *p)
    void glWindowPos3iMESA(GLint x, GLint y, GLint z)
    void glWindowPos3ivMESA(GLint *p)
    void glWindowPos3sMESA(GLshort x, GLshort y, GLshort z)
    void glWindowPos3svMESA(GLshort *p)
    void glWindowPos4dMESA(GLdouble x, GLdouble y, GLdouble z, GLdouble)
    void glWindowPos4dvMESA(GLdouble *p)
    void glWindowPos4fMESA(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glWindowPos4fvMESA(GLfloat *p)
    void glWindowPos4iMESA(GLint x, GLint y, GLint z, GLint w)
    void glWindowPos4ivMESA(GLint *p)
    void glWindowPos4sMESA(GLshort x, GLshort y, GLshort z, GLshort w)
    void glWindowPos4svMESA(GLshort *p)
    void glBeginConditionalRenderNVX(GLuint id)
    void glEndConditionalRenderNVX()
    void glLGPUCopyImageSubDataNVX(GLuint sourceGpu, GLbitfield destinationGpuMask, GLuint srcName, GLenum srcTarget, GLint srcLevel, GLint srcX, GLint srxY, GLint srcZ, GLuint dstName, GLenum dstTarget, GLint dstLevel, GLint dstX, GLint dstY, GLint dstZ, GLsizei width, GLsizei height, GLsizei depth)
    void glLGPUInterlockNVX()
    void glLGPUNamedBufferSubDataNVX(GLbitfield gpuMask, GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glStereoParameterfNV(GLenum pname, GLfloat param)
    void glStereoParameteriNV(GLenum pname, GLint param)
    void glMultiDrawArraysIndirectBindlessNV(GLenum mode, void *indirect, GLsizei drawCount, GLsizei stride, GLint vertexBufferCount)
    void glMultiDrawElementsIndirectBindlessNV(GLenum mode, GLenum type, void *indirect, GLsizei drawCount, GLsizei stride, GLint vertexBufferCount)
    void glMultiDrawArraysIndirectBindlessCountNV(GLenum mode, void *indirect, GLintptr drawCount, GLsizei maxDrawCount, GLsizei stride, GLint vertexBufferCount)
    void glMultiDrawElementsIndirectBindlessCountNV(GLenum mode, GLenum type, void *indirect, GLintptr drawCount, GLsizei maxDrawCount, GLsizei stride, GLint vertexBufferCount)
    GLuint64 glGetImageHandleNV(GLuint texture, GLint level, GLboolean layered, GLint layer, GLenum format)
    GLuint64 glGetTextureHandleNV(GLuint texture)
    GLuint64 glGetTextureSamplerHandleNV(GLuint texture, GLuint sampler)
    GLboolean glIsImageHandleResidentNV(GLuint64 handle)
    GLboolean glIsTextureHandleResidentNV(GLuint64 handle)
    void glMakeImageHandleNonResidentNV(GLuint64 handle)
    void glMakeImageHandleResidentNV(GLuint64 handle, GLenum access)
    void glMakeTextureHandleNonResidentNV(GLuint64 handle)
    void glMakeTextureHandleResidentNV(GLuint64 handle)
    void glProgramUniformHandleui64NV(GLuint program, GLint location, GLuint64 value)
    void glProgramUniformHandleui64vNV(GLuint program, GLint location, GLsizei count, GLuint64 *values)
    void glUniformHandleui64NV(GLint location, GLuint64 value)
    void glUniformHandleui64vNV(GLint location, GLsizei count, GLuint64 *value)
    void glBlendBarrierNV()
    void glBlendParameteriNV(GLenum pname, GLint value)
    void glViewportPositionWScaleNV(GLuint index, GLfloat xcoeff, GLfloat ycoeff)
    void glCallCommandListNV(GLuint list)
    void glCommandListSegmentsNV(GLuint list, GLuint segments)
    void glCompileCommandListNV(GLuint list)
    void glCreateCommandListsNV(GLsizei n, GLuint *lists)
    void glCreateStatesNV(GLsizei n, GLuint *states)
    void glDeleteCommandListsNV(GLsizei n, GLuint *lists)
    void glDeleteStatesNV(GLsizei n, GLuint *states)
    void glDrawCommandsAddressNV(GLenum primitiveMode, GLuint64 *indirects, GLsizei *sizes, GLuint count)
    void glDrawCommandsNV(GLenum primitiveMode, GLuint buffer, GLintptr *indirects, GLsizei *sizes, GLuint count)
    void glDrawCommandsStatesAddressNV(GLuint64 *indirects, GLsizei *sizes, GLuint *states, GLuint *fbos, GLuint count)
    void glDrawCommandsStatesNV(GLuint buffer, GLintptr *indirects, GLsizei *sizes, GLuint *states, GLuint *fbos, GLuint count)
    GLuint glGetCommandHeaderNV(GLenum tokenID, GLuint size)
    GLushort glGetStageIndexNV(GLenum shadertype)
    GLboolean glIsCommandListNV(GLuint list)
    GLboolean glIsStateNV(GLuint state)
    void glListDrawCommandsStatesClientNV(GLuint list, GLuint segment, void **indirects, GLsizei *sizes, GLuint *states, GLuint *fbos, GLuint count)
    void glStateCaptureNV(GLuint state, GLenum mode)
    void glBeginConditionalRenderNV(GLuint id, GLenum mode)
    void glEndConditionalRenderNV()
    void glSubpixelPrecisionBiasNV(GLuint xbits, GLuint ybits)
    void glConservativeRasterParameterfNV(GLenum pname, GLfloat value)
    void glConservativeRasterParameteriNV(GLenum pname, GLint param)
    void glCopyBufferSubDataNV(GLenum readtarget, GLenum writetarget, GLintptr readoffset, GLintptr writeoffset, GLsizeiptr size)
    void glCopyImageSubDataNV(GLuint srcName, GLenum srcTarget, GLint srcLevel, GLint srcX, GLint srcY, GLint srcZ, GLuint dstName, GLenum dstTarget, GLint dstLevel, GLint dstX, GLint dstY, GLint dstZ, GLsizei width, GLsizei height, GLsizei depth)
    void glClearDepthdNV(GLdouble depth)
    void glDepthBoundsdNV(GLdouble zmin, GLdouble zmax)
    void glDepthRangedNV(GLdouble zNear, GLdouble zFar)
    void glDrawBuffersNV(GLsizei n, GLenum *bufs)
    void glDrawArraysInstancedNV(GLenum mode, GLint first, GLsizei count, GLsizei primcount)
    void glDrawElementsInstancedNV(GLenum mode, GLsizei count, GLenum type, void *indices, GLsizei primcount)
    void glDrawTextureNV(GLuint texture, GLuint sampler, GLfloat x0, GLfloat y0, GLfloat x1, GLfloat y1, GLfloat z, GLfloat s0, GLfloat t0, GLfloat s1, GLfloat t1)
    void glDrawVkImageNV(GLuint64 vkImage, GLuint sampler, GLfloat x0, GLfloat y0, GLfloat x1, GLfloat y1, GLfloat z, GLfloat s0, GLfloat t0, GLfloat s1, GLfloat t1)
    GLVULKANPROCNV glGetVkProcAddrNV(GLchar *name)
    void glSignalVkFenceNV(GLuint64 vkFence)
    void glSignalVkSemaphoreNV(GLuint64 vkSemaphore)
    void glWaitVkSemaphoreNV(GLuint64 vkSemaphore)
    void glEvalMapsNV(GLenum target, GLenum mode)
    void glGetMapAttribParameterfvNV(GLenum target, GLuint index, GLenum pname, GLfloat *params)
    void glGetMapAttribParameterivNV(GLenum target, GLuint index, GLenum pname, GLint *params)
    void glGetMapControlPointsNV(GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLboolean packed, void *points)
    void glGetMapParameterfvNV(GLenum target, GLenum pname, GLfloat *params)
    void glGetMapParameterivNV(GLenum target, GLenum pname, GLint *params)
    void glMapControlPointsNV(GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLint uorder, GLint vorder, GLboolean packed, void *points)
    void glMapParameterfvNV(GLenum target, GLenum pname, GLfloat *params)
    void glMapParameterivNV(GLenum target, GLenum pname, GLint *params)
    void glGetMultisamplefvNV(GLenum pname, GLuint index, GLfloat *val)
    void glSampleMaskIndexedNV(GLuint index, GLbitfield mask)
    void glTexRenderbufferNV(GLenum target, GLuint renderbuffer)
    void glDeleteFencesNV(GLsizei n, GLuint *fences)
    void glFinishFenceNV(GLuint fence)
    void glGenFencesNV(GLsizei n, GLuint *fences)
    void glGetFenceivNV(GLuint fence, GLenum pname, GLint *params)
    GLboolean glIsFenceNV(GLuint fence)
    void glSetFenceNV(GLuint fence, GLenum condition)
    GLboolean glTestFenceNV(GLuint fence)
    void glFragmentCoverageColorNV(GLuint color)
    void glGetProgramNamedParameterdvNV(GLuint id, GLsizei len, GLubyte *name, GLdouble *params)
    void glGetProgramNamedParameterfvNV(GLuint id, GLsizei len, GLubyte *name, GLfloat *params)
    void glProgramNamedParameter4dNV(GLuint id, GLsizei len, GLubyte *name, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glProgramNamedParameter4dvNV(GLuint id, GLsizei len, GLubyte *name, GLdouble v[])
    void glProgramNamedParameter4fNV(GLuint id, GLsizei len, GLubyte *name, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glProgramNamedParameter4fvNV(GLuint id, GLsizei len, GLubyte *name, GLfloat v[])
    void glBlitFramebufferNV(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    void glRenderbufferStorageMultisampleNV(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
    void glRenderbufferStorageMultisampleCoverageNV(GLenum target, GLsizei coverageSamples, GLsizei colorSamples, GLenum internalformat, GLsizei width, GLsizei height)
    void glProgramVertexLimitNV(GLenum target, GLint limit)
    void glMulticastBarrierNV()
    void glMulticastBlitFramebufferNV(GLuint srcGpu, GLuint dstGpu, GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
    void glMulticastBufferSubDataNV(GLbitfield gpuMask, GLuint buffer, GLintptr offset, GLsizeiptr size, void *data)
    void glMulticastCopyBufferSubDataNV(GLuint readGpu, GLbitfield writeGpuMask, GLuint readBuffer, GLuint writeBuffer, GLintptr readOffset, GLintptr writeOffset, GLsizeiptr size)
    void glMulticastCopyImageSubDataNV(GLuint srcGpu, GLbitfield dstGpuMask, GLuint srcName, GLenum srcTarget, GLint srcLevel, GLint srcX, GLint srcY, GLint srcZ, GLuint dstName, GLenum dstTarget, GLint dstLevel, GLint dstX, GLint dstY, GLint dstZ, GLsizei srcWidth, GLsizei srcHeight, GLsizei srcDepth)
    void glMulticastFramebufferSampleLocationsfvNV(GLuint gpu, GLuint framebuffer, GLuint start, GLsizei count, GLfloat *v)
    void glMulticastGetQueryObjecti64vNV(GLuint gpu, GLuint id, GLenum pname, GLint64 *params)
    void glMulticastGetQueryObjectivNV(GLuint gpu, GLuint id, GLenum pname, GLint *params)
    void glMulticastGetQueryObjectui64vNV(GLuint gpu, GLuint id, GLenum pname, GLuint64 *params)
    void glMulticastGetQueryObjectuivNV(GLuint gpu, GLuint id, GLenum pname, GLuint *params)
    void glMulticastWaitSyncNV(GLuint signalGpu, GLbitfield waitGpuMask)
    void glRenderGpuMaskNV(GLbitfield mask)
    void glProgramEnvParameterI4iNV(GLenum target, GLuint index, GLint x, GLint y, GLint z, GLint w)
    void glProgramEnvParameterI4ivNV(GLenum target, GLuint index, GLint *params)
    void glProgramEnvParameterI4uiNV(GLenum target, GLuint index, GLuint x, GLuint y, GLuint z, GLuint w)
    void glProgramEnvParameterI4uivNV(GLenum target, GLuint index, GLuint *params)
    void glProgramEnvParametersI4ivNV(GLenum target, GLuint index, GLsizei count, GLint *params)
    void glProgramEnvParametersI4uivNV(GLenum target, GLuint index, GLsizei count, GLuint *params)
    void glProgramLocalParameterI4iNV(GLenum target, GLuint index, GLint x, GLint y, GLint z, GLint w)
    void glProgramLocalParameterI4ivNV(GLenum target, GLuint index, GLint *params)
    void glProgramLocalParameterI4uiNV(GLenum target, GLuint index, GLuint x, GLuint y, GLuint z, GLuint w)
    void glProgramLocalParameterI4uivNV(GLenum target, GLuint index, GLuint *params)
    void glProgramLocalParametersI4ivNV(GLenum target, GLuint index, GLsizei count, GLint *params)
    void glProgramLocalParametersI4uivNV(GLenum target, GLuint index, GLsizei count, GLuint *params)
    void glGetUniformi64vNV(GLuint program, GLint location, GLint64EXT *params)
    void glGetUniformui64vNV(GLuint program, GLint location, GLuint64EXT *params)
    void glProgramUniform1i64NV(GLuint program, GLint location, GLint64EXT x)
    void glProgramUniform1i64vNV(GLuint program, GLint location, GLsizei count, GLint64EXT *value)
    void glProgramUniform1ui64NV(GLuint program, GLint location, GLuint64EXT x)
    void glProgramUniform1ui64vNV(GLuint program, GLint location, GLsizei count, GLuint64EXT *value)
    void glProgramUniform2i64NV(GLuint program, GLint location, GLint64EXT x, GLint64EXT y)
    void glProgramUniform2i64vNV(GLuint program, GLint location, GLsizei count, GLint64EXT *value)
    void glProgramUniform2ui64NV(GLuint program, GLint location, GLuint64EXT x, GLuint64EXT y)
    void glProgramUniform2ui64vNV(GLuint program, GLint location, GLsizei count, GLuint64EXT *value)
    void glProgramUniform3i64NV(GLuint program, GLint location, GLint64EXT x, GLint64EXT y, GLint64EXT z)
    void glProgramUniform3i64vNV(GLuint program, GLint location, GLsizei count, GLint64EXT *value)
    void glProgramUniform3ui64NV(GLuint program, GLint location, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z)
    void glProgramUniform3ui64vNV(GLuint program, GLint location, GLsizei count, GLuint64EXT *value)
    void glProgramUniform4i64NV(GLuint program, GLint location, GLint64EXT x, GLint64EXT y, GLint64EXT z, GLint64EXT w)
    void glProgramUniform4i64vNV(GLuint program, GLint location, GLsizei count, GLint64EXT *value)
    void glProgramUniform4ui64NV(GLuint program, GLint location, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z, GLuint64EXT w)
    void glProgramUniform4ui64vNV(GLuint program, GLint location, GLsizei count, GLuint64EXT *value)
    void glUniform1i64NV(GLint location, GLint64EXT x)
    void glUniform1i64vNV(GLint location, GLsizei count, GLint64EXT *value)
    void glUniform1ui64NV(GLint location, GLuint64EXT x)
    void glUniform1ui64vNV(GLint location, GLsizei count, GLuint64EXT *value)
    void glUniform2i64NV(GLint location, GLint64EXT x, GLint64EXT y)
    void glUniform2i64vNV(GLint location, GLsizei count, GLint64EXT *value)
    void glUniform2ui64NV(GLint location, GLuint64EXT x, GLuint64EXT y)
    void glUniform2ui64vNV(GLint location, GLsizei count, GLuint64EXT *value)
    void glUniform3i64NV(GLint location, GLint64EXT x, GLint64EXT y, GLint64EXT z)
    void glUniform3i64vNV(GLint location, GLsizei count, GLint64EXT *value)
    void glUniform3ui64NV(GLint location, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z)
    void glUniform3ui64vNV(GLint location, GLsizei count, GLuint64EXT *value)
    void glUniform4i64NV(GLint location, GLint64EXT x, GLint64EXT y, GLint64EXT z, GLint64EXT w)
    void glUniform4i64vNV(GLint location, GLsizei count, GLint64EXT *value)
    void glUniform4ui64NV(GLint location, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z, GLuint64EXT w)
    void glUniform4ui64vNV(GLint location, GLsizei count, GLuint64EXT *value)
    void glColor3hNV(GLhalf red, GLhalf green, GLhalf blue)
    void glColor3hvNV(GLhalf *v)
    void glColor4hNV(GLhalf red, GLhalf green, GLhalf blue, GLhalf alpha)
    void glColor4hvNV(GLhalf *v)
    void glFogCoordhNV(GLhalf fog)
    void glFogCoordhvNV(GLhalf *fog)
    void glMultiTexCoord1hNV(GLenum target, GLhalf s)
    void glMultiTexCoord1hvNV(GLenum target, GLhalf *v)
    void glMultiTexCoord2hNV(GLenum target, GLhalf s, GLhalf t)
    void glMultiTexCoord2hvNV(GLenum target, GLhalf *v)
    void glMultiTexCoord3hNV(GLenum target, GLhalf s, GLhalf t, GLhalf r)
    void glMultiTexCoord3hvNV(GLenum target, GLhalf *v)
    void glMultiTexCoord4hNV(GLenum target, GLhalf s, GLhalf t, GLhalf r, GLhalf q)
    void glMultiTexCoord4hvNV(GLenum target, GLhalf *v)
    void glNormal3hNV(GLhalf nx, GLhalf ny, GLhalf nz)
    void glNormal3hvNV(GLhalf *v)
    void glSecondaryColor3hNV(GLhalf red, GLhalf green, GLhalf blue)
    void glSecondaryColor3hvNV(GLhalf *v)
    void glTexCoord1hNV(GLhalf s)
    void glTexCoord1hvNV(GLhalf *v)
    void glTexCoord2hNV(GLhalf s, GLhalf t)
    void glTexCoord2hvNV(GLhalf *v)
    void glTexCoord3hNV(GLhalf s, GLhalf t, GLhalf r)
    void glTexCoord3hvNV(GLhalf *v)
    void glTexCoord4hNV(GLhalf s, GLhalf t, GLhalf r, GLhalf q)
    void glTexCoord4hvNV(GLhalf *v)
    void glVertex2hNV(GLhalf x, GLhalf y)
    void glVertex2hvNV(GLhalf *v)
    void glVertex3hNV(GLhalf x, GLhalf y, GLhalf z)
    void glVertex3hvNV(GLhalf *v)
    void glVertex4hNV(GLhalf x, GLhalf y, GLhalf z, GLhalf w)
    void glVertex4hvNV(GLhalf *v)
    void glVertexAttrib1hNV(GLuint index, GLhalf x)
    void glVertexAttrib1hvNV(GLuint index, GLhalf *v)
    void glVertexAttrib2hNV(GLuint index, GLhalf x, GLhalf y)
    void glVertexAttrib2hvNV(GLuint index, GLhalf *v)
    void glVertexAttrib3hNV(GLuint index, GLhalf x, GLhalf y, GLhalf z)
    void glVertexAttrib3hvNV(GLuint index, GLhalf *v)
    void glVertexAttrib4hNV(GLuint index, GLhalf x, GLhalf y, GLhalf z, GLhalf w)
    void glVertexAttrib4hvNV(GLuint index, GLhalf *v)
    void glVertexAttribs1hvNV(GLuint index, GLsizei n, GLhalf *v)
    void glVertexAttribs2hvNV(GLuint index, GLsizei n, GLhalf *v)
    void glVertexAttribs3hvNV(GLuint index, GLsizei n, GLhalf *v)
    void glVertexAttribs4hvNV(GLuint index, GLsizei n, GLhalf *v)
    void glVertexWeighthNV(GLhalf weight)
    void glVertexWeighthvNV(GLhalf *weight)
    void glVertexAttribDivisorNV(GLuint index, GLuint divisor)
    void glGetInternalformatSampleivNV(GLenum target, GLenum internalformat, GLsizei samples, GLenum pname, GLsizei bufSize, GLint *params)
    void glUniformMatrix2x3fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix2x4fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3x2fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix3x4fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4x2fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glUniformMatrix4x3fvNV(GLint location, GLsizei count, GLboolean transpose, GLfloat *value)
    void glBeginOcclusionQueryNV(GLuint id)
    void glDeleteOcclusionQueriesNV(GLsizei n, GLuint *ids)
    void glEndOcclusionQueryNV()
    void glGenOcclusionQueriesNV(GLsizei n, GLuint *ids)
    void glGetOcclusionQueryivNV(GLuint id, GLenum pname, GLint *params)
    void glGetOcclusionQueryuivNV(GLuint id, GLenum pname, GLuint *params)
    GLboolean glIsOcclusionQueryNV(GLuint id)
    void glProgramBufferParametersIivNV(GLenum target, GLuint buffer, GLuint index, GLsizei count, GLint *params)
    void glProgramBufferParametersIuivNV(GLenum target, GLuint buffer, GLuint index, GLsizei count, GLuint *params)
    void glProgramBufferParametersfvNV(GLenum target, GLuint buffer, GLuint index, GLsizei count, GLfloat *params)
    void glCopyPathNV(GLuint resultPath, GLuint srcPath)
    void glCoverFillPathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLenum coverMode, GLenum transformType, GLfloat *transformValues)
    void glCoverFillPathNV(GLuint path, GLenum coverMode)
    void glCoverStrokePathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLenum coverMode, GLenum transformType, GLfloat *transformValues)
    void glCoverStrokePathNV(GLuint path, GLenum coverMode)
    void glDeletePathsNV(GLuint path, GLsizei range)
    GLuint glGenPathsNV(GLsizei range)
    void glGetPathColorGenfvNV(GLenum color, GLenum pname, GLfloat *value)
    void glGetPathColorGenivNV(GLenum color, GLenum pname, GLint *value)
    void glGetPathCommandsNV(GLuint path, GLubyte *commands)
    void glGetPathCoordsNV(GLuint path, GLfloat *coords)
    void glGetPathDashArrayNV(GLuint path, GLfloat *dashArray)
    GLfloat glGetPathLengthNV(GLuint path, GLsizei startSegment, GLsizei numSegments)
    void glGetPathMetricRangeNV(GLbitfield metricQueryMask, GLuint firstPathName, GLsizei numPaths, GLsizei stride, GLfloat *metrics)
    void glGetPathMetricsNV(GLbitfield metricQueryMask, GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLsizei stride, GLfloat *metrics)
    void glGetPathParameterfvNV(GLuint path, GLenum pname, GLfloat *value)
    void glGetPathParameterivNV(GLuint path, GLenum pname, GLint *value)
    void glGetPathSpacingNV(GLenum pathListMode, GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLfloat advanceScale, GLfloat kerningScale, GLenum transformType, GLfloat *returnedSpacing)
    void glGetPathTexGenfvNV(GLenum texCoordSet, GLenum pname, GLfloat *value)
    void glGetPathTexGenivNV(GLenum texCoordSet, GLenum pname, GLint *value)
    void glGetProgramResourcefvNV(GLuint program, GLenum programInterface, GLuint index, GLsizei propCount, GLenum *props, GLsizei bufSize, GLsizei *length, GLfloat *params)
    void glInterpolatePathsNV(GLuint resultPath, GLuint pathA, GLuint pathB, GLfloat weight)
    GLboolean glIsPathNV(GLuint path)
    GLboolean glIsPointInFillPathNV(GLuint path, GLuint mask, GLfloat x, GLfloat y)
    GLboolean glIsPointInStrokePathNV(GLuint path, GLfloat x, GLfloat y)
    void glMatrixLoad3x2fNV(GLenum matrixMode, GLfloat *m)
    void glMatrixLoad3x3fNV(GLenum matrixMode, GLfloat *m)
    void glMatrixLoadTranspose3x3fNV(GLenum matrixMode, GLfloat *m)
    void glMatrixMult3x2fNV(GLenum matrixMode, GLfloat *m)
    void glMatrixMult3x3fNV(GLenum matrixMode, GLfloat *m)
    void glMatrixMultTranspose3x3fNV(GLenum matrixMode, GLfloat *m)
    void glPathColorGenNV(GLenum color, GLenum genMode, GLenum colorFormat, GLfloat *coeffs)
    void glPathCommandsNV(GLuint path, GLsizei numCommands, GLubyte *commands, GLsizei numCoords, GLenum coordType, void *coords)
    void glPathCoordsNV(GLuint path, GLsizei numCoords, GLenum coordType, void *coords)
    void glPathCoverDepthFuncNV(GLenum zfunc)
    void glPathDashArrayNV(GLuint path, GLsizei dashCount, GLfloat *dashArray)
    void glPathFogGenNV(GLenum genMode)
    GLenum glPathGlyphIndexArrayNV(GLuint firstPathName, GLenum fontTarget, void *fontName, GLbitfield fontStyle, GLuint firstGlyphIndex, GLsizei numGlyphs, GLuint pathParameterTemplate, GLfloat emScale)
    GLenum glPathGlyphIndexRangeNV(GLenum fontTarget, void *fontName, GLbitfield fontStyle, GLuint pathParameterTemplate, GLfloat emScale, GLuint baseAndCount[2])
    void glPathGlyphRangeNV(GLuint firstPathName, GLenum fontTarget, void *fontName, GLbitfield fontStyle, GLuint firstGlyph, GLsizei numGlyphs, GLenum handleMissingGlyphs, GLuint pathParameterTemplate, GLfloat emScale)
    void glPathGlyphsNV(GLuint firstPathName, GLenum fontTarget, void *fontName, GLbitfield fontStyle, GLsizei numGlyphs, GLenum type, void *charcodes, GLenum handleMissingGlyphs, GLuint pathParameterTemplate, GLfloat emScale)
    GLenum glPathMemoryGlyphIndexArrayNV(GLuint firstPathName, GLenum fontTarget, GLsizeiptr fontSize, void *fontData, GLsizei faceIndex, GLuint firstGlyphIndex, GLsizei numGlyphs, GLuint pathParameterTemplate, GLfloat emScale)
    void glPathParameterfNV(GLuint path, GLenum pname, GLfloat value)
    void glPathParameterfvNV(GLuint path, GLenum pname, GLfloat *value)
    void glPathParameteriNV(GLuint path, GLenum pname, GLint value)
    void glPathParameterivNV(GLuint path, GLenum pname, GLint *value)
    void glPathStencilDepthOffsetNV(GLfloat factor, GLfloat units)
    void glPathStencilFuncNV(GLenum func, GLint ref, GLuint mask)
    void glPathStringNV(GLuint path, GLenum format, GLsizei length, void *pathString)
    void glPathSubCommandsNV(GLuint path, GLsizei commandStart, GLsizei commandsToDelete, GLsizei numCommands, GLubyte *commands, GLsizei numCoords, GLenum coordType, void *coords)
    void glPathSubCoordsNV(GLuint path, GLsizei coordStart, GLsizei numCoords, GLenum coordType, void *coords)
    void glPathTexGenNV(GLenum texCoordSet, GLenum genMode, GLint components, GLfloat *coeffs)
    GLboolean glPointAlongPathNV(GLuint path, GLsizei startSegment, GLsizei numSegments, GLfloat distance, GLfloat *x, GLfloat *y, GLfloat *tangentX, GLfloat *tangentY)
    void glProgramPathFragmentInputGenNV(GLuint program, GLint location, GLenum genMode, GLint components, GLfloat *coeffs)
    void glStencilFillPathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLenum fillMode, GLuint mask, GLenum transformType, GLfloat *transformValues)
    void glStencilFillPathNV(GLuint path, GLenum fillMode, GLuint mask)
    void glStencilStrokePathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLint reference, GLuint mask, GLenum transformType, GLfloat *transformValues)
    void glStencilStrokePathNV(GLuint path, GLint reference, GLuint mask)
    void glStencilThenCoverFillPathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLenum fillMode, GLuint mask, GLenum coverMode, GLenum transformType, GLfloat *transformValues)
    void glStencilThenCoverFillPathNV(GLuint path, GLenum fillMode, GLuint mask, GLenum coverMode)
    void glStencilThenCoverStrokePathInstancedNV(GLsizei numPaths, GLenum pathNameType, void *paths, GLuint pathBase, GLint reference, GLuint mask, GLenum coverMode, GLenum transformType, GLfloat *transformValues)
    void glStencilThenCoverStrokePathNV(GLuint path, GLint reference, GLuint mask, GLenum coverMode)
    void glTransformPathNV(GLuint resultPath, GLuint srcPath, GLenum transformType, GLfloat *transformValues)
    void glWeightPathsNV(GLuint resultPath, GLsizei numPaths, GLuint paths[], GLfloat weights[])
    void glFlushPixelDataRangeNV(GLenum target)
    void glPixelDataRangeNV(GLenum target, GLsizei length, void *pointer)
    void glPointParameteriNV(GLenum pname, GLint param)
    void glPointParameterivNV(GLenum pname, GLint *params)
    void glPolygonModeNV(GLenum face, GLenum mode)
    void glGetVideoi64vNV(GLuint video_slot, GLenum pname, GLint64EXT *params)
    void glGetVideoivNV(GLuint video_slot, GLenum pname, GLint *params)
    void glGetVideoui64vNV(GLuint video_slot, GLenum pname, GLuint64EXT *params)
    void glGetVideouivNV(GLuint video_slot, GLenum pname, GLuint *params)
    void glPresentFrameDualFillNV(GLuint video_slot, GLuint64EXT minPresentTime, GLuint beginPresentTimeId, GLuint presentDurationId, GLenum type, GLenum target0, GLuint fill0, GLenum target1, GLuint fill1, GLenum target2, GLuint fill2, GLenum target3, GLuint fill3)
    void glPresentFrameKeyedNV(GLuint video_slot, GLuint64EXT minPresentTime, GLuint beginPresentTimeId, GLuint presentDurationId, GLenum type, GLenum target0, GLuint fill0, GLuint key0, GLenum target1, GLuint fill1, GLuint key1)
    void glPrimitiveRestartIndexNV(GLuint index)
    void glPrimitiveRestartNV()
    void glCombinerInputNV(GLenum stage, GLenum portion, GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage)
    void glCombinerOutputNV(GLenum stage, GLenum portion, GLenum abOutput, GLenum cdOutput, GLenum sumOutput, GLenum scale, GLenum bias, GLboolean abDotProduct, GLboolean cdDotProduct, GLboolean muxSum)
    void glCombinerParameterfNV(GLenum pname, GLfloat param)
    void glCombinerParameterfvNV(GLenum pname, GLfloat *params)
    void glCombinerParameteriNV(GLenum pname, GLint param)
    void glCombinerParameterivNV(GLenum pname, GLint *params)
    void glFinalCombinerInputNV(GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage)
    void glGetCombinerInputParameterfvNV(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLfloat *params)
    void glGetCombinerInputParameterivNV(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLint *params)
    void glGetCombinerOutputParameterfvNV(GLenum stage, GLenum portion, GLenum pname, GLfloat *params)
    void glGetCombinerOutputParameterivNV(GLenum stage, GLenum portion, GLenum pname, GLint *params)
    void glGetFinalCombinerInputParameterfvNV(GLenum variable, GLenum pname, GLfloat *params)
    void glGetFinalCombinerInputParameterivNV(GLenum variable, GLenum pname, GLint *params)
    void glCombinerStageParameterfvNV(GLenum stage, GLenum pname, GLfloat *params)
    void glGetCombinerStageParameterfvNV(GLenum stage, GLenum pname, GLfloat *params)
    void glFramebufferSampleLocationsfvNV(GLenum target, GLuint start, GLsizei count, GLfloat *v)
    void glNamedFramebufferSampleLocationsfvNV(GLuint framebuffer, GLuint start, GLsizei count, GLfloat *v)
    void glGetBufferParameterui64vNV(GLenum target, GLenum pname, GLuint64EXT *params)
    void glGetIntegerui64vNV(GLenum value, GLuint64EXT *result)
    void glGetNamedBufferParameterui64vNV(GLuint buffer, GLenum pname, GLuint64EXT *params)
    GLboolean glIsBufferResidentNV(GLenum target)
    GLboolean glIsNamedBufferResidentNV(GLuint buffer)
    void glMakeBufferNonResidentNV(GLenum target)
    void glMakeBufferResidentNV(GLenum target, GLenum access)
    void glMakeNamedBufferNonResidentNV(GLuint buffer)
    void glMakeNamedBufferResidentNV(GLuint buffer, GLenum access)
    void glProgramUniformui64NV(GLuint program, GLint location, GLuint64EXT value)
    void glProgramUniformui64vNV(GLuint program, GLint location, GLsizei count, GLuint64EXT *value)
    void glUniformui64NV(GLint location, GLuint64EXT value)
    void glUniformui64vNV(GLint location, GLsizei count, GLuint64EXT *value)
    void glCompressedTexImage3DNV(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, void *data)
    void glCompressedTexSubImage3DNV(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, void *data)
    void glCopyTexSubImage3DNV(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glFramebufferTextureLayerNV(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer)
    void glTexImage3DNV(GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, void *pixels)
    void glTexSubImage3DNV(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *pixels)
    void glTextureBarrierNV()
    void glTexImage2DMultisampleCoverageNV(GLenum target, GLsizei coverageSamples, GLsizei colorSamples, GLint internalFormat, GLsizei width, GLsizei height, GLboolean fixedSampleLocations)
    void glTexImage3DMultisampleCoverageNV(GLenum target, GLsizei coverageSamples, GLsizei colorSamples, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedSampleLocations)
    void glTextureImage2DMultisampleCoverageNV(GLuint texture, GLenum target, GLsizei coverageSamples, GLsizei colorSamples, GLint internalFormat, GLsizei width, GLsizei height, GLboolean fixedSampleLocations)
    void glTextureImage2DMultisampleNV(GLuint texture, GLenum target, GLsizei samples, GLint internalFormat, GLsizei width, GLsizei height, GLboolean fixedSampleLocations)
    void glTextureImage3DMultisampleCoverageNV(GLuint texture, GLenum target, GLsizei coverageSamples, GLsizei colorSamples, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedSampleLocations)
    void glTextureImage3DMultisampleNV(GLuint texture, GLenum target, GLsizei samples, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLboolean fixedSampleLocations)
    void glActiveVaryingNV(GLuint program, GLchar *name)
    void glBeginTransformFeedbackNV(GLenum primitiveMode)
    void glBindBufferBaseNV(GLenum target, GLuint index, GLuint buffer)
    void glBindBufferOffsetNV(GLenum target, GLuint index, GLuint buffer, GLintptr offset)
    void glBindBufferRangeNV(GLenum target, GLuint index, GLuint buffer, GLintptr offset, GLsizeiptr size)
    void glEndTransformFeedbackNV()
    void glGetActiveVaryingNV(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name)
    void glGetTransformFeedbackVaryingNV(GLuint program, GLuint index, GLint *location)
    GLint glGetVaryingLocationNV(GLuint program, GLchar *name)
    void glTransformFeedbackAttribsNV(GLuint count, GLint *attribs, GLenum bufferMode)
    void glTransformFeedbackVaryingsNV(GLuint program, GLsizei count, GLint *locations, GLenum bufferMode)
    void glBindTransformFeedbackNV(GLenum target, GLuint id)
    void glDeleteTransformFeedbacksNV(GLsizei n, GLuint *ids)
    void glDrawTransformFeedbackNV(GLenum mode, GLuint id)
    void glGenTransformFeedbacksNV(GLsizei n, GLuint *ids)
    GLboolean glIsTransformFeedbackNV(GLuint id)
    void glPauseTransformFeedbackNV()
    void glResumeTransformFeedbackNV()
    void glVDPAUFiniNV()
    void glVDPAUGetSurfaceivNV(GLvdpauSurfaceNV surface, GLenum pname, GLsizei bufSize, GLsizei *length, GLint *values)
    void glVDPAUInitNV(void *vdpDevice, void *getProcAddress)
    void glVDPAUIsSurfaceNV(GLvdpauSurfaceNV surface)
    void glVDPAUMapSurfacesNV(GLsizei numSurfaces, GLvdpauSurfaceNV *surfaces)
    GLvdpauSurfaceNV glVDPAURegisterOutputSurfaceNV(void *vdpSurface, GLenum target, GLsizei numTextureNames, GLuint *textureNames)
    GLvdpauSurfaceNV glVDPAURegisterVideoSurfaceNV(void *vdpSurface, GLenum target, GLsizei numTextureNames, GLuint *textureNames)
    void glVDPAUSurfaceAccessNV(GLvdpauSurfaceNV surface, GLenum access)
    void glVDPAUUnmapSurfacesNV(GLsizei numSurface, GLvdpauSurfaceNV *surfaces)
    void glVDPAUUnregisterSurfaceNV(GLvdpauSurfaceNV surface)
    void glFlushVertexArrayRangeNV()
    void glVertexArrayRangeNV(GLsizei length, void *pointer)
    void glGetVertexAttribLi64vNV(GLuint index, GLenum pname, GLint64EXT *params)
    void glGetVertexAttribLui64vNV(GLuint index, GLenum pname, GLuint64EXT *params)
    void glVertexAttribL1i64NV(GLuint index, GLint64EXT x)
    void glVertexAttribL1i64vNV(GLuint index, GLint64EXT *v)
    void glVertexAttribL1ui64NV(GLuint index, GLuint64EXT x)
    void glVertexAttribL1ui64vNV(GLuint index, GLuint64EXT *v)
    void glVertexAttribL2i64NV(GLuint index, GLint64EXT x, GLint64EXT y)
    void glVertexAttribL2i64vNV(GLuint index, GLint64EXT *v)
    void glVertexAttribL2ui64NV(GLuint index, GLuint64EXT x, GLuint64EXT y)
    void glVertexAttribL2ui64vNV(GLuint index, GLuint64EXT *v)
    void glVertexAttribL3i64NV(GLuint index, GLint64EXT x, GLint64EXT y, GLint64EXT z)
    void glVertexAttribL3i64vNV(GLuint index, GLint64EXT *v)
    void glVertexAttribL3ui64NV(GLuint index, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z)
    void glVertexAttribL3ui64vNV(GLuint index, GLuint64EXT *v)
    void glVertexAttribL4i64NV(GLuint index, GLint64EXT x, GLint64EXT y, GLint64EXT z, GLint64EXT w)
    void glVertexAttribL4i64vNV(GLuint index, GLint64EXT *v)
    void glVertexAttribL4ui64NV(GLuint index, GLuint64EXT x, GLuint64EXT y, GLuint64EXT z, GLuint64EXT w)
    void glVertexAttribL4ui64vNV(GLuint index, GLuint64EXT *v)
    void glVertexAttribLFormatNV(GLuint index, GLint size, GLenum type, GLsizei stride)
    void glBufferAddressRangeNV(GLenum pname, GLuint index, GLuint64EXT address, GLsizeiptr length)
    void glColorFormatNV(GLint size, GLenum type, GLsizei stride)
    void glEdgeFlagFormatNV(GLsizei stride)
    void glFogCoordFormatNV(GLenum type, GLsizei stride)
    void glGetIntegerui64i_vNV(GLenum value, GLuint index, GLuint64EXT result[])
    void glIndexFormatNV(GLenum type, GLsizei stride)
    void glNormalFormatNV(GLenum type, GLsizei stride)
    void glSecondaryColorFormatNV(GLint size, GLenum type, GLsizei stride)
    void glTexCoordFormatNV(GLint size, GLenum type, GLsizei stride)
    void glVertexAttribFormatNV(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride)
    void glVertexAttribIFormatNV(GLuint index, GLint size, GLenum type, GLsizei stride)
    void glVertexFormatNV(GLint size, GLenum type, GLsizei stride)
    GLboolean glAreProgramsResidentNV(GLsizei n, GLuint *ids, GLboolean *residences)
    void glBindProgramNV(GLenum target, GLuint id)
    void glDeleteProgramsNV(GLsizei n, GLuint *ids)
    void glExecuteProgramNV(GLenum target, GLuint id, GLfloat *params)
    void glGenProgramsNV(GLsizei n, GLuint *ids)
    void glGetProgramParameterdvNV(GLenum target, GLuint index, GLenum pname, GLdouble *params)
    void glGetProgramParameterfvNV(GLenum target, GLuint index, GLenum pname, GLfloat *params)
    void glGetProgramStringNV(GLuint id, GLenum pname, GLubyte *program)
    void glGetProgramivNV(GLuint id, GLenum pname, GLint *params)
    void glGetTrackMatrixivNV(GLenum target, GLuint address, GLenum pname, GLint *params)
    void glGetVertexAttribPointervNV(GLuint index, GLenum pname, void **pointer)
    void glGetVertexAttribdvNV(GLuint index, GLenum pname, GLdouble *params)
    void glGetVertexAttribfvNV(GLuint index, GLenum pname, GLfloat *params)
    void glGetVertexAttribivNV(GLuint index, GLenum pname, GLint *params)
    GLboolean glIsProgramNV(GLuint id)
    void glLoadProgramNV(GLenum target, GLuint id, GLsizei len, GLubyte *program)
    void glProgramParameter4dNV(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glProgramParameter4dvNV(GLenum target, GLuint index, GLdouble *params)
    void glProgramParameter4fNV(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glProgramParameter4fvNV(GLenum target, GLuint index, GLfloat *params)
    void glProgramParameters4dvNV(GLenum target, GLuint index, GLsizei num, GLdouble *params)
    void glProgramParameters4fvNV(GLenum target, GLuint index, GLsizei num, GLfloat *params)
    void glRequestResidentProgramsNV(GLsizei n, GLuint *ids)
    void glTrackMatrixNV(GLenum target, GLuint address, GLenum matrix, GLenum transform)
    void glVertexAttrib1dNV(GLuint index, GLdouble x)
    void glVertexAttrib1dvNV(GLuint index, GLdouble *v)
    void glVertexAttrib1fNV(GLuint index, GLfloat x)
    void glVertexAttrib1fvNV(GLuint index, GLfloat *v)
    void glVertexAttrib1sNV(GLuint index, GLshort x)
    void glVertexAttrib1svNV(GLuint index, GLshort *v)
    void glVertexAttrib2dNV(GLuint index, GLdouble x, GLdouble y)
    void glVertexAttrib2dvNV(GLuint index, GLdouble *v)
    void glVertexAttrib2fNV(GLuint index, GLfloat x, GLfloat y)
    void glVertexAttrib2fvNV(GLuint index, GLfloat *v)
    void glVertexAttrib2sNV(GLuint index, GLshort x, GLshort y)
    void glVertexAttrib2svNV(GLuint index, GLshort *v)
    void glVertexAttrib3dNV(GLuint index, GLdouble x, GLdouble y, GLdouble z)
    void glVertexAttrib3dvNV(GLuint index, GLdouble *v)
    void glVertexAttrib3fNV(GLuint index, GLfloat x, GLfloat y, GLfloat z)
    void glVertexAttrib3fvNV(GLuint index, GLfloat *v)
    void glVertexAttrib3sNV(GLuint index, GLshort x, GLshort y, GLshort z)
    void glVertexAttrib3svNV(GLuint index, GLshort *v)
    void glVertexAttrib4dNV(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertexAttrib4dvNV(GLuint index, GLdouble *v)
    void glVertexAttrib4fNV(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glVertexAttrib4fvNV(GLuint index, GLfloat *v)
    void glVertexAttrib4sNV(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w)
    void glVertexAttrib4svNV(GLuint index, GLshort *v)
    void glVertexAttrib4ubNV(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w)
    void glVertexAttrib4ubvNV(GLuint index, GLubyte *v)
    void glVertexAttribPointerNV(GLuint index, GLint size, GLenum type, GLsizei stride, void *pointer)
    void glVertexAttribs1dvNV(GLuint index, GLsizei n, GLdouble *v)
    void glVertexAttribs1fvNV(GLuint index, GLsizei n, GLfloat *v)
    void glVertexAttribs1svNV(GLuint index, GLsizei n, GLshort *v)
    void glVertexAttribs2dvNV(GLuint index, GLsizei n, GLdouble *v)
    void glVertexAttribs2fvNV(GLuint index, GLsizei n, GLfloat *v)
    void glVertexAttribs2svNV(GLuint index, GLsizei n, GLshort *v)
    void glVertexAttribs3dvNV(GLuint index, GLsizei n, GLdouble *v)
    void glVertexAttribs3fvNV(GLuint index, GLsizei n, GLfloat *v)
    void glVertexAttribs3svNV(GLuint index, GLsizei n, GLshort *v)
    void glVertexAttribs4dvNV(GLuint index, GLsizei n, GLdouble *v)
    void glVertexAttribs4fvNV(GLuint index, GLsizei n, GLfloat *v)
    void glVertexAttribs4svNV(GLuint index, GLsizei n, GLshort *v)
    void glVertexAttribs4ubvNV(GLuint index, GLsizei n, GLubyte *v)
    void glBeginVideoCaptureNV(GLuint video_capture_slot)
    void glBindVideoCaptureStreamBufferNV(GLuint video_capture_slot, GLuint stream, GLenum frame_region, GLintptrARB offset)
    void glBindVideoCaptureStreamTextureNV(GLuint video_capture_slot, GLuint stream, GLenum frame_region, GLenum target, GLuint texture)
    void glEndVideoCaptureNV(GLuint video_capture_slot)
    void glGetVideoCaptureStreamdvNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLdouble *params)
    void glGetVideoCaptureStreamfvNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLfloat *params)
    void glGetVideoCaptureStreamivNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLint *params)
    void glGetVideoCaptureivNV(GLuint video_capture_slot, GLenum pname, GLint *params)
    GLenum glVideoCaptureNV(GLuint video_capture_slot, GLuint *sequence_num, GLuint64EXT *capture_time)
    void glVideoCaptureStreamParameterdvNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLdouble *params)
    void glVideoCaptureStreamParameterfvNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLfloat *params)
    void glVideoCaptureStreamParameterivNV(GLuint video_capture_slot, GLuint stream, GLenum pname, GLint *params)
    void glDepthRangeArrayfvNV(GLuint first, GLsizei count, GLfloat *v)
    void glDepthRangeIndexedfNV(GLuint index, GLfloat n, GLfloat f)
    void glDisableiNV(GLenum target, GLuint index)
    void glEnableiNV(GLenum target, GLuint index)
    void glGetFloati_vNV(GLenum target, GLuint index, GLfloat *data)
    GLboolean glIsEnablediNV(GLenum target, GLuint index)
    void glScissorArrayvNV(GLuint first, GLsizei count, GLint *v)
    void glScissorIndexedNV(GLuint index, GLint left, GLint bottom, GLsizei width, GLsizei height)
    void glScissorIndexedvNV(GLuint index, GLint *v)
    void glViewportArrayvNV(GLuint first, GLsizei count, GLfloat *v)
    void glViewportIndexedfNV(GLuint index, GLfloat x, GLfloat y, GLfloat w, GLfloat h)
    void glViewportIndexedfvNV(GLuint index, GLfloat *v)
    void glViewportSwizzleNV(GLuint index, GLenum swizzlex, GLenum swizzley, GLenum swizzlez, GLenum swizzlew)
    void glFramebufferTextureMultiviewOVR(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint baseViewIndex, GLsizei numViews)
    void glFramebufferTextureMultisampleMultiviewOVR(GLenum target, GLenum attachment, GLuint texture, GLint level, GLsizei samples, GLint baseViewIndex, GLsizei numViews)
    void glAlphaFuncQCOM(GLenum func, GLclampf ref)
    void glDisableDriverControlQCOM(GLuint driverControl)
    void glEnableDriverControlQCOM(GLuint driverControl)
    void glGetDriverControlStringQCOM(GLuint driverControl, GLsizei bufSize, GLsizei *length, GLchar *driverControlString)
    void glGetDriverControlsQCOM(GLint *num, GLsizei size, GLuint *driverControls)
    void glExtGetBufferPointervQCOM(GLenum target, void **params)
    void glExtGetBuffersQCOM(GLuint *buffers, GLint maxBuffers, GLint *numBuffers)
    void glExtGetFramebuffersQCOM(GLuint *framebuffers, GLint maxFramebuffers, GLint *numFramebuffers)
    void glExtGetRenderbuffersQCOM(GLuint *renderbuffers, GLint maxRenderbuffers, GLint *numRenderbuffers)
    void glExtGetTexLevelParameterivQCOM(GLuint texture, GLenum face, GLint level, GLenum pname, GLint *params)
    void glExtGetTexSubImageQCOM(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, void *texels)
    void glExtGetTexturesQCOM(GLuint *textures, GLint maxTextures, GLint *numTextures)
    void glExtTexObjectStateOverrideiQCOM(GLenum target, GLenum pname, GLint param)
    void glExtGetProgramBinarySourceQCOM(GLuint program, GLenum shadertype, GLchar *source, GLint *length)
    void glExtGetProgramsQCOM(GLuint *programs, GLint maxPrograms, GLint *numPrograms)
    void glExtGetShadersQCOM(GLuint *shaders, GLint maxShaders, GLint *numShaders)
    GLboolean glExtIsProgramBinaryQCOM(GLuint program)
    void glFramebufferFoveationConfigQCOM(GLuint fbo, GLuint numLayers, GLuint focalPointsPerLayer, GLuint requestedFeatures, GLuint *providedFeatures)
    void glFramebufferFoveationParametersQCOM(GLuint fbo, GLuint layer, GLuint focalPoint, GLfloat focalX, GLfloat focalY, GLfloat gainX, GLfloat gainY, GLfloat foveaArea)
    void glFramebufferFetchBarrierQCOM()
    void glEndTilingQCOM(GLbitfield preserveMask)
    void glStartTilingQCOM(GLuint x, GLuint y, GLuint width, GLuint height, GLbitfield preserveMask)
    void glAlphaFuncx(GLenum func, GLclampx ref)
    void glClearColorx(GLclampx red, GLclampx green, GLclampx blue, GLclampx alpha)
    void glClearDepthx(GLclampx depth)
    void glColor4x(GLfixed red, GLfixed green, GLfixed blue, GLfixed alpha)
    void glDepthRangex(GLclampx zNear, GLclampx zFar)
    void glFogx(GLenum pname, GLfixed param)
    void glFogxv(GLenum pname, GLfixed *params)
    void glFrustumf(GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat zNear, GLfloat zFar)
    void glFrustumx(GLfixed left, GLfixed right, GLfixed bottom, GLfixed top, GLfixed zNear, GLfixed zFar)
    void glLightModelx(GLenum pname, GLfixed param)
    void glLightModelxv(GLenum pname, GLfixed *params)
    void glLightx(GLenum light, GLenum pname, GLfixed param)
    void glLightxv(GLenum light, GLenum pname, GLfixed *params)
    void glLineWidthx(GLfixed width)
    void glLoadMatrixx(GLfixed *m)
    void glMaterialx(GLenum face, GLenum pname, GLfixed param)
    void glMaterialxv(GLenum face, GLenum pname, GLfixed *params)
    void glMultMatrixx(GLfixed *m)
    void glMultiTexCoord4x(GLenum target, GLfixed s, GLfixed t, GLfixed r, GLfixed q)
    void glNormal3x(GLfixed nx, GLfixed ny, GLfixed nz)
    void glOrthof(GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat zNear, GLfloat zFar)
    void glOrthox(GLfixed left, GLfixed right, GLfixed bottom, GLfixed top, GLfixed zNear, GLfixed zFar)
    void glPointSizex(GLfixed size)
    void glPolygonOffsetx(GLfixed factor, GLfixed units)
    void glRotatex(GLfixed angle, GLfixed x, GLfixed y, GLfixed z)
    void glSampleCoveragex(GLclampx value, GLboolean invert)
    void glScalex(GLfixed x, GLfixed y, GLfixed z)
    void glTexEnvx(GLenum target, GLenum pname, GLfixed param)
    void glTexEnvxv(GLenum target, GLenum pname, GLfixed *params)
    void glTexParameterx(GLenum target, GLenum pname, GLfixed param)
    void glTranslatex(GLfixed x, GLfixed y, GLfixed z)
    void glClipPlanef(GLenum plane, GLfloat *equation)
    void glClipPlanex(GLenum plane, GLfixed *equation)
    void glGetClipPlanef(GLenum pname, GLfloat eqn[4])
    void glGetClipPlanex(GLenum pname, GLfixed eqn[4])
    void glGetFixedv(GLenum pname, GLfixed *params)
    void glGetLightxv(GLenum light, GLenum pname, GLfixed *params)
    void glGetMaterialxv(GLenum face, GLenum pname, GLfixed *params)
    void glGetTexEnvxv(GLenum env, GLenum pname, GLfixed *params)
    void glGetTexParameterxv(GLenum target, GLenum pname, GLfixed *params)
    void glPointParameterx(GLenum pname, GLfixed param)
    void glPointParameterxv(GLenum pname, GLfixed *params)
    void glPointSizePointerOES(GLenum type, GLsizei stride, void *pointer)
    void glTexParameterxv(GLenum target, GLenum pname, GLfixed *params)
    GLchar *glErrorStringREGAL(GLenum error)
    GLboolean glGetExtensionREGAL(GLchar *ext)
    GLboolean glIsSupportedREGAL(GLchar *ext)
    void glLogMessageCallbackREGAL(GLLOGPROCREGAL callback)
    void *glGetProcAddressREGAL(GLchar *name)
    void glDetailTexFuncSGIS(GLenum target, GLsizei n, GLfloat *points)
    void glGetDetailTexFuncSGIS(GLenum target, GLfloat *points)
    void glFogFuncSGIS(GLsizei n, GLfloat *points)
    void glGetFogFuncSGIS(GLfloat *points)
    void glSampleMaskSGIS(GLclampf value, GLboolean invert)
    void glSamplePatternSGIS(GLenum pattern)
    void glInterleavedTextureCoordSetsSGIS(GLint factor)
    void glSelectTextureCoordSetSGIS(GLenum target)
    void glSelectTextureSGIS(GLenum target)
    void glSelectTextureTransformSGIS(GLenum target)
    void glMultisampleSubRectPosSGIS(GLint x, GLint y)
    void glGetSharpenTexFuncSGIS(GLenum target, GLfloat *points)
    void glSharpenTexFuncSGIS(GLenum target, GLsizei n, GLfloat *points)
    void glTexImage4DSGIS(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLsizei extent, GLint border, GLenum format, GLenum type, void *pixels)
    void glTexSubImage4DSGIS(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint woffset, GLsizei width, GLsizei height, GLsizei depth, GLsizei extent, GLenum format, GLenum type, void *pixels)
    void glGetTexFilterFuncSGIS(GLenum target, GLenum filter, GLfloat *weights)
    void glTexFilterFuncSGIS(GLenum target, GLenum filter, GLsizei n, GLfloat *weights)
    void glAsyncMarkerSGIX(GLuint marker)
    void glDeleteAsyncMarkersSGIX(GLuint marker, GLsizei range)
    GLint glFinishAsyncSGIX(GLuint *markerp)
    GLuint glGenAsyncMarkersSGIX(GLsizei range)
    GLboolean glIsAsyncMarkerSGIX(GLuint marker)
    GLint glPollAsyncSGIX(GLuint *markerp)
    void glAddressSpace(GLenum space, GLbitfield mask)
    GLint glDataPipe(GLenum space)
    void glFlushRasterSGIX()
    void glFogLayersSGIX(GLsizei n, GLfloat *points)
    void glGetFogLayersSGIX(GLfloat *points)
    void glTextureFogSGIX(GLenum pname)
    void glFragmentColorMaterialSGIX(GLenum face, GLenum mode)
    void glFragmentLightModelfSGIX(GLenum pname, GLfloat param)
    void glFragmentLightModelfvSGIX(GLenum pname, GLfloat *params)
    void glFragmentLightModeliSGIX(GLenum pname, GLint param)
    void glFragmentLightModelivSGIX(GLenum pname, GLint *params)
    void glFragmentLightfSGIX(GLenum light, GLenum pname, GLfloat param)
    void glFragmentLightfvSGIX(GLenum light, GLenum pname, GLfloat *params)
    void glFragmentLightiSGIX(GLenum light, GLenum pname, GLint param)
    void glFragmentLightivSGIX(GLenum light, GLenum pname, GLint *params)
    void glFragmentMaterialfSGIX(GLenum face, GLenum pname, GLfloat param)
    void glFragmentMaterialfvSGIX(GLenum face, GLenum pname, GLfloat *params)
    void glFragmentMaterialiSGIX(GLenum face, GLenum pname, GLint param)
    void glFragmentMaterialivSGIX(GLenum face, GLenum pname, GLint *params)
    void glGetFragmentLightfvSGIX(GLenum light, GLenum value, GLfloat *data)
    void glGetFragmentLightivSGIX(GLenum light, GLenum value, GLint *data)
    void glGetFragmentMaterialfvSGIX(GLenum face, GLenum pname, GLfloat *data)
    void glGetFragmentMaterialivSGIX(GLenum face, GLenum pname, GLint *data)
    void glFrameZoomSGIX(GLint factor)
    void glIglooInterfaceSGIX(GLenum pname, void *param)
    void glAllocMPEGPredictorsSGIX(GLsizei width, GLsizei height, GLsizei n, GLuint *predictors)
    void glDeleteMPEGPredictorsSGIX(GLsizei n, GLuint *predictors)
    void glGenMPEGPredictorsSGIX(GLsizei n, GLuint *predictors)
    void glGetMPEGParameterfvSGIX(GLenum target, GLenum pname, GLfloat *params)
    void glGetMPEGParameterivSGIX(GLenum target, GLenum pname, GLint *params)
    void glGetMPEGPredictorSGIX(GLenum target, GLenum format, GLenum type, void *pixels)
    void glGetMPEGQuantTableubv(GLenum target, GLubyte *values)
    GLboolean glIsMPEGPredictorSGIX(GLuint predictor)
    void glMPEGPredictorSGIX(GLenum target, GLenum format, GLenum type, void *pixels)
    void glMPEGQuantTableubv(GLenum target, GLubyte *values)
    void glSwapMPEGPredictorsSGIX(GLenum target0, GLenum target1)
    void glGetNonlinLightfvSGIX(GLenum light, GLenum pname, GLint *terms, GLfloat *data)
    void glGetNonlinMaterialfvSGIX(GLenum face, GLenum pname, GLint *terms, GLfloat *data)
    void glNonlinLightfvSGIX(GLenum light, GLenum pname, GLint terms, GLfloat *params)
    void glNonlinMaterialfvSGIX(GLenum face, GLenum pname, GLint terms, GLfloat *params)
    void glPixelTexGenSGIX(GLenum mode)
    void glDeformSGIX(GLbitfield mask)
    void glLoadIdentityDeformationMapSGIX(GLbitfield mask)
    void glMeshBreadthSGIX(GLint breadth)
    void glMeshStrideSGIX(GLint stride)
    void glReferencePlaneSGIX(GLdouble *equation)
    void glSpriteParameterfSGIX(GLenum pname, GLfloat param)
    void glSpriteParameterfvSGIX(GLenum pname, GLfloat *params)
    void glSpriteParameteriSGIX(GLenum pname, GLint param)
    void glSpriteParameterivSGIX(GLenum pname, GLint *params)
    void glTagSampleBufferSGIX()
    void glGetVectorOperationSGIX(GLenum operation)
    void glVectorOperationSGIX(GLenum operation)
    GLboolean glAreVertexArraysResidentSGIX(GLsizei n, GLuint *arrays, GLboolean *residences)
    void glBindVertexArraySGIX(GLuint array)
    void glDeleteVertexArraysSGIX(GLsizei n, GLuint *arrays)
    void glGenVertexArraysSGIX(GLsizei n, GLuint *arrays)
    GLboolean glIsVertexArraySGIX(GLuint array)
    void glPrioritizeVertexArraysSGIX(GLsizei n, GLuint *arrays, GLclampf *priorities)
    void glColorTableParameterfvSGI(GLenum target, GLenum pname, GLfloat *params)
    void glColorTableParameterivSGI(GLenum target, GLenum pname, GLint *params)
    void glColorTableSGI(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, void *table)
    void glCopyColorTableSGI(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width)
    void glGetColorTableParameterfvSGI(GLenum target, GLenum pname, GLfloat *params)
    void glGetColorTableParameterivSGI(GLenum target, GLenum pname, GLint *params)
    void glGetColorTableSGI(GLenum target, GLenum format, GLenum type, void *table)
    void glGetPixelTransformParameterfvSGI(GLenum target, GLenum pname, GLfloat *params)
    void glGetPixelTransformParameterivSGI(GLenum target, GLenum pname, GLint *params)
    void glPixelTransformParameterfSGI(GLenum target, GLenum pname, GLfloat param)
    void glPixelTransformParameterfvSGI(GLenum target, GLenum pname, GLfloat *params)
    void glPixelTransformParameteriSGI(GLenum target, GLenum pname, GLint param)
    void glPixelTransformParameterivSGI(GLenum target, GLenum pname, GLint *params)
    void glPixelTransformSGI(GLenum target)
    void glFinishTextureSUNX()
    void glGlobalAlphaFactorbSUN(GLbyte factor)
    void glGlobalAlphaFactordSUN(GLdouble factor)
    void glGlobalAlphaFactorfSUN(GLfloat factor)
    void glGlobalAlphaFactoriSUN(GLint factor)
    void glGlobalAlphaFactorsSUN(GLshort factor)
    void glGlobalAlphaFactorubSUN(GLubyte factor)
    void glGlobalAlphaFactoruiSUN(GLuint factor)
    void glGlobalAlphaFactorusSUN(GLushort factor)
    void glReadVideoPixelsSUN(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glReplacementCodePointerSUN(GLenum type, GLsizei stride, void *pointer)
    void glReplacementCodeubSUN(GLubyte code)
    void glReplacementCodeubvSUN(GLubyte *code)
    void glReplacementCodeuiSUN(GLuint code)
    void glReplacementCodeuivSUN(GLuint *code)
    void glReplacementCodeusSUN(GLushort code)
    void glReplacementCodeusvSUN(GLushort *code)
    void glColor3fVertex3fSUN(GLfloat r, GLfloat g, GLfloat b, GLfloat x, GLfloat y, GLfloat z)
    void glColor3fVertex3fvSUN(GLfloat *c, GLfloat *v)
    void glColor4fNormal3fVertex3fSUN(GLfloat r, GLfloat g, GLfloat b, GLfloat a, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glColor4fNormal3fVertex3fvSUN(GLfloat *c, GLfloat *n, GLfloat *v)
    void glColor4ubVertex2fSUN(GLubyte r, GLubyte g, GLubyte b, GLubyte a, GLfloat x, GLfloat y)
    void glColor4ubVertex2fvSUN(GLubyte *c, GLfloat *v)
    void glColor4ubVertex3fSUN(GLubyte r, GLubyte g, GLubyte b, GLubyte a, GLfloat x, GLfloat y, GLfloat z)
    void glColor4ubVertex3fvSUN(GLubyte *c, GLfloat *v)
    void glNormal3fVertex3fSUN(GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glNormal3fVertex3fvSUN(GLfloat *n, GLfloat *v)
    void glReplacementCodeuiColor3fVertex3fSUN(GLuint rc, GLfloat r, GLfloat g, GLfloat b, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiColor3fVertex3fvSUN(GLuint *rc, GLfloat *c, GLfloat *v)
    void glReplacementCodeuiColor4fNormal3fVertex3fSUN(GLuint rc, GLfloat r, GLfloat g, GLfloat b, GLfloat a, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiColor4fNormal3fVertex3fvSUN(GLuint *rc, GLfloat *c, GLfloat *n, GLfloat *v)
    void glReplacementCodeuiColor4ubVertex3fSUN(GLuint rc, GLubyte r, GLubyte g, GLubyte b, GLubyte a, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiColor4ubVertex3fvSUN(GLuint *rc, GLubyte *c, GLfloat *v)
    void glReplacementCodeuiNormal3fVertex3fSUN(GLuint rc, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiNormal3fVertex3fvSUN(GLuint *rc, GLfloat *n, GLfloat *v)
    void glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(GLuint rc, GLfloat s, GLfloat t, GLfloat r, GLfloat g, GLfloat b, GLfloat a, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN(GLuint *rc, GLfloat *tc, GLfloat *c, GLfloat *n, GLfloat *v)
    void glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(GLuint rc, GLfloat s, GLfloat t, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN(GLuint *rc, GLfloat *tc, GLfloat *n, GLfloat *v)
    void glReplacementCodeuiTexCoord2fVertex3fSUN(GLuint rc, GLfloat s, GLfloat t, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiTexCoord2fVertex3fvSUN(GLuint *rc, GLfloat *tc, GLfloat *v)
    void glReplacementCodeuiVertex3fSUN(GLuint rc, GLfloat x, GLfloat y, GLfloat z)
    void glReplacementCodeuiVertex3fvSUN(GLuint *rc, GLfloat *v)
    void glTexCoord2fColor3fVertex3fSUN(GLfloat s, GLfloat t, GLfloat r, GLfloat g, GLfloat b, GLfloat x, GLfloat y, GLfloat z)
    void glTexCoord2fColor3fVertex3fvSUN(GLfloat *tc, GLfloat *c, GLfloat *v)
    void glTexCoord2fColor4fNormal3fVertex3fSUN(GLfloat s, GLfloat t, GLfloat r, GLfloat g, GLfloat b, GLfloat a, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glTexCoord2fColor4fNormal3fVertex3fvSUN(GLfloat *tc, GLfloat *c, GLfloat *n, GLfloat *v)
    void glTexCoord2fColor4ubVertex3fSUN(GLfloat s, GLfloat t, GLubyte r, GLubyte g, GLubyte b, GLubyte a, GLfloat x, GLfloat y, GLfloat z)
    void glTexCoord2fColor4ubVertex3fvSUN(GLfloat *tc, GLubyte *c, GLfloat *v)
    void glTexCoord2fNormal3fVertex3fSUN(GLfloat s, GLfloat t, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z)
    void glTexCoord2fNormal3fVertex3fvSUN(GLfloat *tc, GLfloat *n, GLfloat *v)
    void glTexCoord2fVertex3fSUN(GLfloat s, GLfloat t, GLfloat x, GLfloat y, GLfloat z)
    void glTexCoord2fVertex3fvSUN(GLfloat *tc, GLfloat *v)
    void glTexCoord4fColor4fNormal3fVertex4fSUN(GLfloat s, GLfloat t, GLfloat p, GLfloat q, GLfloat r, GLfloat g, GLfloat b, GLfloat a, GLfloat nx, GLfloat ny, GLfloat nz, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glTexCoord4fColor4fNormal3fVertex4fvSUN(GLfloat *tc, GLfloat *c, GLfloat *n, GLfloat *v)
    void glTexCoord4fVertex4fSUN(GLfloat s, GLfloat t, GLfloat p, GLfloat q, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glTexCoord4fVertex4fvSUN(GLfloat *tc, GLfloat *v)
    void glAddSwapHintRectWIN(GLint x, GLint y, GLsizei width, GLsizei height)
    void glAccum(GLenum op, GLfloat value)
    void glAlphaFunc(GLenum func, GLclampf ref)
    GLboolean glAreTexturesResident(GLsizei n, GLuint *textures, GLboolean *residences)
    void glArrayElement(GLint i)
    void glBegin(GLenum mode)
    void glBindTexture(GLenum target, GLuint texture)
    void glBitmap(GLsizei width, GLsizei height, GLfloat xorig, GLfloat yorig, GLfloat xmove, GLfloat ymove, GLubyte *bitmap)
    void glBlendFunc(GLenum sfactor, GLenum dfactor)
    void glCallList(GLuint list)
    void glCallLists(GLsizei n, GLenum type, void *lists)
    void glClear(GLbitfield mask)
    void glClearAccum(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
    void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
    void glClearDepth(GLclampd depth)
    void glClearIndex(GLfloat c)
    void glClearStencil(GLint s)
    void glClipPlane(GLenum plane, GLdouble *equation)
    void glColor3b(GLbyte red, GLbyte green, GLbyte blue)
    void glColor3bv(GLbyte *v)
    void glColor3d(GLdouble red, GLdouble green, GLdouble blue)
    void glColor3dv(GLdouble *v)
    void glColor3f(GLfloat red, GLfloat green, GLfloat blue)
    void glColor3fv(GLfloat *v)
    void glColor3i(GLint red, GLint green, GLint blue)
    void glColor3iv(GLint *v)
    void glColor3s(GLshort red, GLshort green, GLshort blue)
    void glColor3sv(GLshort *v)
    void glColor3ub(GLubyte red, GLubyte green, GLubyte blue)
    void glColor3ubv(GLubyte *v)
    void glColor3ui(GLuint red, GLuint green, GLuint blue)
    void glColor3uiv(GLuint *v)
    void glColor3us(GLushort red, GLushort green, GLushort blue)
    void glColor3usv(GLushort *v)
    void glColor4b(GLbyte red, GLbyte green, GLbyte blue, GLbyte alpha)
    void glColor4bv(GLbyte *v)
    void glColor4d(GLdouble red, GLdouble green, GLdouble blue, GLdouble alpha)
    void glColor4dv(GLdouble *v)
    void glColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
    void glColor4fv(GLfloat *v)
    void glColor4i(GLint red, GLint green, GLint blue, GLint alpha)
    void glColor4iv(GLint *v)
    void glColor4s(GLshort red, GLshort green, GLshort blue, GLshort alpha)
    void glColor4sv(GLshort *v)
    void glColor4ub(GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha)
    void glColor4ubv(GLubyte *v)
    void glColor4ui(GLuint red, GLuint green, GLuint blue, GLuint alpha)
    void glColor4uiv(GLuint *v)
    void glColor4us(GLushort red, GLushort green, GLushort blue, GLushort alpha)
    void glColor4usv(GLushort *v)
    void glColorMask(GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha)
    void glColorMaterial(GLenum face, GLenum mode)
    void glColorPointer(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glCopyPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum type)
    void glCopyTexImage1D(GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLint border)
    void glCopyTexImage2D(GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border)
    void glCopyTexSubImage1D(GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width)
    void glCopyTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
    void glCullFace(GLenum mode)
    void glDeleteLists(GLuint list, GLsizei range)
    void glDeleteTextures(GLsizei n, GLuint *textures)
    void glDepthFunc(GLenum func)
    void glDepthMask(GLboolean flag)
    void glDepthRange(GLclampd zNear, GLclampd zFar)
    void glDisable(GLenum cap)
    void glDisableClientState(GLenum array)
    void glDrawArrays(GLenum mode, GLint first, GLsizei count)
    void glDrawBuffer(GLenum mode)
    void glDrawElements(GLenum mode, GLsizei count, GLenum type, void *indices)
    void glDrawPixels(GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glEdgeFlag(GLboolean flag)
    void glEdgeFlagPointer(GLsizei stride, void *pointer)
    void glEdgeFlagv(GLboolean *flag)
    void glEnable(GLenum cap)
    void glEnableClientState(GLenum array)
    void glEnd()
    void glEndList()
    void glEvalCoord1d(GLdouble u)
    void glEvalCoord1dv(GLdouble *u)
    void glEvalCoord1f(GLfloat u)
    void glEvalCoord1fv(GLfloat *u)
    void glEvalCoord2d(GLdouble u, GLdouble v)
    void glEvalCoord2dv(GLdouble *u)
    void glEvalCoord2f(GLfloat u, GLfloat v)
    void glEvalCoord2fv(GLfloat *u)
    void glEvalMesh1(GLenum mode, GLint i1, GLint i2)
    void glEvalMesh2(GLenum mode, GLint i1, GLint i2, GLint j1, GLint j2)
    void glEvalPoint1(GLint i)
    void glEvalPoint2(GLint i, GLint j)
    void glFeedbackBuffer(GLsizei size, GLenum type, GLfloat *buffer)
    void glFinish()
    void glFlush()
    void glFogf(GLenum pname, GLfloat param)
    void glFogfv(GLenum pname, GLfloat *params)
    void glFogi(GLenum pname, GLint param)
    void glFogiv(GLenum pname, GLint *params)
    void glFrontFace(GLenum mode)
    void glFrustum(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar)
    GLuint glGenLists(GLsizei range)
    void glGenTextures(GLsizei n, GLuint *textures)
    void glGetBooleanv(GLenum pname, GLboolean *params)
    void glGetClipPlane(GLenum plane, GLdouble *equation)
    void glGetDoublev(GLenum pname, GLdouble *params)
    GLenum glGetError()
    void glGetFloatv(GLenum pname, GLfloat *params)
    void glGetIntegerv(GLenum pname, GLint *params)
    void glGetLightfv(GLenum light, GLenum pname, GLfloat *params)
    void glGetLightiv(GLenum light, GLenum pname, GLint *params)
    void glGetMapdv(GLenum target, GLenum query, GLdouble *v)
    void glGetMapfv(GLenum target, GLenum query, GLfloat *v)
    void glGetMapiv(GLenum target, GLenum query, GLint *v)
    void glGetMaterialfv(GLenum face, GLenum pname, GLfloat *params)
    void glGetMaterialiv(GLenum face, GLenum pname, GLint *params)
    void glGetPixelMapfv(GLenum map, GLfloat *values)
    void glGetPixelMapuiv(GLenum map, GLuint *values)
    void glGetPixelMapusv(GLenum map, GLushort *values)
    void glGetPointerv(GLenum pname, void * *params)
    void glGetPolygonStipple(GLubyte *mask)
    GLubyte glGetString(GLenum name)
    void glGetTexEnvfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetTexEnviv(GLenum target, GLenum pname, GLint *params)
    void glGetTexGendv(GLenum coord, GLenum pname, GLdouble *params)
    void glGetTexGenfv(GLenum coord, GLenum pname, GLfloat *params)
    void glGetTexGeniv(GLenum coord, GLenum pname, GLint *params)
    void glGetTexImage(GLenum target, GLint level, GLenum format, GLenum type, void *pixels)
    void glGetTexLevelParameterfv(GLenum target, GLint level, GLenum pname, GLfloat *params)
    void glGetTexLevelParameteriv(GLenum target, GLint level, GLenum pname, GLint *params)
    void glGetTexParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glGetTexParameteriv(GLenum target, GLenum pname, GLint *params)
    void glHint(GLenum target, GLenum mode)
    void glIndexMask(GLuint mask)
    void glIndexPointer(GLenum type, GLsizei stride, void *pointer)
    void glIndexd(GLdouble c)
    void glIndexdv(GLdouble *c)
    void glIndexf(GLfloat c)
    void glIndexfv(GLfloat *c)
    void glIndexi(GLint c)
    void glIndexiv(GLint *c)
    void glIndexs(GLshort c)
    void glIndexsv(GLshort *c)
    void glIndexub(GLubyte c)
    void glIndexubv(GLubyte *c)
    void glInitNames()
    void glInterleavedArrays(GLenum format, GLsizei stride, void *pointer)
    GLboolean glIsEnabled(GLenum cap)
    GLboolean glIsList(GLuint list)
    GLboolean glIsTexture(GLuint texture)
    void glLightModelf(GLenum pname, GLfloat param)
    void glLightModelfv(GLenum pname, GLfloat *params)
    void glLightModeli(GLenum pname, GLint param)
    void glLightModeliv(GLenum pname, GLint *params)
    void glLightf(GLenum light, GLenum pname, GLfloat param)
    void glLightfv(GLenum light, GLenum pname, GLfloat *params)
    void glLighti(GLenum light, GLenum pname, GLint param)
    void glLightiv(GLenum light, GLenum pname, GLint *params)
    void glLineStipple(GLint factor, GLushort pattern)
    void glLineWidth(GLfloat width)
    void glListBase(GLuint base)
    void glLoadIdentity()
    void glLoadMatrixd(GLdouble *m)
    void glLoadMatrixf(GLfloat *m)
    void glLoadName(GLuint name)
    void glLogicOp(GLenum opcode)
    void glMap1d(GLenum target, GLdouble u1, GLdouble u2, GLint stride, GLint order, GLdouble *points)
    void glMap1f(GLenum target, GLfloat u1, GLfloat u2, GLint stride, GLint order, GLfloat *points)
    void glMap2d(GLenum target, GLdouble u1, GLdouble u2, GLint ustride, GLint uorder, GLdouble v1, GLdouble v2, GLint vstride, GLint vorder, GLdouble *points)
    void glMap2f(GLenum target, GLfloat u1, GLfloat u2, GLint ustride, GLint uorder, GLfloat v1, GLfloat v2, GLint vstride, GLint vorder, GLfloat *points)
    void glMapGrid1d(GLint un, GLdouble u1, GLdouble u2)
    void glMapGrid1f(GLint un, GLfloat u1, GLfloat u2)
    void glMapGrid2d(GLint un, GLdouble u1, GLdouble u2, GLint vn, GLdouble v1, GLdouble v2)
    void glMapGrid2f(GLint un, GLfloat u1, GLfloat u2, GLint vn, GLfloat v1, GLfloat v2)
    void glMaterialf(GLenum face, GLenum pname, GLfloat param)
    void glMaterialfv(GLenum face, GLenum pname, GLfloat *params)
    void glMateriali(GLenum face, GLenum pname, GLint param)
    void glMaterialiv(GLenum face, GLenum pname, GLint *params)
    void glMatrixMode(GLenum mode)
    void glMultMatrixd(GLdouble *m)
    void glMultMatrixf(GLfloat *m)
    void glNewList(GLuint list, GLenum mode)
    void glNormal3b(GLbyte nx, GLbyte ny, GLbyte nz)
    void glNormal3bv(GLbyte *v)
    void glNormal3d(GLdouble nx, GLdouble ny, GLdouble nz)
    void glNormal3dv(GLdouble *v)
    void glNormal3f(GLfloat nx, GLfloat ny, GLfloat nz)
    void glNormal3fv(GLfloat *v)
    void glNormal3i(GLint nx, GLint ny, GLint nz)
    void glNormal3iv(GLint *v)
    void glNormal3s(GLshort nx, GLshort ny, GLshort nz)
    void glNormal3sv(GLshort *v)
    void glNormalPointer(GLenum type, GLsizei stride, void *pointer)
    void glOrtho(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar)
    void glPassThrough(GLfloat token)
    void glPixelMapfv(GLenum map, GLsizei mapsize, GLfloat *values)
    void glPixelMapuiv(GLenum map, GLsizei mapsize, GLuint *values)
    void glPixelMapusv(GLenum map, GLsizei mapsize, GLushort *values)
    void glPixelStoref(GLenum pname, GLfloat param)
    void glPixelStorei(GLenum pname, GLint param)
    void glPixelTransferf(GLenum pname, GLfloat param)
    void glPixelTransferi(GLenum pname, GLint param)
    void glPixelZoom(GLfloat xfactor, GLfloat yfactor)
    void glPointSize(GLfloat size)
    void glPolygonMode(GLenum face, GLenum mode)
    void glPolygonOffset(GLfloat factor, GLfloat units)
    void glPolygonStipple(GLubyte *mask)
    void glPopAttrib()
    void glPopClientAttrib()
    void glPopMatrix()
    void glPopName()
    void glPrioritizeTextures(GLsizei n, GLuint *textures, GLclampf *priorities)
    void glPushAttrib(GLbitfield mask)
    void glPushClientAttrib(GLbitfield mask)
    void glPushMatrix()
    void glPushName(GLuint name)
    void glRasterPos2d(GLdouble x, GLdouble y)
    void glRasterPos2dv(GLdouble *v)
    void glRasterPos2f(GLfloat x, GLfloat y)
    void glRasterPos2fv(GLfloat *v)
    void glRasterPos2i(GLint x, GLint y)
    void glRasterPos2iv(GLint *v)
    void glRasterPos2s(GLshort x, GLshort y)
    void glRasterPos2sv(GLshort *v)
    void glRasterPos3d(GLdouble x, GLdouble y, GLdouble z)
    void glRasterPos3dv(GLdouble *v)
    void glRasterPos3f(GLfloat x, GLfloat y, GLfloat z)
    void glRasterPos3fv(GLfloat *v)
    void glRasterPos3i(GLint x, GLint y, GLint z)
    void glRasterPos3iv(GLint *v)
    void glRasterPos3s(GLshort x, GLshort y, GLshort z)
    void glRasterPos3sv(GLshort *v)
    void glRasterPos4d(GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glRasterPos4dv(GLdouble *v)
    void glRasterPos4f(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glRasterPos4fv(GLfloat *v)
    void glRasterPos4i(GLint x, GLint y, GLint z, GLint w)
    void glRasterPos4iv(GLint *v)
    void glRasterPos4s(GLshort x, GLshort y, GLshort z, GLshort w)
    void glRasterPos4sv(GLshort *v)
    void glReadBuffer(GLenum mode)
    void glReadPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glRectd(GLdouble x1, GLdouble y1, GLdouble x2, GLdouble y2)
    void glRectdv(GLdouble *v1, GLdouble *v2)
    void glRectf(GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2)
    void glRectfv(GLfloat *v1, GLfloat *v2)
    void glRecti(GLint x1, GLint y1, GLint x2, GLint y2)
    void glRectiv(GLint *v1, GLint *v2)
    void glRects(GLshort x1, GLshort y1, GLshort x2, GLshort y2)
    void glRectsv(GLshort *v1, GLshort *v2)
    GLint glRenderMode(GLenum mode)
    void glRotated(GLdouble angle, GLdouble x, GLdouble y, GLdouble z)
    void glRotatef(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
    void glScaled(GLdouble x, GLdouble y, GLdouble z)
    void glScalef(GLfloat x, GLfloat y, GLfloat z)
    void glScissor(GLint x, GLint y, GLsizei width, GLsizei height)
    void glSelectBuffer(GLsizei size, GLuint *buffer)
    void glShadeModel(GLenum mode)
    void glStencilFunc(GLenum func, GLint ref, GLuint mask)
    void glStencilMask(GLuint mask)
    void glStencilOp(GLenum fail, GLenum zfail, GLenum zpass)
    void glTexCoord1d(GLdouble s)
    void glTexCoord1dv(GLdouble *v)
    void glTexCoord1f(GLfloat s)
    void glTexCoord1fv(GLfloat *v)
    void glTexCoord1i(GLint s)
    void glTexCoord1iv(GLint *v)
    void glTexCoord1s(GLshort s)
    void glTexCoord1sv(GLshort *v)
    void glTexCoord2d(GLdouble s, GLdouble t)
    void glTexCoord2dv(GLdouble *v)
    void glTexCoord2f(GLfloat s, GLfloat t)
    void glTexCoord2fv(GLfloat *v)
    void glTexCoord2i(GLint s, GLint t)
    void glTexCoord2iv(GLint *v)
    void glTexCoord2s(GLshort s, GLshort t)
    void glTexCoord2sv(GLshort *v)
    void glTexCoord3d(GLdouble s, GLdouble t, GLdouble r)
    void glTexCoord3dv(GLdouble *v)
    void glTexCoord3f(GLfloat s, GLfloat t, GLfloat r)
    void glTexCoord3fv(GLfloat *v)
    void glTexCoord3i(GLint s, GLint t, GLint r)
    void glTexCoord3iv(GLint *v)
    void glTexCoord3s(GLshort s, GLshort t, GLshort r)
    void glTexCoord3sv(GLshort *v)
    void glTexCoord4d(GLdouble s, GLdouble t, GLdouble r, GLdouble q)
    void glTexCoord4dv(GLdouble *v)
    void glTexCoord4f(GLfloat s, GLfloat t, GLfloat r, GLfloat q)
    void glTexCoord4fv(GLfloat *v)
    void glTexCoord4i(GLint s, GLint t, GLint r, GLint q)
    void glTexCoord4iv(GLint *v)
    void glTexCoord4s(GLshort s, GLshort t, GLshort r, GLshort q)
    void glTexCoord4sv(GLshort *v)
    void glTexCoordPointer(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glTexEnvf(GLenum target, GLenum pname, GLfloat param)
    void glTexEnvfv(GLenum target, GLenum pname, GLfloat *params)
    void glTexEnvi(GLenum target, GLenum pname, GLint param)
    void glTexEnviv(GLenum target, GLenum pname, GLint *params)
    void glTexGend(GLenum coord, GLenum pname, GLdouble param)
    void glTexGendv(GLenum coord, GLenum pname, GLdouble *params)
    void glTexGenf(GLenum coord, GLenum pname, GLfloat param)
    void glTexGenfv(GLenum coord, GLenum pname, GLfloat *params)
    void glTexGeni(GLenum coord, GLenum pname, GLint param)
    void glTexGeniv(GLenum coord, GLenum pname, GLint *params)
    void glTexImage1D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLint border, GLenum format, GLenum type, void *pixels)
    void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, void *pixels)
    void glTexParameterf(GLenum target, GLenum pname, GLfloat param)
    void glTexParameterfv(GLenum target, GLenum pname, GLfloat *params)
    void glTexParameteri(GLenum target, GLenum pname, GLint param)
    void glTexParameteriv(GLenum target, GLenum pname, GLint *params)
    void glTexSubImage1D(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, void *pixels)
    void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *pixels)
    void glTranslated(GLdouble x, GLdouble y, GLdouble z)
    void glTranslatef(GLfloat x, GLfloat y, GLfloat z)
    void glVertex2d(GLdouble x, GLdouble y)
    void glVertex2dv(GLdouble *v)
    void glVertex2f(GLfloat x, GLfloat y)
    void glVertex2fv(GLfloat *v)
    void glVertex2i(GLint x, GLint y)
    void glVertex2iv(GLint *v)
    void glVertex2s(GLshort x, GLshort y)
    void glVertex2sv(GLshort *v)
    void glVertex3d(GLdouble x, GLdouble y, GLdouble z)
    void glVertex3dv(GLdouble *v)
    void glVertex3f(GLfloat x, GLfloat y, GLfloat z)
    void glVertex3fv(GLfloat *v)
    void glVertex3i(GLint x, GLint y, GLint z)
    void glVertex3iv(GLint *v)
    void glVertex3s(GLshort x, GLshort y, GLshort z)
    void glVertex3sv(GLshort *v)
    void glVertex4d(GLdouble x, GLdouble y, GLdouble z, GLdouble w)
    void glVertex4dv(GLdouble *v)
    void glVertex4f(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
    void glVertex4fv(GLfloat *v)
    void glVertex4i(GLint x, GLint y, GLint z, GLint w)
    void glVertex4iv(GLint *v)
    void glVertex4s(GLshort x, GLshort y, GLshort z, GLshort w)
    void glVertex4sv(GLshort *v)
    void glVertexPointer(GLint size, GLenum type, GLsizei stride, void *pointer)
    void glViewport(GLint x, GLint y, GLsizei width, GLsizei height)